define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.logger',
    'ganttapi.gantt/enums/barHeight',
    'ganttapi.gantt/enums/barColor',
    'ganttapi.gantt/enums/labelHeight',
    'ganttapi.gantt/enums/labelPosition',
    'ganttapi.gantt/enums/recordTypes',
    'ganttapi.gantt/enums/barStatus',
    'ganttapi.gantt/enums/barProgressStyle',
    'markup!ganttapi.gantt/templates/bars.htm:.htm-bar-tooltip',
    'markup!ganttapi.gantt/templates/bars.htm:.htm-gantt-bar',
    'markup!ganttapi.gantt/templates/bars.htm:.htm-summary-bar',
    'markup!ganttapi.gantt/templates/bars.htm:.htm-milestone-bar',
    'markup!ganttapi.gantt/templates/bars.htm:.htm-bar-label'
], function ($, _, logger, BAR_HEIGHT, BAR_COLOR, LABEL_HEIGHT, LABEL_POSITION, RECORD_TYPES, BAR_STATUS, PROGRESS_STYLE, tooltipTemplate, barTemplate, summaryTemplate, milestoneTemplate, labelTemplate) {
    'use strict';
    var defaults = {
        name : 'No bar name',
        type : 'current',
        render : {
            disableOnSummary : false,
            disableOnMilestone : false,
            classes : []
        }
    };
    var BarRenderer = function (options) {
        var that = this;
        this.ganttView = options.ganttView;
        this.timescale = options.timescale;
        this.widget = options.widget;
        this.definition = _.merge({}, defaults, options.definition, options.config, function (a, b) {
            if (_.isArray(a)) {
                return b;
            }
        });
        this.id = options.id;
        this.config = options.config;
        var baseOverrides = this.ganttView.options.bars.rendererFns;
        // override mediator functions from options
        _.forEach(baseOverrides, function (fnc, fncName) {
            that[fncName] = fnc || that[fncName];
        });
        var rendererFns = this.definition.rendererFns;
        // override mediator functions from options
        _.forEach(rendererFns, function (fnc, fncName) {
            that[fncName] = fnc || that[fncName];
        });
    };

    BarRenderer.prototype = {
        constructor : BarRenderer,

        updateTimescale : _.debounce(function () {
            this.ganttView.updateTimescaleRange();
        }, 150),

        getItemCache : function (options) {
            var itemCache = this.ganttView.ganttItems[options.recordId];
            if (itemCache == null) {
                itemCache = {};
                this.ganttView.ganttItems[options.recordId] = itemCache;
            }
            // if (itemCache[this.id] == null) {
            var barCache = this.timescale.getUnits(options.barAttributes.data);
            if (options.ganttAttributes.type === RECORD_TYPES.START_MILESTONE) {
                barCache.render.right = barCache.render.left + BAR_HEIGHT[this.definition.render.size].height;
                barCache.render.width = BAR_HEIGHT[this.definition.render.size].height;
            } else if (options.ganttAttributes.type === RECORD_TYPES.FINISH_MILESTONE) {
                barCache.render.left = barCache.render.right;
                barCache.render.right = barCache.render.left + BAR_HEIGHT[this.definition.render.size].height;
                barCache.render.width = BAR_HEIGHT[this.definition.render.size].height;
            }

            if (barCache.render.left < 0) {
                this.ganttView.timescale.newMin = options.barAttributes.data.start.isBefore(options.barAttributes.data.end) ? options.barAttributes.data.start : options.barAttributes.data.end;
                this.updateTimescale();
            }

            if (barCache.render.right > this.timescale.settings.width) {
                this.ganttView.timescale.newMax = options.barAttributes.data.end.isAfter(options.barAttributes.data.start) ? options.barAttributes.data.end : options.barAttributes.data.start;
                this.updateTimescale();
            }

            itemCache[this.id] = barCache;
//                }

            return itemCache[this.id];
        },

        makeTooltip : function (options) {
            var tooltip = '';
            if (options.tooltipData != null) {
                var templateOptions = _.defaults(options.tooltipData, {
                    id : this.widget.uuid + 'gantt-bar-tooltip-' + this.id + '-' + options.recordId
                });
                tooltip = tooltipTemplate(templateOptions);
            }
            return tooltip;
        },
        makeBar : function (options) {
            var itemCache = this.getItemCache(options);
            var labelMarkup = this.makeLabels(options);
            var template;
            switch (options.ganttAttributes.type) {
            case RECORD_TYPES.SUMMARY:
                template = summaryTemplate;
                break;
            case RECORD_TYPES.START_MILESTONE:
            case RECORD_TYPES.FINISH_MILESTONE:
                template = milestoneTemplate;
                break;
            case RECORD_TYPES.ACTIVITY :
                /* falls through */
            default:
                template = barTemplate;
                break;
            }
            if (options.barAttributes.data.summary) {
                template = summaryTemplate;
            } else if (options.barAttributes.data.milestone) {
                template = milestoneTemplate;
            }

            var templateOptions = {
                id : this.widget.uuid + 'gantt-bar-' + this.id + '-' + options.recordId,
                recordId : options.recordId,
                tooltipId : this.widget.uuid + 'gantt-bar-tooltip-' + this.id + '-' + options.recordId,
                barId : this.id,
                barClasses : this.getBarClasses(options).join(' '),
                width : itemCache.render.width,
                left : itemCache.render.left,
                percent : itemCache.render.percent,
                labelMarkup : labelMarkup,
                toggleClasses : options.node == null || options.node.expanded ? '' : 'child-bars-collapsed',
                showProgress : options.barAttributes.data.percent != null,
                top : this.definition.render.top
            };
            return template(templateOptions);
        },

        getBarClasses : function (options) {
            var classes = [];
            if (this.definition.render.showCritical && options.barAttributes.data.critical) {
                classes.push('bar-critical');
            }
            if (this.definition.render.alwaysShowProgress) {
                classes.push(BAR_STATUS.IN_PROGRESS.barClass);
            } else if (this.definition.render.showProgress) {
                if (options.ganttAttributes.status != null) {
                    classes.push(options.ganttAttributes.status.barClass);
                }
            }
            if (this.definition.render.progressStyle != null) {
                classes.push(PROGRESS_STYLE[this.definition.render.progressStyle].style);
            } else {
                classes.push(PROGRESS_STYLE.MODERN.style);
            }
            var barColor = this.getBarColor(options);
            if (barColor == null) {
                barColor = this.definition.render.color;
            }
            if (barColor != null && BAR_COLOR[barColor] != null) {
                classes.push(BAR_COLOR[barColor].style);
            }
            if (this.definition.render.progressColor != null && BAR_COLOR[this.definition.render.progressColor] != null) {
                classes.push(BAR_COLOR[this.definition.render.progressColor].progressStyle);
            }
            classes.push(BAR_HEIGHT[this.definition.render.size].style);
            if (options.ganttAttributes.type === RECORD_TYPES.SUMMARY && options.node != null && options.node.expanded) {
                classes.push('summary-bar-expanded');
            }
            return classes;
        },

        getBarColor : function () {
            return this.definition.render.color;
        },

        makeLabels : function (options) {
            var that = this;
            var labels = '';

            _.forEach(options.barAttributes.labels, function (label) {
                var labelClasses = [LABEL_HEIGHT[label.labelOptions.render.size].style, LABEL_POSITION[label.labelOptions.render.position].style];
                if (label.labelOptions.render.verticalPosition != null) {
                    labelClasses.push(LABEL_POSITION[label.labelOptions.render.verticalPosition].style);
                }
                var templateOptions = {
                    id : that.widget.uuid + '-bar-label-' + that.id + '-' + options.recordId,
                    recordId : options.recordId,
                    tooltipId : that.widget.uuid + 'gantt-bar-tooltip-' + that.id + options.recordId,
                    barId : that.id,
                    classes : labelClasses.join(' '),
                    label : label.data,
                    color : label.labelOptions.render.color
                };
                labels += labelTemplate(templateOptions);
            });
            return labels;
        },

        updateBar : function (options) {
            var itemCache = this.getItemCache(options);
            var $bar = options.$bar;
            if (options.ganttAttributes.type !== RECORD_TYPES.MILESTONE) {
                $bar.css('width', itemCache.render.width + 'px');
            }
            $bar.css('left', itemCache.render.left + 'px');
            if (options.ganttAttributes.type === RECORD_TYPES.SUMMARY && options.node != null) {
                $bar.toggleClass('summary-bar-expanded', options.node.expanded === true);
            }
            if (this.definition.render.showProgress) {
                $bar.removeClass(_.map(_.values(BAR_STATUS), function (status) {
                    return status.barClass;
                }).join(' '));
                switch (options.ganttAttributes.status) {
                case BAR_STATUS.COMPLETED :
                    $bar.addClass(BAR_STATUS.COMPLETED.barClass);
                    break;
                case BAR_STATUS.NOT_STARTED :
                    $bar.addClass(BAR_STATUS.NOT_STARTED.barClass);
                    break;
                case BAR_STATUS.IN_PROGRESS :
                    $bar.find('.bar-progress').css('width', itemCache.render.percent + '%');
                    $bar.addClass(BAR_STATUS.IN_PROGRESS.barClass);
                    break;
                }
            }
        }
    };

    return BarRenderer;
});
