define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.moment',
    'gridapi.logger',
    'ganttapi.gantt/defaultTimescale',
    'gridapi.grid/configuration'
], function ($, _, moment, logger, DefaultScale, config) {
    'use strict';

    var PeriodScale = function (options) {
        options.setRange = false;
        DefaultScale.call(this, options);
        var that = this;

        this.periods = this.options.periodOptions.periods;
        this.averageUnits = _.reduce(_.map(this.periods, function (period) {
            return period.endDate.diff(period.startDate, that.options.value.min, true);
        }), function (sum, num) {
            return sum + num;
        }) / this.periods.length;
        this.periodOptions = options.periodOptions;
        this.startIndex = 0;
        this.endIndex = 0;
        if (this.options.granularitySize === 1) {
            this.options.value = {
                min : 'day',
                mid : 'week',
                max : 'month'
            };
            this.options.format = {
                min : 'ddd D',
                mid : 'MMMM DD',
                midPrefix : that.options.locale.timephased.weekOf,
                max : 'MMMM YYYY',
                column : 'MMMM DD YYYY',
                minRange : false
            };
        } else if (this.options.granularitySize < 30) {
            this.options.value = {
                min : 'day',
                mid : 'month',
                max : 'year'
            };
            this.options.format = {
                min : 'ddd D',
                mid : 'MMMM',
                max : 'YYYY',
                column : 'MMMM DD YYYY',
                minRange : true
            };
        } else {
            this.options.value = {
                min : 'day',
                mid : 'year'
            };
            this.options.format = {
                min : 'MMM D',
                mid : 'YYYY',
                column : 'MMMM DD YYYY',
                minRange : true
            };
        }

        if (!_.isEmpty(this.options.range)) {
            this.setRange(this.options.range);
        }
    };

    PeriodScale.prototype = Object.create(DefaultScale.prototype);

    _.assign(PeriodScale.prototype, {
        setRange : function (range) {
            if (range.start.isBefore(this.options.rangeBounds.start)) {
                range.start = this.options.rangeBounds.start.clone();
            }
            this.startIndex = _.findIndex(this.periods, function (period) {
                return !period.startDate.isBefore(range.start) || (period.startDate.isBefore(range.start) && period.endDate.isAfter(range.start));
            });
            this.startIndex = this.startIndex - this.options.buffer;
            if (this.startIndex < 0) {
                this.startIndex = 0;
            }
            this.endIndex = this.periods.length - 1;

            for (var i = this.endIndex; i > this.startIndex; i--) {
                var period = this.periods[i];
                if (!period.endDate.isAfter(this.options.rangeBounds.end) || (!period.startDate.isAfter(this.options.rangeBounds.end) && period.endDate.isAfter(this.options.rangeBounds.end))) {
                    this.endIndex = i;
                    break;
                }
            }

            this.endIndex = this.endIndex + this.options.buffer;
            if (this.endIndex > this.periods.length - 1) {
                this.endIndex = this.periods.length - 1;
            }

            this.options.range.start = this.periods[this.startIndex].startDate.clone();
            this.options.range.end = this.periods[this.endIndex].endDate.clone();
        },

        setViewRange : function (range) {
            if (range.start.isBefore(this.options.rangeBounds.start)) {
                range.start = this.options.rangeBounds.start.clone();
            }
            this.startIndex = _.findIndex(this.periods, function (period) {
                return !period.startDate.isBefore(range.start) || (period.startDate.isBefore(range.start) && period.endDate.isAfter(range.start));
            });
            this.startIndex = this.startIndex - this.options.buffer;
            if (this.startIndex < 0) {
                this.startIndex = 0;
            }
            this.endIndex = this.startIndex + this.options.bucketCount;
            if (this.endIndex > this.periods.length - 1) {
                this.endIndex = this.periods.length - 1;
            }
            for (var i = this.endIndex; i > this.startIndex; i--) {
                var period = this.periods[i];
                if (!period.endDate.isAfter(this.options.rangeBounds.end) || (!period.startDate.isAfter(this.options.rangeBounds.end) && period.endDate.isAfter(this.options.rangeBounds.end))) {
                    this.endIndex = i;
                    break;
                }
            }

            this.endIndex = this.endIndex + this.options.buffer;
            if (this.endIndex > this.periods.length - 1) {
                this.endIndex = this.periods.length - 1;
            }

            this.options.range.start = this.periods[this.startIndex].startDate.clone();
            this.options.range.end = this.periods[this.endIndex].endDate.clone();
        },

        getMinBuckets : function () {
            var that = this;
            var buckets = [];
            if (this.options.value.min != null) {
                var options = this.options;
                for (var i = this.startIndex; i <= this.endIndex; i++) {
                    var period = this.periods[i];
                    buckets.push({
                        label : this.options.usePeriodLabels ? (period.title || period.label) : that.getMinLabel(period.startDate, period.endDate, this.options.granularitySize),
                        start : period.startDate,
                        end : period.endDate,
                        width : options.bucketSize,
                        units : period.endDate.diff(period.startDate, this.options.value.min, true)
                    });
                }
            }
            return buckets;
        },

        getMidBuckets : function (start, end) {
            var buckets = [];
            if (this.options.value.mid != null) {
                var bucketStart = start.clone();
                var bucketEnd = moment(bucketStart)[config.endof](this.options.value.mid).add(1, 'ms');
                while (!bucketStart.isAfter(end)) {
                    var units = bucketEnd.diff(bucketStart, this.options.value.min, true) / this.averageUnits;
                    var bucket = {
                        label : this.getMidLabel(bucketStart[config.startof](this.options.value.mid), bucketEnd),
                        start : moment(bucketStart),
                        end : moment(bucketEnd),
                        units : units,
                        width : Math.floor(units * this.options.bucketSize)
                    };

                    bucketStart = bucketEnd[config.startof](this.options.value.mid);
                    bucketEnd = moment(bucketStart)[config.endof](this.options.value.mid).add(1, 'ms');
                    buckets.push(bucket);
                }
            }
            return buckets;
        },

        getMaxBuckets : function (start, end) {
            var buckets = [];
            if (this.options.value.max != null) {
                var bucketStart = start.clone();
                var bucketEnd = moment(bucketStart)[config.endof](this.options.value.max).add(1, 'ms');

                while (!bucketStart.isAfter(end)) {
                    var units = bucketEnd.diff(bucketStart, this.options.value.min, true) / this.averageUnits;
                    var bucket = {
                        label : this.getMaxLabel(bucketStart[config.startof](this.options.value.max), bucketEnd),
                        start : moment(bucketStart),
                        end : moment(bucketEnd),
                        units : units,
                        width : Math.floor(units * this.options.bucketSize)
                    };

                    bucketStart = moment(bucketEnd).add(1, this.options.value.min)[config.startof](this.options.value.max);
                    bucketEnd = moment(bucketStart)[config.endof](this.options.value.max).add(1, 'ms');
                    buckets.push(bucket);
                }
            }
            return buckets;
        }
    });

    return PeriodScale;
});
