define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils'
], function ($, _, utils) {
    'use strict';

    var formatString = utils.formatString;
    var Mediator = function (array, widget) {
        var that = this;
        this.options = widget.options;
        this.widget = widget;
        var mediatorFns = this.options.mediatorFns;
        // override mediator functions from options
        _.forEach(mediatorFns, function (fnc, fncName) {
            that[fncName] = fnc || that[fncName];
        });

        this.deletedRecords = [];
        this.modified = {};
        this.readOnly = {};
        this.newRecords = [];
        this.invalidFields = {};
        this.dataIcons = {};
        this.recordKeys = this.options.recordKeys;

        this.store = {};
        this.tempStore = {};
        this.originalOrder = [];
        this.fakeAjaxLoad = {};

        this.keyRecordArray(array);
    };

    Mediator.prototype = {
        constructor : Mediator,

        keyRecordArray : function (array) {
            for (var i = 0; i < array.length; i++) {
                var id = this.getValueFromRecord(array[i], this.recordKeys.id).toString();
                this.store[id] = array[i];
                this.originalOrder.push(id);
            }
        },

        sort : function (args) {
            return false;
        },

        group : function (args) {
            return false;
        },

        filter : function (args) {
            return false;
        },

        getRecord : function (id) {
            var record;
            if (id != null) {
                record = this.store[id];
                if (record == null) {
                    record = this.tempStore[id];
                }
            }
            return record;
        },

        getRecords : function (ids) {
            var records = [];
            var that = this;
            if (ids != null) {
                _.forEach(ids, function (id) {
                    records.push(that.getRecord(id));
                });
            } else {
                records = _.difference(_.values(this.store), this.deletedRecords);
            }
            return records;
        },

        newRecord : function (/* options */) {
            var record = {};
            record[this.recordKeys.id] = '' + -(_.uniqueId() + 1);
            return record;
        },

        addRecords : function (records, options) {
            var that = this;
            var isTemp = options && options.isTemp;
            _.forEach(records, function (record) {
                if (!isTemp) {
                    if (record[that.recordKeys.id] < 0) {
                        that.newRecords.push(record);
                    }
                    that.store[record[that.recordKeys.id]] = record;
                } else {
                    that.tempStore[record[that.recordKeys.id]] = record;
                }
            });
            if (!isTemp) {
                this.widget._recordsAdded(records);
            }
        },

        copyRecords : function (records) {
            var that = this;
            var copiedRecords = [];

            for (var i = 0; i < records.length; ++i) {
                var newRecord = _.merge(that.newRecord(), records[i]),
                    newRecordId = '' + -(_.uniqueId());
                newRecord[that.recordKeys.id] = newRecordId;
                copiedRecords.push(newRecord);
            }
            return copiedRecords;
        },

        deleteRecords : function (records, options) {
            var isTemp = options && options.isTemp;
            var record, i;
            for (i = 0; i < records.length; i++) {
                record = records[i];
                delete this.modified[record[this.recordKeys.id]];
            }
            if (!isTemp) {
                var newRecords = _.intersection(records, this.newRecords);
                this.deletedRecords = _.union(this.deletedRecords, _.difference(records, newRecords));
                this.newRecords = _.difference(this.newRecords, newRecords);
                this.widget._recordsRemoved(records);
            }
            // Since we need the records around for proper removal in the ui we have to delete them properly and then check
            // uniqueness again
            for (i = 0; i < records.length; i++) {
                record = records[i];
                if (!isTemp) {
                    delete this.store[record[this.recordKeys.id]];
                } else {
                    delete this.tempStore[record[this.recordKeys.id]];
                }
            }
            this.widget._recheckStoreUniqueness();
        },

        attachListeners : function () {
            // Cant really watch an array now can we ....
        },

        placeRecord : function (record) {
            var location = {
                rowsetId : record.__location__,
                table : '$tbody',
                parentId : record[this.recordKeys.parentId]
            };

            if (location.rowsetId == null || location.rowsetId === '') {
                location.rowsetId = this.widget.defaultRowset;
            }
            return location;
        },

        iterateStore : function (functionToRun, thisContext, args) {
            var that = this;
            // The first time iterating over use the original store order, after
            // the original store order has been done the sibling ids will keep the data correct.
            if (that.originalOrder.length > 0) {
                _.forEach(that.originalOrder, function (recordId) {
                    var record = that.getRecord(recordId);
                    if (that.deletedRecords.indexOf(record) === -1) {
                        functionToRun.call(thisContext, record, args);
                    }
                });
                that.originalOrder = [];
            } else {
                _.forEach(_.values(this.store), function (record) {
                    if (that.deletedRecords.indexOf(record) === -1) {
                        functionToRun.call(thisContext, record, args);
                    }
                });
            }
        },

        updateRecord : function (record, key, data, silent) {
            var originalValue = this.getValueFromRecord(record, key);
            var isTemp = _.has(this.tempStore, record[this.recordKeys.id]);
            this.setRecordValue(record, key, data);

            var newValue = this.getValueFromRecord(record, key);
            if (!isTemp && originalValue !== newValue) {
                var modifiedValues = this.modified[record[this.recordKeys.id]];
                if (modifiedValues == null) {
                    modifiedValues = {};
                    this.modified[record[this.recordKeys.id]] = modifiedValues;
                }
                if (modifiedValues[key] == null) {
                    modifiedValues[key] = originalValue;
                } else if (newValue === modifiedValues[key]) {
                    delete modifiedValues[key];
                    if (_.keys(modifiedValues).length <= 0) {
                        delete this.modified[record[this.recordKeys.id]];
                    }
                }
            }
            if (!silent) {
                this.widget._recordUpdated(record, key, data, true);
            }
        },

        clearInvalid : function (recordId, key, triggerValidationEvent) {
            if (key == null) {
                delete this.invalidFields[recordId];
            } else {
                var invalidFields = this.invalidFields[recordId];
                if (invalidFields != null) {
                    delete invalidFields[key];
                    if (_.keys(invalidFields).length <= 0) {
                        delete this.invalidFields[recordId];
                    }
                }
            }
            if (triggerValidationEvent == null || triggerValidationEvent) {
                this.widget._trigger('validation-update', false, { recordId : recordId, invalidFields : this.invalidFields, isDirty : this.hasChanges() });
            }
        },

        markInvalid : function (recordId, key, messages, append, triggerValidationEvent) {
            var invalidFields = this.invalidFields[recordId];
            if (invalidFields == null) {
                invalidFields = {};
                this.invalidFields[recordId] = invalidFields;
            }
            if (append) {
                invalidFields[key] = _.defaults(messages, invalidFields[key]);
            } else {
                invalidFields[key] = messages;
            }
            if (triggerValidationEvent == null || triggerValidationEvent) {
                this.widget._trigger('validation-update', false, { recordId : recordId, dataIndex : key, errors : messages, invalidFields : this.invalidFields, isDirty : this.hasChanges() });
            }
        },

        unhideAllErrors : function () {
            var that = this;
            _.forEach(_.pairs(this.invalidFields), function (pair) {
                var recordId = pair[0];
                var recordErrors = pair[1];
                if (recordErrors != null) {
                    _.forEach(_.values(recordErrors), function (recordError) {
                        if (recordError != null) {
                            _.forEach(_.values(recordError), function (fieldError) {
                                if (fieldError.hidden) {
                                    fieldError.hidden = false;
                                }
                            });
                        }
                    });
                }
                that.widget._updateRecordAndNode(recordId);
            });
        },

        getUniqueMessage : function (dataIndex, record) {
            return formatString(this.options.locale.uniqueError, this.getValueFromRecord(record, dataIndex));
        },

        checkStoreUniqueness : function (key, arrayOfRecords) {
            var valueToRecordMapping = {};
            var matchingRecords = [];
            for (var i = 0; i < arrayOfRecords.length; i++) {
                var recordToCheck = arrayOfRecords[i];
                var valueToCheck = this.getValueFromRecord(recordToCheck, key);
                if (valueToCheck != null && valueToCheck !== '') {
                    if (valueToRecordMapping[valueToCheck] == null) {
                        valueToRecordMapping[valueToCheck] = [];
                    }
                    valueToRecordMapping[valueToCheck].push(recordToCheck);
                }
            }
            var matches = _.values(valueToRecordMapping);
            for (var j = 0; j < matches.length; j++) {
                if (matches[j].length > 1) {
                    matchingRecords.push(matches[j]);
                }
            }

            return matchingRecords;
        },

        getRecordsForUniqueCheck : function (/* record */) {
            return this.getRecords();
        },

        checkRecordUniqueness : function (dataIndex, record) {
            var that = this;
            if (!_.isFunction(this.checkStoreUniqueness)) {
                return;
            }
            var uniqueMatches = this.checkStoreUniqueness(dataIndex, this.getRecordsForUniqueCheck(record));
            _.forEach(_.pairs(this.invalidFields), function (invalidPair) {
                var invalidMessages = invalidPair[1];
                var invalidRecordId = invalidPair[0];
                if (invalidMessages[dataIndex] != null && invalidMessages[dataIndex].unique != null) {
                    delete invalidMessages[dataIndex].unique;
                    if (_.keys(invalidMessages[dataIndex]).length === 0) {
                        that.clearInvalid(invalidRecordId, dataIndex);
                    }
                    that.widget._updateRecordAndNode(invalidRecordId);
                }
            });
            function markRecord(record) {
                that.markInvalid(record[that.recordKeys.id], dataIndex,
                    {
                        unique : {
                            message : that.getUniqueMessage(dataIndex, record),
                            matches : _.without(matchingRecords, record)
                        }
                    }, true);
                that.widget._updateRecordAndNode(record[that.recordKeys.id]);
            }

            for (var i = 0; i < uniqueMatches.length; i++) {
                var matchingRecords = uniqueMatches[i];
                _.forEach(matchingRecords, markRecord);
            }
        },

        getValueFromRecord : function (record, key) {
            return record[key];
        },

        setRecordValue : function (record, key, data) {
            record[key] = data;
        },

        commit : function () {
            this.modified = {};
            this.readOnly = {};
            this.widget.repopulate();
        },

        // return an array of column keys that will be read only
        checkRecordPermissions : function (/* record */) {
            return [];
        },

        checkRecordIcon : function (/* record */) {
            return {};
        },

        recordMoved : function (record, siblingId, parentId, oldParentId, newRowsetId) {
            var oldRowsetId = record.__location__;
            if (newRowsetId != null) {
                record.__location__ = newRowsetId;
            }

            this.widget._recordMoved(record, oldRowsetId);
        },


        /* Build the record attributes object to help the grid render the rows and columns as a more correct representation of the
         * underlying data. The defaults parameter contains default values, if your mediator becomes outdated you can check
         * the defaults for what the grid expects/can handle.
         * For convenience in overriding behavior breakout methods have been added. They are documented below.
         */
        getRecordAttributes : function (record, defaults) {
            //>>excludeStart("perf", pragmas.compiled)
            window.incrementCallCount('boilerplateArrayMediator#getRecordAttributes');
            //>>excludeEnd("perf")
            var attributes = {};


            attributes.invalid = this.lookupInvalidFields(record);

            attributes.modified = this.lookupModifiedFields(record);

            attributes.locked = this.lookupLockedFields(record);

            attributes.readOnly = this.lookupReadOnlyFields(record);

            attributes.dataIcons = this.lookupRecordIcons(record);

            var treeDetails = this.lookupRecordTreeDetails(record);
            _.assign(attributes, treeDetails);

            var recordDetails = this.lookupRecordDetails(record);
            _.assign(attributes, recordDetails);

            _.defaults(attributes, defaults);

            return attributes;
        },

        /* Should return an object with invalid dataindexes as the keys and an object with errorId as the key and an object with
         * at least a message attribute for the error. This is an object allow for extra information to be passed along
         *
         * return {
         *  name : {
         *      format : {
         *          message : 'Name may not contain numbers',
         *      },
         *      unique : {
         *          message : 'Name must be unique',
         *         matching : [records] an array of other matching records
         *     }
         *  },
         *  type : {
         *     required : {
         *         message : 'Must set a type for the project'
         *     }
         *   }
         * }
         */
        lookupInvalidFields : function (record) {
            return this.invalidFields[this.getValueFromRecord(record, this.recordKeys.id)];
        },

        /* Should return an array of modified fields on the record. This will mark the field as changed via css. This should be
         * cached to maintain performance. Values should be set when data is being changed in updateRecord or as a function of
         * your data store.
         *
         * return ['name','description']
         */
        lookupModifiedFields : function (record) {
            var modifiedFields = [];
            var modifiedValues = this.modified[this.getValueFromRecord(record, this.recordKeys.id)];
            if (modifiedValues != null) {
                modifiedFields = _.keys(modifiedValues);
            }
            return modifiedFields;
        },

        /*
         * Should return an array of locked fields on the record. This will not attempt to render any value from the
         * the record but instead put a locked icon in the column.
         *
         * return ['name','description']
         */
        lookupLockedFields : function (/* record */) {
            return [];
        },

        /* Should return an array of fields that are read only for this record. This is separate from column level where
         * every record on the column is read only. You can use this to check type of object and disable fields. This should be
         * cached to maintain performance. I suggest caching in this method as many it will not change. Example if type is project
         * disable description field. This really only needs to be done one time. If a field that controls readOnlyFields changes,
         * call this method with force set to true to reset.
         *
         *  return ['type','start-date']
         */
        lookupReadOnlyFields : function (record, force) {
            var recordId = this.getValueFromRecord(record, this.recordKeys.id);
            if (force || this.readOnly[recordId] == null) {
                this.readOnly[recordId] = this.checkRecordPermissions(record);
            }
            return this.readOnly[recordId];
        },

        /* This method needs to set the following attributes. They only really need to be set on parent records. They do not
         * need to be set at all for non tree grids.
         *
         * return {
         *  isNode : if it has children / you want the + / icons
         *  hasChildren : determines if a +/- should be there, no children = no icon
         *  expanded : do you want the default to have all the children showing, if not set to false and they will be collapsed
         *  childrenLoaded : set to true unless it is lazyLoaded store. In this case you need a lookup
         *  sortable : Do you want the record to obey column sort. Example Portfolios -> Project -> Activity you may only want activities to obey sort
         *  groupable : Can this record be grouped, in the same concept of sortable often certain types are immune to grouping usually the top most parent
         * }
         */
        lookupRecordTreeDetails : function (/* record */) {
            return {};
        },
        /* Should return the icon object for this record. This record should cache the result of checkRecordIcon, if a
         * field that dictates this icon changes you should call this.checkRecordIcon(record)
         * return { icon }
         */
        lookupRecordIcons : function (record, force) {
            var recordId = this.getValueFromRecord(record, this.recordKeys.id);
            if (force || this.dataIcons[recordId] == null) {
                this.dataIcons[recordId] = this.checkRecordIcon(record);
            }
            return this.dataIcons[recordId];
        },

        /* Record details are for attributes that specifically relate to just the record. Expects to set
         * an object with attributes
         *
         * return {
         *  custom : your own object representation
         *  draggable : true/false if you should be able to drag this row
         *  type : if needed for grouping
         *  newRow : if the record is new
         *  editable : true/false. Can you open an editor for the entire record. This will not add any css
         * }
         */
        lookupRecordDetails : function (record) {
            var attributes = {};
            attributes.type = this.getValueFromRecord(record, this.recordKeys.type);
            if (_.includes(this.newRecords, record)) {
                attributes.newRow = true;
            }
            return attributes;
        },

        // Indicating a row has been dropped into a new rowset
        updateRecordPlacement : function (record, oldRowsetId, newRowsetId) {
            if (oldRowsetId !== newRowsetId) {
                record.__location__ = newRowsetId;
            }
        },

        getChildRecords : function (/* recordId */) {
            return [];
        },

        loadChildren : function (/* recordId */) {
        },

        updateRecordIds : function (record, idMap) {
            var oldRecordKeys = _.keys(idMap);
            var recordId = record[this.recordKeys.id];
            if (_.includes(oldRecordKeys, recordId)) {
                record[this.recordKeys.id] = idMap[recordId];
                this.store[idMap[recordId]] = record;
                delete this.store[recordId];
            }
        },
        commitSuccessful : function (serverResponse) {
            var that = this;
            var recordToSelectId = this.widget.focus.record != null ? this.widget.focus.record[this.recordKeys.id] : null;
            if (serverResponse.updatedIdMap != null && _.keys(serverResponse.updatedIdMap).length) {
                this.iterateStore(this.updateRecordIds, this, serverResponse.updatedIdMap);
                this.widget._updateRecordIds(serverResponse.updatedIdMap);
                if (recordToSelectId != null && serverResponse.updatedIdMap[recordToSelectId] != null) {
                    recordToSelectId = serverResponse.updatedIdMap[recordToSelectId];
                }
            }

            this.modified = {};
            this.readOnly = {};
            _.forEach(that.deletedRecords, function (deletedRecord) {
                delete that.store[deletedRecord.id];
            });
            this.newRecords = [];

            this.deletedRecords = [];
            this.widget.repopulate(recordToSelectId, this.widget.focus.columnId);
            this.widget.clearSearch();
            this.widget._trigger('is-clean');
        },

        reset : function () {
            var modifiedRecords = _.pairs(this.modified);
            for (var i = 0; i < modifiedRecords.length; i++) {
                var modifiedRecordEntry = modifiedRecords[i];
                var record = this.getRecord(modifiedRecordEntry[0]);
                var modifications = _.pairs(modifiedRecordEntry[1]);
                for (var j = 0; j < modifications.length; j++) {
                    var keyValueModification = modifications[j];
                    record[keyValueModification[0]] = keyValueModification[1];
                }
            }
            this.deletedRecords = [];
            this.replacementMap = {};
            this.newRecords = [];
            this.modified = {};
            this.readOnly = {};
            this.widget.repopulate();
            this.widget._trigger('is-clean');
        },

        hasChanges : function () {
            return _.keys(this.modified).length || this.deletedRecords.length || this.newRecords.length;
        },

        getPostData : function () {
            var postData = null;
            if (this.hasChanges()) {
                var updatedRecords = this.getRecords(_.keys(this.modified));
                postData = {
                    added : this.newRecords,
                    updated : _.difference(updatedRecords, this.newRecords.concat(this.deletedRecords)),
                    deleted : _.pluck(this.deletedRecords, this.recordKeys.id)
                };
                if (this.replacementMap != null) {
                    postData.updatedIdMap = this.replacementMap;
                }
            }
            return postData;
        },

        destroy : function () {
            delete this.widget;
            delete this.store;
        }
    };

    return Mediator;
})
;
