define([
    'gridapi.jquery',
    'gridapi.lodash'
], function ($, _) {
    'use strict';

    return {
        /**
         * Create a new sub menu based on the type.
         * @param grid - access grid functionality
         * @param $parentLi - parent list item for sub menu
         * @param type - type of sub menu
         * @param options - any additional params needing to be passed to sub menu
         */
        init : function (grid, $parentLi, type, options) {
            require(['gridapi.columnMenu/menu', 'gridapi.columnMenu/columnMenuActions'],
                function (menu, actions) {
                    var columnToggles = function (grid, column, layoutOptions) {
                        // create toggle column menu
                        var $menu = menu.createMenu({
                            classes : ['sub-menu']
                        });

                        return {
                            render : function () {
                                var that = this,
                                    $ul = $menu.find('ul');

                                $menu.data('menu-api', this);

                                // if grid consumer has not defined a custom menu then use a flat list of all the columns
                                if (_.isEmpty(layoutOptions)) {
                                    layoutOptions = {
                                        dataIndexes : _.without(_.pluck(grid.columnDefinitions, 'dataIndex'), 'row-selector')
                                    };
                                }

                                // create column items
                                _.forEach(layoutOptions.dataIndexes, function (index) {
                                    var col = grid.columnDefinitions[index];
                                    var $li = menu.createMenuItemCheck({
                                        id: 'pgbu-grid-column-menu-check-' + grid.widget.uuid + '-' + index,
                                        name : col.title,
                                        checked : col.visible ? 'checked="checked"' : '',
                                        dataIndex : col.dataIndex,
                                        action : 'column'
                                    });
                                    if (grid.getColumnVisibleCount() <= 1 && col.visible) {
                                        // disable the ability to toggle the last visible column
                                        that.enableCheckboxes($li, false);
                                    }

                                    $ul.append($li);
                                });

                                // create column sub lists
                                _.forEach(layoutOptions.subMenus, function (subList, index) {
                                    menu.addDivider($ul);
                                    $ul.append(menu.createMenuItem({
                                        id: 'pgbu-grid-column-menu-li-' + grid.widget.uuid + '-' + index,
                                        name : subList.label,
                                        action : 'check-column',
                                        classes : ['sub-menu-item'],
                                        data : {
                                            columnLayout : layoutOptions,
                                            dataLayoutIndex : index
                                        }
                                    }));
                                });

                                // attach listeners
                                menu.attachListeners($ul, actions, {
                                    grid : grid,
                                    column : column
                                });

                                return $menu;
                            },

                            disableLastColumnToggle : function () {
                                var $rootMenu = $menu.closest('.root-menu');
                                var $lis = $rootMenu.find('li[data-action="column"]');
                                if (grid.getColumnVisibleCount() <= 1) {
                                    // only one column left so disable the ability to turn column off
                                    var $inputChecked = $lis.find('input[checked="checked"]');
                                    var $li = $inputChecked.closest('li');
                                    this.enableCheckboxes($li, false);
                                } else {
                                    // remove any disabled column toggles
                                    this.enableCheckboxes($lis, true);
                                }
                            },

                            enableCheckboxes : function ($lis, enable) {
                                var $checkboxes = $lis.find('input[type="checkbox"]');
                                if (enable) {
                                    $lis.removeClass('disabled');
                                    $checkboxes.prop('disabled', false);
                                } else {
                                    $lis.addClass('disabled');
                                    $checkboxes.prop('disabled', true);
                                }
                            }
                        };
                    };
                    var subMenu = null;
                    switch (type) {
                    case 'columnToggle':
                        subMenu = columnToggles(grid, options.column, options.toggleLayout);
                        break;
                    case 'filter':
                        break;
                    default:
                        throw new Error('subMenu.js - Invalid column type.');
                    }

                    // create new sub menu
                    var $menu = subMenu.render();

                    // add sub menu after the parent list item list menu - eg after the ul
                    $parentLi.closest('ul').append($menu);

                    // position menu
                    $menu.position({
                        my : 'left top',
                        at : 'right top',
                        of : $parentLi,
                        offset : '0 0',
                        collision : 'fit'
                    });

                    // select the first item in menu
                    menu.selectFirstMenuItem($menu.find('ul'));
                });
        }
    };
});
