define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils/keyCode',
    'gridapi.grid/utils'
], function ($, _, keyCode, utils) {
    'use strict';

    // templates
    var menuTemplate = utils.basicTemplate('<div class="pgbu-grid-menu" tabindex="-1" role="menu"><ul></ul></div>');
    var menuItemTemplate = utils.basicTemplate('<li id="<%=id%>" role="menuitem" data-action="<%=action%>">' +
        '<a href="#" class="name"><%=name%></a></li>');
    var menuItemCheckTemplate = utils.basicTemplate('<li role="menuitem" data-index="<%=dataIndex%>" data-action="<%=action%>"><a href="#"><input id="<%=id%>" type="checkbox" class="icon"' +
        '<%=checked%>/><label for="<%=id%>" class="name"><%=name%></label></a></li>');

    // markup
    var menuItemDividerMarkup = '<li class="divider"></li>';

    return {

        createMenu : function (options) {
            var $menu = $(menuTemplate(options));

            // add classes
            _.forEach(options.classes, function (clazz) {
                $menu.addClass(clazz);
            });

            return $menu;
        },

        createMenuItem : function (options) {
            var $li = $(menuItemTemplate(options));
            if (options.icon == null || options.icon.length === 0) {
                $li.find('img').remove();
            }
            // add classes
            _.forEach(options.classes, function (clazz) {
                $li.addClass(clazz);
            });

            // add data
            _.forEach(options.data, function (data, name) {
                $li.data(name, data);
            });

            return $li;
        },

        createMenuItemCheck : function (options) {
            var $li = $(menuItemCheckTemplate(options));

            // add classes
            _.forEach(options.classes, function (clazz) {
                $li.addClass(clazz);
            });

            // add data
            _.forEach(options.data, function (data, name) {
                $li.data(name, data);
            });

            return $li;
        },

        addDivider : function ($ul) {
            var $divider = $(menuItemDividerMarkup);
            $ul.append($divider);
            return $divider;
        },

        attachListeners : function ($ul, eventActions, options) {
            var that = this,
                $lis = $ul.find('li'),
                $rootMenu = $ul.closest('.root-menu');

            $rootMenu.data('navigation-type', 'keyboard');

            $lis.find('a').on('click', function (event) {
                event.preventDefault();
            });

            // trigger any actions we can
            $lis.on('click', function (event) {
                var $li = $(this);
                if ($li.is('.divider')) {
                    return;
                }
                eventActions[$li.attr('data-action')].call(event, $li, options);
            });

            $lis.on('keydown', function (event) {
                switch (event.keyCode) {
                case keyCode.UP :
                case keyCode.DOWN :
                case keyCode.RIGHT :
                case keyCode.LEFT :
                    event.preventDefault();
                }
            });

            $rootMenu.on('keyup', function (event) {
                // get active list item
                var $li = $ul.find('.item-focused'),
                    action = $li.attr('data-action');

                $rootMenu.data('navigation-type', 'keyboard');
                $rootMenu.find('li.item-focused').removeClass('item-focused-transparent');

                switch (event.keyCode) {
                case keyCode.ENTER :
                        // trigger action and close
                    eventActions[action].call(event, $li, options);
                    break;
                case keyCode.SPACE :
                        // toggle column and dont close
                    eventActions[action].call(event, $li, options);
                    break;
                case keyCode.UP :
                        // move selection up one
                    that.moveSelection($ul, true, options.grid);
                    break;
                case keyCode.DOWN :
                        // move selection down one
                    that.moveSelection($ul, false, options.grid);
                    break;
                case keyCode.RIGHT :
                        // open sub menu if item is a sub menu
                    $li.trigger('mouseover', true);
                    break;
                case keyCode.LEFT :
                        // destroy deepest sub menu
                    var $menu = $li.closest('.sub-menu');
                    if (!$menu.is('.root-menu') && $menu.length) {
                        var $subMenuItem = $menu.parent().find('li.item-focused-transparent');
                        $subMenuItem.removeClass('item-focused-transparent').addClass('item-focused');
                        var id = ($subMenuItem.find('input').length > 0) ? 'a' : $subMenuItem.attr('id');
                        options.grid.widget.view.$grid.attr('aria-activedescendant', id);
                        $('#pgbu-grid-column-menu').attr('aria-activedescendant', id);  // LUX-276
                        $li.parent().remove();
                    }
                    break;
                case keyCode.ESCAPE:
                        // destroy the whole menu
                    $rootMenu = $(this).closest('.root-menu');  // context lost
                    var menuApi = $rootMenu.data('menu-api');
                    menuApi.destroy();
                    break;
                default :
                        // disable all other key events
                    event.stopPropagation();
                    event.preventDefault();
                }
            });

            // to expand sub lists
            $lis.on('mouseover', function (event, fromKeyUp) {
                var $li = $(this),
                    $rootMenu = $li.closest('.root-menu'),
                    action = $li.attr('data-action');

                // if not using the keyboard then remove visual focus
                if (!fromKeyUp) {
                    $rootMenu.data('navigation-type', 'mouse');
                    $rootMenu.find('li.item-focused').addClass('item-focused-transparent');
                }

                // give focus to list item so that if we change to using keys we can move
                that.drawFocus($li);

                if (action === 'check-column') {
                    eventActions[action].call(event, $li, options);
                    setTimeout(function () {
                        var $li_in_rightSubMenu = $li.siblings('div').find('li:eq(0)');
                        $li_in_rightSubMenu.addClass('item-focused');
                        var id = ($li_in_rightSubMenu.find('input').length > 0) ? $li_in_rightSubMenu.find('input').attr('id') : $li_in_rightSubMenu.attr(id);
                        options.grid.widget.view.$grid.attr('aria-activedescendant', id);
                        $('#pgbu-grid-column-menu').attr('aria-activedescendant', id);// LUX-276
                        $li.removeClass('item-focused').addClass('item-focused-transparent');
                    }, 10);
                } else {
                    // destroy deepest sub menu
                    $li.parent().find('.sub-menu').remove();
                }
            });
        },

        moveSelection : function ($ul, up, grid) {
            // use current list item selection and move from there, otherwise start from beginning
            var $li = $ul.find('.item-focused');
            var $lis = $li.parent('ul').children('li');

            if ($li.length) {
                $li = $li.eq(0);
                $li.removeClass('item-focused');
                var $nextLi = up ? $li.prev() : $li.next();
                if (!$nextLi.length) {
                    $nextLi = up ? $lis.last() : $lis.first();
                }
                // select the next item
                $nextLi.addClass('item-focused');
                var id = ($nextLi.find('input').length > 0) ? $nextLi.find('input').attr('id') : $nextLi.attr('id');//  $nextLi.find('input').attr('id') : $nextLi.find('a').attr(id);
                grid.widget.view.$grid.attr('aria-activedescendant', id);
                $('#pgbu-grid-column-menu').attr('aria-activedescendant', id);// LUX-276
                $nextLi.removeClass('item-focused-transparent');
                // skip dividers
                if ($nextLi.is('.divider')) {
                    this.moveSelection($ul, up, grid);
                }
                // draw focus to item
                this.drawFocus($nextLi);
            }
        },

        selectFirstMenuItem : function ($ul) {
            // var $lis = $ul.find('li');

            // select the first item in menu
            // if ($lis.length) {

                // select the first item in the list
                // this.moveSelection($ul, false);

                // draw focus to control
                // this.drawFocus($lis.eq(0));
            // }
        },

        drawFocus : function ($li) {
            // draw focus to anchor tag so we can interact with menu via keyboard
            /* var $anchor = $li.find('a');
             if ($anchor.length) {
             setTimeout(function () {
             $anchor.get(0).focus();
             }, 0);
             }*/
        }
    };
});
