define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.columns/core/editor',
    'gridapi.columns/core/renderer',
    'gridapi.grid/columns/core/defaultFormatter',
    'gridapi.columns/core/headerRenderer',
    'gridapi.columns/defaultColumnOptions',
    'gridapi.logger'
], function ($, _, Editor, Renderer, defaultFormatter, HeaderRenderer, defaultColumnOptions, logger) {
    'use strict';

    /**
     * @typedef {Object} BaseColumnBuilder#args
     * @property {Object} types Map of columnType to columnDefinition
     * @property {Object} definedColumn The column defined by the user
     * @property {Object} modifiers The default column options per the grid widget
     * @private
     */

    /**
     * @class BaseColumnBuilder
     *
     * @param {BaseColumnBuilder#args} args
     * @private
     */
    function BaseColumnBuilder(args) {
        this._init(args);
    }

    BaseColumnBuilder.prototype = _.assign({}, {
        /**
         * @method BaseColumnBuilder#_init
         * @private
         * @param {BaseColumnBuilder#args} args
         */
        _init : function (args) {
            this.types = args.types;
            this.definedColumn = args.definedColumn;
            this.modifiers = args.modifiers;
            this.defaultColumn = this.getDefaultColumn();
            this.newColumn = _.clone(this.defaultColumn);
            this._configure();
        },
        /**
         * @method BaseColumnBuilder#_configure
         * @private
         */
        _configure : function () {
            // The order of these calls is important
            this._validate();
            this._toggleEditor();
            this._setColumnTypeDefaults();
            this._setValidationOptions();
            this._setAccessibilityOptions();
            this._setColumnAlignment();
            this._setCustomOverrides();
            this._setLogicalOverrides();
            this._attachColumnFns();
            this.extend();
        },
        /**
         * @method BaseColumnBuilder#_validate
         * @private
         */
        _validate: function () {
            if (this.definedColumn == null) {
                throw new Error('columnDefintionFns.js - defined column is undefined.');
            }

            if ((this.modifiers == null || !this.modifiers.cellAttrsOnly) && this.definedColumn.dataIndex == null || this.definedColumn.dataIndex === '' || this.definedColumn.dataIndex === '_no-data-index_') {
                throw new SyntaxError('dataIndex property has not been set correctly for column.');
            }

            if (this.definedColumn.displayType === 'select' && !_.isFunction(this.definedColumn.selectValues) && _.isEmpty(this.definedColumn.selectValues)) {
                throw new SyntaxError('selectValues property has not been set correctly for the column displayType select');
            }

            if ((this.modifiers == null || !this.modifiers.cellAttrsOnly) && !this.definedColumn.internalDisplayOnly) {
                if (this.definedColumn.title == null || this.definedColumn.title === '' || this.definedColumn.title === '_no-header_') {
                    throw new SyntaxError('title property has not been set correctly for column.');
                }
            }
        },
        /**
         * @method BaseColumnBuilder#_toggleEditor
         * @private
         */
        _toggleEditor : function () {
            if (this.definedColumn.readOnly || this.definedColumn.displayType === 'check' || this.definedColumn.displayType === 'icon') {
                this.definedColumn.enableEditor = false;
            }
        },
        /**
         * @method BaseColumnBuilder#_setColumnTypeDefaults
         * @private
         */
        _setColumnTypeDefaults : function () {
            var columnType = this.types[this.definedColumn.displayType || this.defaultColumn.displayType];
            if (columnType != null) {
                if (columnType.editor != null) {
                    this.newColumn.editor = new columnType.editor();
                }
                if (columnType.headerRenderer != null) {
                    this.newColumn.headerRenderer = new columnType.headerRenderer();
                }

                if (columnType.renderer != null) {
                    this.newColumn.renderer = new columnType.renderer();
                }

                if (columnType.formatter != null) {
                    this.newColumn.formatter = columnType.formatter;
                }
                if (columnType.defaultOptions != null) {
                    _.assign(this.newColumn, columnType.defaultOptions);
                }
            }
        },
        /**
         * @method BaseColumnBuilder#_setValidationOptions
         * @private
         */
        _setValidationOptions : function () {
            if (this.definedColumn.validator != null) {
                var validator = this.definedColumn.validator,
                    validationOptions = validator.options;
                this.newColumn.required = validationOptions.required;
                this.newColumn.unique = validationOptions.unique;
                switch ((this.definedColumn.displayType || this.defaultColumn.displayType).toLowerCase()) {
                case 'text':
                case 'textarea':
                case 'link':
                    this.newColumn.maxLength = validationOptions.maxValue;
                    break;
                case 'spinner':
                case 'date':
                    this.newColumn.maxValue = validationOptions.maxValue;
                    this.newColumn.minValue = validationOptions.minValue;
                    break;
                }
            }
        },
        /**
         * @method BaseColumnBuilder#_setAccessibilityOptions
         * @private
         */
        _setAccessibilityOptions : function () {
            if (this.definedColumn.describedBy == null) {
                this.definedColumn.describedBy = [(this.definedColumn.displayType || this.defaultColumn.displayType).toLowerCase()];
            }
        },
        /**
         * @method BaseColumnBuilder#_setColumnAlignment
         * @private
         */
        _setColumnAlignment : function () {
            switch (this.definedColumn.displayType) {
            case 'cost' :
            case 'percent' :
            case 'number' :
            case 'date' :
            case 'dateTime' :
                this._fallbackToAlignRight();
                break;
            }
            if (this.definedColumn.spinnerType === 'number') {
                this._fallbackToAlignRight();
            }
        },
        /**
         * @method BaseColumnBuilder#_setColumnOverrides
         * @private
         */
        _setCustomOverrides : function () {
            if (this.definedColumn.headerRenderer != null && _.isFunction(this.definedColumn.headerRenderer)) {
                this.newColumn.headerRenderer = new this.definedColumn.headerRenderer();
                delete this.definedColumn.headerRenderer;
            }

            if (this.definedColumn.renderer != null && _.isFunction(this.definedColumn.renderer)) {
                this.newColumn.renderer = new this.definedColumn.renderer();
                delete this.definedColumn.renderer;
            }

            if (this.definedColumn.editor != null && _.isFunction(this.definedColumn.editor)) {
                this.newColumn.editor = new this.definedColumn.editor();
                delete this.definedColumn.editor;
            }

            if (this.definedColumn.formatter != null) {
                this.newColumn.formatter = this.definedColumn.formatter;
                delete this.definedColumn.formatter;
            }
        },
        /**
         * @method BaseColumnBuilder#_setLogicalOverrides
         * @private
         */
        _setLogicalOverrides : function () {
            if (this.definedColumn.readOnly) {
                this.newColumn.hideAsterisk = true;
            }
            if (this.definedColumn.width != null) {
                var width = parseInt(this.definedColumn.width, 10);
                if (width < 0) {
                    delete this.definedColumn.width;
                    logger.warn('column width negative resetting to default width for column ' + this.definedColumn.title);
                }
            }
        },
        /**
         * @method BaseColumnBuilder#_attachColumnFns
         * @private
         */
        _attachColumnFns : function () {
            _.forEach(this.definedColumn.rendererFns, function (fnc, fncName) {
                this.newColumn.renderer[fncName] = fnc || this.newColumn[fncName];
            }.bind(this));

            _.forEach(this.definedColumn.editorFns, function (fnc, fncName) {
                this.newColumn.editor[fncName] = fnc || this.newColumn[fncName];
            }.bind(this));

            _.forEach(this.definedColumn.headerRendererFns, function (fnc, fncName) {
                this.newColumn.headerRenderer[fncName] = fnc || this.newColumn[fncName];
            }.bind(this));
        },
        /**
         * @method BaseColumnBuilder#extend
         * @private
         */
        extend : _.noop,
        /**
         * @method BaseColumnBuilder#_fallbackToAlignRight
         * @private
         */
        _fallbackToAlignRight : function () {
            if (this.definedColumn.align == null) {
                this.newColumn.align = 'right';
                this.newColumn.headerAlign = 'right';
            }
        },
        /**
         * @method BaseColumnBuilder#getDefaultColumn
         * @returns {Object} default column definition
         * @private
         */
        getDefaultColumn: function () {
            return _.assign(defaultColumnOptions, {
                formatter: defaultFormatter,
                renderer: new Renderer(),
                headerRenderer: new HeaderRenderer(),
                editor: new Editor()
            });
        },
        /**
         * @method BaseColumnBuilder#build
         * @returns {Object} new column
         * @private
         */
        build : function () {
            return _.merge({}, this.newColumn, this.definedColumn, function (a, b) {
                if (_.isArray(a)) {
                    return b;
                }
            });
        }
    });

    return BaseColumnBuilder;
});
