define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/gridMixins'
], function ($, _, mixins) {
    'use strict';

    var renderer = function () {
    };
    var gridHighlights = {
        prefix: '<mark class="grid-highlight">',
        postfix: '</mark>'
    };

    renderer.extend = mixins.extendFn;

    // alignment of text
    var columnAlignment = {
        left : '', // 'cell-align-left', no need to do this, save some css speed
        right : 'cell-align-right',
        center : 'cell-align-center'
    };

    _.assign(renderer.prototype, {
        /**
         * Override to build cell html
         * Must be in string format for performance - NO JQUERY!!
         *
         * @method create
         * @private
         */
        create : function (options) {
            var data = options.data,
                column = options.column,
                groupBandClasses = options.groupBandClasses,
                level = options.level,
                hasChildren = options.hasChildren,
                clickableNode = options.clickableNode,
                loading = options.loading,
                expanded = options.expanded,
                unselectable = options.printable ? 'off' : 'on',
                icon;

            if (options.invalid != null && options.invalid.invalidFormat) {
                options.formattedValue = options.invalid.invalidFormat.invalidData;
            } else {
                options.formattedValue = options.formattedValue ? options.formattedValue : this.format(options, data);
            }

            if (options.autoEscape) {
                options.formattedValue = options.escapeFn(options.formattedValue);
            }

            // Add cell markup if its invalid
            options.showErrors = options.invalid != null && _.some(_.values(options.invalid), function (error) {
                return !error.hidden;
            });

            options.isEmpty = options.column.required && !options.readOnly && options.editable && (options.formattedValue == null || options.formattedValue.length === 0);
            // Add cell, classes and renderer
            var html = '<td unselectable="' + unselectable + '" data-index="' + options.dataIndex + '" id="' + options.id + '"';
            if (options.subRow != null) {
                html += 'data-sub-row="' + options.subRow + '"';
            }

            html += this.getAriaMarkup(options);
            html += this.getClassMarkup(options) + ' ';
            if (options.colSpan) {
                html += ' colspan="' + options.colSpan + '" ';
            }
            html += this.getAttributes(options) + ' ';
            html += ' > <div class="cell-positioner" style="height:' + options.cellHeight + 'px">';
            if (options.expandable && level != null) {
                html += '<div role="presentation" class="cell-spacers">' + this.getSpacers(clickableNode ? level : level - 1, groupBandClasses);

                // Add toggle button if it has children
                if (clickableNode && hasChildren) {
                    html += '<div class="toggle-children"  style="line-height:' + options.cellHeight + 'px">';
                    if (loading) {
                        html += options.markup.cellLoading;
                    } else {
                        html += expanded ? options.markup.expandedRow : options.markup.collapsedRow;
                    }
                    html += '</div>';
                } else if (options.addSpacer) {
                    html += '<div class="toggle-children-spacer"/> ';
                }
                html += '</div>';
            }


            html += '<div unselectable="' + unselectable + '" class="cell-renderer ' + this.getAlignment(options) + '" style="line-height:' + options.cellHeight + 'px">';

            // Add Spacers for tree if neccessary
            if (options.expandable && level != null) {
                html += this.getSpacers(clickableNode ? level : level, groupBandClasses);
            }
            // Add toggle button if it has children
            if (clickableNode && hasChildren || options.addSpacer) {
                html += '<div class="toggle-children-spacer"/>';
            }

            if (options.grabber === 'before' || options.grabber === 'both') {
                html += '<div class="cell-grabber">' + options.markup.rowDragger + '</div>';
            }

            if (options.locked) {
                html += options.markup.lockedCell;
            } else {
                if (column.dataDisplay === 'icon' || column.dataDisplay === 'icon-value') {
                    icon = this.getIconMarkup(options);
                    if (icon != null) {
                        html += icon + ' ';
                    }
                }

                // Add the formatted value and close the renderer
                var renderedValue = (this.getValueMarkup(options) || '').toString(); // Force to a string for replacing
                if (options.found) {
                    renderedValue = options.matcher.surroundMatches(renderedValue, gridHighlights);
                }
                if (options.isEmpty) {
                    var message = (column.autoNumbered && (options.attributes.newRow || options.isDummyRecord)) ? options.locale.autoNumbered : options.locale.required;

                    html += '<span aria-hidden="true" role="presentation" class="cell-required-message">[ ' + message + ' ]</span>';
                }
                if (column.dataDisplay !== 'icon' && !options.locked) {
                    html += renderedValue;
                }
                if (column.dataDisplay === 'value-icon') {
                    icon = this.getIconMarkup(options);
                    if (icon != null) {
                        html += icon + ' ';
                    }
                }
            }
            html += '</div>';

            if (options.grabber === 'after' || options.grabber === 'both') {
                html += '<div class="cell-grabber cell-grabber-after">' + options.markup.rowDragger + '</div>';
            }

            // Add cell markup if its invalid
            if (options.showErrors) {
                if (options.allWarning) {
                    html += '<i class="cell-error-indicator cell-warning-indicator pgbu-icon pgbu-icon-indicator-warning"></i>';
                } else {
                    html += '<div class="cell-error-indicator cell-invalid-indicator"></div>';
                }
            }
            html += this.getCellIndicators(options);
            html += this.getCellToolbar(options);

            // Close the cell
            html += '</div></td>';

            return html;
        },

        getCellToolbar : function (options) {
            var toolbarMarkup = '';
            if (options.cellToolbarActions != null && options.cellToolbarActions.length) {
                toolbarMarkup = '<div class="cell-toolbar">';
                for (var i = 0; i < options.cellToolbarActions.length; i++) {
                    var toolbarItem = options.cellToolbarActions[i];
                    toolbarMarkup += '<a href="#" tabindex="-1"><i ' + toolbarItem.dataTrigger + '="' + toolbarItem.trigger + '" title="' + toolbarItem.title + '" class="toolbar-item pgbu-icon-small ' + toolbarItem.icon + '"></i></a>';
                }
                toolbarMarkup += '</div>';
            }
            return toolbarMarkup;
        },

        getCellIndicators : function (/* renderOptions */) {
            return '';
        },

        getAriaMarkup : function (renderOptions) {
            var ariaMarkup = ' tabindex="-1"';
            var role = 'gridcell';

            if (renderOptions.column.rowHeader) {
                role = 'rowheader';
            }

            ariaMarkup += 'role="' + role + '"';
            ariaMarkup += this.getAriaLabeledBy(renderOptions);
            if (renderOptions.column.required) {
                ariaMarkup += ' aria-required="true" ';
            }
            if (renderOptions.invalid) {
                ariaMarkup += ' aria-invalid="true" ';
            }
            if (renderOptions.readOnly || renderOptions.locked || !renderOptions.editable) {
                ariaMarkup += ' aria-readonly="true" ';
            }

            if (this.isValueChecked != null) {
                ariaMarkup += ' aria-checked="' + this.isValueChecked(renderOptions) + '" ';
            }

            if (renderOptions.clickableNode && renderOptions.hasChildren) {
                ariaMarkup += ' aria-expanded="' + renderOptions.expanded + '" ';
            }
            var describedBy = this.getAriaDescribedBy(renderOptions);
            if (describedBy != null) {
                ariaMarkup += describedBy;
            }

            return ariaMarkup;
        },

        getAriaDescribedBy : function (renderOptions) {
            var describedBy = [];
            var description = this.getDescriptionLabel(renderOptions);
            if (description != null) {
                describedBy.push(description);
            }
            if (renderOptions.showErrors) {
                describedBy.push(renderOptions.descriptionIds.cellState.errorOnCell);
            }
            if (renderOptions.describedBy != null) {
                describedBy = describedBy.concat(renderOptions.describedBy);
            }
            if (describedBy.length) {
                return ' aria-describedby="' + describedBy.join(' ') + '" ';
            }
        },

        getDescriptionLabel : function (renderOptions) {
            if (renderOptions.editable && !renderOptions.readOnly && !renderOptions.locked && renderOptions.column.enableEditor) {
                return renderOptions.descriptionIds.cellState.editableCell;
            }
        },

        getAriaLabeledBy : function (renderOptions) {
            var labeledBy = ' aria-labelledby="';
            var labels = renderOptions.labels;
            if (!renderOptions.isEmpty) {
                labels += ' ' + renderOptions.id;
            }

            if (renderOptions.clickableNode) {
                if (renderOptions.expanded) {
                    labels += ' ' + renderOptions.descriptionIds.cellState.expandedState;
                    labels += ' ' + renderOptions.descriptionIds.cellState.expanded;
                } else {
                    labels += ' ' + renderOptions.descriptionIds.cellState.collapsedState;
                    labels += ' ' + renderOptions.descriptionIds.cellState.collapsed;
                }
            }

            labeledBy += labels + '"';
            return labeledBy;
        },

        getValueMarkup : function (renderOptions) {
            var valueToRender = renderOptions.formattedValue;
            return valueToRender;
        },

        getIconMarkup : function (renderOptions) {
            return renderOptions.iconMarkup;
        },

        getClassMarkup : function (renderOptions) {
            var classes = ' class="pgbu-cell ';
            classes += (renderOptions.modified ? 'cell-modified ' : '');
            classes += (renderOptions.locked ? 'cell-locked ' : '');
            classes += (renderOptions.readOnly || renderOptions.locked ? 'cell-uneditable ' : '');
            classes += (renderOptions.selected && !renderOptions.printable ? (renderOptions.cellSelectedClasses + ' ') : '');
            classes += (!renderOptions.invalid ? '' : 'cell-invalid ');
            classes += (renderOptions.clickableNode && renderOptions.hasChildren ? 'cell-expandable group-row-top ' : '');
            classes += (renderOptions.clickableNode && !renderOptions.expanded && renderOptions.hasChildren ? ' group-row-bottom ' : '');
            classes += renderOptions.cellLocationClasses;
            classes += renderOptions.cutCopyClasses != null ? renderOptions.cutCopyClasses : '';
            classes += renderOptions.column.dataIndex === 'empty-column' ? 'cell-forced-empty ' : '';
            classes += renderOptions.lastRecordAtLevel ? 'group-row-bottom ' : '';
            classes += renderOptions.focused && renderOptions.selected ? 'cell-focused ' : '';
            classes += renderOptions.column.columnClass + ' ';
            classes += this.getClasses(renderOptions);

            if (renderOptions.invalid != null) {
                var allWarnings = _.every(_.values(renderOptions.invalid), function (error) {
                    return error.warning;
                });
                if (allWarnings) {
                    classes += ' cell-warning ';
                    renderOptions.allWarning = true;
                }
            }
            classes += '" ';
            return classes;
        },

        getColumnClasses : function (/* renderOptions */) {
            return ' ';
        },

        getClasses : function (/* renderOptions */) {
            return '';
        },

        getAttributes : function (/* renderOptions */) {
            return '';
        },

        getSpacers : function (level, groupBandClasses) {
            var html = '';
            if (level != null) {
                var nextClassIndex = 0;
                for (var i = 0; i <= level; i++) {
                    if (nextClassIndex > groupBandClasses.length - 1) {
                        nextClassIndex = 0;
                    }
                    html += '<div class="group-spacer ' + groupBandClasses[nextClassIndex] + '"/>';
                    ++nextClassIndex;
                }
            }
            return html;
        },

        getExcelCellData : function (renderOptions) {
            return renderOptions.formattedValue;
        },

        getExcelCell : function (renderOptions) {
            var value = _.escape(_.unescape(this.getExcelCellData(renderOptions)));
            var cell = {};
            if (value) {
                cell.value = value;
                cell.inline = true;
            }
            return cell;
        },

        /**
         * Override to change the format of the data
         * @param column - column definition
         * @param data - value to be formatted
         * @method format
         * @private
         */
        format : function (renderOptions) {
            return renderOptions.data == null ? '' : renderOptions.column.formatter.format(renderOptions.data);
        },

        getAlignment : function (renderOptions) {
            if (renderOptions.locked) {
                return ' ' + columnAlignment.center;
            }
            return ' ' + columnAlignment[renderOptions.column.align] || columnAlignment.left;
        }
    });

    return renderer;
});
