define([
    'ojs/ojcore',
    'knockout',
    'jquery',
    'ojs/ojknockout',
    'ojs/ojinputtext',
    'gridapi.lodash',
    'gridapi.grid/gridMixins',
    'gridapi.grid/eventHandlers',
    'gridapi.grid/utils/keyCode'
], function (oj, ko, $, ojKo, inputText, _, mixins, eventHandlers, keyCode) {
    'use strict';

    var renderer = function () {};

    renderer.extend = mixins.extendFn;

    // alignment of text
    var columnAlignment = {
        left: '', // 'cell-align-left', no need to do this, save some css speed
        right: 'cell-align-right',
        center: 'cell-align-center'
    };

    _.assign(renderer.prototype, {
        /**
         * Override to build cell html
         */
        create: function (column, options) {
            this.searchFields = options.searchFields;
            this.searchFields[column.dataIndex] = {};

            var html = '<th tabindex="-1" role="gridcell" unselectable="on" data-index="' + column.dataIndex + '" id="' + options.id + '" ';

            html += this.getClassMarkup(options) + ' ';
            if (column.abbr != null) {
                html += ' abbr="' + column.abbr + '"';
            }
            html += ' width="' + column.width + '"';

            html += ' >'; // closing the th

            if (!((column.dataIndex === 'empty-column') || (column.dataIndex === 'row-selector') || (column.dataIndex === 'menu-button-column') || (column.disableColumnFilter === true))) {
                html += '<input ' + 'placeholder=' + options.placeHolderText + ' class="lux-columnFilter input" ' + this.getAriaMarkup(options, column.disableColumnFilter) + ' type="text">';
            }

            if (column.disableColumnFilter === true) {
                html += '<div ' + 'placeholder=' + options.placeHolderText + ' class="lux-columnFilter " ' + this.getAriaMarkup(options, column.disableColumnFilter) + ' ></div>';
            }

            html += '</th>';

            this.searchFields[column.dataIndex].$th = $(html);

            return this.searchFields[column.dataIndex].$th;
        },

        moveUp_into: function (args) {
            var $ele = this.searchFields[args.colId].$th.find('input, div');
            this.requestFocus($ele);
        },
        moveDown_into: function (args) {
            var $ele = this.searchFields[args.colId].$th.find('input, div');
            this.requestFocus($ele);
        },
        moveRight: function (args) {
            var thId = this.searchFields[args.colId].$th.attr('id');
            var $input = $('#' + thId).find('input, div');
            this.requestFocus($input);
        },
        moveLeft: function (args) {
            var thId = this.searchFields[args.colId].$th.attr('id');
            var $input = $('#' + thId).find('input, div');
            this.requestFocus($input);
        },

        requestFocus : function ($element) {
            setTimeout(function () {
                $element.focus();
            }, 50);
        },

        attachListeners: function (column, $column, grid) {
            var that = this;

            var $input = this.searchFields[column.dataIndex].$th.find('input');
            var invokeSearch = _.debounce(function () {
                that.searchFields[column.dataIndex].term = $input.val();
                var columnFilter = {};
                _.forEach(that.searchFields, function (value, key) {
                    if (value.term !== undefined && value.term.trim() !== '') { // if pressing backspace makes a string empty, then remove that col
                        var recentlyUpdated = (key === column.dataIndex) ? true : false;
                        columnFilter[key] = { term: value.term, dataIndex: key, recentlyUpdated: recentlyUpdated };
                    }
                });
                grid._columnBasedFilter({ columnFilter: columnFilter });
            }, 50);
            $input.on('keyup', function (event) {
                // don't invoke search for simple navigation between cells.
                if (!((event.keyCode === keyCode.UP) ||
                        (event.keyCode === keyCode.DOWN) ||
                        (event.keyCode === keyCode.SHIFT) ||
                        (event.keyCode === keyCode.TAB)
                    )) {
                    invokeSearch();
                }
            });

            var gridView = grid.view;
            $input.on('keydown', function (event) {
                if ((event.keyCode === keyCode.LEFT) ||
                    (event.keyCode === keyCode.RIGHT) ||
                    (event.keyCode === keyCode.SPACE) ||
                    (event.keyCode === keyCode.BACKSPACE) ||
                    (event.keyCode === keyCode.HOME) ||
                    (event.keyCode === keyCode.END)
                ) {
                    event.stopPropagation();
                } else {
                    /* let events bubble up to scrolly-grid and then
                     * eventHandler--> selectionModel
                     * Handle it in selectionModel
                     */
                    $.proxy(eventHandlers.keydown, gridView)();
                    if (event.keyCode === keyCode.ESCAPE) {
                        gridView.requestFocus();
                    }
                }
            });
            $input.on('mousedown', function (event) {
                var $target = $(event.target);
                if ($target.is('.lux-columnFilter')) {
                    var dataIndex = $target.parent().data('index');
                    gridView.widget.selectColumnFilterCell(dataIndex); // selecting a cell without recordId
                    $input.focus();
                }
            });
        },

        getAriaMarkup: function (renderOptions, disableColumnFilter) {
            var ariaMarkup = ' tabindex="-1"';
            ariaMarkup += this.getAriaLabeledBy(renderOptions, disableColumnFilter);

            var describedBy = this.getAriaDescribedBy(renderOptions, disableColumnFilter);
            if (describedBy != null) {
                ariaMarkup += describedBy;
            }

            return ariaMarkup;
        },

        getAriaDescribedBy: function (renderOptions/*, disableColumnFilter*/) {
            var describedBy = [];
            var description = this.getDescriptionLabel(renderOptions);
            if (description != null) {
                describedBy.push(description);
            }
            if (renderOptions.showErrors) {
                describedBy.push(renderOptions.descriptionIds.cellState.errorOnCell);
            }
            if (renderOptions.describedBy != null) {
                describedBy = describedBy.concat(renderOptions.describedBy);
            }
            if (describedBy.length) {
                return ' aria-describedby="' + describedBy.join(' ') + '" ';
            }
        },

        getDescriptionLabel: function (renderOptions) {
            if (renderOptions.editable && !renderOptions.readOnly && !renderOptions.locked && renderOptions.column.enableEditor) {
                return renderOptions.descriptionIds.cellState.editableCell;
            }
        },

        getAriaLabeledBy: function (renderOptions, disableColumnFilter) {
            var labeledBy = ' aria-labelledby="';
            var labels = renderOptions.descriptionIds.cellState.columnFilter;
            if (!renderOptions.isEmpty) {
                labels += ' ' + renderOptions.headerId;
            }

            if (disableColumnFilter) {
                labels += ' ' + renderOptions.descriptionIds.cellState.disableColumnFilter;
            }

            labeledBy += labels + '"';
            return labeledBy;
        },

        getValueMarkup: function (renderOptions) {
            var valueToRender = renderOptions.formattedValue;
            return valueToRender;
        },

        getClassMarkup: function (renderOptions) {
            var classes = ' class="pgbu-cell ';
            classes += (renderOptions.modified ? 'cell-modified ' : '');
            classes += (renderOptions.locked ? 'cell-locked ' : '');
            classes += (renderOptions.readOnly || renderOptions.locked ? 'cell-uneditable ' : '');
            classes += (renderOptions.selected && !renderOptions.printable ? (renderOptions.cellSelectedClasses + ' ') : '');
            classes += (!renderOptions.invalid ? '' : 'cell-invalid ');
            classes += (renderOptions.clickableNode && renderOptions.hasChildren ? 'cell-expandable group-row-top ' : '');
            classes += (renderOptions.clickableNode && !renderOptions.expanded && renderOptions.hasChildren ? ' group-row-bottom ' : '');
            classes += renderOptions.cellLocationClasses;
            classes += renderOptions.cutCopyClasses != null ? renderOptions.cutCopyClasses : '';
            classes += renderOptions.lastRecordAtLevel ? 'group-row-bottom ' : '';
            classes += renderOptions.focused && renderOptions.selected ? 'cell-focused ' : '';
            classes += this.getClasses(renderOptions);

            if (renderOptions.invalid != null) {
                var allWarnings = _.every(_.values(renderOptions.invalid), function (error) {
                    return error.warning;
                });
                if (allWarnings) {
                    classes += ' cell-warning ';
                    renderOptions.allWarning = true;
                }
            }
            classes += '" ';
            return classes;
        },

        getClasses: function (/* renderOptions */) {
            return '';
        },

        getAttributes: function (/* renderOptions */) {
            return '';
        },

        getSpacers: function (level, groupBandClasses) {
            var html = '';
            if (level != null) {
                var nextClassIndex = 0;
                for (var i = 0; i <= level; i++) {
                    if (nextClassIndex > groupBandClasses.length - 1) {
                        nextClassIndex = 0;
                    }
                    html += '<div class="group-spacer ' + groupBandClasses[nextClassIndex] + '"/>';
                    ++nextClassIndex;
                }
            }
            return html;
        },

        /**
         * Override to change the format of the data
         * @param column - column definition
         * @param data - value to be formatted
         */
        format: function (renderOptions) {
            return renderOptions.data == null ? '' : renderOptions.column.formatter.format(renderOptions.data);
        },

        getAlignment: function (renderOptions) {
            if (renderOptions.locked) {
                return ' ' + columnAlignment.center;
            }
            return ' ' + columnAlignment[renderOptions.column.align] || columnAlignment.left;
        }
    });

    return renderer;
});
