/**
 * columns should be a dataIndex keyed object
 *
 * @typedef module:luxGrid#columnDefinition
 * @public
 * @property {String} dataIndex data field name in the store
 * @property {String} title column header text
 * @property {String} [tooltip=''] text displayed when hovered over column header
 * @property {String} [headerAlign='left'] alignment of column header text 'left', 'right', 'center'
 * @property {String} [align='left'] alignment of column data text 'left', 'right', 'center'
 * @property {String} [width='100px'] width of the column
 * @property {Boolean} [resizable=false] can the column be resized
 * @property {Boolean} [draggable=false] can the column be moved to a new column position
 * @property {String} [dataType='string'] data type inside of column
 <h6>Possible values for this option are:</h6>
 <ul>
<table>
 <tr>
   <th>dataType</th>
   <th>displayType</th>
 </tr>
 <tr>
   <td>string</td>
   <td>text, textArea, link</td>
 </tr>
 <tr>
   <td>date</td>
   <td>date</td>
 </tr>
 <tr>
   <td>boolean</td>
   <td>check</td>
 </tr>
 <tr>
   <td>float</td>
   <td>number, cost, percent</td>
 </tr>
</table>
</ul>
 * @property {Boolean} [readOnly=false] force the column data not to be editable
 * @property {Boolean} [filterable=false] can the column be filtered
 * @property {Boolean} [sortable=false] can the column be sorted
 * @property {Boolean} [hideable=''] can the column be hidden
 * @property {Boolean} [menu=true] display column menu
 * @property {Boolean} [formatter] a function used to format data in the column
 * @property {Boolean} [rowHeader] a boolean flag used to determine if the this column should be used as ARIA row-header
 * @property {Boolean} [formattedSearch=true]  Specifies how the cell values in the column are formatted for filter matching. If true, the cell values are formatted per the applied formatters, then the filter matching is performed; this is essentially filter matching on the displayed values (what the user sees). If false, the unformatted data values (the raw data as they appear in the JSON data input to the grid) are used for filter matching.
 * @property {Boolean} [disableColumnFilter] a boolean flag used to determine if the column-filter text input be added to this column
 <h6>The default values used for this option specific to the column type are:</h6>
 <ul>
<table>
 <tr>
   <th>displayType</th>
   <th>disableColumnFilter</th>
 </tr>
 <tr>
   <td>check</td>
   <td>true</td>
 </tr>
 <tr>
   <td>text</td>
   <td>false</td>
 </tr>
 <tr>
   <td>textArea</td>
   <td>false</td>
 </tr>
 <tr>
   <td>link</td>
   <td>true</td>
 </tr>
  <tr>
   <td>number</td>
   <td>false</td>
 </tr>
  <tr>
   <td>cost</td>
   <td>false</td>
 </tr>
  <tr>
   <td>percent</td>
   <td>false</td>
 </tr>
  <tr>
   <td>date</td>
   <td>false</td>
 </tr>
  <tr>
   <td>select</td>
   <td>false</td>
 </tr>
  <tr>
   <td>selectMultiple</td>
   <td>false</td>
 </tr>
</table>
</ul>
 * @property {String} displayType type of column to be used for renderer and editor
 *
<div>
<h6>Possible values for this option are:</h6>
 <ul>
<li>
    <b>text</b>
    <p>
        One of the simple, common columns is the text column. It is exactly what it sounds like: A
        column made to contain short blurbs of text (think of it as a regular text input).
    </p>
    <p>An example configuration</p>
    <pre class="prettyprint" style="display: inline-block">
textCol: {
    title: 'Text Column',
    displayType: 'text', // default value
    dataType: 'string', // default value
    width: '150px'
}</pre>
</li>
<li>
    <b>textArea</b>
    <p>
        A text area column is similar to a text column except that it is made with larger blurbs of text in mind.
        While a text column is fit to handle single words or similarly short lenghts of text, a text area column
        is better suited to handle something like a full description or a long address. When double clicked,
        rather than having a simple text input, it displays a text area that the user can expand/contract to fit
        their needs.
    </p>
    <p>An example configuration</p>
    <pre class="prettyprint" style="display: inline-block">
textAreaCol: {
    title: 'Text Area Column',
    displayType: 'textArea',
    dataType: 'string', // default value
    width: '240px'
}</pre>
</li>
<li>
    <b>date</b>
    <p>
        The date column is made for handling formatted dates. Date values should be in local ISO strings. See i18n demo for formatting for the locale. However timezone is not supported.
    </p>
    <p>
        See ojet validation and formatters for formatting and validation.
    </p>
    <pre class="prettyprint" style="display: inline-block">
dateCol: { // unique key
    title: 'Date Column',
    displayType: 'date', //column type. Required, Default value is 'text'
    dataType: 'date', //type of data, Default value is 'string'
    dateType: 'date', // Used to specify whether you want a date column or a date/time column.
    width: '200px',
    formatter: oj.Validation.converterFactory(oj.ConverterFactory.CONVERTER_TYPE_DATETIME)
        .createConverter({
            formatType: 'date',
            dateFormat: 'long'
        }),
    validators: [{
            type: 'dateTimeRange',
            options: {
                min: oj.IntlConverterUtils.dateToLocalIso(new Date(2004, 10, 25)),
                max: oj.IntlConverterUtils.dateToLocalIso(new Date(2016, 11, 6))
            }
        }]
}</pre>
</li>

<li>
    <b>check</b>
    <p>The check column is fairly simple. It simply provides a cell with a checkbox. Data is formatted as
        either 'true' or 'false'.
    </p>
    <pre class="prettyprint" style="display: inline-block">
checkCol: {
    title: 'Check Column',
    displayType: 'check',
    dataType: 'boolean',
    align: 'center',
    width: '100px'
}</pre>
</li>
<li>
    <b>link</b>
    <p>This can be used to create columns with links.
    </p>
    <pre class="prettyprint" style="display: inline-block">
linkCol: {
    title: 'Link Column',
    displayType: 'link', // default value
    dataType: 'string', // default value
    width: '98px'
}</pre>
</li>
<li>
    <b>number</b>
    <p>Use <code>(displayType: 'number', spinner: false)</code></p>
    <pre class="prettyprint" style="display: inline-block">
'numberCol': {
    title: 'Number',
    displayType: 'number',
    //dataType: 'float',
    width: '98px'
},
</pre>
    <p>
        Default cost and percent 'converters' have been set in columnDefFunctions.js. Please refer to OJET number converters and validators for more options on formatting and validation.
    </p>
</li>
<li>
    <b>cost</b>
    <p>Use <code>(displayType: 'number', spinner: false)</code></p>
    <pre class="prettyprint" style="display: inline-block">
'costCol': {
    title: 'Cost',
    displayType: 'cost',
    dataType: 'float',
    width: '130px',
    // LUX 0.7 //formatter : numberFormatter({     format : '$0,0.00',        symbol : '$'     }),
    formatter: oj.Validation.converterFactory('number')
        .createConverter({
            style: 'currency',
            currency: 'USD',
            currencyDisplay: 'code',
            pattern: '¤ ##,##0.00'
        }),

    validators: [{
        type: 'numberRange',
        options: {
            min: 10000.05,
            max: 25000.95,
            hint: {
                inRange: 'Enter a value between {min} and {max}.'
            }
        }
    }]
}
</pre>
    <p>
        Default cost and percent 'converters' have been set in columnDefFunctions.js. Please refer to OJET number converters and validators for more options on formatting and validation.
    </p>
</li>
<li>
    <b>percent</b>
    <p>Use <code>(displayType: 'number', spinner: false)</code></p>
    <pre class="prettyprint" style="display: inline-block">
'percentCol': {
    title: 'Percent',
    displayType: 'percent',
    dataType: 'float',
    width: '100px'
}</pre>
    <p>
        Default cost and percent 'converters' have been set in columnDefFunctions.js. Please refer to OJET number converters and validators for more options on formatting and validation.
    </p>
</li>
<li>
    <b>number</b> (spinner)
    <p>Use of spinner column is deprecated.</p>
    <p>Use <code>(displayType: 'number', spinner: true)</code> to achive similar effect.</p>
    <pre class="prettyprint" style="display: inline-block">
'spinnerNumberCol': {
    title: 'Number (Spinner)',
    displayType: 'number',
    spinner: true,
    width: '150px'
}</pre>
    <p>
        Default cost and percent 'converters' have been set in columnDefFunctions.js. Please refer to OJET number converters and validators for more options on formatting and validation.
    </p>
</li>

<li>
    <b>cost</b> (spinner)
    <p>Use of spinner column is deprecated.</p>
    <p>Use <code>(displayType: 'number', spinner: true)</code> to achive similar effect.</p>
    <pre class="prettyprint" style="display: inline-block">
'spinnerCostCol': {
    title: 'Cost (Spinner)',
    displayType: 'cost',
    spinner: true,
    width: '160px',
    formatter: oj.Validation.converterFactory('number')
        .createConverter({
            style: 'currency',
            currency: 'USD',
            currencyDisplay: 'code',
            pattern: '¤ ##,##0.00'
        }),

    validators: [{
        type: 'numberRange',
        options: {
            min: 15.05,
            max: 125000.95,
            hint: {
                inRange: 'Enter a value between {min} and {max}.'
            }
        }
    }]
}</pre>
    <p>
        Default cost and percent 'converters' have been set in columnDefFunctions.js. Please refer to OJET number converters and validators for more options on formatting and validation.
    </p>
</li>
<li>
    <b>percent</b> (spinner)
    <p>Use of spinner column is deprecated.</p>
    <p>Use <code>(displayType: 'number', spinner: true)</code> to achive similar effect.</p>
    <pre class="prettyprint" style="display: inline-block">
'spinnerPercentCol': {
    title: 'Percent (Spinner)',
    displayType: 'percent',
    spinner: true,
    width: '150px',
    formatter: oj.Validation.converterFactory('number')
        .createConverter({
            style: 'percent',
            maximumFractionDigits: 2,
            useGrouping: false
        })
}</pre>
    <p>
        Default cost and percent 'converters' have been set in columnDefFunctions.js. Please refer to OJET number converters and validators for more options on formatting and validation.
    </p>
</li>
<li>
    <b>select</b>
    <p>The select column allows a user to select a value from a set list of values,
        displayed in a drop-down select menu.
    </p>
    <p> column options specific to this column  type are:
    <ul style="list-style: none;">
        <li>selectValues : The formatted list of values used to populate the list.</li>
        <li>disableTypeahead : Boolean flag used to determine whether the typeahead is allowed.
            Default is false.
        </li>
        <li>selectMultiple (this option is not supported after LUX .7, instead use displayType: 'selectMultiple') : Boolean flag used to determine whether the select column allows multiple
            selections. Default is false.
        </li>
    </ul>
    </p>
    <p>First, we need to have a set list of values to pick from. They should be formated as:
    </p>
    <pre class="prettyprint" style="display: inline-block">
var fruitValues = {
    "_apple_" : {
        "parentId" : "0",
        "id" : "_apple_",
        "name" : "Apple"
    },
    ...,
    "_melon_" : {
        "parentId" : "0",
        "id" : "_melon_",
        "name" : "Melon"
    }
}</pre>
    <p>
        Then we set our options as follows:
    </p>
    <pre class="prettyprint" style="display: inline-block">
selectCol: {
    title: 'Select Column',
    displayType: 'select',
    width: '220px',
    selectValues: fruitValues,
    disableTypeahead: true
}</pre>

</li>

<li>
    <b>selectMultiple</b>
    <p>Select column with multi-select functionality. </p>
    <p> column options specific to this column  type are:
    <ul style="list-style: none;">
        <li><b>selectValues</b> : The formatted list of values used to populate the list.</li>
        <li><b>disableTypeahead</b> : Boolean flag used to determine whether the typeahead is allowed.
            Default is false.
        </li>
    </ul>
    </p>
   <p>First, we need to have a set list of values to pick from. They should be formated as:
</p>
    <pre class="prettyprint" style="display: inline-block">
var fruitValues = {
    "_apple_" : {
        "parentId" : "0",
        "id" : "_apple_",
        "name" : "Apple"
    },
    ...,
    "_melon_" : {
        "parentId" : "0",
        "id" : "_melon_",
        "name" : "Melon"
    }
}</pre>
    <p>
        Then we set our options as follows:
    </p>
    <pre class="prettyprint" style="display: inline-block">
selectMultiCol: {
    title: 'Select Multi Column',
    displayType: 'selectMultiple',
    selectMultiple: true,
    width: '220px',
    selectValues: fruitValues,
    disableTypeahead: true
}</pre>

</li>

<li>
    <b>Icon Column</b>
    <p>
        The icon column is actually somewhat of a misnomer; Any column can be made to have icons. All you have
        to do is set 'showIcon' to true in the column options, configure the 'type' attribute of 'recordKeys' to
        match the dataIndex of the column you want to show icons, and set which icons you want to use for which
        records in the 'recordIcons' attribute.
    </p>
    <p> specific options that can be used with icon column are:
    <ul style="list-style: none;">
        <li>dataDisplay : Specifies the order in which icon and value are displayed. Possible values are 'icon-value' and 'value-icon'
        </li>
    </ul>
    <pre class="prettyprint" style="display: inline-block">$target.grid({
    ...,
    recordKeys : {
        id : 'id',
        type : 'selectCol'
    },
    recordIcons : {
        default : {
            imageSource : 'img/resource-labor.png',
            classes : ''
        },
        '_melon_' : {
            imageSource : 'img/yellow_smile.gif',
            classes : ''
        }
    },
    columns : {
        groups : [
            {
                id : 'bar',
                type : 'flex',
                columns : [
                    {
                        title : 'Icon Column',
                        dataIndex : 'selectCol',
                        displayType : 'select',
                        width : '200px',
                        selectValues : fruitValues,
                        disableTypeahead : true,
                        showIcon : true
                    }
                ]
            }
        ]
    }
});</pre>
</li>
</ul>
</div>

 * @property {Boolean} [maxLength=null]
 * @property {Boolean} [hasDisabledMode=false]
 * @property {Boolean} [rollupRole=null]
 * @property {Number} [rollupOrder=0]
 * @property {Boolean} [enableEditor=true]
 * @property {String} [dataDisplay='value']
 * @property {Boolean} [visible=true]
 * @property {String} [minWidth=null]
 * @property {Boolean} [filldown=true]
 * @property {Array} [joinedColumns]
 * @property {String} [columnClass='']
 * @property {Boolean} [searchable=true]
 * @property {Boolean} [hideAsterisk=false]
 */

 define(['gridapi.lodash'], function (_) {
     'use strict';
     return {
         dataIndex: undefined, // data name in object
         title: '_no-header_', // column header text
         tooltip: '', // text displayed when hovered over column header
         headerAlign: 'left', // alignment of column header text 'left', 'right', 'center'
         align: 'left', // alignment of column data text 'left', 'right', 'center'
         width: '100px', // width of the column
         resizable: false, // can the column be resized
         draggable: false, // can the column be moved to a new column position
         displayType: 'text', // type of column to be used for renderer and editor
         dataType: 'string', // data type inside of column
         readOnly: false, // force the column data not to be editable
         filterable: false, // can the column be filtered
         sortable: false, // can the column be sorted
         hideable: false, // can the column be hidden
         menu: true, // display column menu
         maxLength: null,
         hasDisabledMode: false,
         rollupRole: null,
         rollupOrder: 0,
         enableEditor: true,
         dataDisplay: 'value',
         visible: true, // internal use only - for column menu
         minWidth: null,
         filldown: true,
         joinedColumns: [],
         columnClass: ' ',
         searchable: true,
         hideAsterisk: false,
         formattedSearch: true,
         grouper: {
             getGroupValue: function (value) {
                 return value;
             }
         },
         advancedFilters : {
             enabled : true
         },
         escaping : {
             autoEscapeRender : true,
             autoEscapeEditor : true,
             escape : _.escape.bind(_),
             unescape : _.unescape.bind(_)
         },
         allowClearCells : true
     };
 });
