define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils/keyCode',
    'gridapi.columns/core/editor',
    'gridapi.columns/core/renderer',
    'gridapi.columns/core/headerRenderer',
    'gridapi.oui'
], function ($, _, keyCode, editor, renderer, headerRenderer, oui) {
    'use strict';

    /*
     * Picker column assumes that data is held in an array.  Objects in the array at the minimal should have id, parentId and name
     * [{
     *     id : 'fooId',
     *     parentId : 'fooParentId',
     *     name : 'fooName'
     * }]
     */

    return {
        defaultOptions : {
            width : '150px',
            columnClass : 'picker-column',
            listDelim : ','
        },
        formatter : {
            format : function (datas, column) {
                var values = '';
                if (!_.isArray(datas)) {
                    datas = [datas];
                }
                _.forEach(datas, function (data, index) {
                    if (data != null) {
                        values += data.name;

                        if (index < datas.length - 1) {
                            // if not last item then add a delimiter
                            values += column.listDelim;
                        }
                    }
                });
                return values;
            },

            validate : function () {
                return true;
            }
        },

        headerRenderer : headerRenderer.extend({}),

        renderer : renderer.extend({
            format : function (renderOptions) {
                var column = renderOptions.column;
                return column.formatter.format(renderOptions.data, column);
            }
        }),

        editor : editor.extend({
            show : function (startString) {
                this.render(startString);

                var $editor = this.getEditor();

                $editor.show();

                this.drawFocusToInput();
            },

            setupModal : function () {

            },

            initModalComponents : function () {

            },

            init : function (options) {
                // have to clone as data will be an array for multi-select
                this.newData = _.clone(options.data);
                this.persistData = false;// By default only persist if they select a new item

                var that = this;
                var properties = [];

                _.forEach(options, function (value, name) {
                    that[name] = value;
                    properties.push(name);
                });

                // use to determine if commit is already in process
                // this is just used to prevent commit being called again from the grid
                this.commitStarted = false;
                this.ignoreSelectText = false;
                // cache the property names we have set so we can delete later on
                that.prop = properties;

                if (this.column.sharedPicker) {
                    if (this.column.staticComponents == null) {
                        this.column.staticComponents = {};
                    }

                    this.column.staticComponents.setNewPickerData = _.noop;
                    this.column.staticComponents.setNewTypeAheadData = _.noop;
                }
            },

            destroy : function () {
                if (this.column.sharedPicker) {
                    this.column.staticComponents.innerEditor.detach();

                    this.column.staticComponents.setNewPickerData = _.noop;
                    this.column.staticComponents.setNewTypeAheadData = _.noop;
                } else {
                    if (this.typeahead != null) {
                        this.typeahead.destroy();
                    }

                    // Remove the modal that gets created
                    if (this.picker != null) {
                        this.picker.destroy();
                    }
                }

                var $editor = this.getEditor();
                if ($editor.length) {
                    $editor.remove();
                }
            },

            onKeyDown : function (event) {
                // Not calling prototype keydown when typeahead is on the picker due to the triggers
                // on change
                switch (event.keyCode) {
                case keyCode.TAB :
                    this.onTab(event);
                    break;
                    /* falls through */
                case keyCode.ESCAPE:
                case keyCode.ENTER:
                    this.fromKeyboard = true;
                    editor.prototype.onKeyDown.apply(this, arguments);
                    break;
                }
            },

            makeStaticPickerElement : _.once(function () {
                var that = this,
                    initData = {
                        selected: this.data || []
                    },
                    $input;

                var lazyLoad = (this.column.doublePicker.shared && this.column.doublePicker.shared.lazyLoad) ||
                    (this.column.doublePicker.source && this.column.doublePicker.source.tree.lazyLoad) || false;

                if (!lazyLoad && this.column.doublePicker.source && this.column.doublePicker.source.tree &&
                    this.column.doublePicker.source.tree.storeFns &&
                    this.column.doublePicker.source.tree.storeFns.load) {
                    initData.available = this.column.doublePicker.source.tree.storeFns.load();
                }

                this.column.staticComponents.innerEditor = $('<div class="cell-editor-picker-wrapper"><input type="text" /></div>');
                $input = this.column.staticComponents.innerEditor.find('input');

                if (this.column.typeAhead == null && this.column.doublePicker.createTypeAhead == null) {
                    this.column.doublePicker.createTypeAhead = {
                        appendTo : this.column.staticComponents.innerEditor
                    };
                } else if (this.column.typeAhead != null) {
                    // Grid was given typeahead options so using that to construct the typeahead
                    this.column.doublePicker.createTypeAhead = false;
                }

                this.column.staticComponents.picker = new oui.DoublePicker(this.column.doublePicker, $input, initData);

                if (_.isObject(this.column.doublePicker.modal)) {
                    var components = this.setupModal(this.column.doublePicker.modal);

                    this.column.doublePicker.modal.one('shown.modal-init', function () {
                        that.initModalComponents(that.column.staticComponents.picker, components);
                    });
                }

                if (this.column.typeAhead != null) {
                    this.column.typeAhead.appendTo = this.column.staticComponents.innerEditor;
                    this.column.staticComponents.typeAhead = new oui.TypeAhead(this.column.typeAhead, $input, initData);
                }

                this.column.staticComponents.picker.on('accept', function (nodes) {
                    that.column.staticComponents.setNewPickerData(nodes);
                });

                if (this.column.staticComponents.typeAhead) {
                    this.column.staticComponents.typeAhead.on('select', function (nodes) {
                        that.column.staticComponents.setNewTypeAheadData(nodes);
                    });
                }
            }),

            makeNewPicker : function () {
                var that = this,
                    $editor,
                    initData = {
                        selected : this.data || []
                    };

                var lazyLoad = (this.column.doublePicker.shared && this.column.doublePicker.shared.lazyLoad) ||
                    (this.column.doublePicker.source && this.column.doublePicker.source.tree.lazyLoad) || false;

                if (!lazyLoad && this.column.doublePicker.source && this.column.doublePicker.source.tree &&
                    this.column.doublePicker.source.tree.storeFns &&
                    this.column.doublePicker.source.tree.storeFns.load) {
                    initData.available = this.column.doublePicker.source.tree.storeFns.load();
                }
                $editor = $('<div class="cell-editor cell-editor-picker"><input type="text"/></div>');
                var $input = $editor.find('input');

                if (this.column.typeAhead == null && this.column.doublePicker.createTypeAhead == null) {
                    this.column.doublePicker.createTypeAhead = {
                        appendTo : $editor
                    };
                } else if (this.column.typeAhead != null) {
                    // Grid was given typeahead options so using that to construct the typeahead
                    this.column.doublePicker.createTypeAhead = false;
                }

                this.picker = new oui.DoublePicker(this.column.doublePicker, $input, initData);

                if (_.isObject(this.column.doublePicker.modal)) {
                    var components = this.setupModal(this.column.doublePicker.modal);

                    this.column.doublePicker.modal.one('shown.modal-init', function () {
                        that.initModalComponents(that.picker, components);
                    });
                }

                if (this.column.typeAhead != null) {
                    this.column.typeAhead.appendTo = $editor;
                    this.typeahead = new oui.TypeAhead(this.column.typeAhead, $input, initData);
                }

                return $editor;
            },

            make : function () {
                this.fromKeyboard = false;
                if (this.column.sharedPicker) {
                    var $editor = $('<div class="cell-editor cell-editor-picker"></div>');
                    this.makeStaticPickerElement();
                    this.column.staticComponents.innerEditor.appendTo($editor);
                    return $editor;
                } else {
                    return this.makeNewPicker();
                }
            },

            renderStaticPickerElement : function (/* startString */) {
                var that = this,
                    $editorInput = this.getEditorInput().val('');

                this.column.staticComponents.picker.init();

                if (this.column.staticComponents.typeAhead && this.column.typeAhead != null) {
                    this.column.staticComponents.typeAhead.init();
                }

                this.column.staticComponents.setNewPickerData = function (newData) {
                    that.newData = newData;

                    if (!that.typeahead) {
                        $editorInput.val(that.column.formatter.format(that.newData, that.column));
                    }
                    that.persistData = true;
                    that.commit({});
                };

                this.column.staticComponents.setNewTypeAheadData = function (newData) {
                    that.newData = newData;
                    that.persistData = true;
                };

                this.setDataOnPicker();
            },

            renderNewPickerElement : function (/* startString */) {
                this.picker.init();

                if (this.typeahead) {
                    this.typeahead.init();
                }

                // TypeAhead may have been created by default. Let's see if it exists
                if (this.typeahead == null) {
                    this.typeahead = oui.get(this.getEditorInput(), oui.TypeAhead.prototype.name());
                }

                this.setDataOnPicker();
                this.attachPickerListener();
            },

            render : function (startString) {
                if (this.column.sharedPicker) {
                    this.renderStaticPickerElement(startString);
                } else {
                    this.renderNewPickerElement(startString);
                }

                if (startString != null && this.column.typeAhead != null) {
                    // this.getEditorInput().select();
                    this.getEditorInput().trigger($.Event('keypress', { which : startString.charCodeAt(0) }));
                    this.getEditorInput().val(startString);
                } else if (this.column.typeAhead == null) {
                    // if the data is not valid then just display it
                    this.getEditorInput().val(this.formatValueForInput()); // if typeahead dont do this
                }
            },

            formatValueForInput : function () {
                if (!this.invalid) {
                    return this.column.formatter.format(this.newData, this.column);
                } else {
                    return this.data;
                }
            },

            /*
             * You have this.record, this.data, this.column, this.column.staticComponents.picker and more available.
             */
            setDataOnPicker : function () {
                var data = _.isArray(this.data) ? this.data : [this.data];
                data = _.compact(data);
                if (this.column.sharedPicker) {
                    if (this.column.staticComponents.typeAhead) {
                        this.column.staticComponents.typeAhead.value(data);
                    } else {
                        this.column.staticComponents.picker.value(data);
                    }
                } else if (this.typeahead) {
                    this.typeahead.value(data);
                } else {
                    this.picker.value(data);
                }
            },

            attachPickerListener : function () {
                var that = this;
                var $editorInput = this.getEditorInput();

                if (!this.column.sharedPicker) {
                    this.picker.on('accept', function (nodes) {
                        that.newData = nodes;

                        if (!that.typeahead) {
                            $editorInput.val(that.column.formatter.format(that.newData, that.column));
                        }
                        that.persistData = true;
                        that.commit({});
                    });

                    if (this.typeahead) {
                        this.typeahead.on('select', function (nodes) {
                            that.newData = nodes;
                            that.persistData = true;
                        });
                    }
                }
            },

            getNewData : function () {
                return {
                    value : this.newData || (this.column.sharedPicker ? this.column.staticComponents.picker.selected() : this.picker.selected()),
                    valid : true
                };
            },

            commit : function () {
                this.commitStarted = true;
                if (this.persistData || this.column.required) { // By default only persist if they select a new item
                    var checkUserInput = this.getNewData();
                    var persist = {
                        record : this.record,
                        value : checkUserInput.value,
                        valid : checkUserInput.valid,
                        column : this.column
                    };
                    // persist the new value
                    this.onPersistFn(persist);
                }
                this.commitFinished = true;
                // close editor
                this.close();
                return true;
            }
        })
    };
});
