define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.moment',
    'gridapi.logger',
    'gridapi.grid/constants',
    'gridapi.grid/RollupBase',
    'gridapi.grid/rollupUtils'
], function ($, _, moment, logger, constants, RollupBase, rollupUtils) {
    'use strict';

    var rollups = rollupUtils.rollups;
    var normalizeDataIndices = rollupUtils.normalizeDataIndices;
    var CLEAR_AND_FILL = { clear : true, fill : true };

    var DataRollup = function (args) {
        var that = this;
        this.widget = args.widget;
        this.dataCache = args.widget.dataCache;

        // override mediator functions from optionsmn
        _.forEach(this.widget.options.dataRollup.dataRollupFns, function (fnc, fncName) {
            that[fncName] = fnc || that[fncName];
        });

        RollupBase.call(this);
    };

    DataRollup.prototype = _.create(RollupBase.prototype, {
        constructor : DataRollup,

        setStore : function (store) {
            this.store = store;
        },

        buildAndRollupTree : function (args) {
            if (this.widget.groupingApplied) {
                return;
            }
            if (this.widget.options.dataRollup.enabled || this.widget.options.data != null) {
                this.buildTree();
            }
            if (this.widget.options.dataRollup.enabled) {
                this.forceAllRollups(args);
            }
        },

        buildTree : function () {
            var that = this;
            _.forEach(this.getRootIds(), function (root) {
                that.addTreeNode({ recordId : root });
            });
        },

        getRootIds : function () {
            var roots = [];
            if (this.widget.options.data != null) {
                roots = this.widget._getRecordsByData({ types : this.widget.options.data.rootTypes });
            } else {
                this.store.iterateStore(this.widget._getRecordFromStore, this.widget);
                roots = this.widget.storeRecords;
            }
            return this.widget._getRecordIds(roots);
        },

        addTreeNode : function (args) {
            var dataCache = this.dataCache;
            var widget = this.widget;
            var recordId = args.recordId;
            dataCache.getRecordCache(recordId, CLEAR_AND_FILL);
            var children = widget._getChildRecords(recordId);
            var recordIds = widget._getRecordIds(children);
            if (_.has(args, 'parentId')) {
                dataCache.setParent({
                    recordId : recordId,
                    parentId : args.parentId
                });
            }
            dataCache.setChildren({
                recordId : recordId,
                childIds : recordIds
            });
            var toAdd = { parentId : recordId, recordId : null };
            for (var i = 0, len = children.length; i < len; ++i) {
                toAdd.recordId = recordIds[i];
                this.addTreeNode(toAdd);
            }
        },

        getRecordsFromNode : function (node) {
            var that = this;
            var records = [];
            _.forEach(node.childIds, function doGetRecordsFromNode(recordId) {
                var recordCache = that.dataCache.getRecordCache(recordId);
                if (!recordCache.attributes.isNode && !recordCache.hasChildren || !recordCache.attributes.rollup) {
                    var record = that.widget._getRecord(recordId);
                    if (record) {
                        records.push(record);
                    }
                }
            });
            return records;
        },

        _enabled : function () {
            return this.widget.options.dataRollup.enabled;
        },
        _getNode : function (nodeId) {
            return this.dataCache.getRecordCache(nodeId);
        },
        _getChildNodesFromNode : function (node) {
            return this.getChildNodesFromNode(node);
        },
        _getRecordsFromNode : function (node) {
            return this.getRecordsFromNode(node);
        },
        _getNodeId : function (node) {
            return node.recordId;
        },
        _makeRollupArgs : function (node, dataIndices, records, childNodes) {
            return {
                node : node,
                dataIndices : normalizeDataIndices(dataIndices),
                records : records,
                childNodes : childNodes
            };
        },
        rollupNodeValues : function (groupId, dataIndices, rowsetId) {
            logger.warn('rollupNodeValues is deprecated - use rollupFromLeafs(<Array<Id>, dataIndices, rowsetId) instead');
            this.rollupFromLeafs([groupId], dataIndices, rowsetId);
        },


        getChildNodesFromNode : function (node) {
            var that = this;
            var childNodes = [];
            _.forEach(node.childIds, function doGetChildNodes(recordId) {
                var recordCache = that.dataCache.getRecordCache(recordId);
                if (recordCache.attributes.rollup && (recordCache.attributes.isNode || recordCache.hasChildren)) {
                    childNodes.push(recordCache);
                }
            });
            return childNodes;
        },

        rollupStackedColumns : function (args, commonArgs) {
            if (!this.widget.options.stackedCells.enabled || !this.widget.options.stackedCells.rollupEnabled) {
                return;
            }

            var stackedColumns = commonArgs.stackedColumns;
            var stackedSubRows = commonArgs.stackedSubRows,
                records = args.records,
                childNodes = args.childNodes,
                node = args.node,
                that = this;
            _.forEach(stackedSubRows, function (stackedSubRow) {
                _.forEach(stackedColumns, function (stackedColumn) {
                    var valueKey = stackedColumn.dataIndex + '-' + stackedSubRow.dataIndex;
                    var rollupValue = null;
                    var values = _.map(records, function (record) {
                        return that.widget._getStackedValueFromRecord(record, stackedColumn.dataIndex, stackedSubRow.dataIndex);
                    });

                    if (childNodes.length) {
                        // Average median and range need to look at all records to properly calculate
                        if (stackedSubRow.rollupRole === 'average' || stackedSubRow.rollupRole === 'median' || stackedSubRow.rollupRole === 'range') {
                            logger.warn('average, median and range unsupported for tree rollups at this time.');
                            return;
                        } else {
                            _.forEach(childNodes, function (childCache) {
                                values.push(childCache.stackedRollupValues[valueKey]);
                            });
                        }
                    }

                    values = _.filter(values, function (value) {
                        return value != null;
                    });

                    if (values.length || stackedSubRow.rollupRole === 'store' || stackedSubRow.rollupRole === 'function') {
                        if (stackedSubRow.rollupRole === 'store') {
                            if (that.store.rollupValues != null) {
                                rollupValue = that.store.rollupValues(values, stackedSubRow.dataIndex, {
                                    records : records,
                                    childGroups : childNodes, // childGroups for consistency if people are reusing logic in mediator
                                    childNodes : childNodes,
                                    values : values,
                                    node : node,
                                    group : node, // group for consistency
                                    rollupValues : node.rollupValues,
                                    stackedRollupValues : node.stackedRollupValues
                                });
                            }
                        } else if (stackedSubRow.rollupRole === 'function') {
                            if (stackedSubRow.rollupFn != null) {
                                rollupValue = stackedSubRow.rollupFn({
                                    values : values,
                                    records : records,
                                    childGroups : childNodes,
                                    group : node,
                                    node : node,
                                    rollupValues : node.rollupValues,
                                    stackedRollupValues : node.stackedRollupValues
                                });
                            }
                        } else {
                            rollupValue = rollups[stackedSubRow.rollupRole](values);
                        }
                        node.stackedRollupValues[valueKey] = rollupValue;
                    } else if (node.stackedRollupValues && node.stackedRollupValues[valueKey]) {
                        delete node.stackedRollupValues[valueKey];
                    }
                });
            });
        },

        rollupStandardColumns : function (args, commonArgs) {
            var that = this;
            var standardColumns = commonArgs.standardColumns,
                records = args.records,
                childNodes = args.childNodes,
                node = args.node;
            _.forEach(standardColumns, function rollupStandardLoop(column) {
                var rollupValue = null;
                var values = _.map(records, function (record) {
                    return that.store.getValueFromRecord(record, column.dataIndex);
                });

                if (childNodes.length) {
                    // Average median and range need to look at all records to properly calculate
                    if (column.rollupRole === 'average' || column.rollupRole === 'median' || column.rollupRole === 'range') {
                        logger.warn('average, median and range unsupported for tree rollups at this time.');
                        return;
                    } else {
                        _.forEach(childNodes, function (childNode) {
                            values.push(childNode.rollupValues[column.dataIndex]);
                        });
                    }
                }

                if (column.rollupRole !== 'minDateNull' && column.rollupRole !== 'maxDateNull') {
                    values = _.filter(values, function (value) {
                        return value != null;
                    });
                }

                if (values.length || column.rollupRole === 'store' || column.rollupRole === 'function') {
                    if (column.rollupRole === 'store') {
                        if (that.store.rollupValues != null) {
                            rollupValue = that.store.rollupValues(values, column.dataIndex, {
                                records : records,
                                childNodes : childNodes,
                                childGroups : childNodes, // Group for backwards compatability in mediator
                                values : values,
                                node : node,
                                group : node, // Group for backwards compatability in mediator
                                rollupValues : node.rollupValues
                            });
                        }
                    } else if (column.rollupRole === 'function') {
                        if (column.rollupFn != null) {
                            rollupValue = column.rollupFn({
                                values : values,
                                records : records,
                                childNodes : childNodes,
                                childGroups : childNodes, // Group for backwards compatability in mediator
                                node : node,
                                group : node, // Group for backwards compatability in mediator
                                rollupValues : node.rollupValues
                            });
                        }
                    } else {
                        rollupValue = rollups[column.rollupRole](values);
                    }
                    node.rollupValues[column.dataIndex] = rollupValue;
                } else if (node.rollupValues && node.rollupValues[column.dataIndex]) {
                    delete node.rollupValues[column.dataIndex];
                }
            });
        },

        rollupTimephasedColumns : function (args, commonArgs) {
            if (!this.widget.options.timephased.enabled) {
                return;
            }
            var timephasedBuckets = commonArgs.timephasedBuckets;
            var timephasedSubRows = commonArgs.timephasedSubRows,
                records = args.records,
                childNodes = args.childNodes,
                node = args.node,
                that = this;
            _.forEach(timephasedSubRows, function (timephasedSubRow) {
                _.forEach(timephasedBuckets, function (timephasedBucket) {
                    var rollupValue = null;
                    var bucketKey = timephasedBucket.granularity + '-' + timephasedBucket.bucketId;
                    var valueKey = bucketKey + '.' + timephasedSubRow.dataIndex;
                    var values = _.map(records, function (record) {
                        return that.widget._getTimephasedValueFromRecord({
                            recordId : that.widget._getRecordId(record),
                            record : record,
                            dataIndex : timephasedSubRow.dataIndex,
                            bucketId : timephasedBucket.bucketId,
                            granularity : timephasedBucket.granularity
                        });
                    });

                    if (childNodes.length) {
                        // Average median and range need to look at all records to properly calculate
                        if (timephasedSubRow.rollupRole === 'average' || timephasedSubRow.rollupRole === 'median' || timephasedSubRow.rollupRole === 'range') {
                            logger.warn('average, median and range unsupported for tree rollups at this time.');
                            return;
                        } else {
                            _.forEach(childNodes, function (childCache) {
                                values.push(_.get(childCache.timephasedRollupValues, valueKey));
                            });
                        }
                    }

                    values = _.filter(values, function (value) {
                        return value != null;
                    });

                    if (values.length || timephasedSubRow.rollupRole === 'store' || timephasedSubRow.rollupRole === 'function') {
                        if (timephasedSubRow.rollupRole === 'store') {
                            if (that.store.rollupValues != null) {
                                rollupValue = that.store.rollupValues(values, timephasedSubRow.dataIndex, {
                                    records : records,
                                    childGroups : childNodes, // Groups for backwards compatability in mediator
                                    childNodes : childNodes,
                                    values : values,
                                    group : node, // Groups for backwards compatability in mediator
                                    node : node,
                                    rollupValues : node.rollupValues,
                                    timephasedRollupValues : node.timephasedRollupValues[bucketKey]
                                });
                            }
                        } else if (timephasedSubRow.rollupRole === 'function') {
                            if (timephasedSubRow.rollupFn != null) {
                                rollupValue = timephasedSubRow.rollupFn({
                                    values : values,
                                    records : records,
                                    childGroups : childNodes, // Groups for backwards compatability in mediator
                                    childNodes : childNodes,
                                    group : node, // Groups for backwards compatability in mediator
                                    node : node,
                                    rollupValues : node.rollupValues,
                                    timephasedRollupValues : node.timephasedRollupValues[bucketKey]
                                });
                            }
                        } else {
                            rollupValue = rollups[timephasedSubRow.rollupRole](values);
                        }
                        _.set(node.timephasedRollupValues, valueKey, rollupValue);
                    } else if (_.get(node.timephasedRollupValues, valueKey)) {
                        delete node.timephasedRollupValues[bucketKey][timephasedSubRow.dataIndex];
                    }
                });
            });
        },

        recordsAdded : function (toAdd) {
            var that = this;
            var toRollup = [];
            var rowset;
            _.forEach(toAdd, function (args) {
                var recordCache = that.dataCache.getRecordCache(args.recordId, { clear : true, fill : true });
                if (recordCache.parentId != null) {
                    var parentCache = that.dataCache.getRecordCache(recordCache.parentId, {
                        clear : true,
                        fill : true
                    });
                    parentCache.childIds.push(args.recordId);
                    parentCache.hasChildren = true;
                    rowset = rowset || parentCache.placement.rowsetId;
                    toRollup.push(parentCache.recordId);
                }
            });
            that.rollupFromLeafs(toRollup, null, rowset);
        },

        recordsRemoved : function (recordIds) {
            var that = this;
            var recordIdMap = _.reduce(recordIds, function (memo, id) {
                memo[id] = true;
                return memo;
            }, {});
            var toRollup = [];
            var rowsetId;
            _.forEach(recordIds, function (recordId) {
                var recordCache = that.dataCache.getRecordCache(recordId);
                if (recordCache == null) { // already removed
                    logger.warn('Trying to remove a record that isnt here, you might be duplicating calls');
                    return;
                }

                var newRowset = recordCache.placement && recordCache.placement.rowsetId;
                //>>excludeStart("validation", pragmas.production)
                logger.assert(rowsetId == null || rowsetId === newRowset, 'all deleted records must belong to the same rowset - how was this violated?');
                //>>excludeEnd("validation")
                rowsetId = newRowset;

                that.dataCache.resetRecordCache(recordId);
                var parentId = recordCache.parentId;
                if (parentId != null) {
                    var parentCache = that.dataCache.getRecordCache(recordCache.parentId, {
                        clear : true,
                        fill : true
                    });
                    parentCache.childIds = _.without(parentCache.childIds, recordId);

                    // if parent wasn't queued for deletion already, queue rollup
                    if (!recordIdMap[parentId]) {
                        toRollup.push(parentId);
                    }
                }
            });

            that.rollupFromLeafs(toRollup, null, rowsetId);
        },

        recordUpdated : function (args) {
            var recordCache = this.dataCache.getRecordCache(args.recordId);
            if (recordCache == null) {
                logger.warn('Updated record does not exist ');
                return;
            }
            if (recordCache.parentId != null) {
                var parentCache = this.dataCache.getRecordCache(recordCache.parentId, {
                    clear : true,
                    fill : true
                });
                this.rollupFromLeafs([parentCache.recordId], [args.dataIndex], parentCache.placement.rowsetId);
            }
        },

        recordMoved : function (args) {
            var oldRecordCacheParent = this.dataCache.getRecordCache(args.recordId).parentId;
            var recordCache = this.dataCache.getRecordCache(args.recordId, { clear : true, fill : true });
            var toRollup = [];
            var rowset;
            if (oldRecordCacheParent !== recordCache.parentId) {
                var oldParentCache = this.dataCache.getRecordCache(oldRecordCacheParent);
                var newParentCache = this.dataCache.getRecordCache(recordCache.parentId);
                if (oldParentCache != null) {
                    oldParentCache.childIds = _.without(oldParentCache.childIds, args.recordId);
                    toRollup.push(oldParentCache.recordId);
                    rowset = oldParentCache.placement.rowsetId;
                }
                if (newParentCache != null) {
                    newParentCache.childIds.push(args.recordId);
                    toRollup.push(newParentCache.recordId);
                    rowset = newParentCache.placement.rowsetId;
                }
            }
            if (toRollup.length) {
                this.rollupFromLeafs(toRollup, null, rowset);
            }
        },

        forceAllRollups : function (args) {
            if (this.widget.options.dataRollup.enabled) {
                var leafsToRollup = [];
                _.forEach(this.dataCache.recordCache, function (recordCache) {
                    if (recordCache.attributes && recordCache.attributes.rollup && !_.isEmpty(recordCache.childIds)) {
                        leafsToRollup.push(recordCache.recordId);
                    }
                });
                var dataIndices = args && args.dataIndices || null;
                this.rollupFromLeafs(leafsToRollup, dataIndices);
                this.widget._forceViewRender();
            }
        }
    });

    return DataRollup;
});
