define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.grid/utils/keyCode',
    'gridapi.grid/gridMixins',
    'gridapi.logger',
    'gridapi.grid/touchInteractions/utils',
    'gridapi.grid/touchInteractions/gestures',
    'gridapi.grid/touchInteractions/dragHandles',
    'gridapi.grid/rows/rowMoveFns',
    'gridapi.grid/constants'
], function ($, _, keyCode, mixins, logger, utils, Gestures, DragHandles, rowMoveFns, constants) {
    'use strict';

    var TouchInteractions = function (args) {
        var that = this;
        that.eventHandlers = args.eventHandlers;
        that.options = args.options;
        that.widget = args.widget;
        that.view = args.gridView;

        that.logger = new utils.LoggerOnScreen({ height: 200, width: 300, showMessages: false });
        that.dH = new DragHandles(args, that.logger);

        that._isTouch = utils.isTouchDevice();
        that.rowMoveFns = rowMoveFns(args.gridView);
    };


    TouchInteractions.prototype = {

        constructor: TouchInteractions,

        // last column which is used as visual cue for DnD, class-'cell-grabber' would make make the cursor= move;
        isRowMoverTd : function (event) {
            var $target = $(event.originalEvent.target);
            var $td = $target.closest('td');
            var cellGrabber = $td.children().find('.cell-grabber').length;
            return (cellGrabber) ? $td : false;
        },

        // called from selectionModel.focusCell (after editing dragHandles needs to be adjusted.)
        resetDragHandles: function () {
            var that = this;
            if (that.options.enabled && that._isTouch) {
                setTimeout(function () {
                    if (that.dH.exist()) {
                        that.dH.repositionDragHandles();
                    }
                }, 100);
            }
        },

        clearFocus: function () {
            var that = this;
            // clear previous selection
            that.view.selectionModel.clearFocus();
        },

        makeSelection: function (event, $td) {
            var that = this;
            // clear previous selection
            that.clearFocus();

            that.view.selectionModel.cell.cellSelected($td, {
                append: false,
                event: event
            });
        },

        singleTapHandler: function (event, multiSelect) {
            var that = this;
            //that.logger.log('singleTapHandler');
            var $target = $(event.target);

            if (!that.dH.isItDragHandle($target)) {
                var $td = $target.closest('td');
                if (!$td.is('.editing') && !$target.is('.cell-grabber') && !$target.parents().is('.viewport-headers')) { // viewport-headers
                    that.makeSelection(event, $td);
                    if (multiSelect) {
                        if (!that.dH.exist()) {
                            that.dH.createDragHandles();
                            that.dH.trackTouchMove();
                        }
                        that.dH.repositionDragHandles();
                    }
                }// else --> let mousedown do it's job.  // eventHander.mousedown should not be prevented. It breaks : editing, columnMenu ...
            } //else  // clickign on drag handle
        },


        initListeners: function () {
            var that = this;
            var rowMoveShadeOn = false;

            if (that.options.enabled && that._isTouch) {
                var $grid = that.view.$grid;
                // do not attach handlers to column-filter and headers
                var $list = $grid.find('.grid-rowset').not('.rowset-' + constants.COLUMNFILTER_ROWSET_ID).not('.rowset-' + constants.HEADERS_ROWSET_ID);
                var gesture = new Gestures($list, that.logger);

                // immediate DnD when on the column with hamburger icons, singleTap is noticed after a delay.
                gesture.register('touchStart', function (event) {
                    if (that.isRowMoverTd(event)) {
                        that.singleTapHandler(event, that.widget.options.multiSelect);
                    }
                });

                gesture.register('singleTap', function (event) {
                    if (!that.isRowMoverTd(event)) {
                        that.singleTapHandler(event, that.widget.options.multiSelect);
                    }
                });

                gesture.register('doubleTap', function (event) {
                    var colType = that.widget.columnDefinitions[that.view.selectionModel.focus.columnId].displayType;
                    var isAndroid = utils.isAndroid();
                    if (!isAndroid) { // for iPad
                        $.proxy(that.eventHandlers.doubleClick, that.view)()(event);
                        that.view.currentEditor.drawFocusToInput();
                    }

                    if (!that.dH.exist()) {  // if direct double-tap; dragHandles were never crated, however they need to apper after done editing.
                        that.dH.createDragHandles();
                        that.dH.trackTouchMove();
                    }

                    that.dH.repositionDragHandles();

                    // for date column
                    if (colType && colType === 'date') {
                        setTimeout(function () {
                            if (!isAndroid) { // for iPad
                                that.view.requestFocus();
                            }
                            that.view.currentEditor.dateField.ojInputDate('show');
                        }, 170); // the 170 ms wait time is based on the testing on iPad and Andriod tablet.
                    }
                });

                gesture.register('touchMoveAll', function (event) {
                    if (that.isRowMoverTd(event)) {
                        if (!rowMoveShadeOn) {
                            var $target = $(event.target);
                            var $td = $target.closest('td');
                            that.rowMoveFns.initMoveRow($td.closest('div.grid-rowset'), $td.parent(), event.originalEvent.touches[0].clientY);
                            rowMoveShadeOn = true;
                            // stream of throttled functions can fire after mouseup, so cancel it.
                            that.throttled = _.throttle(that.rowMoveFns.mousemoveFn(5), 20);
                            that.mouseupFn = that.rowMoveFns.mouseupFn(that.throttled);
                        }
                        that.throttled(event.originalEvent.touches[0]);
                        event.preventDefault();
                        event.stopPropagation();
                    }
                });

                gesture.register('touchEnd', function (event) {
                    if (rowMoveShadeOn) {
                        that.mouseupFn(event.originalEvent);
                        rowMoveShadeOn = false;
                    }
                });

                gesture.register('allComplete', function (event) {
                    that.dH.showDragHandles(event);
                });
            }
        },


        attachColumnListeners: function (column) {
            var that = this;
            var columnMenuOpen, gesture;

            if (that.options.enabled && that._isTouch) {
                columnMenuOpen = false;

                gesture = new Gestures(column, that.logger);

                gesture.register('touchStart', function (event) {
                    that.view.closeMenus();
                    if (that.view.selectionModel.focus.rowsetId === constants.COLUMNFILTER_ROWSET_ID) {
                        that.view.selectionModel.cell.firstCellInGrid();  // don't want to select columnFilter row
                    } else {
                        that.widget.selectCell();  // 'row-selected-bottom and row-selected-top classes'
                    }
                });

                gesture.register('press', function (event) {
                    var $target = $(event.target);
                    if (!utils.isAndroid()) { // for iPad
                        that.view.openColumnMenu($target.closest('th'), event, false);
                        columnMenuOpen = true;
                        event.preventDefault();
                        event.stopPropagation();
                    }

                    setTimeout(function () {
                        var columnMenuId = that.view.componentIds.columnMenu;
                        var $menu = $('#' + columnMenuId);
                        var position = $target.closest('th').offset();
                        $menu.css('left', position.left);
                        $menu.css('top', position.top + $target.closest('th').outerHeight());
                    }, 700);  // wait for the keybaord to go away when repositioing the column menu.
                });

                gesture.register('touchEnd', function (event) {
                    if (columnMenuOpen) { // prevent sorting
                        event.preventDefault();
                        event.stopPropagation();
                    }
                    columnMenuOpen = false;
                    return event;
                });
            }
        }
    };

    return TouchInteractions;
});
