
define([
    'gridapi.lodash',
    'gridapi.logger',
    'gridapi.grid/utils/regex',
    'gridapi.grid/utils/strings'
], function (_, logger, regexes, strings) {
    'use strict';

    var HTML_TAG_CONTENTS = '(<[^>]*>)';

    
    function Matcher(components) {
        var that = this;
        function wrapTester(tester) {
            if (!tester) { return null; }

            var term = components.term;
            return function wrappingTester(val) {
                if (_.isFunction(val)) {
                    return val(term, that);
                }
                return tester.test(val);
            };
        }

        var priv = {};
        Object.defineProperty(this, '_priv', {
            enumerable : false,
            value : priv
        });
        priv.matchOnAny = components.matchOnAny;
        priv.positive = wrapTester(components.positive);
        priv.negative = wrapTester(components.negative);
        priv.positiveRaw = components.positive;
        priv.negativeRaw = components.negative;
        priv.passesPos = function (str) {
            return passesPosTerms(priv, str);
        };
        priv.passesNeg = function (str) {
            return passesNotTerms(priv, str);
        };

        if (!components.matchOnAny) {
            priv.all = _.map(components.positiveTerms, function (term) {
                var re = new RegExp(term, 'i');
                var tester = {
                    test : function (string) {
                        return string != null && re.test(string);
                    }
                };
                return wrapTester(tester);
            });
        }
    }

    var testString = function (priv, str) {
        if (!passesNotTerms(priv, str)) {
            return false;
        }
        return passesPosTerms(priv, str);
    };
    var passesPosTerms = function (priv, str) {
        if (priv.matchOnAny) {
            return priv.positive(str);
        } else {
            return _.every(priv.all, function (matcher) {
                return matcher(str);
            });
        }
    };
    var passesNotTerms = function (priv, str) {
        var neg = priv.negative;
        if (neg != null && neg(str)) {
            return false;
        }
        return true;
    };
    var lazyAny = function (chain, fn) {
        var res = chain.filter(fn).first();
        if (res instanceof _) { res = res.value(); }
        return res != null;
    };
    var lazyAll = function (chain, fn) {
        var res = chain.reject(fn).first();
        if (res instanceof _) { res = res.value(); }
        return res == null;
    };

    Matcher.prototype = {
        constructor : Matcher,

        
        exec : function (str) {
            //>>excludeStart("validation", pragmas.production)
            logger.assert(strings.isString(str) || (_.isNumber(str) && !_.isNaN(str)), 'exec must be called with strings or non-NaN numbers');
            //>>excludeEnd("validation")
            str = String(str);
            if (!str || !testString(this._priv, str)) {
                return null;
            }
            return _.map(this._priv.positiveRaw.exec(str), _.trim);
        },

        
        test : function (str) {
            if (!str) {
                return false;
            }

            var priv = this._priv;
            if (strings.isString(str) || _.isNumber(str)) {
                return testString(priv, String(str));
            } else if (_.isArray(str)) {
                if (priv.negative && !_.every(str, priv.passesNeg)) {
                    return false;
                }

                if (priv.matchOnAny) {
                    return _.some(str, priv.passesPos);
                } else {
                    return _.every(priv.all, function (tester) {
                        return _.some(str, tester);
                    });
                }
            } else if (str instanceof _) {
                if (priv.negative && !lazyAll(str, priv.passesNeg)) {
                    return false;
                }

                if (priv.matchOnAny) {
                    return lazyAny(str, priv.passesPos);
                } else {
                    var tests = priv.all;
                    return lazyAny(str, function (recordVal) {
                        return _.every(tests, function (tester) {
                            if (tester(recordVal)) {
                                tests = _.without(tests, tester);
                                return true;
                            }
                        });
                    });
                }
            }
            //>>excludeStart("validation", pragmas.production)
            logger.warn('Unexpected type of input given as test: ' + typeof str);
            //>>excludeEnd("validation")
            return false;
        },

        
        surroundMatches : function (str, opts) {
            //>>excludeStart("validation", pragmas.production)
            logger.assert(strings.isString(str), 'surroundMatches must be called with strings');
            //>>excludeEnd("validation")
            if (!testString(this._priv, str)) {
                return str;
            }

            var prefix = opts.prefix;
            var postfix = opts.postfix;
            var globalMatcher = new RegExp(HTML_TAG_CONTENTS + '|(' + this._priv.positiveRaw.source + ')', 'gi');
            return str.replace(globalMatcher, function (match) {
                if (!match) { return ''; }
                if (match[0] === '<' && match[match.length - 1] === '>') {
                    return match;
                }

                var parts = /^(\s*)([^]*?)(\s*)$/.exec(match);
                return parts[1] + prefix + parts[2] + postfix + parts[3];
            });
        }
    };

    // \b does NOT recognize boundaries on unicode characters; sub this in instead
    //  not perfect, since the prefix will include the space in the match.  not ideal, but...maybe acceptable?  better than not having exact matches on unicode at all
    var wordBoundaryPrefix = '(?:^|\\s)';
    var wordBoundaryPostfix = '(?=\\s|$)';
    var wordBoundarySearch = /\\w/;
    function markTermBoundary(word) {
        if (!word || !word.trim()) { return; }

        var firstChar = word[0];
        var startsWhitespace = wordBoundarySearch.test(firstChar);
        var endsWhitespace = wordBoundarySearch.test(word[word.length - 1]);
        var prefix = startsWhitespace ? '' : wordBoundaryPrefix;
        var postfix = endsWhitespace ? '' : wordBoundaryPostfix;

        return prefix + word + postfix;
    }

    function swapNumericStartsWith(term) {
        if (/^\d+$/.test(term)) {
            return wordBoundaryPrefix + '-?' + term;
        }
        return term;
    }

    var DEFAULT_OPTIONS = {
        matchOnAny : false,
        numbersUseStartsWith : true
    };

    
    function matcherFactory(term, args) {
        args = _.defaults({}, args, DEFAULT_OPTIONS);
        if (!strings.isString(term) || !term.trim().length) {
            return new Matcher({
                positiveTerms : [],
                positive : /^$/,
                negativeTerms : [],
                negative : false
            });
        }

        term = term.trim();
        term = regexes.escapeLiterals(term);
        term = term.replace(/&quot;/g, '"');

        var notTerms = [];
        var not = /-(?!\d)(?:"(.*?)"|(\S*))/g;
        var notMatch;
        
        while (notMatch = not.exec(term)) {
            // guard against terms like "foo-thing", which should be searched as a whole word, not considered a negative term
            var notPart = notMatch[0];
            var notInd = term.indexOf(notPart);
            if (notInd > 0 && /\S/.test(term[notInd - 1])) {
                continue;
            }

            term = term.replace(notMatch[0], '');
            not.lastIndex = 0;
            //>>excludeStart("validation", pragmas.production)
            logger.assert((notMatch[2] == null) !== (notMatch[1] == null), 'matched as both quoted and unquoted...how?');
            //>>excludeEnd("validation")

            var termPart = notMatch[2] || markTermBoundary(notMatch[1]);
            if (termPart) {
                notTerms.push(termPart);
            }
        }
        

        var searchTerms = [];
        var terms = _.groupBy(term.split('"'), function (value, i) {
            return i % 2 === 0 ? 'unquoted' : 'quoted';
        });
        terms.unquoted = _.flatten(_.map(terms.unquoted, function (term) {
            return term.split(' ');
        }));

        _.forEach(terms.quoted, function (word) {
            if (word.length) {
                searchTerms.push(markTermBoundary(word));
            }
        });

        _.forEach(terms.unquoted, function (word) {
            word = word.trim();
            if (word.length) {
                searchTerms.push(word);
            }
        });

        if (args.numbersUseStartsWith) {
            searchTerms = _.map(searchTerms, swapNumericStartsWith);
            notTerms = _.map(notTerms, swapNumericStartsWith);
        }

        var positiveMatcher = new RegExp(searchTerms.join('|'), 'i');
        var negativeMatcher = notTerms.length && (new RegExp(notTerms.join('|'), 'i'));
        return new Matcher({
            term : term,
            matchOnAny : args.matchOnAny,
            numbersUseStartsWith : args.numbersUseStartsWith,
            positiveTerms : searchTerms,
            positive : positiveMatcher,
            negativeTerms : notTerms,
            negative : negativeMatcher
        });
    }
    matcherFactory.Matcher = Matcher;

    return matcherFactory;
});
