/**
 * Copyright (c) 2013 - 2015 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Implements ojet data tree (ojtree) using luxnavigator as input. This widget takes a
 * luxhierarchy object as "data" input and converts to a ojtree.
 *
 * @author Joe McClintock
 /**
 * The luxnavtree widget
* @module luxnavtree
* @requires ojet
* @requires jquery
* @requires knockout
* @requires underscore
* @requires luxhierarchy
 */
define(['ojs/ojcore',
    'knockout',
    'jquery',
    'underscore',
    'luxhierarchy',
    'ojs/ojtree'],
function(oj, ko, $, _) {

    'use strict';
    (function() {
        $.widget('GBU.luxnavtree', {
            version:           '5.1.0',
            defaultElement:    '<div>',
            widgetEventPrefix: 'lux',
            /**
            * Options
            * <dl>
            *     <dt>data</dt>
            *     <dd>
            *         Use this option to pass in the LuxHierarchy object.</br>
            *         Type: String</br>
            *         Default: null</br>
            *         Example:</br>
            *         <code>data:</code>
            *         <pre class="prettyprint" style="overflow-x:visible;">
            *             var nodeid_0 = new LuxHierarchy().parseJSON('{"action": "main-content", "actionLabel": "Home", "childrenLabel": "Home Children Label"}'),
            *                 nodeid_0_0 = nodeid_0.addChild().parseJSON('{"action": "lux-introduction", "actionLabel": "Introduction", "childrenLabel": "Introduction Clildren Label"}'),
            *                 nodeid_0_0_0 = nodeid_0_0.addChild().parseJSON('{"action": "lux-introduction_goals", "actionLabel": "Goals"}'),
            *                 nodeid_0_0_1 = nodeid_0_0.addChild().parseJSON('{"action": "lux-introduction_characteristics", "actionLabel": "Characteristics"}'),
            *                 nodeid_0_0_2 = nodeid_0_0.addChild().parseJSON('{"action": "lux-introduction_starting-points", "actionLabel": "Starting points"}'),
            *                 node_0_1 = nodeid_0.addChild().parseJSON('{"action": "lux-standards", "actionLabel": "External requirements and Oracle standards",  "childrenLabel": "External requirements and Oracle standards Children"}'),
            *                 nodeid_0_1_0 = node_0_1.addChild().parseJSON('{"action": "lux-standards_logo-usage", "actionLabel": "Logo usage"}'),
            *                 nodeid_0_1_1 = node_0_1.addChild().parseJSON('{"action": "lux-standards_notices", "actionLabel": "Notices and disclaimers"}'),
            *                 nodeid_0_1_2 = node_0_1.addChild().parseJSON('{"action": "lux-standards_accessibility", "actionLabel": "Accessibility requirements"}'),
            *                 nodeid_0_2 = nodeid_0.addChild().parseJSON('{"action": "lux-ui-page-layout", "actionLabel": "UI page layout", "childrenLabel":""}'),
            *                 nodeid_0_3 = nodeid_0.addChild().parseJSON('{"action": "lux-theme", "actionLabel": "The LUX theme", "childrenLabel":""}'),
            *                .......
            *         </pre>
            *     </dd>
            *     <dt>optionChange</dt>
            *     <dd>
            *         Custom function for handling tree node clicks. Input parameters are "event" and "data".</br>
            *         Type: function<br />
            *         Default: null<br />
            *         Example:<br />
            *         <code>optionChange:
            *             NavigatorClick = function(event, data) {
            *                 var selected = self.arrayToStr(data.value);
            *
            *                 console.log(value + " clicked");
            *             }.bind(self);
            *         </code>
            *     </dd>
            *     <dt>currentState</dt>
            *     <dd>
            *         String representation of the currently selected tree node.  Use this in conjunction with
            *         a Knockout observable subscription to do something appropriate.<br />
            *         Type: String<br />
            *         Default: null<br />
            *         Example:<br />
            *         <code>currentState:
            *             self.currentState = ko.observable("home");
            *         </code>
            *     </dd>
            *     <dt>icons</dt>
            *     <dd>
            *         To show or not, the default ojtree icons.<br />
            *         Type: Boolean<br />
            *         Default: null<br />
            *         Example: icons: false<br />
            *     </dd>
            * </dl>
            */
            options:           {
                expandParents:          false,
                initExpanded:           null,
                initLoaded:             [],
                selection:              [],
                selectionMode:          'single',
                selectedParentCollapse: false,
                selectedParentExpand:   true,
                selectPrevOnDelete:     false,
                data:                   null,
                emptyText:              null,
                icons:                  true,
                types:                  null,
                before:                 null,
                collapse:               null,
                create:                 null,
                collapseAll:            null,
                cut:                    null,
                dehover:                null,
                remove:                 null,
                destroy:                null,
                expand:                 null,
                expandAll:              null,
                hover:                  null,
                loaded:                 null,
                move:                   null,
                optionChange:           null,
                paste:                  null,
                refresh:                null,
                rename:                 null
            },
            selectedNode: null,
            prefixId:     '',

            _create: function() {
                this.el = this.element;
                this._setup();
                this._refresh();
            },

            _setup: function() {
                this.data = this.options.data;
            },

            _refresh: function() {
                if (this.data.luxhierarchy) {
                    this.prefixId = 'tree-';
                    this.options.data = this._parseLuxHierarchy();
                }

                this._navTree = this.el.ojTree(this.options);
                this._onStateChange(this.options.currentState);
            },

            _parseLuxHierarchy: function() {
                var self = this,
                    luxhierarchy = this.data.luxhierarchy,
                    ojetTree = {title: '', attr: ''},
                    tree = [],
                    parseHierarchy = function(item, tree) {
                        var child = tree || {title: '', attr: ''},
                            itemLength = item.children.length;

                        child.title = item.actionLabel;
                        // create unique id with prefix 'tree-'
                        child.attr = {'title': item.actionLabel, 'id': self.prefixId + item.id, type: item.type};
                        if (item.children && itemLength > 0) {
                            child.children = [];
                            for (var i = 0; i < itemLength; i += 1) {
                                child.children.push(parseHierarchy(item.children[i]));
                            }
                        }
                        return child;
                    };

                if (_.isArray(luxhierarchy)) {
                    for (var i = 0; i < luxhierarchy.length; i += 1) {
                        tree.push(parseHierarchy(luxhierarchy[i]));
                    }
                } else {
                    tree.push(parseHierarchy(luxhierarchy.findRoot(), ojetTree));
                }
                return tree;
            },

            _onStateChange: function(value) {
                var self = this,
                    // create unique id with prefix 'tree-'
                    selectedNode = $('#' + self.prefixId + (_.isObject(value) ? value.id : value));

                if (self._navTree.ojTree('isSelected', selectedNode)) {
                    // node has already been expanded and selected so don't do anything
                    return;
                }

                self.selectedNode = selectedNode;

                self._navTree.ojTree('select', self.selectedNode);
                self._expandParents();
            },

            _expandParents: function() {
                var self = this,
                    parents = [],
                    node = this.selectedNode;

                while (node) {
                    node = self._navTree.ojTree('getParent', node);
                    if (node) {
                        parents.push(node);
                    }
                }

                parents.forEach(function(node) {
                    self._navTree.ojTree('expand', node);
                });
            },

            /**
             * _setOption: evaluates the options and calls the
             *             appropriate function to handle change
             *             event.
             * @param {String} key   key of the changed option
             * @param {String} value new value of the option
             */
            _setOption: function(key, value) {
                var self = this;
                // if the currentState option has changed pass
                // value to onStateChange function for action.
                if (key === 'currentState') {
                    self._onStateChange(value);
                }
            },

            /**
             * _setOptions: iterates over all changed options and passes
             *              key values to setOption for evaluation.
             *
             * @param {Object} options array of options
             */
            _setOptions: function(options) {
                var self = this;

                $.each(options, function(key, value) {
                    self._setOption(key, value);
                });
            },

            _destroy: function() {
                var self = this;

                self.el.remove();
            }

        });

    }());

});