/*jshint loopfunc: true */
/*global define */
/**
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Components that provide context, including many primary navigation components such as navigation lists, breadcrumbs and navigation trees, provide the most benefit when they are always visible to the user.
 * The luxpin functionality is used to keep a specific UI element (which could contain an entire component) visible within the viewport at all times, or as much as possible given other constraints. The luxpin
 * also makes sure that more important components in the viewport context like header and footer are not overlapped by the component that is using luxpin (we like to think of it as a componet that is pinned on the view-port!).

 * @author Shailesh Zirpe
 */

/**
 * The jQuery plugin namespace.
 * @external "jQuery.fn"
 * @see {@link http://docs.jquery.com/Plugins/Authoring The jQuery Plugin Guide}
 */

/**
 * The luxpin plugin
 * @module luxpin
 * @augments external:"jQuery.fn"
 * @requires jquery
 */

define(['jquery'], function($) {
    'use strict';

    var applyPin = function($this, settings, pvtData) {
        var scrollTop = $(window).scrollTop();
        // threshold is critical point where scrolling further would overlap the nav onto footer.
        var threshold = $(document).height() - (settings.offsetBottom + $this.outerHeight(true) + settings.marginTop);
        var thresholdOverflow = scrollTop - threshold;

        if (scrollTop < settings.offsetTop) {
            $this.css({'position': 'static'});
            pvtData.pinnedState = true; if (pvtData.unpinnedState === true && $.isFunction(settings.unpinnedCallback)) {
                settings.unpinnedCallback();
            } pvtData.unpinnedState = false;
        } else if (scrollTop < threshold) {
            $this.css({'position': 'fixed', 'top': settings.marginTop, 'bottom': 'auto'});
            pvtData.unpinnedState = true; if (pvtData.pinnedState === true && $.isFunction(settings.pinnedCallback)) {
                settings.pinnedCallback();
            } pvtData.pinnedState = false;
        } else {
            $this.css({'position': 'fixed', 'top': -thresholdOverflow});
        }
    };

    /**
 * Make it a JSDoc comment, if you would like to document plugins as methods on $.fn
 *
 * @function external:"jQuery.fn".luxpin
 * @param {object} options
 * @param  {Integer} options.offsetTop  offsetTop
 * @param  {Integer} options.offsetBottom  offsetBottom
 * @param  {Integer} options.marginTop  marginTop
 * @param  {function} options.pinnedCallback  pinnedCallback
 * @param  {function} options.unpinnedCallback  unpinnedCallback
 */

    $.fn.luxpin = function(options) {
        //make sure you have int values
        $.each(options, function(key, value) {
            if (typeof value !== 'function') {
                options[key] = parseInt(value, 10);
            }
        });

        return this.each(function() {
            var $this = $(this);
            var pvtData = {
                height:        $(document).height(),
                pinnedState:   false,
                unpinnedState: false
            };
            /** @lends module:luxpin */
            var defaults = {
                /**
                 * Options
                 * <dl>
                 *     <dt>offsetTop</dt>
                 *     <dd>
                 *        Specify the distance in pixels from the top of the page to avoid overlapping the page header. The plug-in tries to guess and avoid overlapping with header element, the default value for this options is the
                 *        distance of the element from top of the page when luxpin is not applied. </br>
                 *        Type: Integer  </br>
                 *        Default: the elements default distance from top of the page, if the plug-in had not been applied.  </br>
                 *        Example:  </br>
<pre class="prettyprint">
// luxpin
    $(&#39;.lux-docs-sidenav&#39;).luxpin({
        offsetTop: $(&#39;.hero-unit&#39;).outerHeight(true),
        offsetBottom: 20,
        marginTop: $(&#39;.hero-unit&#39;).css(&#39;margin-bottom&#39;)//,
        //pinnedCallback: function() { console.log(&#39;pinnedCallback 1 executing&#39;);   },
        //unpinnedCallback: function() { console.log(&#39;unpinnedCallback 1 executing&#39;); }
    });
</pre>
                 *     </dd>
                 *     <dt>offsetBottom</dt>
                 *     <dd>
                 *        Specify the distance in pixels from the bottom of the page to avoid overlapping the page footer. The plug-in tries to guess and avoid overlapping with footer element, the default value for this options is the
                 *        height of the footer element on the page.  </br>
                 *        Type: Integer  </br>
                 *        Default: height of the footer element.  </br>
                 *        Example:    </br>
                 *          <code class="prettyprint">
                 *              offsetBottom: 20
                 *          </code>
                 *     </dd>
                 *     <dt>marginTop</dt>
                 *     <dd>
                 *         Specify the distance from the top of the viewport, when the page is scrolled such as header is not displayed.    </br>
                 *         Type: Interger   </br>
                 *         Default: 200   </br>
                 *         Example:  </br>
                 *         <code class="prettyprint">marginTop: $('.hero-unit').css('margin-bottom')</code>
                 *     </dd>
                 *     <dt>pinnedCallback</dt>
                 *     <dd>
                 *         Using luxpin would toggle the pinned element (element that is using luxpin) between two states. When the position of the top of view-port is less than the offset-top
                            value you have set, the pinned element stays in the normal flow of the page, otherwise the luxpin would stop the element from scrolling. We like to think of it as the element is pinned on the view-port. You can
                            provide a callback function to be executed when the pinning triggers.   </br>
                 *         Type: function   </br>
                 *         Default: null    </br>
                 *         Example:    </br>
                 *         <code class="prettyprint">
                 *          pinnedCallback: function() { console.log('pinnedCallback executing');   }
                 *         </code>
                 *     </dd>
                 *     <dt>unpinnedCallback</dt>
                 *     <dd>
                 *         Using luxpin would toggle the pinned element (element that is using luxpin) between two states. When the position of the top of view-port is less than the offset-top
                            value you have set, the pinned element stays in the normal flow of the page, otherwise the luxpin would stop the element from scrolling. We like to think of it as the element is pinned on the view-port. You can
                            provide a callback function to be executed when pinning is released and the pinned element comes back in the default scrolling flow.     </br>
                 *         Type: function    </br>
                 *         Default: null     </br>
                 *         Example:    </br>
                 *         <code class="prettyprint">
                 *          pinnedCallback: function() { console.log('un pinned callback  executing, element is in normal scrolling flow');   }
                 *         </code>
                 *     </dd>
                 *
                 * </dl>
                 */
                options: {
                    offsetTop:        $this.offset().top,
                    offsetBottom:     $('footer:first').length ? $('footer:first').outerHeight(true) : 10,
                    marginTop:        200,
                    pinnedCallback:   function() {},
                    unpinnedCallback: function() {}
                }
            };
            var settings = $.extend({}, defaults.options, options);
            $this.css({'-webkit-transform': 'translateZ(0)'});
            $(window).on('scroll', function() {
                applyPin($this, settings, pvtData);
            });
            $(document).ready(function() {
                applyPin($this, settings, pvtData);
            });
        });
    };

    // options using data-*
    /*
    $(document).ready(function() {
        $("[data-bind*='luxpin']").each(function() {
            var jq_nav = $(this);
            var options = jq_nav.data('bind');
            if (options.component === 'luxpin') {
                var settings = $.extend(defaults.options, options);
                jq_nav.luxpin(options);
            }
        });
    });
    */

}); //end of define