/*jshint loopfunc: true */
/*global define */
/**
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * If you want to provide lots of information on single page, you would end up with long page and lots of scrolling.
 * To organize information on the page it is common practice to divide page content into multiple sections with id’s which
 * work as page markers and then provide navigation list so that users can directly jump to area of their interest.
 * luxscrollposition plugin adds context to the scrolled location on the page by highlighting the appropriate item in
 * the list while scrolling. Getting visual feedback from the page is a nice way to improve  responsiveness of the page,
 * making it more friendly to work with and more readable at the same time.
 *
 * @author Shailesh Zirpe
 */

/**
 * The luxscrollposition plugin
 * @module luxscrollposition
 * @augments external:"jQuery.fn"
 * @requires jquery
 */

define(['jquery'], function($) {
    $(document).ready(function() {
        //"use strict";
        var pluginName = 'luxscrollposition';
        /** @lends module:luxscrollposition */
        var defaults = {
            /**
                     * Options
                     * <dl>
                     *     <dt>offsetTop</dt>
                     *     <dd>
                     *        Specify the distance in pixels from the top of the viewport to add as an offset when calculating position of scroll. </br>
                     *        Type: Integer  </br>
                     *        Default: 100  </br>
                     *        Example:  </br>
<pre class="prettyprint">
    //luxscrollposition
    $('.lux-docs-sidenav').luxscrollposition({
        offsetTop:200,
        //highlightCallback: function(id){console.log("highlitedCallback:  " + id);},
        //unhighlightCallback: function(id){console.log("un-highlitedCallback:  " + id);}
    });
</pre>
                     *     </dd>
                     *     <dt>highlightCallback</dt>
                     *     <dd>
                     *         This is trigged when a particular list item is being highlighted while scrolling the page. The id of the 'li' being highlighted is available as callback argument. </br>
                     *         Type: function   </br>
                     *         Default: null    </br>
                     *         Example:    </br>
                     *         <code class="prettyprint">
                     *          highlightCallback: function(id){console.log("highlitedCallback:  " + id);}
                     *         </code>
                     *     </dd>
                     *     <dt>unhighlightCallback</dt>
                     *     <dd>
                     *         When you scroll away from a particular section, unhighlightCallback is trigged, you get the id of un-highlighted 'li' in the callback as argument.   </br>
                     *         Type: function    </br>
                     *         Default: null     </br>
                     *         Example:    </br>
                     *         <code class="prettyprint">
                     *          unhighlightCallback: function(id){console.log("un-highlitedCallback:  " + id);}
                     *         </code>
                     *     </dd>
                     *
                     * </dl>
                     */
            options: {
                offsetTop:           100,
                highlightCallback:   function(){},
                unhighlightCallback: function(){}
            }
        };
        // plugin constructor
        function Plugin(element, options) {
            this.$el = $(element);
            this.settings = $.extend({}, defaults.options, options) ;
            this.pvtData = {
                height:             $(document).height(),
                highlightedState:   false,
                unhighlightedState: false
            };
            //this.map= new Array();
            this._defaults = defaults.options;
            this._name = pluginName;
            this.init();
        }

        Plugin.prototype = {
            init: function() {
                var that = this;
                this.$el.css({'-webkit-transform': 'translateZ(0)'});
                this.createMap();
                $(window).on('scroll', function(){
                    that.applyhighlight();
                });
                $(document).ready(function(){
                    that.applyhighlight();
                });
            },
            createMap: function () {
                var that = this;
                var map = [];
                this.$el.find('a[href^=\'#\']').each(function(){
                    var jump = ($(this).attr('href').replace(/^#/g, ''));
                    if ($('[id=' + jump + ']').length > 0){
                        map.push({
                            id:     jump,
                            min:    Math.round($('[id=' + jump + ']').offset().top) - that.settings.offsetTop,
                            max:    $(document).height(),
                            active: false
                        });
                    }
                });
                map.sort(function(a, b) {
                    return a.min - b.min;
                });
                $.each(map, function(index, value) { // set the max of item to min on next item, so you have proper intervals
                    if (index < map.length - 1){
                        map[index].max = map[index + 1].min - 1;
                    }
                });
                this.map = map;
            },
            applyhighlight: function() {
                var that = this;
                var scrollTop = $(window).scrollTop();
                $.each(this.map, function(index, value) {
                    if (value.min < scrollTop && scrollTop < value.max){
                        if (value.active !== true){
                            $('a[href=\'#' + value.id.toString() + '\']').parent('li').addClass('active');
                            value.active = true;
                            if ($.isFunction(that.settings.highlightCallback)) {
                                that.settings.highlightCallback(value.id);
                            }
                        }
                    } else {
                        if (value.active !== false){
                            $('a[href=\'#' + value.id.toString() + '\']').parent('li').removeClass('active');
                            value.active = false;
                            if ($.isFunction(that.settings.unhighlightCallback)) {
                                that.settings.unhighlightCallback(value.id);
                            }
                        }
                    }
                });
            },
            /**
             *  Internally the plugin creates an array of objects which establish relationship between each id in the navigation list and respective scroll positions on
             *  the page, then it adds ‘active’ class to item in the navigation list depending upon the section under viewport. (id’s are also available in the callbacks)
             *  However if an action on the page (such as ajax request)results in changing the length in the section or page, you need to call refresh array of mapping objects by calling the refresh method.
             *
             * @memberof! module:luxscrollposition
             * @alias refresh
             * @type {string}
             * @example <caption>luxscrollposition refresh</caption>
             *   require(['jquery', 'luxscrollposition'], function($) {
             *       $('.lux-docs-sidenav').luxscrollposition('refresh');
             *   });
             */
            refresh: function(){
                this.createMap();
                this.applyhighlight();
            }
        };

        /**
     * Make it a JSDoc comment, if you would like to document plugins as methods on $.fn
     * @function external:"jQuery.fn".luxscrollposition
     * @param {object} options
     * @param  {Integer} options.offsetTop  offsetTop
     * @param  {function} options.highlightCallback  highlightCallback
     * @param  {function} options.unhighlightCallback  unhighlightCallback
     */
        $.fn[pluginName] = function (options) {
            return this.each(function () {
                var data = $.data(this, 'plugin_' + pluginName); //getter
                if (!data) {
                    var newPlugin = new Plugin(this, options);
                    $.data(this, 'plugin_' + pluginName, newPlugin);
                } else {
                    if (options === 'refresh'){
                        $.each(data.map, function(index, value) {
                            $('a[href=\'#' + value.id.toString() + '\']').parent('li').removeClass('active');
                        });
                        data.refresh();
                    }
                }

            });
        };
    });
}); //end of define