/**
 * Copyright (c) 2013 - 2015 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Implements luxsearchresults widget for client-side interaction
 * with REST services providing search results
 * @author Joe McCLintock
 */

/**
* The luxsearch widget
* @module luxsearchresults
* @jqbase
* @requires ojet
* @requires jquery
* @requires knockout
* @requires underscore
* @requires jqueryui
 */
define(['ojs/ojcore', 'knockout', 'jquery', 'underscore',
    'ojs/ojknockout', 'ojs/ojtabs', 'ojs/ojpagingcontrol', 'ojs/ojarraypagingdatasource'],

function(oj, ko, $, _) {
    'use strict';
    /**
     * The Search Results widget listens for this event at the document level in order
     * to render a progress message (presumably when the search request is set to the server)
     *
     * @event module:luxsearchresults#lux_search_start
     * @param {jqEvent} e
     * @param {object} data
     * @param {String} data.term the text being searched
     */
    var SEARCH_START_EVENTID = 'lux_search_start';

    /**
     * The Search Results widget listens for this event at the document level
     * and passes the results to the .data method for rendering
     *
     * @event module:luxsearchresults#lux_search_results
     * @param {jqEvent} e
     * @param {object} data
     * @param {String} data.term the text being searched
     * @param {Array} data.searchResults Array of search results. See the data() method for details on child properties.
     */
    var SEARCH_RESULTS_EVENTID = 'lux_search_results';

    (function() {
        $.widget('GBU.luxsearchresults', {
            version:           '5.1.0',
            defaultElement:    '<div>',
            widgetEventPrefix: 'lux',

            /**
            * Options
            * <dl>
            *     <dt>cssClass</dt>
            *     <dd>
            *         Use this option to use your own css class for the login form<br />
            *             Type: String<br />
            *             Default: 'searchResults'<br />
            *             Example:<br />
            *             <code>cssClass: 'searchResults'</code>
            *     </dd>
            *     <dt>pagingEnabled</dt>
            *     <dd>
            *         Set this option to true for paging results<br />
            *             Type: Boolean<br />
            *             Default: true<br />
            *             Example:<br />
            *             <code>pagingEnabled: true</code>
            *     </dd>
            *     <dt>pagingOnTop</dt>
            *     <dd>
            *         Set this option to true for paging that appear above the results<br />
            *             Type: Boolean<br />
            *             Default: true<br />
            *             Example:<br />
            *             <code>pagingOnTop: true</code>
            *     </dd>
            *     <dt>pageSize</dt>
            *     <dd>
            *         Set this option to set paging size<br />
            *             Type: numeric<br />
            *             Default: 10<br />
            *             Example:<br />
            *             <code>pageSize: 10</code>
            *     </dd>
            *     <dt>highlightMatchingText</dt>
            *     <dd>
            *         Set this option to lighlight the value of the search in the search results<br />
            *             Type: Boolean<br />
            *             Default: true<br />
            *             Example:<br />
            *             <code>highlightMatchingText: true</code>
            *     </dd>
            *     <dt>groupByCategory</dt>
            *     <dd>
            *         Set this option to Tab the resulst by the category column in the search results<br />
            *             Type: Boolean<br />
            *             Default: true<br />
            *             Example:<br />
            *             <code>groupByCategory: true</code>
            *     </dd>
            *     <dt>selectionEnabled</dt>
            *     <dd>
            *         Number of characters to display for summary text.<br />
            *             Type: numeric<br />
            *             Default: 200<br />
            *             Example:<br />
            *             <code>summaryDisplayCharacters: 200</code>
            *     </dd>
            *     <dt>searchStatusFormatter</dt>
            *     <dd>
            *         Function to format the localized search status message<br />
            *             Type: function<br />
            *             Default: function(term) { return 'Searching for "' + term + '"...'; }<br />
            *             Example:<br />
            *             <code>searchStatusFormatter:
            *                 function(term) { return 'Searching for "' + term + '"...'; }
            *             </code>
            *     </dd>
            *     <dt>searchHeadingFormatter</dt>
            *     <dd>
            *         Function to format the localized search results heading<br />
            *             Type: function<br />
            *             Default: function(term) { return 'Search results for "' + term + '"'; }<br />
            *             Example:<br />
            *             <code>searchHeadingFormatter:
            *                 function(term) { return 'Search results for "' + term + '"'; }
            *             </code>
            *     </dd>
            *     <dt>pageSizeOptions</dt>
            *     <dd>
            *         Numeric options to show in 'Items per page' selector<br />
            *             Type: array of numeric<br />
            *             Default: [10, 50, 250, 1000]<br />
            *             Example:<br />
            *             <code>pageSizeOptions: [10, 50, 250, 1000]</code>
            *     </dd>
            * </dl>
            */

            options: {
                cssClass:                 'searchResults',
                pagingEnabled:            true,
                pagingOnTop:              false,
                pagingOverflow:           'fit',
                pageSize:                 10,
                highlightMatchingText:    true,
                groupByCategory:          true,
                selectionEnabled:         true,
                summaryDisplayCharacters: 200,
                searchStatusFormatter:    function(term) {
                    return 'Searching for "' + term + '"...';
                },
                searchHeadingFormatter: function(term) {
                    return 'Search results for "' + term + '"';
                },
                pageSizeOptions: [10, 50, 250, 1000]
            },

            /**
             * @override
             * @private
             */
            _create: function () {
                var self = this;

                self.element.addClass(this.options.cssClass);
                self._headingWrapper = $('<div/>', {'aria-live': 'polite'}).appendTo(self.element);
                self._contentContainer = $('<div/>').appendTo(self.element);

                $(document).on(SEARCH_START_EVENTID, function(e, data) {
                    self._searchTerm = data.term;
                    if (! self._headingElement) {
                        self._headingElement = $('<h1/>, {"id": "resultHeading"}').appendTo(self._headingWrapper);
                    }
                    self._headingElement.text(self.options.searchStatusFormatter(self._searchTerm));
                });

                $(document).on(SEARCH_RESULTS_EVENTID, function(e, data) {
                    self.data(data);
                });
            },

            _refresh: function() {
                var self = this;

                self._contentContainer.empty();
                if (!self._currentData) {
                    return;
                }

                if (self._currentData.length > 0) {
                    if (self.options.groupByCategory) {
                        self._renderResultsByCategory();
                    } else if (self.options.pagingEnabled) {
                        self._renderPagedResults();
                    } else {
                        self._renderResultsPlain();
                    }
                }

                // set the results header now that we've actually rendered the results
                if (! self._headingElement) {
                    self._headingElement = $('<h1/>, {"id": "resultHeading"}').appendTo(self._headingWrapper);
                }
                self._headingElement.text(self.options.searchHeadingFormatter(self._searchTerm));
            },

            _currentData:   null,
            _allData:       null,
            _searchTerm:    '',
            _categories:    [],
            _tabPagingData: [],

            /**
             * Set the current data to display in the search results
             *
             * @param {Object} data data Object with the following properties:
             * @param {String} data.term The term that was searched for
             * @param {Array} data.searchResults Array of search results having title, summary, url, and breadcrumbs properties
             * @param {String} data.searchResults.title Short name
             * @param {String} data.searchResults.summary Summary paragraph
             * @param {String} data.searchResults.url Link target to navigate to this result
             * @param {String} data.searchResults.breadcrumbs Contextual information about this result's category
             */
            data: function(data) {
                var self = this;
                data = data || {term: '', searchResults: []};

                self._allData = data.searchResults || [];
                self._searchTerm = data.term || '';
                self._selectCurrentData();
                self._refresh();
            },

            _selectCurrentData: function() {
                var self = this;

                self._currentData = self._allData;
                if (self.options.groupByCategory) {
                    // chained functions to extract unique rows of data, sorted by default category,
                    // mapped to a collection of categories
                    self._categories = _.chain(self._currentData).pluck('category').uniq().sort().map(
                        function(category, i) {
                            return {id: 'category' + i, name: category};
                        }
                    ).value();
                }
            },

            _renderResultsByCategory: function() {
                var self = this,
                    tabCnt = 0,
                    tabsContainer = $('<div/>', {'id': 'tabs-container'}),
                    // ojTabs expects the container to be a DIV of id "tabs"
                    searchCategories = $('<div/>', {'id': 'tabs'}),
                    uOlist = $('<ul/>');

                searchCategories.append(document.createComment('tab bar'));
                uOlist.appendTo(searchCategories);

                _.each(self._categories, function(category) {
                    var item = $('<li />').append($('<span />', {'text': category.name}));

                    item.appendTo(uOlist);
                });

                searchCategories.appendTo(tabsContainer);
                self._contentContainer = tabsContainer.appendTo(self.element);
                searchCategories.append(document.createComment('tab contents'));

                // iterate over each category creating the html markup that will
                // be translated by ojTabs.
                _.each(self._categories, function(category) {
                    var tabID = 'tabs-' + (tabCnt = tabCnt + 1), // ojTabs expects each tabe to a DIV of id "tabs-" sequence number
                        categoryDiv = $('<div />', {id: tabID}).appendTo(searchCategories),
                        tabContainer = $('<div/>', {'class': 'tabContent', 'role': 'list'}),
                        items = _.filter(self._currentData, function(item) {
                            return item.category === category.name;
                        });

                    if (self.options.pagingEnabled) {
                        self._renderPaging(tabID, categoryDiv, items);
                    } else {
                        categoryDiv.append(tabContainer);
                        _.each(items, function(item) {
                            self._renderResultItem(item, tabContainer);
                        });
                    }
                }, self);

                searchCategories.ojTabs();
            },

            _renderResultsPlain: function() {
                var self = this,
                    // create the div to hold results for this category
                    categoryDiv = $('<div/>', {'class': 'tabContent', 'role': 'list'});

                _.each(self._currentData, function(item) {
                    self._renderResultItem(item, categoryDiv);
                });
                categoryDiv.appendTo(self._contentContainer);
            },

            _renderResultItem: function(item, container) {
                var self = this,
                    term = self._searchTerm,
                    itemDiv = $('<div/>', {'class': 'searchResultItem', 'role': 'listitem'}).appendTo(container);

                $('<a/>', {'class': 'name', href: item.url, html: this._buildHighlightedContent(item.title, term)}).appendTo(itemDiv);
                $('<a/>', {'class': 'itemContainer', href: item.url, html: this._buildHighlightedContent(item.breadcrumbs, term)}).appendTo(itemDiv);
                $('<span/>', {'class': 'itemContent', html: this._buildHighlightedContent(item.summary, term, self.options.summaryDisplayCharacters)}).appendTo(itemDiv);
            },

            // embed span tags inside text for highlighting search terms
            _buildHighlightedContent: function(originalText, term, maxContentLength) {
                if (this.options.highlightMatchingText &&
                        originalText && originalText.length && originalText.length !== 0 &&
                        term && term.length && term.length !== 0) {
                    if (maxContentLength) {
                        originalText = originalText.slice(0, maxContentLength);
                    }
                    var expression = new RegExp('(' + term + ')', 'i');
                    var nodes = [];
                    var match = originalText.match(expression);
                    if (match) {
                        nodes.push(originalText.slice(0, match.index));
                        // hackish way to use jQuery to do the encoding of any embedded html
                        nodes.push($('<span>').append($('<span>', {'class': 'textMatch', text: match[0]})).html());
                        nodes.push(this._buildHighlightedContent(originalText.slice(match.index + match[0].length, originalText.length), term));
                        return nodes.join('');
                    }
                }
                return originalText;
            },

            _renderPagedResults: function() {
                var self = this;

                self._renderPaging(null, self._contentContainer, self._currentData);
            },

            /**
             * Creates paged results in the passed in containerDiv
             * @param {string} tabID - required to create pages with a tabbed container to
             *                       index the page data source per each tab.  Is "null" if
             *                       simple paging.
             * @param {string} containerDiv "div" container of pages.
             * @param {array} items search results
             */
            _renderPaging: function(tabID, containerDiv, items) {
                var self = this,
                    dataIdx = tabID || 'tab-0',
                    pagingContainer = $('<div/>', {'id': 'pages', 'class': 'paging-container'}),
                    // create the div to hold results for this category
                    categoryDiv = $('<div/>', {'class': 'tabContent', 'role': 'list'}),
                    goToPage = function() {
                        // refresh the page with current page from data source object
                        self._refreshTabbedPage(categoryDiv, self._tabPagingData[dataIdx].getWindow());
                    };

                // Create a paging data source object with current items
                self._tabPagingData[dataIdx] = new oj.ArrayPagingDataSource(items);

                pagingContainer.ojPagingControl({
                    data:     self._tabPagingData[dataIdx],
                    pageSize: self.options.pageSize,
                    overflow: self.options.pagingOverflow
                });

                // Using the optionChange to decide that the page has change.
                // This will include change to the page number in the Page input
                // field which is listening to the on click event.
                pagingContainer.on({'ojoptionchange': function(/*event, data*/) {
                    goToPage();
                }});

                this._refreshTabbedPage(categoryDiv, self._tabPagingData[dataIdx].getWindow());

                if (self.options.pagingOnTop) {
                    categoryDiv.appendTo(pagingContainer);
                } else {
                    categoryDiv.appendTo(containerDiv);
                }
                pagingContainer.appendTo(containerDiv);
            },

            _refreshTabbedPage: function(categoryDiv, data) {
                var self = this;

                categoryDiv.empty();

                _.each(data, function(item) {
                    self._renderResultItem(item, categoryDiv);
                });
            }

        });
    }());
});
