/**
 * Copyright (c) 2013 - 2015 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Implements signin control using ojet ojInputText, ojInputPassword and ojButton.

 * The sigin widget provides you options to customize the login form to match
 * the UX guidelines of your organization. The widget comes along with a luxlogininfo
 * module (please make sure you include LUXUtils library in your project) which
 * allows you to centrally organize your login related information
 * and loosely coupling your code from the authentication mechanism.
 * Another way of putting it, it would not affect your code if you choose to
 * migrate away from the widget in future if your code is mainly using luxlogininfo API.
 *
 * You can use the sigin widget to ease the task of authenticating the user by
 * providing the username and password. At it’s heart the login widget uses WebLogic
 * form based authentication. You may choose to override the default authentication
 * mechanism of the widget if you like to keep the styling aspect of the widget
 * but implement your own logic for authentication.

 * @author Joe McClintock
 */

/**
 * The luxsignin widget
 * @module luxsignin
 * @jqbase
 * @requires luxlogininfo
 * @requires ojet
 * @requires knockout
 * @requires jquery
 * @requires jqueryui
 * @requires underscore
 */
define(['ojs/ojcore',
    'knockout',
    'jquery',
    'underscore',
    'ojs/ojbutton',
    'ojs/ojinputtext',
    'luxlogininfo',
    'jqueryui-amd/widget'],
function(oj, ko, $, _, OjButton, OjInputText, luxlogininfo) {

    'use strict';
    (function() {
        $.widget('GBU.luxsignin', {
            version:           '5.1.0',
            defaultElement:    '<div>',
            widgetEventPrefix: 'lux',

            /**
            * Options
            * <dl>
            *     <dt>baseURL</dt>
            *     <dd>
            *        Specify the base URL of your application. This URL will be used to submit the username and
            *        password to server where j_security_check servlet exits. The URL values will be appended by
            *        ‘j_security_check’.  This option must be provided in order to use the default functionality
            *        of the widget unless you decide to provide  overrideDefaultSubmit callback as options parameter.</br>
            *        Type: String</br>
            *        Default: ‘ ‘</br>
            *        Example:</br>
            *        <code>baseURL: 'http://localhost:7001/lux_login'</code>
            *     </dd>
            *     <dt>formClass</dt>
            *     <dd>
            *         Use this option to use your own css class for the login form</br>
            *         Type: String</br>
            *         Default: ‘signInForm’</br>
            *         Example:</br>
            *         <code>formClass: 'signInForm'</code>
            *     </dd>
            *     <dt>signInHeading</dt>
            *     <dd>
            *         Text heading for the sign form</br>
            *         Type: String</br></br>
            *         Default: 'Sign In'</br>
            *         Example:</br>
            *         <code>signInHeading: 'Sign In'</code>
            *     </dd>
            *     <dt>signedOutMsg</dt>
            *     <dd>
            *         Text heading message that appears on signing out</br>
            *         Type: String</br></br>
            *         Default: 'Your are now signed out'</br>
            *         Example:</br>
            *         <code>signedOutMsg: 'Your are now signed out'</code>
            *     </dd>
            *     <dt>containerClass</dt>
            *     <dd>
            *        Use this option to style the sign in form element container</br>
            *        Type:String</br>
            *        Default:  'oj-form oj-panel lux-sign-in-block', // space seperated string</br>
            *        Example:</br>
            *        <code>containerClass: 'oj-form oj-panel lux-sign-in-block' </code> // space seperated string
            *     </dd>
            *     <dt>buttonClass</dt>
            *     <dd>
            *        Use this option to use your own css class for the submit button</br>
            *        Type:String</br>
            *        Default:  'lux-sign-in-button lux-blue-alternate lux-38px-height', // space seperated string</br>
            *        Example:</br>
            *        <code>buttonClass: 'lux-sign-in-button lux-blue-alternate lux-38px-height' </code> // space seperated string
            *     </dd>
            *     <dt>errorMessageClass</dt>
            *     <dd>
            *         Use this option to use your own css class for the error message.</br>
            *         Type:String</br>
            *         Default:"text-error"</br>
            *         Example:</br>
            *         <code> errorMessageClass: "text-error" </code>
            *     </dd>
            *     <dt>invalidLoginError</dt>
            *     <dd>
            *         The provided message will be displayed when user enters incorrect username and password
            *         (For security reasons, it is assumed we don't want to give more granular information as
            *          to whether the Username or Password (or both) was the incorrect item)</br>
            *          Type: String</br>
            *          Default:'The Username or Password is incorrect'</br>
            *          Example:</br>
            *          <code> invalidLoginError: 'The Username or Password is incorrect' </code>
            *     </dd>
            *     <dt>defaultUsername</dt>
            *     <dd>
            *         Developer can provide default username to be displayed in the username field.</br>
            *         Type:String</br>
            *         Default: null</br>
            *         Example:</br>
            *         <code> defaultUsername: 'SystemAdmin'</code>
            *     </dd>
            *     <dt>defaultUsername</dt>
            *     <dd>
            *         Developer can provide default username to be displayed in the username field.</br>
            *         Type:String</br>
            *         Default: null</br>
            *         Example:</br>
            *         <code> defaultUsername: 'SystemAdmin'</code>
            *     </dd>
            *     <dt>hideOnSuccess</dt>
            *     <dd>
            *         When set to true, the widget will hide itself on successful login.</br>
            *         Type: Boolean</br>
            *         Default: false</br>
            *         Example:</br>
            *         <code> hideOnSuccess: false </code>
            *     </dd>
            *     <dt>destroyOnSuccess</dt>
            *     <dd>
            *         When set to true, the widget will destroy itself on successful login. The widget will set username
            *         and loggedIn values in luxlogininfo module for further use before destroying the widget.</br>
            *         Type: Boolean</br>
            *         Default: false</br>
            *         Example:</br>
            *         <code> destroyOnSuccess: false </code>
            *     </dd>
            *     <dt>disableValidation</dt>
            *     <dd>
            *         Set to true to disable validation</br>
            *         Type: Boolean</br>
            *         Default: false</br>
            *         Example:</br>
            *         <code> disableValidation: false </code>
            *     </dd>
            *     <dt>overrideDefaultSubmit</dt>
            *     <dd>
            *        You may choose to override the default authentication mechanism of the widget if you like to keep the
            *        styling aspect of the widget but implement your own logic for authentication. This boolean tells the
            *        signin widget not to execute the server side authentication request through the baseURL.  Instead the basic
            *        username and password validation is done but authentication is left to the submitCallBack option (see
            *        below).</br>
            *
            *         Type: boolean</br>
            *         Default: false</br>
            *         Example: overrideDefaultSubmit: true</br>
            *         <code>
            *              overrideDefaultSubmit: true
            *         </code>
            *     </dd>
            *     <dt>submitCallBack</dt>
            *     <dd>
            *        This required call back function is called either to override the default widget authentication (if overrideDefaultSubmit
            *        is true) or to take the results of the default authentication to do what you need to do in the contect your application.
            *        The callback gets a data object.  The data object has two propertites j_username and j_password, which refers
            *        to the values the users has entered in the username and password fields. Note: overriding default submit behaviour will not
            *        disable input validation, to disable input validation set disableValidation :false in options hash. The callback gets data object,
            *        data object has two propertites j_username and j_password, which refers to the values the users has
            *        entered in the username and password fields.</br>
            *
            *         Type: function</br>
            *         Default: null</br>
            *         Example:</br>
            *         <code>
            *           submitCallBack: function(event, data) {  // over ride default, preventing default submit action
            *               console.log('over ride default, preventing default');
            *           }
            *
            *           submitCallBack: function(data) { // process results of widget authentication
            *               .... application logic .....  NOTE: the event object is not passed with the default authentication.
            *           }
            *         </code>
            *     </dd>
            *     <dt>labelAlignment</dt>
            *     <dd>
            *         Developer specified field label alignment options.</br>
            *         Type: String</br>
            *         Default: null (: 'top')</br>
            *         values: null, 'top', 'right', 'embedded' (as placeholder)</br>
            *         Example:</br>
            *         <code> labelAlignment: 'right'</code>
            *     </dd>
            *     <dt>usernameId</dt>
            *     <dd>
            *         Developer can provide default username text field id.</br>
            *         Type:String</br>
            *         Default: 'username_id'</br>
            *         Example:</br>
            *         <code> usernameId: 'username_id'</code>
            *     </dd>
            *     <dt>usernameLabel</dt>
            *     <dd>
            *         Developer can provide default label or placeholder text for the username input text.</br>
            *         Type:String</br>
            *         Default: 'User Name'</br>
            *         Example:</br>
            *         <code> usernameLabel: 'User Name'</code>
            *     </dd>
            *     <dt>usernameCallback</dt>
            *     <dd>
            *         Developer can provide callack function that executes when a "option" change</br>
            *         event is detected on the username input text.</br>
            *         Type: function</br>
            *         Default: null</br>
            *         Example:</br>
            *         <code> usernameCallback: function() { console.log('usename entered'); }</code>
            *     </dd>
            *     <dt>passwordId</dt>
            *     <dd>
            *         Developer can provide default password field id.</br>
            *         Type:String</br>
            *         Default: 'password_id'</br>
            *         Example:</br>
            *         <code> passwordId: 'password_id'</code>
            *     </dd>
            *     <dt>passwordLabel</dt>
            *     <dd>
            *         Developer can provide default label or placeholder text for the password input text.</br>
            *         Type:String</br>
            *         Default: 'User Password'</br>
            *         Example:</br>
            *         <code> passwordLabel: 'User Password' </code>
            *     </dd>
            *     <dt>passwordCallback</dt>
            *     <dd>
            *         Developer can provide callack function that executes when a "option" change
            *         event is detected on the password input text.</br>
            *         Type: function</br>
            *         Default: null</br>
            *         Example:</br>
            *         <code> passwordCallback: function() { console.log('password entered'); }</code>
            *     </dd>
            *     <dt>errorMessageId</dt>
            *     <dd>
            *         Developer can provide default error message field id.</br>
            *         Type:String</br>
            *         Default: 'error-message-id'</br>
            *         Example:</br>
            *         <code> errorMessageId: 'error-message-id'</code>
            *     </dd>
            *     <dt>errorMessageVisableClass</dt>
            *     <dd>
            *         Developer can provide css class style for displaying error messages.</br>
            *         Type:String</br>
            *         Default: 'text-error-visible'</br>
            *         Example:</br>
            *         <code> errorMessageVisableClass: 'text-error-visible'</code>
            *     </dd>
            *     <dt>serverError</dt>
            *     <dd>
            *         Developer message when server side authentication fails.</br>
            *         Type:String</br>
            *         Default: 'Server Error'</br>
            *         Example:</br>
            *         <code> serverError: 'Server Error'</code>
            *     </dd>
            *     <dt>usernameFocused</dt>
            *     <dd>
            *         When the signin form initially loads, sets the focus on the username field.</br>
            *         Type: boolean</br>
            *         Default: true</br>
            *         Example:</br>
            *         <code> usernameFocused: true</code>
            *     </dd>
            *     <dt>includeDomain</dt>
            *     <dd>
            *         Set to true to include "domain" text input</br>
            *         Type: Boolean</br>
            *         Default: false</br>
            *         Example:</br>
            *         <code> includeDomain: true </code>
            *     </dd>
            *     <dt>idDomianId</dt>
            *     <dd>
            *         Developer can provide default domain field id.</br>
            *         Type:String</br>
            *         Default: 'domain_id'</br>
            *         Example:</br>
            *         <code> domainId: 'domain_id'</code>
            *     </dd>
            *     <dt>idDomainLabel</dt>
            *     <dd>
            *         Developer can provide default label or placeholder text for the domain input text.</br>
            *         Type:String</br>
            *         Default: 'Domain'</br>
            *         Example:</br>
            *         <code> idDomainLabel: 'Domain' </code>
            *     </dd>
            *     <dt>idDomainCallback</dt>
            *     <dd>
            *         Developer can provide callack function that executes when a "option" change
            *         event is detected on the domain input text.</br>
            *         Type: function</br>
            *         Default: null</br>
            *         Example:</br>
            *         <code> domainCallback: function() { console.log('domain entered'); }</code>
            *     </dd>
            *     <dt>signInButtonId</dt>
            *     <dd>
            *         Developer can provide default id for the signin submit button.</br>
            *         Type:String</br>
            *         Default: 'signin-button'</br>
            *         Example:</br>
            *         <code> signInButtonId: 'signin-button' </code>
            *     </dd>
            *     <dt>signInButtonLabel</dt>
            *     <dd>
            *         Developer can provide default label for the signin submit button.</br>
            *         Type:String</br>
            *         Default: 'Sign-In'</br>
            *         Example:</br>
            *         <code> signInButtonLabel: 'Sign-In' </code>
            *     </dd>
            *     <dt>accessAccountLink</dt>
            *     <dd>
            *         Developer can provide default link text for access account help.</br>
            *         Type:String</br>
            *         Default: false</br>
            *         Example:</br>
            *         <code> accessAccountLink: 'Can\'t access your account?' </code>
            *     </dd>
            *     <dt>accessAccountHelp</dt>
            *     <dd>
            *         Developer can provide callback function that executes when the "access account link" is clicked.</br>
            *         Type: function</br>
            *         Default: null</br>
            *         Example:</br>
            *         <code> accessAccountHelp: function() { console.log('user clicked help'); }</code>
            *     </dd>
            *     <dt>cancelLink</dt>
            *     <dd>
            *         Developer text for the cancel button at the bottom of the sign in form.</br>
            *         Type: string</br>
            *         Default: 'Cancel'</br>
            *         Example:</br>
            *         <code> cancelLink: 'Cancel'</code>
            *     </dd>
            *     <dt>cancelButton</dt>
            *     <dd>
            *         Developer can provide default cancel button at the bottom of the sign in form.</br>
            *         Type:boolean</br>
            *         Default: false</br>
            *         Example:</br>
            *         <code> cancelButton: true</code>
            *     </dd>
            *     <dt>cancelCallback</dt>
            *     <dd>
            *         Developer can provide callack function that executes when the cancel button is clicked.</br>
            *         Type: function</br>
            *         Default: null</br>
            *         Example:</br>
            *         <code> cancelCallback: function() { console.log('user clicked cancel'); }</code>
            *     </dd>
            *     <dt>capslockDetection</dt>
            *     <dd>
            *         Developer specify to turn on caps lock detetion. Internet Explorer already has this "feature".</br>
            *         Type: function</br>
            *         Default: null</br>
            *         Example:</br>
            *         <code> capslockDetection: true</code>
            *     </dd>
            *     <dt>capsLockWarning</dt>
            *     <dd>
            *         Developer specify caps lock warning message.</br>
            *         Type: function</br>
            *         Default: 'CAPS LOCK IS ON'</br>
            *         Example:</br>
            *         <code> capsLockWarning: 'CAPS LOCK IS ON'</code>
            *     </dd>
            * </dl>
            */
            options: {
                baseURL:                  '',
                formClass:                'signInForm',
                signInHeading:            'Sign In',
                signedOutMsg:             'Your are now signed out',
                errorMessageId:           'error-message-id',
                errorMessageClass:        'text-error',
                errorMessageVisableClass: 'text-error-visible',
                invalidLoginError:        'The Username or Password is incorrect',
                serverError:              'Server Error',
                capsLockWarning:          'CAPS LOCK IS ON',
                capslockDetection:        null,
                defaultUsername:          null,
                usernameFocused:          true,
                hideOnSuccess:            false,
                destroyOnSuccess:         false,
                disableValidation:        false,
                overrideDefaultSubmit:    false,
                submitCallBack:           null,
                usernameId:               'username_id',
                userNameLabel:            'User Name',
                usernameCallback:         null,
                passwordId:               'password_id',
                passwordLabel:            'Password',
                passwordCallback:         null,
                signInButtonId:           'signin-button',
                signInButtonLabel:        'Sign-In',
                buttonClass:              'lux-sign-in-button lux-blue-alternate lux-38px-height',
                containerClass:           'oj-form oj-panel lux-sign-in-block',
                includeDomain:            false,
                idDomianId:               'domain_id',
                idDomainLabel:            'Domain',
                idDomainCallback:         null,
                accessAccountHelp:        null,
                accessAccountLink:        'Can\'t access your account?',
                cancelLink:               'Cancel',
                cancelButton:             false,
                cancelCallback:           null,
                labelAlignment:           'top', // top, right, embedded,
                cssRow:                   'oj-flex',
                cssCol:                   'oj-flex-item'
            },
            _customMessage:  [],
            _observablesMap: {},

            _create: function() {
                var self = this;

                self._setup();
                self._refresh();
            },

            _setup: function() {
                var self = this;

                self._customMessage = [];
            },

            _refresh: function() {
                var self = this,
                    accessAccountHelpContainer,
                    accessAccountHelp,
                    clearLink,
                    holdEl = [],
                    cssRow = this.options.cssRow,
                    cssCol = this.options.cssCol,
                    applyLabel = function(label, id) {
                        if (self.options.labelAlignment === 'right') {
                            holdEl[0] = $('<div/>').addClass(cssRow).appendTo(self._form);
                            holdEl[1] = $('<div/>').addClass(cssCol).appendTo(holdEl[0]);
                        }
                        if (!self.options.labelAlignment || (self.options.labelAlignment !== 'embedded')) {
                            // create the form input element and save off element container
                            // for later localization changes.
                            self._observablesMap[label] = $('<label/>',
                                {'for': self.options[id], text: self.options[label]})
                                .addClass('oj-component-initnode')
                                .appendTo(holdEl[1] || self._form);
                        }
                        if (self.options.labelAlignment === 'right') {
                            holdEl[1] = $('<div/>').addClass(cssCol).appendTo(holdEl[0]);
                        }
                    };

                // create the form container
                self._form = $('<form/>', {action:  'j_security_check',
                    method:  'POST',
                    'class': self.options.formClass
                })
                    .addClass(self.options.containerClass)
                    .attr('role', 'group')
                    .attr('aria-labelledby', 'sign-in-heading-top')
                    .appendTo(self.element);

                // create h2 heading
                if (this.options.labelAlignment === 'right') {
                    holdEl[0] = $('<div/>').addClass(cssRow).appendTo(self._form);
                    holdEl[1] = $('<div/>').addClass(cssCol + ' lux-oj-col-empty').appendTo(holdEl[0]);
                    holdEl[2] = $('<div/>').addClass(cssCol).appendTo(holdEl[0]);
                }

                // create heading elements and save off a reference to each
                // container for later localization changes.
                self._observablesMap.signedOutMsg = $('<h1/>',
                    {text: self.options.signedOutMsg})
                    .addClass('oj-header-border lux-signed-out-message')
                    .appendTo(holdEl[2] || self._form);

                if (self.options.signedOut) {
                    self._observablesMap.signedOutMsg.show();
                }

                self._observablesMap.signInHeading = $('<h2/>',
                    {text: self.options.signInHeading})
                    .addClass('oj-header')
                    .appendTo(holdEl[2] || self._form);

                // create hidden error message
                if (this.options.labelAlignment === 'right') {
                    holdEl[0] = $('<div/>').addClass(cssRow).appendTo(self._form);
                    holdEl[1] = $('<div/>').addClass(cssCol + ' lux-oj-col-empty').appendTo(holdEl[0]);
                    holdEl[2] = $('<div/>').addClass(cssCol).appendTo(holdEl[0]);
                }
                self._errorMessage = $('<p/>', {text: self.options.invalidLoginError})
                    .addClass(self.options.errorMessageClass)
                    .attr('id', self.options.errorMessageId)
                    .attr('role', 'alert')
                    .appendTo(holdEl[2] || self._form);

                self._observablesMap.invalidLoginError = self._errorMessage;

                //*************************************************************//
                // User name elements
                // ************************************************************//
                applyLabel('userNameLabel', 'usernameId');

                self._usernameElement = $('<input/>', {type: 'text', name: 'j_username'});
                self._usernameElement.attr('id', self.options.usernameId);
                self._usernameElement.attr('aria-label', self.options.userNameLabel);
                self._usernameElement.appendTo(holdEl[1] || self._form);
                self._usernameElement.ojInputText({
                    required: true,
                    type:     'text'
                });

                if (this.options.defaultUsername) {
                    self._usernameElement.ojInputText('value', this.options.defaultUsername);
                }

                if (this.options.labelAlignment && this.options.labelAlignment === 'embedded') {
                    self._usernameElement.ojInputText('option', 'placeholder', self.options.userNameLabel);
                    self._observablesMap.userNameLabel = self._usernameElement;
                }

                self._usernameElement.on({'ojoptionchange': function(event, data) {
                    if (self.options.usernameCallback && _.isFunction(self.options.usernameCallback)) {
                        self.options.usernameCallback(event, data);
                    }
                }
                });

                // allow for turning on or off the username focus, useful with more than one login form on a page
                // where you only want the first form to have the username focus.
                if (_.isUndefined(self.options.usernameFocused) || self.options.usernameFocused) {
                    self._usernameElement.focus();
                }

                // keyup makes sure the character just entered is validated.
                self._usernameElement.on('keyup', function(event) {
                    // vaidate now so pervious error messages are removed
                    if (self._isNotControlKeys(event)) {
                        self._usernameElement.ojInputText('validate');
                    }
                });

                // on leaving the username make sure it's valid.
                self._usernameElement.on('blur', function(/*event*/) {
                    self._usernameElement.ojInputText('validate');
                });

                //*************************************************************//
                // Password elements
                // ************************************************************//
                applyLabel('passwordLabel', 'passwordId');

                self._observablesMap.capsLockWarning = self.options.capsLockWarning;
                self._customMessage.push({summary: self.options.capsLockWarning, detail: ' ', severity: oj.Message.SEVERITY_TYPE.WARNING});

                self._passwordElement = $('<input/>', {type: 'text', name: 'j_password'});
                self._passwordElement.attr('id', self.options.passwordId);
                self._passwordElement.attr('aria-label', self.options.passwordLabel);
                self._passwordElement.appendTo(holdEl[1] || self._form);
                self._passwordElement.ojInputPassword({
                    required: true,
                    type:     'text'
                });

                if (self.options.labelAlignment && self.options.labelAlignment === 'embedded') {
                    self._passwordElement.ojInputPassword('option', 'placeholder', self.options.passwordLabel);
                    self._observablesMap.passwordLabel = self._passwordElement;
                }

                self._passwordElement.on({'ojoptionchange': function(event, data) {
                    if (self.options.passwordCallback && _.isFunction(self.options.passwordCallback)) {
                        self.options.passwordCallback(event, data);
                    }
                }
                });

                /*******************************************************************************************
                * The 2 keyboard events:
                *  keypress: Fired when the key which is being pressed results in a character being sent.
                *            This excludes modifier keys, directional keys, and keys which remove text.
                *
                *  keyup: Fired after the user has released the key, telling you what key was pressed but
                *         because its being fired after the fact you are unable to prevent this event.
                ********************************************************************************************/

                // on keyup the charater input is known so if the password is not currently valid validate it again.
                self._passwordElement.on('keyup', function(event) {
                    // if not tab vaidate now so pervious error messages are removed
                    if (self._isNotControlKeys(event) && !self._passwordElement.ojInputPassword('isValid')) {
                        self._passwordElement.ojInputPassword('validate');
                    }
                });

                if (self.options.capslockDetection) {
                    // on keypress the charater input is not known so we only check for caps lock.
                    self._passwordElement.on('keypress', function(event) {
                        self._passwordElement.ojInputPassword('option', 'messagesCustom', []);
                        // If the password input is valid (as checked on the keyup event), check for
                        // capslock.  NOTE: that keypress event gives us the charater case while keyup does
                        // not.
                        if (self._passwordElement.ojInputPassword('isValid') && self._capsLockOn(event)) {
                            self._passwordElement.ojInputPassword('option', 'messagesCustom', self._customMessage);
                        }
                    });
                }
                // on leaving password make sure it's valid
                self._passwordElement.on('blur', function(/*event*/) {
                    self._passwordElement.ojInputPassword('validate');
                });

                //*************************************************************//
                // User domain elements
                // ************************************************************//
                if (self.options.includeDomain) {

                    applyLabel('idDomainLabel', 'idDomianId');

                    self._domainElement = $('<input/>, {type: "text", name: "j_domain"}');
                    self._domainElement.attr('id', self.options.idDomianId);
                    self._domainElement.attr('aria-label', self.options.idDomainLabel);
                    self._domainElement.appendTo(holdEl[1] || self._form);
                    self._domainElement.ojInputText({
                        required: true,
                        type:     'text'
                    });

                    if (this.options.labelAlignment && this.options.labelAlignment === 'embedded') {
                        self._domainElement.ojInputText('option', 'placeholder', self.options.idDomainLabel);
                        self._observablesMap.idDomainLabel = self._domainElement;
                    }

                    self._domainElement.on({'ojoptionchange': function(event, data) {
                        if (self.options.idDomainCallback && _.isFunction(self.options.idDomainCallback)) {
                            self.options.idDomainCallback(event, data);
                        }
                    }
                    });
                }

                //*************************************************************//
                // Access help elements
                // ************************************************************//

                self._accessAccountHelp = $('<div/>').addClass('oj-flex lux-access-help');
                self._accessAccountHelp.appendTo(self._form);
                $('<div/>').addClass('oj-flex-item lux-oj-col-empty').appendTo(self._accessAccountHelp);
                accessAccountHelpContainer = $('<div/>').addClass('oj-flex-item').appendTo(self._accessAccountHelp);
                accessAccountHelp = $('<a/>', {href: '#'}).text(self.options.accessAccountLink).appendTo(accessAccountHelpContainer);

                // save off the help link text container for later localization changes.
                self._observablesMap.accessAccountLink = accessAccountHelp;

                accessAccountHelp.on('click', function(event) {
                    if (_.isFunction(self.options.accessAccountHelp)) {
                        self.options.accessAccountHelp(event);
                    }
                });

                //*************************************************************//
                // Button elements
                // ************************************************************//

                if (this.options.labelAlignment === 'right') {
                    holdEl[0] = $('<div/>').addClass(cssRow).appendTo(self._form);
                    holdEl[1] = $('<div/>').addClass(cssCol + ' lux-oj-col-empty').appendTo(holdEl[0]);
                    holdEl[2] = $('<div/>').addClass(cssCol).appendTo(holdEl[0]);
                }

                self._buttonElement = $('<button/>')
                    .addClass(self.options.buttonClass);
                self._buttonElement.attr('id', self.options.signInButtonId);
                self._buttonElement.appendTo(holdEl[2] || self._form);
                self._buttonElement.ojButton({
                    label: self.options.signInButtonLabel
                });

                // save off the button text span container for later localization changes.
                self._observablesMap.signInButtonLabel = self._buttonElement.find('span.oj-button-text');

                self._buttonElement.on('click', function(event) {
                    self._observablesMap.signedOutMsg.hide();
                    self._subBtnClick(event);
                });

                //*************************************************************//
                // Is the a clear link
                //*************************************************************//
                if (self.options.cancelButton &&
                   (self.options.cancelCallback && _.isFunction(self.options.cancelCallback))) {
                    //*************************************************************//
                    // clear form
                    // ************************************************************//
                    clearLink = $('<a/>', {href: '#'}).text(self.options.cancelLink).appendTo(holdEl[2] || self._form);
                    clearLink.addClass('lux-signin-cancel-link');
                    self._observablesMap.cancelLink = clearLink;
                    clearLink.on('click', function(event) {
                        var data = {};

                        data.value = '';
                        // empty username value
                        self._usernameElement.ojInputText('reset');
                        self._usernameElement.val(data.value);
                        // set focus to username
                        self._usernameElement.focus();
                        // empty password value
                        self._passwordElement.ojInputPassword('reset');
                        self._passwordElement.val(data.value);
                        // clear messages from password
                        if (self.options.includeDomain) {
                            self._domainElement.ojInputText('reset');
                            self._domainElement.val(data.value);
                        }
                        // remove error display
                        self._errorMessage.removeClass(self.options.errorMessageVisableClass);
                        self.options.cancelCallback(event, data);
                    });
                }
            },

            _enterKeyHandling: function(event) {
                var focusedEle;

                if (event.which === 13) {
                    event.preventDefault();
                    focusedEle = $(':focus');
                    if (focusedEle.attr('name') === this._usernameElement.attr('name')) {
                        this._passwordElement.focus();
                    } else { //this means either password field or sub btn is focused
                        this._subBtnClick(event);
                    }
                }
            },

            _subBtnClick: function(event) {
                var self = this,
                    success = true,
                    data = {j_username: this._usernameElement.val(), j_password: this._passwordElement.val()};

                event.preventDefault();

                if (self.options.includeDomain) {
                    data.j_domain = self._domainElement.val();
                }

                self._errorMessage.text(self.options.invalidLoginError);

                if (self._usernameElement.val() === '') {
                    self._usernameElement.ojInputText('validate');
                    self._usernameElement.focus();
                    return;
                } else if (self._passwordElement.val() === ''){
                    self._passwordElement.ojInputPassword('validate');
                    self._passwordElement.focus();
                    return;
                } else if (self.options.includeDomain && self._domainElement.val() === '') {
                    self._domainElement.ojInputText('validate');
                    self._domainElement.focus();
                    return;
                }

                if ((this.options.disableValidation) || this._isValid()) {
                    self._toggleErrorMsg();
                    if (this.options.overrideDefaultSubmit && _.isFunction(this.options.submitCallBack)) {
                        success = this.options.submitCallBack(event, data);
                    } else {
                        this._submitForm();
                    }
                }

                if (!success) {
                    self._errorMessageDisplay();
                    return;
                }

            },

            _toggleErrorMsg: function() {
                var self = this;

                if (self._errorMessage.hasClass(self.options.errorMessageVisableClass)) {
                    // move focus to User Name field, and select any value in the field
                    self._usernameElement.focus().select();
                    // Add aria-describedby attribute to User Name and Password fields, pointing to error message
                    self._usernameElement.attr('aria-describedby', 'sign-in-error-top');
                    self._passwordElement.attr('aria-describedby', 'sign-in-error-top');
                } else {
                    // Remove aria-describedby attribute from User Name and Password fields
                    self._usernameElement.removeAttr('aria-describedby');
                    self._passwordElement.removeAttr('aria-describedby');
                }
            },

            _submitForm: function() {
                var self = this,
                    data = {j_username: self._usernameElement.val(), j_password: self._passwordElement.val()};

                if (self.options.includeDomain) {
                    data.j_domain = self._domainElement.val();
                }

                $.ajax({
                    url:     self.options.baseURL + '/j_security_check',
                    type:    'POST',
                    timeout: 15000,
                    data:    data
                }).done(function(jqXHR) {
                    luxlogininfo.setUsername(self._usernameElement.val());
                    luxlogininfo.setLoginStatus(true);
                    /**
                     * This event is triggered on the widget element when the server acknowledges that the username and password
                     * provided by the user are correct. jQuery event object and data object are passed as parameters to the
                     * event handler. The data object has the two properties ‘loggedIn’ and ‘username’. For success event the
                     * loggedIn property is always true. The username is what user enters in the username field. The widget will
                     * also set these values of loggedIn and username in the luxlogininfo module.
                     */
                    if (self.options.submitCallBack && _.isFunction(self.options.submitCallBack)) {
                        self.options.submitCallBack({loggedIn: true, username: self._usernameElement.val(), luxlogininfo: luxlogininfo});
                    }
                    self._usernameElement.val('');
                    self._passwordElement.val('');
                    if (self.options.hideOnSuccess === true) {
                        self.element.hide();
                    }
                    if (self.options.destroyOnSuccess === true) {
                        self._destroy();
                    }
                }).fail(function(jqXHR, textStatus, errorThrown) {
                    if (jqXHR.status === 403) {
                        self._errorMessage.show();
                        self._passwordElement.val('');
                        luxlogininfo.setLoginStatus(false);
                        luxlogininfo.setUsername('Anonymous');
                        /**
                         * This event is triggered on the widget element when the provided username or password are not accepted
                         * by server as valid username and password. jQuery event object and data object are passed as parameters
                         * to the event handler. The data object has the two properties ‘loggedIn’ and ‘username’. For fail
                         * event the loggedIn property is always false. The username is always ‘Anonymous’. The widget will also
                         * set these values of loggedIn and username in the luxlogininfo module.
                         */
                        if (self.options.submitCallBack && _.isFunction(self.options.submitCallBack)) {
                            self.options.submitCallBack({loggedIn: false, username: 'Anonymous'});
                            self._errorMessageDisplay();
                        }
                    } else {
                        /**
                         * This event is triggered on the widget element when the widget can not connect to server to varify the
                         * validity of username and password. jQuery event object and standard object passed to jqXHR.fial are passed as parameters
                         * to the event handler.
                         */
                        if (self.options.submitCallBack && _.isFunction(self.options.submitCallBack)) {
                            self.options.submitCallBack({jqXHR: jqXHR, textStatus: textStatus, errorThrown: errorThrown});
                            self._errorMessage.text(self.options.serverError);
                            self._errorMessageDisplay();
                        }
                    }
                });
            },

            _errorMessageDisplay: function() {
                var self = this;

                // Display invalid login message
                self._errorMessage.addClass(self.options.errorMessageVisableClass);
                // move focus to User Name field, and select any value in the field
                self._usernameElement.focus().select();
                // clear Password field
                self._passwordElement.val('');
                // Add aria-describedby attribute to User Name and Password fields, pointing to error message
                self._usernameElement.attr('aria-describedby', 'sign-in-error-top');
                self._passwordElement.attr('aria-describedby', 'sign-in-error-top');
                // display CAPS LOCK warning message
                //self._passwordElement.ojInputPassword('option', 'messagesCustom', self._customMessage);
                return;
            },

            _destroy: function() {
                this._form.remove();
            },

            _isValid: function(){
                var self = this;

                if ((self._usernameElement.val() === '') || (self._passwordElement.val() === '')) {
                    self._errorMessage.addClass(self.options.errorMessageVisableClass);
                    self._errorMessage.show();
                    return false; // not valid
                } else {
                    if (self.options.includeDomain && (self._domainElement.val() === '')) {
                        self._errorMessage.addClass(this.options.errorMessageVisableClass);
                        self._errorMessage.show();
                        return false;
                    } else {
                        self._errorMessage.removeClass(self.options.errorMessageVisableClass);
                        return true; // valid
                    }
                }
            },

            _isNotControlKeys: function(e) {
                var regex = new RegExp('^[a-zA-Z0-9]+$'),
                    str = String.fromCharCode(!e.charCode ? e.which : e.charCode);

                if (regex.test(str)) {
                    return true;
                }

                e.preventDefault();
                return false;
            },

            _capsLockOn: function(e) {
                var charCode = false,
                    shifton = false;

                e = (e) ? e : window.event;

                charCode = e.which ? e.which : (e.keyCode ? e.keyCode : false);

                shifton = e.shiftKey ? e.shiftKey : ((charCode === 16));

                if (charCode >= 97 && charCode <= 122 && shifton) {
                    return true;
                }

                if (charCode >= 65 && charCode <= 90 && !shifton) {
                    return true;
                }

                return false;
            },

            _changeLabels: function(key, value) {
                var self = this,
                    inputField;

                if (self._observablesMap[key] && !_.contains(['capsLockWarning'], key)) {
                    self._observablesMap[key].text(value);
                    self.options[key] = value;
                }

                // check for text input
                if (_.contains(['userNameLabel', 'passwordLabel', 'idDomainLabel'], key)) {

                    // and for embedded labels in text input
                    if (self.options.labelAlignment &&
                        self.options.labelAlignment === 'embedded') {

                        self._observablesMap[key].attr('placeholder', value);
                    }
                    // update the aria-label attribute on the text input
                    inputField = $(self._observablesMap[key]).attr('for');
                    inputField = $('#' + inputField);
                    inputField.attr('aria-label', value);
                }

                if (_.contains(['capsLockWarning'], key)) {
                    self._observablesMap.capsLockWarning = value;
                    self.options.capsLockWarning = value;
                    self._customMessage = [];
                    self._customMessage.push({summary: self.options.capsLockWarning, detail: ' ', severity: oj.Message.SEVERITY_TYPE.WARNING});
                }
            },

            /**
             * _setOption: evaluates the options and calls the
             *             approprate function to handle change
             *             event.
             * @param {String} key   key of the changed option
             * @param {String} value new value of the option
             */
            _setOption: function(key, value) {
                var self = this;

                self._changeLabels(key, value);
            },

            /**
             * _setOptions: iterates over all changed options and passes
             *              key values to setOption for evaluation.
             *
             * @param {Object} options array of options
             */
            _setOptions: function(options) {
                var self = this;

                $.each(options, function(key, value) {
                    self._setOption(key, value);
                });

            }

        });

    }());
});