/**
 * @fileOverview
 * <p>The module acts as a wrapper for the <a href="http://mathjs.org/" target="_blank">mathjs</a> library. Mathjs contains
 * all the math functions associated with the base JavaSctipt math object and more to operate in numbers with greater than 15 total digits,
 * the limit imposed by JavaScript.</p>
 *
 * <p>&quot;Math.js is an extensive math library for JavaScript and Node.js. It features a flexible expression parser with support for symbolic
 * computation, comes with a large set of built-in functions and constants, and offers an integrated solution to work with different
 * data types like numbers, big numbers, complex numbers, fractions, units, and matrices. Powerful and easy to use.&quot;</p>
 */
/**
 * @module lux-bignumber
 * @requires knockout
 * @requires jquery
 * @requires lodash
 * @requires mathjs
 * @requires ojcore
*/
define(['ojs/ojcore',
    'knockout',
    'jquery',
    'lodash',
    'mathjs'
], function(oj, ko, $, _, math) {
    'use strict';

    var obj = /** @lends module:lux-bignumber */ {
    /**
    *    Options
    *        <dl>
    *            <dt>Configuration</dt>
    *            <dd>
    *                <p>Math.js contains a number of configuration options. There are two ways to configure math.js:</p>
    *                <ul>
    *                    <li>Configure an existing instance of math.js using <code class="highlighter-rouge">math.config(options)</code>,
    *                    for example <code class="highlighter-rouge">math.config({number: 'BigNumber'})</code> to change to BigNumbers.</li>
    *                    <li>Create and configure a new instance of math.js using <code class="highlighter-rouge">math.create([options])</code>,
    *                    for example <code class="highlighter-rouge">var bigmath = math.create({number: 'BigNumber'})</code> to create a new
    *                    instance configured to use BigNumbers.</li>
    *                </ul>
    *            </dd>
    *            <dt>epsilon</dt>
    *            <dd>
    *                The minimum relative difference used to test equality between two compared values. This value is used by all relational functions.<br/>
    *                Default: 1e-14.
    *            </dd>
    *            <dt>matrix</dt>
    *            <dd>
    *                The default type of matrix output for functions. Available values are: 'Matrix' (default) or 'Array'. Where possible,
    *                the type of matrix output from functions is determined from the function input: An array as input will return an Array,
    *                a Matrix as input will return a Matrix. In case of no matrix as input, the type of output is determined by the option matrix.
    *                In case of mixed matrix inputs, a matrix will be returned always.
    *            </dd>
    *            <dt>number</dt>
    *            <dd>
    *                The default type of numbers. This setting is used by functions
    *                         like <code class="highlighter-rouge">eval</code> which cannot determine the correct type of output from the
    *                         functions input. For most functions though, the type of output is determined
    * from the the input: a number as input will return a number as output,
    * a BigNumber as input returns a BigNumber as output.
    * Available values are: <code class="highlighter-rouge">'number'</code> (default), <code class="highlighter-rouge">'BigNumber'</code>, or <code class="highlighter-rouge">'Fraction'</code>.
    * <a href="http://mathjs.org/examples/bignumbers.js.html" target="_blank">BigNumbers</a> have higher precision than the default
    * numbers of JavaScript, and <a href="http://mathjs.org/examples/fractions.js.html" target="_blank"><code class="highlighter-rouge">Fractions</code></a> store
    * values in terms of a numerator and denominator.
    *            </dd>
    *            <dt>precision</dt>
    *            <dd>
    *                The maximum number of significant digits for bigNumbers. This setting only applies to BigNumbers, not to numbers. Default value is 64.
    *            </dd>
    *            <dt>predictable</dt>
    *            <dd>
    *                Predictable output type of functions. When true, output type depends only on the input types. When false (default),
    *                output type can vary depending on input values. For example math.sqrt(-4) returns complex('2i')
    *                when predictable is false, and returns NaN when true. Predictable output can be needed when programmatically
    *                handling the result of a calculation, but can be inconvenient for users when evaluating dynamic equations.
    *            </dd>
    *            <dt>randomSeed</dt>
    *            <dd>
    *                Set this option to seed pseudo random number generation, making it deterministic. The pseudo random number
    *                generator is reset with the seed provided each time this option is set. For example, setting it to 'a' will
    *                cause math.random() to return 0.43449421599986604 upon the first call after setting the option every time.
    *                Set to null to seed the pseudo random number generator with a random seed. Default value is null.
    *            </dd>
    *        </dl>
    *
    */
        options: {}
    };

    var lux = {}; // create LUX name space

    /**
     * @constructor
     * @alias module:luxBignumber
     * @method module:luxBignumber#BigNumber
     * @description Constructor, setups up the parameters for using mathjs
     * @param {object} options - name value pairs used to configure mathjs
     */
    lux.BigNumber = function (options) {
        this.options = options || {number: 'BigNumber', precision: 64};
        this.options.number = options && options.number || 'BigNumber';
        this.options.precision = options && options.precision || 64;
        this.type = {type: 'number'};
        // extend the math object with "this"
        _.assign(this, math);
        // configure mathjs with options
        this.config(this.options);
        this.Init();
    };

    /**
     *  @description Initialize mathjs with disabled javascript functions that could cause security issues.
     *  @returns {undefined}
     */
    lux.BigNumber.prototype.Init = function() {
        // disable use of these functions for security
        this.import({
            'createUnit': function () {
                throw new Error('Function createUnit is disabled');
            },
            'eval': function () {
                throw new Error('Function eval is disabled');
            },
            'parse': function () {
                throw new Error('Function parse is disabled');
            },
            'simplify': function () {
                throw new Error('Function simplify is disabled');
            },
            'derivative': function () {
                throw new Error('Function derivative is disabled');
            }
        }, {override: true});
    };

    /**
     *  @method module:luxBignumber#getOptions
     *  @description Get options used to create the mathjs object.
     *  @returns {object} options - mathjs options
     */
    lux.BigNumber.prototype.getOptions = function() {
        return this.options;
    };

    /**
     *  @description Method controls how mathjs object is created.
     *  @method module:luxBignumber#BigNumberFactory
     *  @returns {object} mathjs - mathjs object
     *  @example
     *  luxBignum.BigNumberFactory().createBigNumber({number: luxBignum.BigNumberFactory.BIGNUMBER, precision: 64, epsilon: 1e-25});
     */
    lux.BigNumberFactory = function() {

        function _createBigNumber(options) {
            var bignum = new lux.BigNumber(options);

            return bignum;
        }

        return {
            /**
             * @description Instantiates the mathjs wrapper object
             * @param {object} options name value pairs describing how to construct the mathjs object
             * @returns {object} mathjs - mathjs object
             */
            createBigNumber: function(options) {
                return _createBigNumber(options);
            }
        };
    };

    lux.BigNumberFactory.BIGNUMBER = 'BigNumber';

    return lux;
});