/**
 * Copyright (c) 2013 - 2015 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * The purpose of this utility is to enable Knockout.js declarative bindings for lux patterns (or any other jQueryUI widget factory widget).
 * This utility offers two-way binding for KO observables with jQueryUI widget factory widgets.  The LUX BindingBridge binds application data
 * in the ViewModel  (Knockout observable or otherwise) to widget factory options.  If the bound data in the ViewModel is observable, then the
 * bridge fires widget factory’s ‘option’ method whenever the value of the observable is modified.  (If the bound data in ViewModel is not
 * observable then the BindingBridge does not notify the jQuery widget about the state change in the application ViewModel).
 * Usage: This utility is a require.js module, so the application should just include this module in the require (or define) call.
 * This utility then automatically notifies the widget (widget factory) if any observable in the ViewModel is changed. This utility
 * also works in the other direction  by modifying  the ViewModel observable if the user action leads to change of state of the widget.
 * Hence the two-way binding.
 * Note for jQueryUI widget developers:
 * When you are developing widgets using jQueryUI widget factory, a user action that leads to change of widget state (i.e. widget option) should be notified to the LUX BindingBridge by using ‘luxoptionchange’ event.  LUX BindingBridge observes this event and updates any Knockout observables in the ViewModel, effectively communicating the state of the widget back to the application. For API consistency with LUX and OJET, the data passed to the ‘luxoptionchange’ event should have ‘option’ and ‘value’ properties, where ‘option’ is the name of the option that has changed in widget factory and ‘value’ is the new value of the option that has changed.
 * luxoptionchange" should have following signature for the  data passed.
 * data = {  option: ,       previousValue: ,       value:     ,      optionMetadata:      }

 * @author Shailesh Zirpe
 */

require([
    'knockout',
    'underscore',
    'jquery',
    'jqueryui-amd/widget'
],
function(ko, _, $) {
    ko.bindingHandlers.luxComponent = {
        init: function(element, valueAccessor, allBindings, viewModel, bindingContext) {
            var valueUnwrapped = ko.unwrap(valueAccessor());
            var optionsObj = {}; _.map(valueUnwrapped, function(val, key) {
                optionsObj[key] = ko.unwrap(val);
            });
            var WidgetName = valueUnwrapped.component;
            var luxComponentLoaded = valueUnwrapped.luxComponentLoaded;
            var writeBackMode = false; // writeBackMode is used to avoide  'option' method firing, during writing back to VM
            var subStore = []; // Subscription store. (upon destory the option change should not fire)

            /*
                 * initial invocation of the widget factory
                 * $(element).luxBreadcrumbs(optionsObj);
                 */
            $(element)[WidgetName](optionsObj);
            // run the after load callback
            if (typeof luxComponentLoaded === 'function') {
                luxComponentLoaded(element, WidgetName, optionsObj, allBindings, bindingContext);
            }

            /*
                 * app -->> component
                 * Following code is responsible for calling widget-factory's 'option' method, whenever the value of observable is changed in VM
                 * console.log("app -->> bridge -->> component");
                 * $(element).luxBreadcrumbs( "option", { disabled: true } );
                 */
            _.each(valueAccessor(), function(value, key, list) {
                if (ko.isObservable(value)) {
                    var sub = value.subscribe(function(newVal) {
                        if (!writeBackMode) {
                            var newOption = {}; newOption[key] = newVal;
                            $(element)[WidgetName]('option', newOption);
                        }
                        writeBackMode = false;
                    });
                    subStore.push(sub);
                }
            });

            /*
                 * app <<-- component
                 * console.log("app <<-- bridge <<-- component");
                 * signature of data object per OJET specification.
                 * data = {  option: ,       previousValue: ,       value:     ,      optionMetadata:      }
                 */
            ko.utils.registerEventHandler(element, 'luxoptionchange', function(e, data) {
                // bindingBridge instantiated a widget with a UUID, so bBrige can only wirteBack to what it instantiated, nothing else
                if (data.optionMetadata.uuid === $(element).data('GBU-' + WidgetName).uuid) {
                    var newVal = data.value;
                    var nameOfOption = data.option;
                    writeBackMode = true;
                    valueAccessor()[nameOfOption](newVal);
                }
            });

            /*
                 * destroy the widget:
                 * http://knockoutjs.com/documentation/custom-bindings-disposal.html
                 * This will be called when the element is removed by Knockout or if some other part of your code calls ko.removeNode(element)
                 * todo : e.g pogressbar may fire this.events = ['change', 'create', 'complete']; events
                 * remove the event handler
                 * $(element).off(eventName);
                 */
            ko.utils.domNodeDisposal.addDisposeCallback(element, function() {
                $(element)[WidgetName]('destroy');
                $.each(subStore, function(index, value) {
                    if (typeof value !== 'undefined') {
                        value.dispose();
                    }
                });
                subStore.length = 0;
            });

        }
    };
}
);
