/**
 * @module gridMediatorArray
 */

/**
 * gridMediatorArray is an interface to any defined store of your choosing.
 *
 * You store could be as simple as an array, a custom solution or backbone implementation
 *
 * To use a custom store on a grid do
 *
 *      grid.setStore(data, myCustomStoreWrapper);
 *
 * Here is the interface for your storeWrapper.  The store for this storeWrapper is just a simple array
 *
 *      var myCustomStoreWrapper = function (store, widget) {
 *          this.store = store;
 *          this.options = widget.options;
 *          this.widget = widget;
 *          var storeFns = this.options.storeFns;
 *          this.group = storeFns.group || this.group;
 *          this.sort = storeFns.sort || this.sort;
 *          this.filter = storeFns.filter || this.filter;
 *          this.getRecord = storeFns.getRecord || this.getRecord;
 *          this.getRecords = storeFns.getRecords || this.getRecords;
 *          this.newRecord = storeFns.newRecord || this.newRecord;
 *          this.cutRecords = storeFns.cutRecords || this.cutRecords;
 *          this.copyRecords = storeFns.copyRecords || this.copyRecords;
 *          this.deleteRecords = storeFns.deleteRecords || this.deleteRecords;
 *          this.attachListeners = storeFns.attachListeners || this.attachListeners;
 *          this.placeRecord = storeFns.placeRecord || this.placeRecord;
 *          this.iterateStore = storeFns.iterateStore || this.iterateStore;
 *          this.updateRecord = storeFns.updateRecord || this.updateRecord;
 *          this.commit = storeFns.commit || this.commit;
 *          this.reset = storeFns.reset || this.reset;
 *          this.deletedRecords = [];
 *          this.getRecordAttributes = storeFns.getRecordAttributes || this.getRecordAttributes;
 *          this.checkRecordPermissions = storeFns.checkRecordPermissions || this.checkRecordPermissions;
 *          this.modified = {};
 *          this.readOnly = {};
 *
 *          this.newRecords = {};
 *      };
 *
 *      myCustomStoreWrapper.prototype = {
 *          constructor : StoreWrapper,
 *
 *          sort : function (args) {
 *              return false;
 *          },
 *
 *          group : function (args) {
 *              return false;
 *          },
 *
 *          filter : function (args) {
 *              return false;
 *          },
 *
 *          getRecord : function (id) {
 *              if(id < 0){
 *                  return this.newRecords[id];
 *              }
 *              return this.store[id];
 *          },
 *
 *          getRecords : function (ids) {
 *              var records = [];
 *              var that = this;
 *              _.each(ids, function (id) {
 *                  records.push(that.getRecord(id));
 *              });
 *              return records;
 *          },
 *
 *          newRecord : function (type) {
 *              var record = {};
 *              record.id = -(_.uniqueId() + 1);
 *              record.title = 'You should probably overwrite this method !!';
 *              record.type = type;
 *              return record;
 *          },
 *
 *          addRecords : function (records) {
 *              var that = this;
 *              var stripOut = [];
 *              _.each(records, function (record) {
 *                  var recordIndex = _.indexOf(that.deletedRecords, record);
 *                  if (recordIndex !== -1) {
 *                      delete that.deletedRecords[recordIndex];
 *                      stripOut.push(record);
 *                  }
 *                  if(record.id < 0){
 *                      that.newRecords[record.id] = record;
 *                  }
 *              });
 *              this.store.push(_.difference(records, stripOut));
 *              this.widget._recordsAdded(records);
 *          },
 *
 *          cutRecords : function (records, destination) {
 *
 *          },
 *
 *          copyRecords : function (records, destination) {
 *              var that = this;
 *              var newRecords = [];
 *              _.each(records, function (record) {
 *                  var newRecord = $.extend(true, {}, record);
 *                  newRecord.id = -(_.uniqueId());
 *                  that.store.push(newRecord);
 *                  newRecords.push(newRecord);
 *              });
 *              return newRecords;
 *          },
 *
 *          deleteRecords : function (records) {
 *              this.deletedRecords = _.union(this.deletedRecords, records);
 *              this.widget._recordsRemoved(records);
 *          },
 *
 *          attachListeners : function () {
 *
 *          },
 *
 *          placeRecord : function (record) {
 *              var location = {
 *                  waffer : TODO: wafferId,
 *                  table : '$tbody'
 *              };
 *              return location;
 *          },
 *
 *          iterateStore : function (functionToRun, thisContext, args) {
 *              for (var i = 0; i < this.store.length; ++i) {
 *                  functionToRun.call(thisContext, this.store[i], args);
 *              }
 *          },
 *
 *          updateRecord : function (record, key, data) {
 *              record[key] = data;
 *
 *              if(this.modified[record.id]!=null){
 *                  this.modified[record.id].push(key);
 *              } else {
 *                  this.modified[record.id] = [key];
 *              }
 *              this.widget._recordUpdated(record, key, data, true);
 *          },
 *
 *          getValueFromRecord : function (record, key) {
 *              return record[key];
 *          },
 *
 *          commit : function(){
 *              this.widget.repopulate();
 *              this.modified = {};
 *              this.readOnly = {};
 *          },
 *
 *          reset : function(){
 *              this.widget.repopulate();
 *              this.modified = {};
 *              this.readOnly = {};
 *          },
 *
 *          checkRecordPermissions : function(record){
 *              return [];
 *          },
 *
 *          getRecordAttributes : function(record){
 *              return { newRow : false, modified : [], readOnly : []};
 *          }
 *      };
 *
 *      return myCustomStoreWrapper;
 *

 * @constructor
 *
 *
 * @param {object} store - where the data is kept
 * @param {object} widget - reference to the grid widget
 *
 */

/**
 * Define your own custom sort method
 *
 * EG Server side sorts
 *
 * @method module:gridMediatorArray#sort
 * @param {object} args - sort criteria
 * @return {boolean} whether to use custom sort or the default client sort
 * @public
 */

/**
 * Define your own custom filter method
 *
 * EG Server side filters
 *
 * @method module:gridMediatorArray#filter
 * @param {object} args - filter criteria
 * @return {boolean} whether to use custom filer or the default client filter
 * @public
 */

/**
 * Define your own custom group method
 *
 * EG Server side grouping
 *
 * @method module:gridMediatorArray#group
 * @param {object} args - group criteria
 * @return {boolean} whether to use custom group or the default client group
 * @public
 */

/**
 * Get the record for the given id.
 *
 * @method module:gridMediatorArray#getRecord
 * @param {object} id - id for record
 * @return {object} record
 * @public
 */

/**
 * Return a collection of records for the given collection of ids
 *
 * @method module:gridMediatorArray#getRecords
 * @param {array} ids
 * @return {array} collection of records
 * @public
 */

/**
 * Define the contents of a new record
 *
 * @method module:gridMediatorArray#newRecord
 * @param {object} type
 * @return {object} record
 * @public
 */

/**
 * Add a collection of new records to the store
 *
 * @method module:gridMediatorArray#addRecords
 * @param {array} records
 * @public
 */

/**
 * Move records from one location to another in the store
 *
 * @method module:gridMediatorArray#cutRecords
 * @param {array} records
 * @param {object} destination
 * @public
 */

/**
 * Duplicate records in the store
 *
 * @method module:gridMediatorArray#copyRecords
 * @param {array} records
 * @param {object} destination
 * @public
 */

/**
 * Remove records from the store
 *
 * @method module:gridMediatorArray#deleteRecords
 * @param {array} records
 * @public
 */

/**
 * Attach possible listeners to the store
 *
 * @method module:gridMediatorArray#attachListeners
 * @param {array} records
 * @public
 */

/**
 * Return the position of the record for the grid
 *
 * @method module:gridMediatorArray#placeRecord
 * @param {object} record
 * @public
 */

/**
 * Traverse the store collection
 *
 * @method module:gridMediatorArray#iterateStore
 * @param {function} functionToRun
 * @param {object} context
 * @param {object} args
 * @public
 */

/**
 * Update the record with the corisponding data for the key given
 *
 * @method module:gridMediatorArray#updateRecord
 * @param {object} record
 * @param {object} key
 * @param {object} data
 * @public
 */

/**
 * Retrieve corisponding data for the key given from record
 *
 * @method module:gridMediatorArray#getValueFromRecord
 * @param {object} record
 * @param {object} key
 * @return {object} data to return from record
 * @public
 */

/**
 * Commit the data to store.  Probably involves some server participation
 *
 * @method module:gridMediatorArray#commit
 * @public
 */

/**
 * Reset the data in the grid to the given state in the store
 *
 * @method module:gridMediatorArray#reset
 * @public
 */

/**
 * Return a field name to make the field non editable
 *
 * @method module:gridMediatorArray#checkRecordPermissions
 * @param {object} record
 * @return {array} list of field names
 * @public
 */

/**
 * Build the record attributes to see if a record is new, has readonly fields and if it contains modified values
 *
 * @method module:gridMediatorArray#checkRecordAttributes
 * @param {object} record
 * @return {object} - { newRow : false, modified : [], readOnly : []}
 * @public
 */