
/**
 * @module luxGridStore
 */
define('luxGridStore/orderedMap', ['underscore'], function (_) {

    /**
     * Notes:
     * collection = ordered list of items
     * indexes = lookup map between item key and collection index
     * TODO - could probably use some underscore functions here for managing the index
     */

    return function (options) {

        var that = this,
            defaults = _.defaults(options, {
                key: 'key'
            });
        that.keys = {};
        that.collection = [];

        var populateIndexes = function () {
            that.keys = {};
            for (var index = 0; index < that.collection.length; index++) {
                var item = that.collection[index];
                that.keys[item[defaults.key]] = index;
            }
        };

        var isKeyValid = function (key) {
            if (key == null) {
                throw new Error('key undefined', 'orderedMap.js');
            }
            return true;
        };

        return {

            /**
             * Return all items
             * @return array
             */
            items: function () {
                return that.collection;
            },

            /**
             * Return item referenced by key
             * @param key - string
             * @return object
             */
            item: function (key) {
                if (isKeyValid(key)) {
                    var item = that.collection[that.keys[key]];
                    return item == null ? null : item;
                }
                return null;
            },

            /**
             * Add array of items
             * @param items - array
             */
            addAll: function (items) {
                for (var index = 0; index < items.length; index++) {
                    this.add(items[index]);
                }
            },

            /**
             * Add item
             * @param item - object
             */
            add: function (item) {
                that.collection.push(item);
                that.keys[item[defaults.key]] = that.collection.length - 1;
            },

            /**
             * Insert item into a specific position
             * Note - SLOW due to repopulate of indexes
             * @param item - object
             * @param position - integer
             */
            insertAt: function (item, position) {
                if (position < 0 || position > that.collection.length - 1) {
                    throw new RangeError('position out of index bounds', 'orderedmap.js');
                }

                that.collection.splice(position, 0, item);
                populateIndexes();
            },

            /**
             * Remove item
             * Note - SLOW due to repopulate of indexes
             * @param key - string
             */
            remove: function (key) {
                if (isKeyValid(key)) {
                    var position = this.position(key);
                    if (position > -1) {
                        delete that.keys[key];
                        that.collection.splice(position, 1);
                        populateIndexes();
                    }
                }
            },

            /**
             * Return the position of the item
             * @param key - integer
             * @return integer
             */
            position: function (key) {
                if (isKeyValid(key)) {
                    var thisKey = that.keys[key];
                    return thisKey == null ? -1 : thisKey;
                }
                return -1;
            },

            /**
             * Return whether the ordered map is empty
             * @return boolean
             */
            isEmpty: function () {
                return this.count() === 0;
            },

            /**
             * Return the size of the ordered map
             * @return integer
             */
            count: function () {
                return that.collection.length;
            },

            /**
             * Remove all items from ordered map
             */
            clear: function () {
                that.keys = {};
                that.collection = [];
            }
        };
    };
});

define('luxGridStore/storeCollection', ['underscore', 'luxGridStore/orderedMap'], function (_, OrderedMap) {

    var collection = function (name, options) {
        this.name = name;
        this.map = {};
        this.options = _.defaults(options, {
            key:      'id',
            typeKeys: {}
        });
    };

    var getKey = function (type) {
        var key = this.options.typeKeys[type];
        return key == null ? this.options.key : key;
    };

    collection.prototype.add = function (type, item) {
        if (!this.hasType(type)) {
            this.options.key = getKey.call(this, type);
            this.map[type] = new OrderedMap(this.options);
        }
        this.map[type].add(item);
        return item;
    };

    collection.prototype.insertAt = function (type, item, position) {
        if (!this.hasType(type)) {
            this.options.key = getKey(type);
            this.map[type] = new OrderedMap(this.options);
        }
        this.map[type].insertAt(item, position);
        return item;
    };

    collection.prototype.position = function (type, item) {
        if (this.hasType(type)) {
            return this.map[type].position(item);
        }
        return -1;
    };

    collection.prototype.remove = function (type, key) {
        if (this.hasType(type)) {
            this.map[type].remove(key);

            //            if (this.isEmpty(type)) {
            //                this.clear(type);
            //            }
        }
    };

    collection.prototype.item = function (type, key) {
        if (this.hasType(type)) {
            return this.map[type].item(key);
        }

        return null;
    };

    collection.prototype.items = function (type) {

        if (this.hasType(type)) {
            return this.map[type].items();
        }
        return [];
    };

    collection.prototype.hasType = function (type) {
        if (type == null) {
            throw new Error('storeCollection.js - "type" parameter not defined');
        }

        return _.has(this.map, type);
    };

    collection.prototype.clear = function (type) {
        if (this.hasType(type)) {
            delete this.map[type];
        }
    };

    collection.prototype.clearAll = function () {
        this.map = {};
    };

    collection.prototype.count = function (type) {
        if (this.hasType(type)) {
            return this.map[type].count();
        }
        return 0;
    };

    collection.prototype.isEmpty = function (type) {
        if (this.hasType(type)) {
            return this.map[type].isEmpty();
        }
        return true;
    };

    collection.prototype.isEmptyAll = function () {
        var that = this;
        var empty = true;
        _.each(this.map, function (value, type) {
            if (that.count(type) > 0) {
                empty = false;
                return true;
            }
        });
        return empty;
    };

    collection.prototype.types = function () {
        return _.keys(this.map);
    };

    return collection;
});

define('luxGridStore/store', ['luxGridCoreAPI.jquery', 'underscore', 'luxGridStore/storeCollection'], function ($, _, StoreCollection) {

/**
 * Store allows you to store different types of objects for your page.  Some of benefits of using store are:
 * <ul>
 *     <li>Order list that is indexed.  Allows item retrieval via id</li>
 *     <li>Storing multiple object types.  EG collection of projects, collection of categories, etc</li>
 *     <li>Knowing whether an item or item property (field) has changed from its original value</li>
 *     <li>Production of save json for batch page saving</li>
 * </ul>
 *
 *      options : {
 *          key : 'id'  // field used to index the object - field will need to be of type string
 *      }
 *
 *
 * @class store
 * @constructor
 * @alias module:luxGridStore.store
 * @param options Options for the store.
 */

    var defaults = {};

    var store = function (options) {

        defaults = _.defaults(options, {
            key: 'id'
        });

        // store meta information
        this._meta = {};

        this.storeCollection = new StoreCollection('store', options);
        this.modifyStoreCollection = new StoreCollection('modify', options);
        this.modifyStoreCollection.add = function (type, id, field, value) {
            var item = this.item(type, id);
            if (item != null) {
                var oldValues = item.oldValues;
                // commit old value - only to done if no original value is found
                if (oldValues != null && !_.has(oldValues, field)) {
                    oldValues[field] = value;
                }
            } else {
                item = {
                    oldValues: {}
                };
                item[defaults.key] = id;
                item.oldValues[field] = value;
                StoreCollection.prototype.add.call(this, type, item);
            }
        };
        this.modifyStoreCollection.remove = function (type, id, field, purge) {

            var item = this.item(type, id);
            if (item != null) {
                var oldValues = item.oldValues;
                if (oldValues != null) {
                    delete oldValues[field];
                }

                // remove the item if no modified fields exist
                // If purge is true, get rid of the whole record - this is
                //     because the remove method for this collection is by
                //     field and not by id.
                if (_.isEmpty(oldValues) || purge === true) {
                    StoreCollection.prototype.remove.call(this, type, id);
                }
            }

            // remove the type if type is empty
            if (this.isEmpty(type)) {
                this.clear(type);
            }
        };

        this.deleteStoreCollection = new StoreCollection('delete', options);
        this.newStoreCollection = new StoreCollection('new', options);
    };

    store.prototype.utilities = {
        /**
         * Is item new
         *
         * @method module:luxGridStore#isNew
         * @param {object} item
         * @return boolean
         */
        isNew: function (item) {
            if (item != null) {
                return parseInt(this.itemId(item), 10) < 0;
            }

            throw new Error('item undefined', 'store.js');
        },

        /**
         * Return item id
         *
         * @method module:luxGridStore#itemId
         * @param {object} item
         * @return string
         */
        itemId: function (item) {
            if (item != null) {
                return item[defaults.key];
            }

            throw new Error('item undefined', 'store.js');
        },

        /**
         * Return the options
         *
         * @method module:luxGridStore#options
         * @return object
         */
        options: function () {
            return defaults;
        },

        /**
         * Constant ajax get call.  Should be used with $.when()
         *
         * @property GET
         * @param {string} url
         * @return {function}
         */
        GET: function (url) {
            return $.ajax({
                url:      url,
                dataType: 'json',
                type:     'GET'
            });
        },

        /**
         * Constant ajax put call.  Should be used with $.when()
         *
         * @property PUT
         * @param {string} url
         * @param {object} data
         * @return {function}
         */
        PUT: function (url, data) {
            return $.ajax({
                url:         url,
                type:        'PUT',
                data:        JSON.stringify(data),
                contentType: 'application/json'
            });
        }
    };

    /**
     * Add meta data to for type
     *
     * @method module:luxGridStore#addMeta
     * @param {string} type
     * @param {object} data
     */
    store.prototype.addMeta = function (type, data) {
        this._meta[type] = data;
    };

    /**
     * Clear meta data for type
     *
     * @method module:luxGridStore#clearMeta
     * @param {string} type
     */
    store.prototype.clearMeta = function (type) {
        this._meta[type] = {};
    };

    /**
     * Return meta data for type
     *
     * @method module:luxGridStore#meta
     * @param {string} type
     * @return object
     */
    store.prototype.meta = function (type) {
        return this._meta[type];
    };

    /**
     * Is the store type empty
     *
     * @method module:luxGridStore#isEmpty
     * @param {string} type
     * @return boolean
     */
    store.prototype.isEmpty = function (type) {
        return this.storeCollection.isEmpty(type);
    };

    /**
     * Is the store empty
     *
     * @method module:luxGridStore#isEmptyAll
     * @return boolean
     */
    store.prototype.isEmptyAll = function () {
        return this.storeCollection.isEmptyAll();
    };

    /**
     * Count amount of items of a type in the store
     *
     * @method module:luxGridStore#count
     * @param {string} type
     * @return integer
     */
    store.prototype.count = function (type) {
        return this.storeCollection.count(type);
    };

    /**
     * Add an array of items to the store
     *
     * @method module:luxGridStore#addAll
     * @param {string} type
     * @param {array} items
     */
    store.prototype.addAll = function (type, items) {
        var that = this;
        _.each(items, function (item) {
            that.add(type, item);
        });
    };

    /**
     * Add item to store
     *
     * @method module:luxGridStore#add
     * @param {string} type
     * @param {object} item
     */
    store.prototype.add = function (type, item) {

        this.storeCollection.add(type, item);

        if (this.utilities.isNew(item)) {
            this.newStoreCollection.add(type, item);
        }
    };

    /**
     * Merge an array of items into the store. If the item is in one of the store's collections, it is ignored.
     * Otherwise, it is added.
     *
     * @method module:luxGridStore#merge
     * @param {string} type
     * @param {array} items
     */
    store.prototype.merge = function (type, items) {
        var that = this;

        _.each(items, function (item) {
            var id = that.utilities.itemId(item);
            if (that.storeCollection.item(type, id) == null &&
                that.deleteStoreCollection.item(type, id) == null &&
                that.modifyStoreCollection.item(type, id) == null) {
                that.add(type, item);
            }
        });
    };

    /**
     * Insert item to store
     *
     * @method module:luxGridStore#insertAt
     * @param {string} type
     * @param {object} item
     * @param {integer} position
     */
    store.prototype.insertAt = function (type, item, position) {

        this.storeCollection.insertAt(type, item, position);

        if (this.utilities.isNew(item)) {
            this.newStoreCollection.add(type, item);
        }
    };

    /**
     * Position of item in store
     *
     * @method module:luxGridStore#position
     * @param {string} type
     * @param {object} item
     * @return integer
     */
    store.prototype.position = function (type, item) {
        return this.storeCollection.position(type, item);
    };

    /**
     * Return array of items for the type
     *
     * @method module:luxGridStore#items
     * @param {string} type
     * @param {function} filterFn - a callback to a function to filter items
     * @return array
     */
    store.prototype.items = function (type, filterFn) {

        var items = this.storeCollection.items(type);

        if (_.isFunction(filterFn)) {
            var tmpItems = [];
            _.each(items, function (item) {
                if (filterFn(item)) {
                    tmpItems.push(item);
                }
            });
            items = tmpItems;
        }

        return items;
    };

    /**
     * Return item for that key and type
     *
     * @method module:luxGridStore#item
     * @param {string} type
     * @param {string} key
     * @return object
     */
    store.prototype.item = function (type, key) {
        return this.storeCollection.item(type, key);
    };

    /**
     * Update item field for that key and type
     *
     * @method module:luxGridStore#update
     * @param {string} type
     * @param {string} key
     * @param {string} field
     * @param {string} value
     */
    store.prototype.update = function (type, key, field, value) {

        var item = this.storeCollection.item(type, key);
        if (item != null) {
            // tell the modify store value has changed
            if (!this.utilities.isNew(item)) {

                var modifiedItem = this.modifyStoreCollection.item(type, key);
                if (modifiedItem != null && _.has(modifiedItem.oldValues, field)) {
                    var originalValue = modifiedItem.oldValues[field];
                    // is the updated value the same as the original
                    if (originalValue === value){
                        // remove value from modified collection
                        this.modifyStoreCollection.remove(type, key, field);
                    }
                } else {
                    // store the original value
                    var oldValue = item[field];
                    this.modifyStoreCollection.add(type, key, field, oldValue);

                }
            }

            // change the value
            item[field] = value;
        }
    };

    /**
     * Remove item for that key and type
     *
     * @method module:luxGridStore#remove
     * @param {string} type
     * @param {string} key
     */
    store.prototype.remove = function (type, key) {

        var item = this.item(type, key);
        if (item != null) {

            // if the item is not new then add to deleted store
            if (!this.utilities.isNew(item)) {
                this.deleteStoreCollection.add(type, item);
            }

            // remove item from new store
            this.newStoreCollection.remove(type, key);

            // remove item from modify store
            this.modifyStoreCollection.remove(type, key, null, true);

            this.storeCollection.remove(type, key);
        }
    };

    /**
     * Has item been modified for that key and type
     *
     * @method module:luxGridStore#isModified
     * @param {string} type
     * @param {string} key
     * @return boolean
     */
    store.prototype.isModified = function (type, key) {
        return !_.isEmpty(this.modifyStoreCollection.item(type, key));
    };

    /**
     * Has the store been modified
     *
     * @method module:luxGridStore#isModifiedAll
     * @return boolean
     */
    store.prototype.isModifiedAll = function () {
        return !(this.newStoreCollection.isEmptyAll() && this.modifyStoreCollection.isEmptyAll() && this.deleteStoreCollection.isEmptyAll());
    };

    /**
     * Has the item field been modified for that key, type and field
     *
     * @method module:luxGridStore#isModifiedField
     * @param {string} type
     * @param {string} key
     * @param {string} field
     * @return boolean
     */
    store.prototype.isModifiedField = function (type, key, field) {

        var item = this.modifyStoreCollection.item(type, key);
        if (item != null && item.oldValues != null) {
            return _.has(item.oldValues, field);
        }

        return false;
    };

    /**
     * Reset item for that key and type
     *
     * @method module:luxGridStore#reset
     * @param {string} type
     * @param {string} key
     */
    store.prototype.reset = function (type, key) {
        var that = this;
        var item = that.modifyStoreCollection.item(type, key);
        if (item != null) {
            _.each(item.oldValues, function (value, field) {
                that.storeCollection.item(type, key)[field] = value;
                that.modifyStoreCollection.remove(type, key, field);
            });
        }
    };

    /**
     * Reset all items
     *
     * @method module:luxGridStore#resetAll
     */
    store.prototype.resetAll = function () {

        var that = this;

        // put all delete item back
        _.each(this.deleteStoreCollection.map, function (orderedMap, type) {
            _.each(orderedMap.items(), function (item) {
                that.add(type, item);
            });
        });

        // put all old values back
        _.each(this.modifyStoreCollection.map, function (orderedMap, type) {
            _.each(orderedMap.items(), function (item) {
                _.each(item.oldValues, function (value, field) {
                    that.storeCollection.item(type, item[defaults.key])[field] = value;
                });
            });
        });

        // remove new items
        _.each(this.newStoreCollection.map, function (orderedMap, type) {
            _.each(orderedMap.items(), function (item) {
                that.remove(type, item[defaults.key]);
            });
        });

        that.modifyStoreCollection.clearAll();
        that.deleteStoreCollection.clearAll();
        that.newStoreCollection.clearAll();
    };

    /**
     * Get save object for all item types
     * Example:
     *
     *      {
     *          foo : {
     *              added: [],      // array of objects
     *              modified: [],   // array of objects
     *              deleted: []     // array of ids
     *          },
     *          bar : {
     *              added: [],      // array of objects
     *              modified: [],   // array of objects
     *              deleted: []     // array of ids
     *          }
     *      }
     *
     * @method module:luxGridStore#saveAll
     * @return object
     */
    store.prototype.saveAll = function () {
        var that = this;
        var saveData = {};

        _.each(this.storeCollection.types(), function (type) {
            var saveObj = that.save(type);
            if (saveObj != null) {
                saveData[type] = saveObj[type];
            }
        });

        return saveData;
    };

    /**
     * Get save object for item type
     *
     * @method module:luxGridStore#save
     * @param {string} type
     * @return object
     */
    store.prototype.save = function (type) {
        var that = this;
        if (this.storeCollection.hasType(type)) {

            var saveData = {};
            saveData[type] = {
                added:    [],
                deleted:  [],
                modified: []
            };

            // new items - whole object
            _.each(this.newStoreCollection.items(type), function (item) {
                saveData[type].added.push(item);
            });

            // removed items - just the id
            _.each(this.deleteStoreCollection.items(type), function (item) {
                saveData[type].deleted.push(item[defaults.key]);
            });

            // modified items
            _.each(this.modifyStoreCollection.items(type), function (item, id) {
                saveData[type].modified.push(that.storeCollection.item(type, item[defaults.key]));
            });

            return saveData;
        }

        return null;
    };

    /**
     * Commit entire store
     *
     * @method module:luxGridStore#commitAll
     * @param {object} data - response data from server
     */
    store.prototype.commitAll = function (data) {
        var that = this;
        _.each(data, function (data, type) {
            that.commit(type, data);
        });
        that.modifyStoreCollection.clearAll();
        that.deleteStoreCollection.clearAll();
        that.newStoreCollection.clearAll();
    };

    /**
     * Commit item(s) to store
     *
     * @method module:luxGridStore#commit
     * @param {string} type
     * @param {object} data - response data from server
     */
    store.prototype.commit = function (type, data) {
        var that = this;
        // update the new item ids
        _.each(data, function (newId, oldId) {
            var item = that.storeCollection.item(type, oldId);
            that.storeCollection.remove(type, oldId);
            item[defaults.key] = newId;
            that.storeCollection.add(type, item);
        });

        // clear new
        that.newStoreCollection.clear(type);

        // clear modified
        that.modifyStoreCollection.clear(type);

        // clear deleted
        that.deleteStoreCollection.clear(type);

    };

    return store;

});

define('luxGridStoreAPI', ['luxGridStore/store'],
    function () {

        return {
        };
    }
);
