/*global define */
/**
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * SubscriptionStoreFactory can be used to create subscription-store for the type of provided observable-array
 * and adapter. Once the subscription-store is created, the subscriptions can be created or destoryed using
 * the 'setWriteBack' method. Whenever value of an observable changes, it fires 'update' method on the provided
 * adapter. Use of SubscriptionStoreFactory in your applicatin de-couples the burden of recording changes in
 * your view-model data and allows you to create and destroy subscriptions for an observable-array at run time.
 * @author Shailesh Zirpe
 */

/**
 * Store of subscriptions for provided observable array.
 * @module luxSubscriptionStoreFactory
 * @requires jquery
 * @requires knockout
 */
define(['knockout', 'jquery'], function(ko, $) {

    var SubscriptionStore = function(obArray, storeAdapter, modelId) {
        this.subStore = [];
        this.obArray = obArray;
        this.storeAdapter = storeAdapter;
        this.modelId = modelId;
    };

    /**
     * Create or destory subscriptions based on the passed parameter to this method.
     * @method module:luxSubscriptionStoreFactory#setWriteBack
     * @param {Boolean} trueFalse - Boolean value that decides whether to create or destory subcriptions for the array.
     */
    SubscriptionStore.prototype.setWriteBack = function(trueFalse) {
        var self = this;
        if (trueFalse) {
            $.each(self.obArray(), function(index, observableObj) {
                $.each(observableObj, function(key, val) {
                    if (ko.isObservable(val)) {
                        var sub = val.subscribe(function(newVal) {
                            var modelId = observableObj[self.modelId]();
                            var data = {
                                modelId:      modelId,
                                key:          key,
                                newVal:       newVal,
                                storeAdapter: self.storeAdapter
                            };
                            /**
                             * This event is triggered on document node when the loggedIn status is set to true using the
                             * luxlogininfo.setLoginStatus() method. jQuery event object and data object are passed as parameters
                             * to the event handler. The data object has the two properties ‘loggedIn’ and ‘username’.
                             * For loginsuccess event the loggedIn property is always true.
                             * @event module:luxSubscriptionStoreFactory.storeUpdate
                             * @param {jqEvent} event
                             * @param {object} data
                             * @param {Boolean} data.modelId modelId
                             * @param {string} data.key key
                             * @param {string} data.newVal newVal
                             * @example
                             * <caption>  Bind an event listener to the event:</caption>
                             *   $(document).on('storeUpdate', function(event, data) {
                             *      console.log(data);
                             *  });
                             *
                             */
                            $(document).trigger('storeUpdate', data);
                            self.storeAdapter.update(modelId, key, newVal);
                        });
                        self.subStore.push(sub);
                    }
                });
            });
        } else {
            $.each(self.subStore, function(index, value) {
                if (typeof value !== 'undefined') {
                    value.dispose();
                }
            });
            self.subStore.length = 0;
        }
    };

    /**
     * Constructor for the luxSubscriptionStoreFactory.
     * SubscriptionStoreFactory can be used to create subscription-store for the type of provided observable-array
     * and adapter. Once the subscription-store is created, the subscriptions can be created or destoryed using
     * the 'setWriteBack' method. Whenever value of an observable changes, it fires 'update' method on the provided
     * adapter. Use of SubscriptionStoreFactory in your applicatin de-couples the burden of recording changes in
     * your view-model data and allows you to create and destroy subscriptions for an observable-array at run time.
     * @alias module:luxSubscriptionStoreFactory
     * @returns {luxSubscriptionStoreFactory} The newly created object of luxSubscriptionStoreFactory
     */
    var SubscriptionStoreFactory = function() {};

    /**
     * The method can be used to create an instance of subscription store for the type of provided observable-array
     * and adapter.
     *
     * @method module:luxSubscriptionStoreFactory#createSubscriptionStore
     * @public
     * @param obArray Knockout observable array of observables
     * @param storeAdapter an adapter on which to call the update method
     * @param modelId Identifier key in the object of observable array
     */
    SubscriptionStoreFactory.prototype.createSubscriptionStore = function(obArray, storeAdapter, modelId) {
        return new SubscriptionStore(obArray, storeAdapter, modelId);
    };

    return SubscriptionStoreFactory;

});
