/**
 * Copyright (c) 2013 - 2018 Oracle and/or its affiliates. All rights reserved.
 * @author Daniel Hoang, Shailesh Zirpe
 */
/* eslint-disable max-params */
define(['./viewModelMetaData',
    'ojs/ojcore',
    'jquery',
    'lodash',
    'knockout',
    './utils/preventChildrenBinding',
    'text!./adminCtrl.tmpl.html',
    'text!./contextMenu.tmpl.html',
    'text!./slot.tmpl.html',
    './defaultStore/metaCollection',
    './defaultStore/ctrlCollection',
    'jqueryui-amd/widget'
],
function(vmMetaData, oj, $, _, ko, utils, adminCtrl, contextMenu, slotTemplate, MetaCollection, CtrlCollection) {
    'use strict';
    /*
     * This utility can be utilized for converting current lux jQueryUI widget to CCA.
     */
    var viewModel = function(context, meta) {
        var self = this;
        self.ce = context.element;
        self.$ce = $(self.ce);
        self.meta = meta;
        var dbObj = {};

        dbObj.uuid = context.unique;

        dbObj.element = self.$ce;

        // i18n related options // LUX-573 translations option
        var i18nStringsDefaults = {
            'removeTile':                   'Remove',
            'configureTile':                'Configure Tile',
            'flipBack':                     'Flip Back',
            'flipForward':                  'Flip Forward',
            'reorderTitle':                 'Drag to reorder',
            'reorderLabel':                 'Drag to reorder, context menu available',
            'resizeCollapse':               'Collapse tile',
            'resizeExpand':                 'Expand tile',
            'tileDialogTitle':              'Click on tiles to add them',
            'addDashboardTilesButtonLabel': 'Add tiles to dashboard',
            'dashboardTileLabel':           'tile'
        };
        var i18nStringsNLSBundleDefined = {};
        _.each(i18nStringsDefaults, function(value, key, list) {
            var keyStr = 'luxDashboard' + '.' + key;
            var valueFromNLSBundle = oj.Translations.getTranslatedString(keyStr);
            if (valueFromNLSBundle !== keyStr) { // defined in NLS Bundle
                i18nStringsNLSBundleDefined[key] = valueFromNLSBundle;
            }
        });

        // Cache the context properties for later use.
        self.properties = context.properties;

        // Create "options" object for the dashboard (since we just reuse the dashboard widget code)
        // from the CCA properties. Since dashboard CCA has API changes for the ctrlStore and metaStore
        // properties, we don't create "ctrlStore" and "metaStore" options here.
        self.options = _.omit(context.properties, ['ctrlStore', 'metaStore']);

        self._internalStoreUpdate = function(ctrlStore, metaStore) {
            var _location;

            // Here we parse ctrlStore property to create the dashboard internal "ctrlStore" related options accordingly.
            if (!_.isUndefined(ctrlStore)) {
                _location = ko.unwrap(ctrlStore.location);
                if (_.isString(_location)) {
                    self.options.ctrlStore = undefined;
                    self.options.defaultCtrlStoreURL = _location;
                    self.options.defaultCtrlStoreExtender = ko.unwrap(ctrlStore.defaultStoreExtender);
                } else { // Assuming object represents custom store
                    self.options.defaultCtrlStoreURL = undefined;
                    self.options.ctrlStore = _location;
                }
                // These are common options for both stores
                self.options.ctrlStoreAdapterOverrides = ko.unwrap(ctrlStore.adapterOverrides);
                var _admin = ko.unwrap(ctrlStore.admin);
                self.options.ctrlWriteBack = _.isUndefined(_admin) ? false : _admin;
            }

            // Here we parse metaStore property to create the dashboard internal "metaStore" related options accordingly.
            if (!_.isUndefined(metaStore)) {
                _location = ko.unwrap(metaStore.location);
                if (_.isString(_location)) {
                    self.options.metaStore = undefined;
                    self.options.defaultMetaStoreURL = _location;
                    self.options.defaultMetaStoreExtender = ko.unwrap(metaStore.defaultStoreExtender);
                } else { // Assuming object represents custom store
                    self.options.defaultMetaStoreURL = undefined;
                    self.options.metaStore = _location;
                }
                // These are common options for both stores
                self.options.metaStoreAdapterOverrides = ko.unwrap(metaStore.adapterOverrides);
                var _persist = ko.unwrap(metaStore.persist);
                self.options.metaWriteBack = _.isUndefined(_persist) ? true : _persist;
            }
        };
        // Initialize component's internal stores
        self._internalStoreUpdate(context.properties.ctrlStore, context.properties.metaStore);

        self.options.translations = $.extend({}, i18nStringsDefaults, i18nStringsNLSBundleDefined, self.options.translations);

        // Listen for changes in the application '*Changed' event and modify the observable
        self.propertyChanged = function(ctx) {
            oj.Logger.info('propertyChanged: ' + ctx.property + '(' + JSON.stringify(ctx.value) + ')');
            if (ctx.updatedFrom === 'external') {
                if (ctx.property !== 'metaStore' && ctx.property !== 'ctrlStore') {
                    self[ctx.property](ctx.value);
                } else {
                    self._internalStoreUpdate(self.properties.ctrlStore, self.properties.metaStore);
                    if (ctx.property === 'ctrlStore' && ko.isObservable(self.defaultCtrlStoreURL)) {
                        self.defaultCtrlStoreURL(self.options.defaultCtrlStoreURL);
                        self.ctrlWriteBack(self.options.ctrlWriteBack);
                    }
                    if (ctx.property === 'metaStore' && ko.isObservable(self.defaultMetaStoreURL)) {
                        self.defaultMetaStoreURL(self.options.defaultMetaStoreURL);
                        self.metaWriteBack(self.options.metaWriteBack);
                    }
                }
            }
        };

        /*
         * The following callback is triggered when the view is attached to the DOM
         */
        self.connected = function() {

            //append the admin control dialog KO tempalte, the idea is to use same markup for mutliple dashboards adminCtrl
            if (!$(document.body).find('#adminCtrlLuxDashboard').length) { // add only one instance
                $('body').append($(adminCtrl).get(0));
            }
            if (!$(document.body).find('#luxDashboardContextMenuTemplate').length){ // add only one instance
                $('body').append($(contextMenu).get(0));
            }

            // parent call
            vmMetaData.call(self, dbObj);
        };

        self.getTemplateContent = function() {
            var template = context.element.querySelector('template');
            // Browsers that support the HTML template element will have content property.
            // Otherwise just return the element as HTML template is not supported.
            return template && template.content ? template.content : template;
        };

        self.bindingsApplied = function() {

            self.dataReady.subscribe(function(isDataReady){
                // option update 4 (ko if refresh)
                var templateContent = self.getTemplateContent();
                var tileIds = self.filteredMetaArray.map(function(obj){
                    return obj.tileId();
                });

                // This will initialize all the lux-slots based on the tileIds.
                // The elements are being loaded from the template tag
                // Most of this code will get removed once the oj-slots are working
                tileIds.forEach(function(tileId){
                    var slotName = self.getTileId(tileId);
                    var luxSlotEle = templateContent.querySelector('[lux-slot=' + slotName + ']');
                    var luxSlotEleClone = document.importNode(luxSlotEle, true);

                    var containerParent = $('.' + slotName).find('.luxSlotHook').get(0);
                    var data = ko.dataFor(self.ce);

                    if (isDataReady){ // option update 5 (ko if refresh : add DOM back)
                        if (containerParent !== undefined){
                            self.customBindingsApplied = true;
                            containerParent.appendChild(luxSlotEleClone);
                            ko.applyBindings(data, containerParent);
                        }
                    } else { // option update 6 (ko if refresh : DOM does not exist, removed by ko-if)
                        self.customBindingsApplied = false;
                        // since DOM does not exist, it is not required to ko.cleanNode(luxSlotEle); and containerParent.remove(luxSlotEle);
                    }
                });

            });

            // when the tile is removed by pressing the 'x' icon, ko-clean it
            context.element.addEventListener('beforeTileRemoved', function(e){
                var tileId = e.detail.tileId;
                var slotName = self.getTileId(tileId);
                var containerParent = $('.' + slotName).find('.luxSlotHook').get(0);
                ko.cleanNode(containerParent);
            });

            self.openTheDialogLazyLoader.subscribe(function(isTileAdderDialogOpen){
                // option update 4 (ko if refresh)
                var templateContent = self.getTemplateContent();
                // tileIds that are removed.
                var tileIds = self.metaArray()
                    .filter(function(obj){
                        return !obj.inserted();
                    })
                    .map(function(obj){
                        return obj.tileId();
                    });

                setTimeout(function(){

                    tileIds.forEach(function(tileId){
                        var slotName = self.getTileId(tileId);
                        var luxSlotEle = templateContent.querySelector('[lux-slot=' + slotName + ']');
                        var luxSlotEleClone = document.importNode(luxSlotEle, true);
                        var containerParent = $('.' + slotName).find('.luxSlotHook').get(0);
                        var data = ko.dataFor(self.ce);

                        if (isTileAdderDialogOpen){ // option update 5 (ko if refresh : add DOM back)
                            if (containerParent !== undefined){
                                self.customBindingsApplied = true;
                                containerParent.appendChild(luxSlotEleClone);
                                ko.applyBindings(data, containerParent);
                            }
                        }
                    });

                }, 0);

            });

        };

        // some of the external API
        self.getDefaultMetaStore = function(options) {
            return MetaCollection;
        };

        self.getDefaultCtrlStore = function(options) {
            return CtrlCollection;
        };

    };

    viewModel.prototype = Object.create(vmMetaData.prototype);
    viewModel.prototype.constructor = vmMetaData;

    return viewModel;
});

