/*
 * Copyright (c) 2017 Oracle and/or its affiliates. All rights reserved.
 *
 * @fileOverview
 * Setup a default view's data loader and methods to load data from a given entity.
 *
 * @author Daniel Hoang
 */
define([
    'ojs/ojcore',
    'knockout',
    'jquery'
], function (
    oj, ko, $
) {
    'use strict';

    var Logger = oj.Logger;

    var LuxGridData = function () {

        var self = this;

        self.dataObservable = undefined;
    };

    LuxGridData.prototype._fetchData = function(initialized) {
        var self = this;

        self.gridElement.setLoading(true);
        if (self.pageSize()) {
            // Hide pagination when fetching data (otherwise we'll see NaN in the number fields)
            var container = $(self.composite).find('.oj-lux-grid-basic-page-control-container')[0];

            // Consumer should use CollectionPagingDataSource or ArrayPagingDataSource
            Logger.info('Fetching pages from data-source ...');
            if (initialized) {
                // Get the data window to observe for the collection data changes
                // fetched by the pagination control.
                self.dataObservable = self.dataSource.getWindowObservable();

                // Define a computed pagination to react when data window changes,
                // with rate limit so we would only react when data changes are all
                // loaded in the collection's window.
                self.gridPaginationComputed = ko.computed(function() {
                    // This will trigger this computed based on the data observable changes.
                    var data = ko.toJS(self.dataObservable());
                    // reload the grid data
                    self._reloadGrid(data);
                    if (data && data.length) {
                        // set the style to show the pagination control.
                        container.classList.add('oj-lux-grid-basic-page-control-complete');
                    } else {
                        // set the style to hide the pagination control if there is no data.
                        container.classList.remove('oj-lux-grid-basic-page-control-complete');
                    }
                }).extend({rateLimit: {timeout: 10, method: 'notifyWhenChangesStop'}});
            } else {
                // request data source to refetch the data
                self.dataSource.fetch().then(function(/*result*/) {
                    // This fetch result should reflect in the data source window's observable,
                    // and we would handle the grid reload in the gridPaginationComputed above.
                    // So here we just refresh the grid pagination control.
                    self.gridPaginationControlElement.refresh();
                });
            }
        } else {
            // Consumer should use CollectionTableDataSource or ArrayTableDataSource
            Logger.info('Fetching all from data-source ...');
            self.dataSource.fetch().then(function(result) {
                self._reloadGrid(result.data);
            });
        }
    };

    LuxGridData.prototype._reloadGrid = function(data) {
        try {
            this.gridElement.setStore(data);
            this.gridElement.setLoading(false);
        } catch (e) {
            oj.Logger.warn('The LuxGridData._reloadGrid() threw an error: ', e);
        }
    };

    return LuxGridData;
});
