/**
 * Copyright (c) 2018 Oracle and/or its affiliates. All rights reserved.
 * @author Daniel Hoang
 */
define([
    'ojs/ojcore',
    'knockout',
    'jquery',
    'lodash',
    'text!./component.json',
    'ojL10n!./resources/nls/oj-lux-grid-strings',
    'AbstractCCAWidget'
],
function(oj, ko, $, _, metadata, gridStrings, AbstractCCAWidget) {
    'use strict';

    var Logger = oj.Logger;

    var ViewModel = function(context) {
        var self = this;

        // Initialize CCA widget
        self.widgetName = 'grid';
        AbstractCCAWidget.call(self, context, metadata);

        // Creata element ID for the widnget container element
        var widgetContainerId = self._uniqueId('oj-lux-grid-id-');

        // Register the widget container ID
        self.registerWidgetContainerId(widgetContainerId);

        // Initialize grid translation strings
        self.translations = gridStrings.luxGrid;
    };

    // Make LuxGridModel a subclass of AbstractCCAWidget
    ViewModel.prototype = Object.create(AbstractCCAWidget.prototype);
    ViewModel.prototype.constructor = ViewModel;

    // Internal method to handle component's whenReady API.
    // Return a promise to resolve when the grid's widget instance is instantiated.
    ViewModel.prototype._whenReady = function() {
        var self = this;
        return new Promise(function(resolve) {
            if (self.widgetReady()) {
                resolve();
            } else {
                var subscription = self.widgetReady.subscribe(function(isReady) {
                    if (isReady) {
                        resolve();
                        subscription.dispose();
                    }
                });
            }
        });
    };

    // Overriding abstract class' bindingsApplied() life-cycle in order to handle the grid's locale config option.
    ViewModel.prototype.bindingsApplied = function(context) {
        var self = this;

        // Pick up per instance's translation overrides. We do this by checking for explicit overrides supplied
        // on the tag instance and apply those. They are the most specific.
        var instanceTranslations = (context.properties.translations) ? context.properties.translations : {};

        // Update the grid's translations by merging this grid instance's translations with the global translations.
        Logger.info('Updating grid locale=' + oj.Config.getLocale());

        // options is defined by the abstract class to hold grid widget's options.
        // So here we update the grid's "locale" option with the translations.
        // Also, we need to **CLONE** the translations object (by merge({})) so as to decouple it from other instances.
        self.setWidgetOption('locale', _.merge({}, self.translations, instanceTranslations));
    };

    // Override to return proper event name format as required by the widget event naming convention.
    // For grid widget, the event name requires prefix 'grid-', follow with event name in kebab-case-format.
    ViewModel.prototype.getWidgetEventName = function(eventName) {
        return ('grid-' + _.kebabCase(eventName));
    };

    return ViewModel;
});
