/**
    * @fileOverview
    * This Composite Component implements a "Big Number" UI component that allows for the entry of numbers larger than the
    * JavaScript 15 digit limitation.  Taking inspiration from ojInputNumber, it combines the functionality of ojInputNumber with the
    * ability to enter arbitrary large numbers.  It uses ojInputText as it's basis.  Numbers are treated as strings and passed to luxBigNumber
    * to do calculations.  Comparisons are made for max and min values (if specified) using comparator functions from luxBigNumber.
    *
    * Functionality from ojInputNumber has been copied and modified to fit the needs of oj-lux-input-bignumber.
    * <h2>Instantiating the oj-lux-input-bignumber CCA</h2>
    * <p>A basic HTML markup with properties looks likes this:</p>
    * <pre class="prettyprint" style="overflow-x:visible;">
    *     <code>
    *           &lt;oj-lux-inputnumber
    *                id="currency2"
    *                value="{{decimal}}"
    *                min="[[min]]"
    *                min="[[max]]"
    *                step="{{step}}"
    *                root-attributes="{{style}}"
    *                converter="[[converterConfiguration]]"/>
    *            &lt;/oj-lux-inputnumber&gt;
    *     </code>
    * </pre>
    * <p>The id property is optional. To associate a label (see below) with the oj-lux-input-bignumber the id is required.
    * In the case of above example, the value, step, root-attributes and converter are knockout observables defined in the view model code but
    * they could be hard coded in the markup.  However Knockout observables allow for 2-way binding when using the CCA {{variable}} notation.
    *  </p>
    * <pre class="prettyprint" style="overflow-x:visible;">
    *     self.decimal = ko.observable('56.78');
    *     self.converterConfigJSON =  {style: 'currency', 'currency': 'USD', maximumFractionDigits: 2};
    *     self.style = {'style': 'max-width:65%'};
    *     self.step = "1.5";
    *     self.max = "12,000,000,000,000";
    *     self.min = "1200";
    * </pre>
    */

/**
    * @module oj-lux-input-bignumber
    * @requires knockout
    * @requires jquery
    * @requires lodash
    * @requires luxBignumber
    * @requires text!./component.json
    * @requires oL10n!resources/nls/oi18nLocaleElements
    * @requires ojs/ojcore
    * @requires ojs/ojvalidation-base
    * @requires ojs/ojknockout
    * @requires ojs/ojbutton
    * @requires ojs/ojinputtext
    * @requires promise
    */
define([
    'ojs/ojcore',
    'knockout',
    'jquery',
    'lodash',
    'luxBignumber',
    'text!./component.json',
    'oL10n!oi18nResources/nls/oi18nLocaleElements',
    './utils/NumberUtils',
    './oj-lux-input-bignumber-methods',
    'intlBigNumberConverter',
    'ojs/ojvalidation-base',
    'ojs/ojknockout',
    'ojs/ojbutton',
    'ojs/ojinputtext',
    'promise'
], function(oj, ko, $, _, luxBignum, meta, oi18nLocaleElements, NumberUtils, BigNumberMethods) {
    'use strict';

    var obj = /** @lends module:oj-lux-input-bignumber */ {
        /**
        * Options
        * <dl>
        *     <dt>HTML-Markup</dt>
        *     <dd>
        *         <p>A number converter instance that duck types int-bignumber-converter. Or an object literal containing the properties listed below. </p>
        *     <pre class="prettyprint" style="overflow-x:visible;">
        *     <code>
        *     &lt;oj-lux-input-bignumber
        *            id="currency2"
        *            value="{{decimal}}"
        *            min="[[min]]"
        *            min="[[max]]"
        *            step="{{step}}"
        *            root-attributes="{{style}}"
        *            converter="[[converterConfiguration]]"/>
        *     &lt;/oj-lux-input-bignumber&gt;
        *     </code>
        *     </pre>
        *     </dd>
        *     <dt>converter</dt>
        *     <dd>
        *         <p>The converter properties are the same as for ojInputNumber. For details <a href="http://jet.us.oracle.com/5.1.0/jsdocs/oj.ojInputNumber.html#converter" target="_blank">JET Docs</a></p>
        *         <p>When converter option changes due to programmatic intervention, the component performs various tasks based on the current state it is in.</p>
        *         <p>The properties for oj-lux-input-bignumber CCA mirror the options for JET's <a href="http://jet.us.oracle.com/5.1.0/jsdocs/oj.ojInputNumber.html" target="_blank">ojInputNumber</a> options. Please
        *            reference the JET Documentation for details. Below is an outline of the oj-lux-input-bignumber properties.  NOTE property names are shown as they would
        *            appear in html markup. Example:</p>
        *         <ul>
        *             <li>style</li>
        *             <li>currency</li>
        *             <li>currencyDisplay</li>
        *             <li>decimalFormat</li>
        *             <li>minimumIntegerDigits</li>
        *             <li>minimumFractionDigits</li>
        *             <li>maximumFractionDigits</li>
        *             <li>useGrouping</li>
        *             <li>pattern</li>
        *             <li>separators</li>
        *             <li>lenientParse</li>
        *         </ul><br />
        *         Default: creates the default converter<br />
        *         <strong>Example:</strong> {style: 'currency', currency: 'EUR', maximumFractionDigits: 2}<br />
        *     </dd>
        *     <dt>id</dt>
        *     <dd>
        *        <p>Include the "id" for label reference.
        *        </p>
        *         Type: string<br />
        *         Default: null<br />
        *         <strong>Example:</strong> id="myid"<br />
        *     </dd>
        *     <dt>inputElementId</dt>
        *     <dd>
        *        <p>ID applied to the internal html input element created by the ojinputtext component.
        *        </p>
        *         Type: string<br />
        *         Default: 'oj-oj-lux-input-bignumber'<br />
        *         <strong>Example:</strong> inputElementId="myInputId"<br />
        *     </dd>
        *     <dt>disabled</dt>
        *     <dd>
        *        <p>If permanently disabled the value can be simple boolean value (disabled=true) otherwise to
        *         allow dynamically toggling use a knockout observable ( viewmodel: var diadisabledObservable =
        *         ko.observable(false)) and surround the markup property with double brackets ({{}})
        *         to get 2-way binding from the CCA.</p>
        *         Type: boolean<br />
        *         Default: false<br />
        *         <strong>Example:</strong> disabled="{{disabledObservable}}"<br />
        *     </dd>
        *     <dt>value</dt>
        *     <dd>
        *         <p>Contains the value of component. Can have an initial value.</p>
        *         Type: string / knockout observable containing a string representing a number<br />
        *         Default: <br />
        *         <strong>Example:</strong> value="{{'9475757'}}" or value="{{myobservable}}"<br />
        *     </dd>
        *     <dt>required</dt>
        *     <dd>
        *         <p>Determines if a value is required to be entered</p>
        *         Type: boolean<br />
        *         Default: false<br />
        *         <strong>Example:</strong> value="{{true}}" or value="{{requireVariable}}"<br />
        *     </dd>
        *     <dt>max</dt>
        *     <dd>
        *         <p>Sets the upper limit for input. Must be a numeric value with no commas or spaces. Can except a decimal point.</p>
        *         Type: string<br />
        *         Default: undefined<br />
        *         <strong>Example:</strong> max="{{'9475757'}}" or max="{{'999.99'}}" or max="{{maxObservable}}"<br />
        *     </dd>
        *     <dt>min</dt>
        *     <dd>
        *         <p>Sets the lower limit for input. Must be a numeric value with no commas or spaces. Can except a decimal point.</p>
        *         Type: string<br />
        *         Default: undefined<br />
        *         <strong>Example:</strong> max="{{'90'}}" or max="{{'10.99'}}" or max="{{minObservable}}"<br />
        *     </dd>
        *     <dt>step</dt>
        *     <dd>
        *         <p>Sets the numeric amount incremented on a up or down step. Must be a numeric value with no commas or spaces. Can include a decimal point.</p>
        *         Type: string<br />
        *         Default: 1<br />
        *         <strong>Example:</strong> step="{{'10'}}" or step="{{'1.5'}}" or step="{{stepObservable}}"<br />
        *         <strong>NOTE:</strong> Decimal separators varies depending on locale. However for the step property use the period
        *         to indicate the factional part of a number.<br />
        *     </dd>
        *     <dt>justifyInput</dt>
        *     <dd>
        *         <p>Defines which side of the input box characters are justified. Default is right.</p>
        *         Type: string<br />
        *         Default: "left"<br />
        *         <strong>Example:</strong> justify-input="left"<br />
        *     </dd>
        *     <dt>autoComplete</dt>
        *     <dd>
        *         <p>Turns the html input element browser suggestions.</p>
        *         Type: boolean<br />
        *         Default: false<br />
        *         <strong>Example:</strong> auto-complete=false<br />
        *     </dd>
        *     <dt>translations</dt>
        *     <dd>
        *          <p>A collection of translated resources from the translation bundle, or null if this component has no resources. Resources may be accessed and overridden individually or collectively, as seen in the examples</p>
        *          Type: JSON Object<br />
        *          Default: an object containing all resources relevant to the component and all its superclasses, or null if none<br />
        *          <strong>Example</strong>
        *          <pre class="prettyprint" style="overflow-x:visible;">
        *          <code>
        *          translations = {
        *               numberRange: {
        *                  hint: {
        *                     inRange: "Please enter a number between {min} and {max}"
        *                  },
        *                  messageDetail: {
        *                     rangeUnderflow: "Your entered value must be greater than or equal to {min}",
        *                     rangeOverflow: "Your entered value must be less than or equal to {max}"
        *                  },
        *                  messageSummary: {
        *                      rangeUnderflow: "Your entered value is too low",
        *                      rangeOverflow: "Your entered value is too high"
        *                  }
        *              }
        *          };
        *          </code>
        *          </pre><br />
        *     </dd>
        * </dl>
        */
        options: {}
    };

    // An application will use this object to get locale data that need to be passed to the converter functions
    var oi18nResources = {};

    oi18nResources.getOi18nLocaleElements = function () {
        return oi18nResources.oi18nLocaleElements;
    };

    var CONSTANTS = {
        STEP_UP:           'increment',
        STEP_DOWN:         'decrement',
        DEFAULT_STEP:      '1',
        LABELWRAPPER:      '<div class="oj-label oj-component oj-inputtext-label"><div class="oj-label-group"></div></div>',
        TEXT_ALIGNMENT:    'text-align: ',
        DEFAULT_ALIGNMENT: 'left',
        ALLOWED_ALIGNMENT: ['left', 'right'],
        DEFAULT_INPUT_ID:  'oj-lux-input-bignumber'
    };

    /**
    * oj-lux-input-bignumber constructor.
    * @description Initializes the component using the properties key-value pair of the context to setup the conditions
    *              behaviors, converter and validation message translations. Gets the oi18n locale elements needed by
    *              intlBigNumberConverter to format currencies per the locale defined in the HTML language attribute.  Then
    *              returns a &quot;promise&quot; to process and configure CCA properties, get the converter, validate properties
    *              include custom validators.
    * @alias module:oj-lux-input-bignumber
    * @param {object} context - contains the following key-value pairs: element, props, slotNodeCounts, unique
    * @return {undefined}
    *
    */
    var luxInputBigNumber = function(context) {
        var self = this,
            properties = context.properties,
            metaProperties = JSON.parse(meta).properties;

        self.properties = {};
        // Create unique element IDs
        self.uniqueNumber = parseInt(context.unique.replace(/[^0-9\.]/g, ''), 10);
        self.inputId = self._uniqueId(CONSTANTS.DEFAULT_INPUT_ID);
        oj.Logger.info('************ generated unique id:', self.uniqueNumber, self.inputId);

        // *************************
        // Create mathjs instance from luxBignumber wrapper.
        // number: Create Mathjs instance of type BigNumber
        // precision: The maximum number of significant digits for bigNumbers. This setting only applies to BigNumbers, not to numbers. Default value is 64.
        // epsilon: The minimum relative difference used to test equality between two compared values.
        self.bigNum = luxBignum.BigNumberFactory().createBigNumber({number: luxBignum.BigNumberFactory.BIGNUMBER, precision: 128, epsilon: 1e-25});
        // *************************
        self.rawValue = null; // holds the raw value for the text input. used to determine the which step button to disable.

        oj.Logger.info('********** instantiate CCA for ', context.element.id);
        // save reference to oi18nLocaleElements function
        oi18nResources.oi18nLocaleElements = oi18nLocaleElements;
        self.oi18nResources = oi18nResources;

        self._validateAndConfigureProperties(properties);
        self.props = self._getActiveProperties(self.properties, metaProperties);

        self._setUpInputElementId();

        oj.Logger.info('********** instantiate CCA got properties', self.props);
    };

    luxInputBigNumber.prototype._setUpInputElementId = function() {
        var self = this;
        // If the default inputElementId, the value of inputElementId is applied
        // to the input element incremented by CCA sequence number parsed for the
        // context.unique id.
        if (CONSTANTS.DEFAULT_INPUT_ID === self.props.inputElementId) {
            self.inputId = self._uniqueId(self.props.inputElementId + '-');
        } else {
            // otherwise use the id supplied in the CCA properties.
            self.inputId = self.props.inputElementId;
        }
    };

    /**
     * @method module:oj-lux-input-bignumber#bindingsApplied
     * @description This function is invoked after the bindings are applied on the view as part of the Composite Component life cycle.
     * A reference to "input" element defined in the view is obtained using the context.unique value as the id.
     * With this reference, the ojInputText component is instantiated with options in the constructor. Listeners are
     * created on the "step" buttons described in the view that increment and decrement the number value.  Change
     * listeners are created on each component property to update the property when external events make changes to the
     * property.  A OnChange event is applied to the ojInputText to manage changes to options, particularly the value
     * option. Finally jquery DOM manipulation is done to wrap the input text with the decrement, increment buttons
     * that follows how the current ojInputNumber laid out.  If a "label" element is defined "for" the composite component
     * the 'for' attribute value is changed to point to actual "input" element defined in the view.
     * @param  {object} context - contains the following key-value pairs: element, props, slotNodeCounts, unique
     * @return {undefined}
     */
    luxInputBigNumber.prototype.bindingsApplied = function (context) {
        var self = this;

        oj.Logger.info('********** bindingsApplied CCA for', context.element.id);

        self._container = $('#' + self.inputId);
        self._container.attr('style', self.inputAligment);
        // default turns of the browser autocomplete.
        if (!self.properties.autoComplete) {
            self._container.attr('autocomplete', 'off');
        }
        // Setup button pointers.
        self.decButton = document.getElementById(self.inputId + '-dec');
        self.incButton = document.getElementById(self.inputId + '-inc');
        // Setup spinner button event listeners.
        BigNumberMethods.setUpSpinnerButtonMouseEventListeners.call(self, CONSTANTS);
        // Setup inputText field.
        BigNumberMethods.setUpInputTextField.call(self);
        // Add the property change listeners.
        BigNumberMethods.setUpPropertyChangeListeners.call(self, context);
        // Add the JET HTML wrapper and css classes to replicate the look and feel of oj-input-number spinners
        BigNumberMethods.setUpSpinnerButtons.call(self);
        // Add the JET HTML wrapper and css classes to replicate the correct layout for ojInputtext and Label tag.
        BigNumberMethods.setUpInputTextLabel.call(self, context, CONSTANTS);
        // The input can be disabled
        if (self.properties.disabled) {
            self._disable(context); // disabled
        } else {
            self._setButtonStates(self.properties.value); // enable both buttons.
        }
    };

    /**
    *  Validates the component's display value using the converter and all validators registered on the component and updates the value option by performing the following steps.
    *  <ul>
    *     <li>All messages are cleared, including custom messages added by the app.</li>
    *     <li>The UI value is first converted (i.e., parsed). If there is a parse error then the messagesShown option is updated and method returns false.</li>
    *     <li>If there are no validators setup for the component the value option is updated using the display value and the method returns true. Otherwise all validators are run in sequence using the parsed value from the previous step. The implicit required validator is run first if the component is marked required. When a validation error is encountered it is remembered and the next validator in the sequence is run. </li>
    *     <li>At the end of validation if there are errors, the messagesShown option is updated and method returns false. If there were no errors, then the value option is updated and method returns true.</li>
    *  </ul>
    *  @return {boolean}  true if valid, false if not
    */
    luxInputBigNumber.prototype.validate = function() {
        var self = this,
            valid = self._container.ojInputText('validate');

        if (valid) {
            self.properties.value = self._container.ojInputText('option', 'value');
        }

        return valid;
    };

    /**
     *    Whether the component is currently valid. It is valid if it doesn't have any errors.
     *    @return {Boolean} true if isValid, false if not
     */
    luxInputBigNumber.prototype.isValid = function() {
        var self = this;

        if (!self._container) {
            return false;
        }

        return self._container.ojInputText('isValid');
    };

    /**
     *    Takes all deferred messages and shows them. If there were no deferred messages this method simply returns.
     *    @return {string} messages.
     */
    luxInputBigNumber.prototype.showMessages = function() {
        var self = this;

        return self._container.ojInputText('showMessages');
    };

    /**
     *    Refreshes the component. Usually called after dom changes have been made.
     *    @returns {undefined}
     */
    luxInputBigNumber.prototype.refresh = function() {
        var self = this;

        self._container.ojInputText('refresh');
    };

    /**
     *   Resets the component by clearing all messages and messages attributes - messagesCustom - and updates the component's display value using the attribute value. User entered values will be erased when this method is called.
     *   @returns {undefined}
     */
    luxInputBigNumber.prototype.reset = function() {
        var self = this;

        self._container.ojInputText('reset');
    };

    /**
     *    Returns a un-formatted string representation of the current value. Removes all currency and separators leaving only the decimal point
     *    @return {object} representing decimal number.
     */
    luxInputBigNumber.prototype.normalize = function() {
        var self = this;

        if (self.properties.value === '0') {
            return '0';
        }

        return self.properties.converter ?
            self.properties.converter.normalize(self.properties.value, oi18nResources.getOi18nLocaleElements()) :
            self.properties.value;
    };
    /**
     *    Returns a formatted string representation of the current value based on the converter.
     *    @return {object} representing decimal number.
     */
    luxInputBigNumber.prototype.format = function() {
        var self = this;

        if (self.properties.value === '0') {
            return '0';
        }

        return self.properties.value;
    };

    /**
     *  Formats the form input value per locale
     *  @param  {string} newVal the value to be formatted
     *  @return {string}        formatted value
     */
    luxInputBigNumber.prototype._formatFieldInput = function(newVal) {
        var self = this,
            value = self._normalizeInput(newVal);

        if (self._isEmpty(newVal)) {
            return;
        }

        return self.properties.converter ?
            self.properties.converter.format(value, oi18nResources.getOi18nLocaleElements()) :
            value;
    };

    luxInputBigNumber.prototype._formatInput = function(newVal) {
        var self = this,
            value = newVal;

        if (!self.properties.converter.format) {
            return;
        }

        if (self._isEmpty(newVal)) {
            return;
        }

        return self._formatValue(value);
    };

    luxInputBigNumber.prototype._normalizeInput = function(newVal) {
        var self = this;

        if (!self.properties.converter.normalize) {
            return;
        }

        if (self._isEmpty(newVal)) {
            return;
        }

        return self.properties.converter ?
            self.properties.converter.normalize(newVal, oi18nResources.getOi18nLocaleElements()) :
            newVal;
    };

    luxInputBigNumber.prototype._isEmpty = function(value) {

        if (_.isNull(value)) {
            return true;
        } else if (_.isUndefined(value)) {
            return true;
        } else if (value.trim().length === 0) {
            return true;
        }
        return false;
    };

    /**
     * This factory method creates update methods for the various CCA properties.
     * @param {object} context - CCA context, used by update methods to get a handle to the CCA in the DOM
     * @param {string} eventType - indicates what has changed, [property name]Changed.
     * @param {string} key - property key in the javascript property object
     * @return {undefined}
     * TODO: refactor this function into the bignumber-methods file.
     */
    luxInputBigNumber.prototype._eventListenerFactory = function eventListenerFactory(context, eventType, key) {
        var self = this;
        // When a attribute name is changed the
        // *Changed event is fired by CCA infrastructure.
        if (eventType === 'valueChanged') {

            return BigNumberMethods.updateValue.bind(self, key);

        } else if (eventType === 'disabledChanged') {
            /*
             *    returns listener function
             *    @param  {object} event - ui event object
             *    @return {function}   listener that updates the disabled property
             */
            return function updateDisabled(event) {
                var object = {};

                object[key] = event.detail.value;
                self._toggleEnableDisable(context, object.disabled);
                // NOTE: do not update the ojInputText disabled option
                // as this will not dim the input field.
            };
        } else if (eventType === 'translationsChanged') {
            /*
             *    returns listener function
             *    @param  {object} event - ui event object
             *    @return {function}   listener that updates the element style Translations
             */
            return function updateTranslations() {
                self._setValidators();
            };
        } else if (eventType === 'converterChanged') {
            /*
             *    returns listener function
             *    @param  {object} event - ui event object
             *    @return {function}   listener that updates the converter property
             */
            return function updateConverter(event) {

                if (event.detail.updatedFrom === 'internal') {
                    return;
                }

                // apply the new converter object
                self._getConverter();

                // update the option
                self._container.ojInputText('option', 'converter', self.properties.converter);
            };
        } else if (eventType === 'minChanged') {

            return function updateConverterMinChange(event) {

                if (event.detail.updatedFrom === 'internal') {
                    return;
                }

                self._updateMinMax(event, key);
            };
        } else if (eventType === 'maxChanged') {

            return function updateConverterMaxChange(event) {

                if (event.detail.updatedFrom === 'internal') {
                    return;
                }

                self._updateMinMax(event, key);
            };
        } else {
            /*
             * returns listener function for all other properties updates
             */
            return BigNumberMethods.updateProperties.bind(self, key, CONSTANTS);
        }
    };

    luxInputBigNumber.prototype._updateMinMax = function(event, key) {
        var self = this,
            object = {};

        oj.Logger.info('******* ', self.inputId, 'property', key, 'changed', event.detail.value);

        object[key] = event.detail.value;

        self.properties[key] = self._parse(key, self.properties[key]);
        self._setValidators();

        if (self.properties.max) {
            document.getElementById(self.inputId).setAttribute('aria-valuemax', self.properties.max);
        } else {
            document.getElementById(self.inputId).removeAttribute('aria-valuemax');
        }
        if (self.properties.min) {
            document.getElementById(self.inputId).setAttribute('aria-valuemin', self.properties.min);
        } else {
            document.getElementById(self.inputId).removeAttribute('aria-valuemin');
        }
    };

    /**
     * Private method.
     * @description Properties from the context object are saved to the model object.
     * Some minor changes are made to properties. If properties.value does not exist it is created as an empty
     * string. If properties.step does not exist it is created with default value of 1. If properties.step does
     * exist it is coerced to a string. If no element ID is specified for the parent CCA, one is created here.
     * @param  {object} properties - contains the following properties key-value pairs
     * @return {undefined}
     */
    luxInputBigNumber.prototype._validateAndConfigureProperties = function(properties) {
        var self = this;

        //Save the resolved properties for later access
        self.properties = properties;

        // trim off any spaces since this is a string.
        if (self.properties.value) {
            if (_.isNumber(self.properties.value)) {
                self.properties.value = self.properties.value.toString();
            }
            // trim off spaces before or after the value
            self.properties.value = self.properties.value.trim();
        }
        // if the input value does not exist, create it with
        // an empty value.
        if (!self.properties.value) {
            self.properties.value = '';
        }

        self._getConverter();

        // if no step property create it with default value
        if (!self.properties.step) {
            self.properties.step = CONSTANTS.DEFAULT_STEP;
        }

        self._validateProperties();
        self._setValidators();

        self.inputAligment = '';
        if (_.isUndefined(self.properties.justifyInput)) {
            self.inputAligment = CONSTANTS.TEXT_ALIGNMENT + CONSTANTS.DEFAULT_ALIGNMENT;
        } else if (_.includes(CONSTANTS.ALLOWED_ALIGNMENT, self.properties.justifyInput.toLowerCase())) {
            self.inputAligment = CONSTANTS.TEXT_ALIGNMENT + self.properties.justifyInput + ';';
        } else {
            self.inputAligment = CONSTANTS.TEXT_ALIGNMENT + CONSTANTS.DEFAULT_ALIGNMENT;
        }
    };

    /**
     * Private method.
     * @description Determines how to create the converter object. If converter the property
     * is supplied the converter is created using the converter property. If an actual converter
     * object is supplied, the converter object is passed on.  If no converter object or properties
     * are supplied, a default converter is created.
     * @private
     * @returns {undefined}
     */
    luxInputBigNumber.prototype._getConverter = function() {
        var self = this,
            luxConverterFactory,
            //math = self.bigNum,
            parsedOptins,
            customParse = function(p) {
                if (_.isString(p)) {
                    return p;
                }

                return p.toString();
            };

        // *******************************************************************
        // Configure the intlNumberConverter property
        // *******************************************************************
        luxConverterFactory = oj.Validation.converterFactory('bignumber');
        if (!self.properties.converter) {
            self.properties.converter = {'customParse': customParse};
            // nothing set to create a default converter
            oj.Logger.info('*** NO CONVERTER SPECIFIED, using default');
            self.properties.converter = luxConverterFactory.createConverter(self.properties.converter);
        } else if (_.isPlainObject(self.properties.converter)) {
            // JSON object parameters sent so create converter with theses parameters
            oj.Logger.info('*** CONVERTER PARAMETERS SPECIFIED');
            self.properties.converter.customParse = customParse;
            self.properties.converter = luxConverterFactory.createConverter(self.properties.converter);
        } else if (_.isString(self.properties.converter)) {
            // plane JSON string parameters sent so create converter with theses parameters
            oj.Logger.info('*** CONVERTER STRING PARAMETERS SPECIFIED');
            try {
                parsedOptins = JSON.parse(self.properties.converter);
                parsedOptins.customParse = customParse;
                self.properties.converter = luxConverterFactory.createConverter(parsedOptins);
            } catch (e) {
                oj.Logger.error('Invalid converter parameters,', self.properties.converter, 'message:', e);
            }
        } else {
            // if the converter object already exists, need to add the customParse function, so
            // get the converter properties, add the customParse function and re-create the object.
            parsedOptins = self.properties.converter.getOptions();
            parsedOptins.customParse = customParse;
            self.properties.converter = luxConverterFactory.createConverter(parsedOptins);
        }
        // get the converter options object, this information is used in formatting output.
        self._converterProps = self.properties.converter.getOptions();
    };

    /**
     * @description Makes sure the values, step, min, and max properties, if provided, are numeric values.
     * @private
     * @returns {undefined}
     */
    luxInputBigNumber.prototype._validateProperties = function() {
        var self = this,
            toParse = ['value', 'step', 'min', 'max'];

        // validate inputs are numeric.
        toParse.forEach(function(property) {
            var value = self.properties[property];

            if (!_.isUndefined(value)) {
                if (property === 'step') {
                    self.properties[property] = value !== CONSTANTS.DEFAULT_STEP ? self._parseStep(value) : value;
                } else if (property === 'min' || property === 'max') {
                    self.properties[property] = self._parse(property, value);
                } else {
                    if (value.length > 0) {
                        self.properties[property] = self._parse(property, value);
                    } else {
                        self.properties[property] = value;
                    }
                }
            }
        });
    };

    /**
     * @description Function to toggle the disable on or off base on the
     * state of the disabled property.
     * @param  {object} context - contains the following key-value pairs: element, props, slotNodeCounts, unique.
     * @private
     * @returns {undefined}
     */
    luxInputBigNumber.prototype._toggleEnableDisable = function(context) {
        var self = this,
            inputEl = $('#' + self.inputId);

        if (self.properties.disabled) {
            self._disable(context);
        } else {
            $('#' + self.inputId + '-dec').removeClass('oj-disabled');
            $('#' + self.inputId + '-inc').removeClass('oj-disabled');
            inputEl.removeClass('oj-inputnumber-input');
            $('#' + context.element.id).children().first().removeClass('oj-disabled');
            document.getElementById(self.inputId).disabled = false;
            inputEl.ojInputText('option', 'disabled', false);
        }
    };

    /**
     * @description Function to disable the input text and step buttons.
     * @param  {object} context - contains the following key-value pairs: element, props, slotNodeCounts, unique.
     * @private
     * @returns {undefined}
     */
    luxInputBigNumber.prototype._disable = function(context) {
        var self = this,
            inputEl = $('#' + self.inputId);

        $('#' + self.inputId + '-dec').addClass('oj-disabled');
        $('#' + self.inputId + '-inc').addClass('oj-disabled');
        inputEl.addClass('oj-inputnumber-input');
        $('#' + context.element.id).children().first().addClass('oj-disabled');
        document.getElementById(self.inputId).disabled = true;
        inputEl.ojInputText('option', 'disabled', true);
    };

    /**
     * Private method.
     * @description Sets up the validator array that is passed as an option to the ojInputText.  Number validation
     * is allows pushed. Range validation is push if there is max and or min properties. Finally any developer supplied
     * validations are pushed to the array. NOTE: A reference is saved to the oi18nResources. oi18nResources is in the
     * _createRangeValidator method to create the validation with oi18nResources.
     * @returns {undefined}
     */
    luxInputBigNumber.prototype._setValidators = function() {
        var self = this, validators = [];

        self.properties.bigNum = self.bigNum;
        self.properties.oi18nResources = oi18nResources.getOi18nLocaleElements();
        if (self.properties.max || self.properties.min) {
            validators.push(oj.Validation.validatorFactory(oj.ConverterFactory.VALIDATOR_TYPE_BIGNUMBERRANGE).createValidator(self._createRangeValidator()));
        }
        // include any custom validation
        if (self.properties.validators) {
            validators.push(self.properties.validators);
        } else {
            self.properties.validators = [];
        }

        self.properties.validators = validators;
    };

    /**
     * @description Copied from ojInputNumber with some modifications.  Added options for
     * oi18nResource and mathjs big number utility.
     * @return {object} JSON Object containing range validation options
     */
    luxInputBigNumber.prototype._createRangeValidator = function() {
        var self = this,
            properties = self.properties,
            // Make sure the normalized max min values is used
            minOpt = properties.min && self._normalizeInput(self._normalizeInput(properties.min)),
            maxOpt = properties.max && self._normalizeInput(self._normalizeInput(properties.max)),
            newMin = minOpt !== null ? minOpt : undefined,
            newMax = maxOpt !== null ? maxOpt : undefined,
            translations = properties.translations,
            translationsOptionNumberRange = translations ? translations.numberRange || {} : {},
            hintMin,
            hintMax,
            hintInRange,
            hintExact,
            messageDetailRangeOverflow,
            messageDetailRangeUnderflow,
            messageDetailExact,
            messageSummaryRangeOverflow,
            messageSummaryRangeUnderflow,
            translationsHint = translationsOptionNumberRange.hint || {},
            translationsMessageDetail = translationsOptionNumberRange.messageDetail || {},
            translationsMessageSummary = translationsOptionNumberRange.messageSummary || {},
            numberRangeValidatorOptions;

        if (translationsHint !== null) {
            hintMin = translationsHint.min || null;
            hintMax = translationsHint.max || null;
            hintInRange = translationsHint.inRange || null;
            hintExact = translationsHint.exact || null;
        }

        if (translationsMessageDetail !== null) {
            messageDetailRangeOverflow = translationsMessageDetail.rangeOverflow || null;
            messageDetailRangeUnderflow = translationsMessageDetail.rangeUnderflow || null;
            messageDetailExact = translationsMessageDetail.exact || null;
        }
        if (translationsMessageSummary !== null) {
            messageSummaryRangeOverflow = translationsMessageSummary.rangeOverflow || null;
            messageSummaryRangeUnderflow = translationsMessageSummary.rangeUnderflow || null;
        }

        // unless the translations options for the number range were set, the hints and messageDetails
        // and messageSummaries will be null and we'll pick up the default strings from the
        // NumberRangeValidator.
        numberRangeValidatorOptions = {
            'min':            newMin,
            'max':            newMax,
            'oi18nResources': oi18nResources.getOi18nLocaleElements(),
            'bigNum':         self.bigNum,
            'hint':           {
                'min':     hintMin || null,
                'max':     hintMax || null,
                'inRange': hintInRange || null,
                'exact':   hintExact || null
            },
            'messageDetail': {
                'rangeOverflow':  messageDetailRangeOverflow || null,
                'rangeUnderflow': messageDetailRangeUnderflow || null,
                'exact':          messageDetailExact || null
            },
            'messageSummary': {
                'rangeOverflow':  messageSummaryRangeOverflow || null,
                'rangeUnderflow': messageSummaryRangeUnderflow || null
            },
            'converter': self.properties.converter
        };

        return numberRangeValidatorOptions;
    };

    /**
     * @description Checks keypress for keycode 40 (up arrow) and keycode 38
     * (down arrow) and executes the appropriate function to
     * step the input value.
     * @param  {object} event - object representing the key press event.
     * @returns {undefined}
     */
    luxInputBigNumber.prototype._keypress = function(event) {
        var self = this,
            keyCode = event.keyCode;

        if (keyCode === 40) {
            self._stop(event);
            self.stepDown(event);
        } else if (keyCode === 38) {
            self._stop(event);
            self.stepUp(event);
        }
    };

    luxInputBigNumber.prototype._repeat = function(repeat, direction, event) {
        var self = this,
            stopRepeat = false,
            isDisabled = function(el) {
                return el.classList.contains('oj-disabled');
            };

        if (direction === CONSTANTS.STEP_DOWN) {
            if (isDisabled(document.getElementById(self.inputId + '-dec'))) {
                stopRepeat = true;
            }
        } else if (direction === CONSTANTS.STEP_UP) {
            if (isDisabled(document.getElementById(self.inputId + '-inc'))) {
                stopRepeat = true;
            }
        }

        // repeat spinning as long as the key is down and min/max isn't reached
        repeat = repeat || 500;

        clearTimeout(self.timer);
        // this.timer will be cleared elsewhere, like when the user stops holding down the up/down
        // arrows. See luxInputBigNumber.prototype._stop
        self.timer = _.delay(function() {
            if (!stopRepeat) {
                self._repeat(40, direction, event);
            }
        }, repeat);

        self._step(event, direction);
    };

    luxInputBigNumber.prototype._stop = function() {
        var self = this;

        clearTimeout(self.timer);
    };

    /**
     * @description Decrements the value by the specified number of steps. If the resulting value is above the max, below the min, or results in a step mismatch, the value will be adjusted to the closest valid value.
     * @method module:oj-lux-input-bignumber#stepDown
     * @param  {object} event passed to the _step private method.
     * @return {undefined} if the button is disabled step does not execute
     */
    luxInputBigNumber.prototype.stepDown = function(event) {
        var self = this;

        if (self._buttonIsDisabled(event)) {
            return;
        }

        self._step(event, CONSTANTS.STEP_DOWN);
    };

    /**
     * @description Increments the value by the specified number of steps. If the resulting value is above the max, below the min, or results in a step mismatch, the value will be adjusted to the closest valid value.
     * @method module:oj-lux-input-bignumber#stepUp
     * @param  {object} event passed to the _step private method.
     * @return {undefined} if the button is disabled step does not execute
     */
    luxInputBigNumber.prototype.stepUp = function(event) {
        var self = this;

        if (self._buttonIsDisabled(event)) {
            return;
        }

        self._step(event, CONSTANTS.STEP_UP);
    };

    /**
     * @description generalized function called by stepUp and stepDown functions
     * @param  {object} event - used to console display the current target id
     * @param  {string} direction - describes the step direction, increment or decrement.
     * @return {undefined}   if current value is up against the max / min range the step request
     *                  does not execute.
     */
    luxInputBigNumber.prototype._step = function(event, direction) {
        var self = this;

        if (event.currentTarget && event.currentTarget.id) {
            // if the button is disabled do not do anything.
            if (self._buttonIsDisabled(event)) {
                return;
            }
            if (!self._container.ojInputText('isValid')) {
                self.properties.value = '0';
            }
        }

        self.stepping = true;
        if (self._adjustValue(direction)) {
            self._setValueProperty(self._stepNumber(direction, self.properties.value), false);
        }
        self.stepping = false;
    };

    /**
     * @description Given the source of the new value (from input, from increment / decrement spinner, or from the saved
     * rawValue this function determines what the starting point of the new value should be. Then given the direction
     * of the step, this function checks the value against the max / min range and sets the value to the max or min value
     * (depending of the direction) when the value is up against the max / min range.  Disables the max / min buttons when
     * the value not within the maximum and or minimum ranges.
     *
     * @param  {string} direction - describes the step direction, increment or decrement.
     * @return {boolean} when the value is maxed or mined return false otherwise returns true.
     */
    luxInputBigNumber.prototype._adjustValue = function(direction) {
        var self = this,
            value = self.properties.value,
            stringValue = _.isString(value) ? value : value ? value.toString() : value,
            validInput = self._container.ojInputText('isValid'),
            paramsObj = {};

        paramsObj.direction = direction;
        paramsObj.normalized = self._normalizeInput(stringValue);
        paramsObj.math = self.bigNum;
        paramsObj.min = self.properties.min !== undefined ? paramsObj.math.bignumber(self._normalizeInput(self.properties.min)) : null;
        paramsObj.max = self.properties.max !== undefined ? paramsObj.math.bignumber(self._normalizeInput(self.properties.max)) : null;
        paramsObj.step = paramsObj.math.bignumber(self._normalizeInput(self.properties.step));

        // if no min or max don't need to run this function.
        if (!self.properties.min && !self.properties.max) {
            return true;
        }

        // if there is no value to compare, there is no input
        // and nothing to compare.
        if (!value || paramsObj.normalized.length < 1) {
            paramsObj.normalized = '0';
        }

        // is there a rawValue possibly because the user has entered a value pressed the up key or button
        // without leaving the input field
        if (self.rawValue) {
            // get a normalized version of the value (removes none locale formatting characters)
            self.rawValue = NumberUtils.normalizeLocale(self.rawValue, oi18nResources.getOi18nLocaleElements());

            if (self.rawValue.length > 0) {
                paramsObj.normalized = self._normalizeInput(self.rawValue);
            }
            // save the converter normalized rawValue over the parameter value.
            if (!validInput) {
                self.properties.value = '';
            }
        }

        oj.Logger.info('******** withinRange, input is valid?', paramsObj.normalized, validInput);

        paramsObj.normalized = paramsObj.normalized.length > 0 ? paramsObj.math.bignumber(paramsObj.normalized) : '0';

        // normalized the increment / decrement buttons
        self._enableButton(self.decButton);
        self._enableButton(self.incButton);

        // **************************************************************
        // now check for adjustments due to max / min range constraints
        // **************************************************************
        return self._maxMinRangeAdjustments(paramsObj);
    };

    luxInputBigNumber.prototype._maxMinRangeAdjustments = function(paramsObj) {
        var self = this,
            ok = true,
            newValue,
            params = paramsObj,
            updateValueProperty = function(value, button) {
                // set the value property to 0 so the next update fires
                self._setValueProperty(self._formatValue('0'), false);
                // set the value property to the max or min of the range.
                self._setValueProperty(self._formatValue(self._checkRange(value)), false);
                self._disableButton(button);
                self.rawValue = null;
                ok = false;
            };

        if (self.properties.min && params.direction === CONSTANTS.STEP_DOWN) {
            newValue = params.math.subtract(params.normalized, params.step);
            if ((params.math.smallerEq(newValue, params.min))) {
                updateValueProperty(params.min, self.decButton);
            }
        }

        if (self.properties.max && params.direction === CONSTANTS.STEP_UP) {
            newValue = params.math.add(params.normalized, params.step);
            if ((params.math.larger(newValue, params.max))) {
                updateValueProperty(params.max, self.incButton);
            }
        }

        if (params.direction === CONSTANTS.STEP_UP && self.properties.min) {
            if ((params.math.smaller(params.normalized, params.min))) {
                updateValueProperty(params.min, self.decButton);
            }
        }

        if (params.direction === CONSTANTS.STEP_DOWN && self.properties.max) {
            if ((params.math.larger(params.normalized, params.max))) {
                updateValueProperty(params.max, self.incButton);
            }
        }

        return ok;
    };

    /**
     * @description Updates the value property for the latest step.
     * @param {string} value - the new step value.
     * @param {boolean} allowNextUpdate - sets global flag to the value update listener that will allow or not allow
     *                                    the next update ot the value property.
     * @private
     * @returns {*}
     */
    luxInputBigNumber.prototype._setValueProperty = function(value, allowNextUpdate) {
        var self = this;

        self.properties.value = value;

        // set the global valueUpdated flag to false so next update to properties.value happens.
        if (!(_.isUndefined(allowNextUpdate))) {
            self.valueUpdated = allowNextUpdate;
        }
    };

    /**
     * @description Does the actual math to step the value (increment or decrement). Uses luxBigNumber (a wrapper for mathjs)
     * to set the value to a mathjs bignumber, subtract or add the step value and return the formatted per oi18nResources
     * locale elements.
     * @param  {string} direction  - the step direction, increment or decrement.
     * @param  {string} currentVal - copy of the properties.value.
     * @return {string}             - the new step value formatted per oi18nResources locale elements
     */
    luxInputBigNumber.prototype._stepNumber = function(direction, currentVal) {
        var self = this,
            math = self.bigNum,
            bigNumValue,
            precision, stepPrecision,
            normalized = (_.isString(currentVal) && currentVal.length === 0) ? null : currentVal,
            currentValue;

        if (currentVal && _.isString(currentVal)) {
            normalized = self._normalizeInput(currentVal);
        }

        if (self.rawValue) {
            // user entered value but is still in the input field
            normalized = self._normalizeInput(self.rawValue);
        }

        // if rawValue is an empty string the end user has blanked
        // out the field but not left the field.
        if (!_.isNull(self.rawValue) && self.rawValue.length === 0) {
            bigNumValue = self.properties.min ? self.properties.min : '0';
            bigNumValue = math.bignumber(self._normalizeInput(bigNumValue));
            self.rawValue = null;
            normalized = self._normalizeInput(bigNumValue.toString());
            return self._formatValue(normalized, NumberUtils.getPrecision(normalized));
        }

        self.rawValue = null;

        currentValue = normalized ? math.bignumber(normalized) : math.bignumber('0');

        if (direction === CONSTANTS.STEP_DOWN) {
            currentValue = math.subtract(currentValue, math.bignumber(self.properties.step));
        }

        if (direction === CONSTANTS.STEP_UP) {
            currentValue = math.add(currentValue, math.bignumber(self.properties.step));
        }

        precision = NumberUtils.getPrecision(normalized);
        stepPrecision = NumberUtils.getPrecision(self.properties.step);
        if (stepPrecision > precision) {
            precision = NumberUtils.getPrecision(self.properties.step);
        }

        return self._formatValue(currentValue, precision);
    };

    /**
     *    Determines if converter style of "percent" and
     *    and multiplies the range boundary by 100 to get
     *    actual percent value that is used to set the
     *    boundary.
     *    @param  {object} value - a mathjs bignumber
     *    @return {object}  a mathjs bignumber
     */
    luxInputBigNumber.prototype._checkRange = function(value) {
        var self = this, math = self.bigNum;

        if (self._converterProps.style === 'percent') {
            return math.multiply(value, 100);
        }
        return value;
    };

    /**
     * @description Sets the oj-disabled class to the button
     * @param  {object} button - the element
     * @returns {undefined} nothing
     */
    luxInputBigNumber.prototype._disableButton = function(button) {
        $(button).addClass('oj-disabled');
    };

    /**
     * @description Sets the oj-enabled class to the button
     * @param  {object} button - the element
     * @returns {undefined} nothing
     */
    luxInputBigNumber.prototype._enableButton = function(button) {
        $(button).removeClass('oj-disabled');
    };

    /**
     * @description Checks a button for the oj-disabled class
     * @param  {object} event - the currentTarget id
     * @return {boolean} if the oj-disabled class exists returns true otherwise false
     */
    luxInputBigNumber.prototype._buttonIsDisabled = function(event) {
        var buttonId = event.currentTarget.id,
            buttonElement = document.getElementById(buttonId);

        return buttonElement.classList.contains('oj-disabled');
    };

    /**
     * @description Formats Mathjs values.  The minimumFractionDigits and maximumFractionalDigits described in the
     * converter are used to determine the actual fractional digit. If actual precision is greater than the
     * maximumFractionalDigits, the maximumFractionalDigits is used. This will round the entered fractionalDigit. If
     * actual precision is less than the minimumFractionDigits the minimumFractionDigits is used. This will append
     * a 0 the fractionalDigit.
     *
     * @param  {object} value a bignumber object of Mathjs
     * @param {number} factionalDigits - precision after the decimal point including zero padding.
     * @return {string}       formatted value as a string
     */
    luxInputBigNumber.prototype._formatValue = function(value, factionalDigits) {
        var self = this,
            actualPrecision = self._actualPrecision(value, factionalDigits),
            math = self.bigNum,
            mathMaximumFractionDigits = self._converterProps.maximumFractionDigits,
            mathMinimumFractionDigits = self._converterProps.minimumFractionDigits,
            // determine the fraction digit sized and if there is one.
            precision = actualPrecision || self._converterProps.minimumFractionDigits || 0,
            formattedValue;

        // if maximumFractionalDigits are less than minimumFractionalDigits
        // for purpose of determining the rounding precision, set the maximumFractionalDigits
        // to maximumFractionalDigits.
        if (mathMinimumFractionDigits > mathMaximumFractionDigits) {
            mathMaximumFractionDigits = mathMinimumFractionDigits;
        }

        // check decimal precision.
        if (actualPrecision > mathMaximumFractionDigits) {
            // If actual precision is greater than the maximumFractionalDigits force the maximumFractionalDigits.
            // This will round the entered fractionalDigit.
            precision = mathMaximumFractionDigits;
        } else if (actualPrecision < mathMinimumFractionDigits) {
            // If actual precision is less than the minimumFractionDigits force use the minimumFractionDigits.
            // This will append a 0 the fractionalDigit.
            precision = mathMinimumFractionDigits;
        }

        if (_.isNull(value) || _.isUndefined(value) || (_.isString(value) && value.length === 0)) {
            value = '0';
        }

        formattedValue = math.format(math.bignumber(value), {'notation': 'fixed', 'precision': precision});

        return self.properties.converter ?
            self.properties.converter.format(formattedValue, oi18nResources.getOi18nLocaleElements()) :
            formattedValue;
    };

    /**
     * Function to get the factional digits.
     * @param {string | object} value - either a simple string or a Bignumber object
     * @param {number | null} factionalDigits - fractional digits from existing value that
     *                                          spin up or down.
     * @returns {undefined}
     */
    luxInputBigNumber.prototype._actualPrecision = function(value, factionalDigits) {
        var self = this,
            actualPrecision = 0,
            //digits,
            stringValue,
            //digitArray,
            math = self.bigNum;

        if (factionalDigits && !_.isUndefined(factionalDigits)) {
            actualPrecision = factionalDigits;
        } else if (_.isString(value) && value.length > 0) {
            actualPrecision = value.indexOf('.') > -1 ?
                (value.length - value.indexOf('.') - 1) : 0;
        } else if (_.isObject(value) && math.typeof(value) === 'BigNumber') {
            stringValue = value.toString();
            // **********************************************
            // mathjs bignumber toString() returns with
            // exponential notation when the number of
            // digits in the number string is 21 or greater.
            // **********************************************
            // get the exponential notation amount

            stringValue = NumberUtils.getNumberFromExponent(stringValue);
            actualPrecision = NumberUtils.getPrecision(stringValue);
        }

        return actualPrecision;
    };

    /**
     * @description Returns a filtered properties object that match the meta data properties from
     * the component.json file.
     * @param  {object} properties     properties object for the CCA context
     * @param  {object} metaProperties properties meta data
     * @return {object}                filtered properties.
     */
    luxInputBigNumber.prototype._getActiveProperties = function (properties, metaProperties) {
        var props = {};

        for (var key in metaProperties) {
            if (metaProperties.hasOwnProperty(key)) {
                // do not include CCA
                if (!(key === '_BRIDGE' || key === '_ELEMENT')) {
                    props[key] = properties[key];
                }
            }
        }

        return props;
    };

    /**
     * @description Sets the state (enabled / disabled) of the step buttons based on the value in relation
     * to the max / min range.  If no max / min, just returns. If the value as a string is empty
     * (length is zero) return.
     * @param  {string} value - current value of the oj-lux-input-bignumber
     * @return {undefined}
     */
    luxInputBigNumber.prototype._setButtonStates = function(value) {
        var self = this,
            math = self.bigNum,
            min = self.properties.min !== undefined ? math.bignumber(self._normalizeInput(self.properties.min)) : null,
            max = self.properties.max !== undefined ? math.bignumber(self._normalizeInput(self.properties.max)) : null,
            stringValue = _.isString(value) ? value : value ? value.toString() : value,
            normalized = self._normalizeInput(stringValue);

        if (!self.properties.min && !self.properties.max) {
            return;
        }

        if (normalized && normalized.length === 0) {
            if (_.isString(value)) {
                if (value.length > 0){
                    // Indicates entered all non numeric characters
                    // so need to disable the step buttons
                    self._disableButton(self.decButton);
                    self._disableButton(self.incButton);
                }
                if (value.length === 0) {
                    // Indicates the input is empty so
                    // enable the step buttons ***
                    // if there is a min, disable the
                    // spin down button.
                    if (self.properties.min) {
                        self._disableButton(self.decButton);
                        self._enableButton(self.incButton);
                    } else {
                        // otherwise enable both
                        self._enableButton(self.decButton);
                        self._enableButton(self.incButton);
                    }
                }
            }
            return;
        }

        normalized = normalized ? math.bignumber(normalized) : math.bignumber('0');

        self._enableButton(self.decButton);
        self._enableButton(self.incButton);

        if (self.properties.min) {
            if (value.length === 0) {
                self._disableButton(self.decButton);
                return;
            }
            if ((math.smallerEq(normalized, min))) {
                self._disableButton(self.decButton);
                return;
            }
        }

        if (self.properties.max) {
            // uses the decimal.js gte function
            // math.largerEq(normalized, max) not correct
            // all the time.
            if (normalized.gte(max)) {
                self._disableButton(self.incButton);
            }
        }
    };

    // The user can clear out min/max by setting the option to null, so we
    // do not coerce null.
    /**
     * @description NOTE: Copied from ojInputNumber and modified.
     * @param {string} option name of the option. this will show up in the error if thrown
     * @param {string} val to parse
     * @returns {number}
     * @throws {Error} if option value is invalid
     */
    luxInputBigNumber.prototype._parse = function(option, val) {
        var self = this,
            math = self.bigNum,
            returnValue = val,
            msg = 'oj-lux-input-bignumber\'s ' + option + ' option is not a string representing a number';

        try {
            math.bignumber(self._normalizeInput(val));
        } catch (e) {
            oj.Logger.error(msg);
            returnValue = undefined;
        }

        return returnValue;
    };

    /**
     * @description NOTE: Copied from ojInputNumber and modified.
     * parse the step's value
     * @throws {Error} if option value is invalid
     * @private
     * @param {string} val - step value as a string
     * @returns {string}
     */
    luxInputBigNumber.prototype._parseStep = function(val) {
        var defaultStep = CONSTANTS.DEFAULT_STEP,
            math = this.bigNum,
            parsedValue = this._parse('step', val),
            mathValue;

        if (val === null) {
            return defaultStep;
        }

        if (!parsedValue) {
            return defaultStep;
        }

        mathValue = math.bignumber(parsedValue);
        if (math.smallerEq(mathValue, 0)) {
            oj.Logger.warn('default value of "1" is set');
            return defaultStep;
        }

        return parsedValue;
    };

    /**
     *    Creates uniqueId
     *    @param  {String} idPrefix prefix to attach the unique id.
     *    @return {String}          Generated Id
     */
    luxInputBigNumber.prototype._uniqueId = function(idPrefix) {
        var self = this;

        return (idPrefix + self.uniqueNumber);
    };

    return luxInputBigNumber;
});
