/**
 * @class grid
 * @example <caption>Usage of Grid Widget</caption>
 * // calling method on grid element directly in the view
 * this._$grid.grid('repopulate');
 *
 * // calling method on gridWidget property in the view
 * this._gridWidget.repopulate();
 *
 */
define([
    'gridapi.jquery',
    'gridapi.lodash',
    'gridapi.moment',
    'gridapi.namespace',
    'gridapi.grid/gridWidgetOptions',
    'gridapi.grid/gridView',
    'gridapi.grid/data/attrFns',
    'gridapi.grid/view/ViewManager',
    'gridapi.grid/boilerplateArrayMediator',
    'gridapi.grid/boilerplateTreeMediator',
    'gridapi.grid/sorters',
    'gridapi.grid/data/ColumnMatcher',
    'gridapi.grid/groupers',
    'gridapi.grid/recordGrouper',
    'gridapi.grid/dataCache',
    'gridapi.grid/dataRollup',
    'gridapi.grid/rollups',
    'gridapi.grid/columnFilter',
    'gridapi.i18n/localizedStrings',
    'gridapi.grid/constants',
    'gridapi.logger',
    'gridapi.grid/utils',
    'gridapi.timsort',
    'gridapi.grid/utils/strings',
    'gridapi.grid/utils/generateSearchableColumns',
    'gridapi.grid/utils/searchTermRegExpBuilder',
    'gridapi.jqueryui',
    'gridapi.grid/config/themeConfigurator',
    'gridapi.grid/configuration'
], function ($, _, moment, widgetNamespace, gridWidgetOptions, GridView,
             attrFns, ViewManager, DefaultMediator, DefaultTreeMediator, sorters,
             ColumnMatcher, groupers, RecordGrouper, DataCache, DataRollup,
             rollups, ColumnFilterCtr, locale, constants, logger,
             utils, timsort, strings, generateSearchableColumns, searchBuilder) {
    'use strict';

    //>>excludeStart("demoDebugOnly", pragmas.compiled)
    window.callCounts = {};
    window.clearCallCount = function () {
        window.callCounts = {};
    };
    window.incrementCallCount = function (key) {
        if (window.callCounts[key]) {
            window.callCounts[key]++;
        } else {
            window.callCounts[key] = 1;
        }
    };
    window.printCallCounts = function (mod) {
        mod = mod || 1.5;
        var timeTakenModifier = mod / 1000;
        var output = '';
        _.forEach(window.callCounts, function (count, key) {
            if (output) {
                output += '\n';
            }

            output += [key, 'was invoked', count, 'times, which is like', count * timeTakenModifier, 'seconds (assuming', mod + 'ms', 'per call)'].join(' ');
        });
        logger.info('\n' + output + '\n');
    };
    //>>excludeEnd("demoDebugOnly")

    /**
     * @typedef grid#viewConfig
     *
     * @property {Array<grid#columnGroup>} [columnGroups]
     * @property {Object} [columns]
     * @property {Object} [theme]
     * @property {String} [theme.colors='GREY']
     * @property {String} [theme.style='MODERN_NOTCHES']
     * @property {Object} [timephased]
     * @property {Number} [timephased.bucketWidth=100]
     * @property {timestamp} [timephased.end]
     * @property {String} [timephased.scale='MONTH_WEEK']
     * @property {timestamp} [timephased.start]
     * @property {String} [timephased.usePeriodLabels=false]
     * @property {Array} [sortBy]
     * @property {Object} [sorters] All overriding sorting for different column datatypes. Set sorters.string : function (valueA, valueB, asc)
     * @property {Array} [groupBy]
     * @property {Object} [groupingOptions]
     * @property {Boolean} [groupingOptions.showColumnTitle=false]
     * @property {Boolean} [groupingOptions.enableRollup=true]
     */

    //>>excludeStart("logging", pragmas.production)
    logger.debug('gridWidget | loaded');
    //>>excludeEnd("logging")

    /**
     * Data Related Actions: Cut, Copy, Paste, Delete, New    (Is this only for toolbar)
     * Store Related : Sort, Filter, Page, Load, Save, Cancel, Update, Undo, Redo, Load Children,
     * View Related : Resize, Select, ColumnToggle, Column Moved, Row Moved, Group
     *
     * Callable: Find, Find Next, Find Prev
     *
     *
     */
    var digitCheck = /^-?\d/;
    var UTILS = {
        determineRowsetPercents : function (rowsets) {
            var setPercentage = 0;
            var alreadySet = 0;

            var flexRowsets = _.filter(rowsets, function (rowset) {
                return rowset.type === 'flex';
            });
            _.forEach(flexRowsets, function (rowset) {
                if (rowset.percent != null) {
                    rowset.percent = parseInt(rowset.percent, 10);
                    setPercentage += rowset.percent;
                    alreadySet++;
                }
            });
            if (flexRowsets.length > alreadySet) {
                var percentPerRowset = (100 - setPercentage) / flexRowsets.length - alreadySet;
                _.forEach(flexRowsets, function (rowset) {
                    if (rowset.percent == null) {
                        rowset.percent = percentPerRowset;
                    }
                });
            }
        }
    };
    var FOR_SEARCH_ARG = { forSearch : true };
    var SKIP_FOCUS_ARG = { skipFocus : true };
    var FROM_EMPTY_TERM_ARG = { fromEmptyTerm : true };
    var FORCE_CLEAR_ARG = { force : true };

    //>>excludeStart("validation", pragmas.production)
    var hasWarned = false;
    //>>excludeEnd("validation")
    function genSearchFnFactory(column, value, args) {
        //>>excludeStart("validation", pragmas.production)
        if (!hasWarned) {
            logger.warn('column.generateSearchFn is deprecated - prefer defining column.getSearchValue, which should return the string representation of the column');
            hasWarned = true;
        }
        //>>excludeEnd("validation")
        return function (term, searchMatcher) {
            try {
                return column.generateSearchFn(term, searchMatcher, column, args)(value);
            } catch (e) {
                return false;
            }
        };
    }

    function bulkModificationOperationDecorator(fn) {
        return function bulkModificationOperationDecoratorInner() {
            this._preBulkModifyOperationProcessor();
            var result = fn.apply(this, arguments);
            this._postBulkModifyOperationProcessor();
            return result;
        };
    }
    function addBulkOperationMethods(grid) {
        var swapInvokeStates;
        function makeRollupHandler(swapInvokeKey, apiFn) {
            var precedence = 0;
            return {
                api : apiFn,
                fnName : 'rollupFromLeafs',
                cacheName : swapInvokeKey,
                replacement : function (leafIds, dataIndices, rowsetId, args) {
                    var leafCalls = swapInvokeStates[swapInvokeKey];
                    var callsKey = rowsetId + '.' + String(dataIndices);
                    var callsOfRowset = leafCalls[callsKey];
                    if (!callsOfRowset) {
                        leafCalls[callsKey] = callsOfRowset = {
                            rowsetId : rowsetId,
                            precedence : precedence++,
                            leafs : leafIds,
                            dataIndices : dataIndices,
                            args : args
                        };
                    } else {
                        callsOfRowset.leafs = callsOfRowset.leafs.concat(leafIds);
                    }
                },
                handlePost : function () {
                    var leafCalls = swapInvokeStates[swapInvokeKey];
                    _.forEach(_.sortBy(leafCalls, _.property('precedence')), function (call) {
                        apiFn().rollupFromLeafs(call.leafs, call.dataIndices, call.rowsetId, call.args);
                    });
                }
            };
        }
        var swapHandlers = [
            {
                api : function () { return grid.store; }, // not _.constant since setStore is done after init
                fnName : 'checkRecordUniqueness',
                replacement : function (dataIndex, record) {
                    if (!grid.store.checkUniquenessStyle) {
                        var original = swapInvokeStates.originals.checkRecordUniqueness;
                        if (original) {
                            return original.apply(this, arguments);
                        }
                    }

                    var uniqueProcs = swapInvokeStates.checkRecordUniqueness;
                    uniqueProcs.push({ record : record, dataIndex : dataIndex });
                },
                handlePost : function () {
                    if (grid.store.checkUniquenessStyle) {
                        var uniqueProcs = swapInvokeStates.checkRecordUniqueness;
                        var indices = _.uniq(_.pluck(uniqueProcs, 'dataIndex'));
                        var records = _.uniq(_.pluck(uniqueProcs, 'record'));
                        grid.store[grid.store.checkUniquenessStyle](records, indices);
                    }
                }
            },
            makeRollupHandler('groupedRollup', function () { return grid.recordGrouper; }),
            makeRollupHandler('treeRollup', function () { return grid.dataRollup; }),
            {
                api : function () { return grid; },
                fnName : '_rollupRowsetTotal',
                replacement : function (group) {
                    swapInvokeStates.rollupRowsetTotal.push(group);
                },
                handlePost : function () {
                    _.forEach(_.uniq(swapInvokeStates.rollupRowsetTotal), function (groupId) {
                        var rowsetId = grid.recordGrouper.recordGroups[groupId].rowsetId;
                        grid.recordGrouper.rollupFromLeafs([groupId], null, rowsetId);
                    });
                }
            }, {
                api : function () { return grid.view; },
                fnName : 'renderRowset',
                replacement : function (rowsetId, args) {
                    var seenRowsets = swapInvokeStates.renderRowset;
                    if (!_.includes(seenRowsets, rowsetId)) {
                        seenRowsets.push(rowsetId);
                    }
                    if (args) {
                        swapInvokeStates.renderRowsetFlags[rowsetId] = args;
                    }
                },
                handlePost : function () {
                    var seenRowsets = swapInvokeStates.renderRowset;
                    _.forEach(seenRowsets, function (rowset) {
                        var flag = !!swapInvokeStates.renderRowsetFlags[rowset];
                        grid.view.renderRowset(rowset, flag);
                    });
                }
            }, {
                api : function () { return grid.view; },
                fnName : 'updateRowsetScrollbar',
                replacement : function (rowsetId) {
                    var seenRowsets = swapInvokeStates.updateRowsetScrollbar;
                    if (!_.includes(seenRowsets, rowsetId)) {
                        seenRowsets.push(rowsetId);
                    }
                },
                handlePost : function () {
                    var seenRowsets = swapInvokeStates.updateRowsetScrollbar;
                    _.forEach(seenRowsets, function (rowset) {
                        grid.view.updateRowsetScrollbar(rowset);
                    });
                }
            }, {
                api : function () { return grid.view; },
                fnName : 'reselectRecords',
                replacement : function (args) {
                    if (args && args.skipFocus) {
                        swapInvokeStates.reselectRecordsSkipFocus = true;
                    } else {
                        swapInvokeStates.reselectRecords = true;
                    }
                },
                handlePost : function () {
                    if (swapInvokeStates.reselectRecords) {
                        grid.view.reselectRecords();
                    } else if (swapInvokeStates.reselectRecordsSkipFocus) {
                        grid.view.reselectRecords(SKIP_FOCUS_ARG);
                    }
                }
            }, {
                api : function () { return grid.view; },
                fnName : 'resize',
                replacement : function () {
                    swapInvokeStates.resize = true;
                },
                handlePost : function () {
                    if (swapInvokeStates.resize) {
                        grid.view.resize();
                    }
                }
            }
        ];
        function swap(handler) {
            var api = handler.api();
            swapInvokeStates.originals[handler.cacheName || handler.fnName] = api[handler.fnName];
            api[handler.fnName] = handler.replacement;
        }
        function restore(handler) {
            handler.api()[handler.fnName] = swapInvokeStates.originals[handler.cacheName || handler.fnName];
        }

        grid._preBulkModifyOperationProcessor = function _preBulkModifyOperationProcessor() {
            if (grid.__bulkOperationFlag) {
                return;
            }

            grid.__bulkOperationFlag = true;
            grid._trigger('pre-bulk-add-operation');
            swapInvokeStates = {
                renderRowset : [],
                renderRowsetFlags : {},
                checkRecordUniqueness : [],
                rollupRowsetTotal : [],
                groupedRollup : {},
                treeRollup : {},
                originals : {},
                updateRowsetScrollbar : []
            };
            _.forEach(swapHandlers, function (handler) {
                swap(handler);
            });
        };
        grid._postBulkModifyOperationProcessor = function _postBulkModifyOperationProcessor() {
            _.forEach(swapHandlers, function (handler) {
                restore(handler);
                handler.handlePost();
            });
            grid._trigger('post-bulk-add-operation');
            grid.__bulkOperationFlag = false;
        };

        /**
         * Allows processing bulk operations in the most efficient manner possible.
         *
         * @method grid#processBulkOperation
         * @param wrapped {Function} a method to be invoked, as `wrapped.apply(null, args)` for processing some operation
         * @param args {Array} the args to be passed to apply
         * @public
         */
        grid.processBulkOperation = function processBulkOperation(wrapped, args) {
            grid._preBulkModifyOperationProcessor();
            var result = wrapped.apply(null, args);
            grid._postBulkModifyOperationProcessor();
            return result;
        };
    }

    /**
     * @typedef grid#defaultRecordAttributes
     *
     * @property {Boolean} [newRow=false] Whether or not the record is new, and was recently added to the grid.
     * @property {Array} [modified] The modified fields on the record.
     * @property {Array} [readOnly] The readonly fields on the record.
     * @property {Boolean} [isNode=false] Determine if the record can have children added to it.
     * @property {Boolean} [hasChildren=false] Whether the record node, has children nodes.  Only applicable to Tree Grids.
     * @property {Boolean} [editable=true] Determines if the user can edit the record.
     * @property {Boolean} [expanded=false] Whether the record is expanded or collapsed.  Only applicable to Tree Grids.
     * @property {Boolean} [sortable=true] Determines if the record can be sorted when the user clicks on column headers.
     * @property {Boolean} [invalid=null] Reports errors for the grid to render on the cells.
     * @property {Boolean} [childrenLoaded=false] Whether the child records have been loaded for this record or not, used in lazy loaded tree grids.
     * @property {Array<String>} [locked] The fields in the record that should display the locked icon.
     * @property {Boolean} [draggable=true] Turn on letting the rows be dragged around. Unused in prime so far.
     * @property {Number} [precedence] In a tree grid this will determine the order of the children. Higher precedence go first, all of the same precedence will
     *     group together.
     * @property {Boolean} [disableRowMenu=false] Disable the cogwheel from showing for the record.
     */
    var defaultRecordAttributes = function () {
        return {
            newRow : false,
            modified : [],
            readOnly : [],
            isNode : false,
            hasChildren : false,
            editable : true,
            expanded : false,
            sortable : true,
            invalid : null,
            childrenLoaded : false,
            locked : [],
            draggable : true,
            precedence : 1,
            disableRowMenu : false,
            rollup : false
        };
    };

    var defaultRecordTreeAttributes = function () {
        return {
            sortable : true,
            isNode : false,
            hasChildren : false,
            childrenLoaded : false,
            precedence : 1,
            rollup : false,
            expanded : false
            // canHaveChildren : undefined
        };
    };

    // default sort attrs : { sortable : true, precedence : 1 }

    /**
     * @typedef grid#defaultTimephasedAttributes
     *
     * @property {Array<String>} [readOnlyStacked] Contains stacked cell data indexes that should be readonly.  These values correspond directly to the record data
     *     indexes.
     * @property {Array<String>} [readOnlyBuckets] Contains bucket ids that correspond to individual buckets that should be readonly.
     * @property {Object} [readOnlyMatrix] Used for setting individual readonly cells in a timephase column group.  Grid will use both arrays to find only cells
     *     that exist in a cross section of stacked cells and buckets.<br/>Use with caution and only in complex readonly situations.
     * @property {Array<String>} [readOnlyMatrix.bySubRow] Contains stacked cell data indexes that should be read only.
     * @property {Array<String>} [readOnlyMatrix.byBucket] Contains bucket ids that correspond to buckets that should be readonly.
     * @property {Object} [modified] The modified values on the record.
     */
    var defaultTimephasedAttributes = function () {
        return {
            readOnlyStacked : [],
            readOnlyBuckets : [],
            lockedBuckets : [],
            lockedStacked : [],
            readOnlyMatrix : {
                bySubRow : [],
                byBucket : []
            },
            modified : {}
        };
    };

    /**
     * @typedef grid#defaultStackedCellAttributes
     *
     * @property {Object} [modified] The modified values on the record.
     * @property {Array<String>} [readOnlyStacked] Contains stacked cell data indexes that should be readonly.  These values correspond directly to the record data
     *     indexes.
     * @property {Array<String>} [readOnlyColumns] Contains column data indexes from a stacked cell column group, that should be readonly.
     * @property {Object} [readOnlyMatrix} Used for setting individual readonly cells in a stacked cell column group.  Grid will use both arrays to find only cells
     *     that exist in a cross section of stacked cells and columns.<br/>Use with caution and only in complex readonly situations.
     * @property {Array<String>} [readOnlyMatrix.bySubRow] Contains stacked cell data indexes that should be readonly.  These values correspond directly to the
     *     record data indexes.
     * @property {Array<String>} [readOnlyMatrix.byColumn] Contains column data indexes from a stacked cell column group, that should be readonly.
     */
    var defaultStackedCellAttributes = function () {
        return {
            modified : {},
            readOnlyStacked : [],
            readOnlyColumns : [],
            lockedStacked : [],
            lockedColumns : [],
            readOnlyMatrix : {
                bySubRow : [],
                byColumn : []
            }
        };
    };

    var grid = {

        options : gridWidgetOptions,

        version : '@VERSION',

        _updateOldOptions : function () {
            if (this.options.storeFns != null) {
                logger.warn('options.storeFns is deprecated, please use options.mediatorFns');
                this.options.mediatorFns = this.options.storeFns;
            }
            if (this.options.columns.groups != null) {
                logger.warn('options.columns.groups is deprecated, please use options.layout.columnGroups and options.columns');
                var newColumnsConfig = {};
                var newGroupConfig = [];
                _.forEach(this.options.columns.groups, function (columnGroup) {
                    var columns = _.clone(columnGroup.columns);
                    var group = _.clone(columnGroup);
                    var newGroupColumns = [];
                    _.forEach(columns, function (column) {
                        newColumnsConfig[column.dataIndex] = column;
                        newGroupColumns.push(column.dataIndex);
                    });
                    group.columns = newGroupColumns;
                    newGroupConfig.push(group);
                });
                this.options.layout.columnGroups = newGroupConfig;
                this.options.columns = newColumnsConfig;
            }
            _.forEach(this.options.layout.columnGroups, function (columnGroup) {
                var duplicates;
                if (columnGroup.columns.length !== _.uniq(columnGroup.columns).length) {
                    duplicates = _(columnGroup.columns)
                            .groupBy()
                            .map(function (value, key) {
                                if (value.length > 1) {
                                    return key;
                                }

                                return null;
                            })
                            .compact()
                            .value();

                    logger.warn('options.layout.columns includes duplicate columns. Stripping: ' + duplicates.join(', '));
                    columnGroup.columns = _.uniq(columnGroup.columns);
                }
            });

            if (_.isArray(this.options.columns)) {
                var columnObject = {};
                _.forEach(this.options.columns, function (column) {
                    columnObject[column.dataIndex] = column;
                });

                this.options.columns = columnObject;
                logger.warn('options.columns should be a dataIndex keyed object, for convience the given array has been converted');
            }

            if (this.options.layout.waffers != null) {
                logger.warn('options.layout.waffers is deprecated, please use options.layout.rowsets ... happy now :) ?');
                this.options.layout.rowsets = this.options.layout.waffers;
            }

            if (this.options.cellSelectedClass != null) {
                this.options.classes.cellSelected = this.options.cellSelectedClass;
                this.options.classes.cellSelected = this.options.cellSelectedClass;
                logger.warn('options.cellSelectedClass is deprecated move to options.classes.cellSelected');
            }
            if (this.options.rowSelectedClass != null) {
                this.options.classes.rowSelected = this.options.rowSelectedClass;
                logger.warn('options.rowSelectedClass is deprecated move to options.classes.rowSelected');
            }

            if (this.options.enableQuickAddRow) {
                this.options.quickAdd.enabled = this.options.enableQuickAddRow;
                logger.warn('options.enableQuickAddRow is deprecated move to options.quickAdd.enabled');
            }
            if (this.options.search.columnFilter) { // LUX-506 want to preserve the focus to current columnFilter field
                this.options.selectRowByDefault = false;
            }
            if (this.options.stackedCells.enabled) {
                var maxStackedCells = 0;
                for (var i = 0; i < this.options.layout.columnGroups.length; i++) {
                    var columnGroup = this.options.layout.columnGroups[i];
                    if (columnGroup.stackedCells != null) {
                        maxStackedCells = Math.max(maxStackedCells, columnGroup.stackedCells.length);
                    }
                }
                if (maxStackedCells > 1) {
                    this.options.borders.row = 2;
                    if (this.options.classes.grid.indexOf('timephased-grid') === -1) {
                        this.options.classes.grid.push('timephased-grid');
                    }
                }
            }
            if (this.options.data) {
                this.updateDataOptions();
            }
            if (this.options.timephased.enabled && !this.options.timephased.supportsTimephased) {
                logger.warn('options.timephased.supportsTimephased should be set to true for timephased grids');
            }
        },

        updateDataOptions : function () {
            var baseTypes = this.options.data.baseTypes;
            var types = this.options.data.types;
            var typeKeys = _.keys(types);
            var currentType = null;
            var currentTypeKey = null;

            function isChainedGroup(currentType, typeKey) {
                if (_.intersection(currentType.childTypes, baseTypes).length) {
                    return true;
                } else {
                    var subChildTypes = _.without(currentType.childTypes, typeKey);
                    if (subChildTypes.length === 0) {
                        return false;
                    } else {
                        var isChained = false;
                        for (var i = 0; i < subChildTypes.length; i++) {
                            var nextTypeKey = subChildTypes[i];
                            isChained = isChainedGroup(types[nextTypeKey], nextTypeKey);
                            if (isChained) {
                                break;
                            }
                        }
                        return isChained;
                    }
                }
            }

            for (var i = 0; i < typeKeys.length; i++) {
                currentTypeKey = typeKeys[i];
                currentType = types[currentTypeKey];
                currentType.chainedGroup = isChainedGroup(currentType, currentTypeKey);
            }
        },

        _create : function () {
            var that = this;
            this.id = this.uuid + 'grid';
            this._updateOldOptions();
            this._debouncedFns = [];
            var $window = $(window);
            if (this.options.layout.autoResize) {
                var autoResizeDebounce = _.debounce(function () {
                    that.resize();
                }, that.options.layout.autoResizeDebounce, that.options.layout.autoResizeImmediate);
                this._debouncedFns.push(autoResizeDebounce);
                $window.on('resize' + this.eventNamespace + '-main', autoResizeDebounce);
            }
            $window.on('unload' + that.eventNamespace + '-tear-down', function () {
                that.destroy();
                $window.off('unload' + that.eventNamespace + '-tear-down');
            });
            if (this.options.export.enabled && !('download' in document.createElement('a')) && !window.navigator.msSaveOrOpenBlob) {
                    // forcibly disable download in environments that don't support Blob download [PRIMCR-3855]
                this.options.export.enabled = false;
            }
            this.widgetEventPrefix = 'grid-';
            this.config = {
                rowsets : [],
                groups : [],
                contextMenu : this.options.contextMenu,
                sortBy : []
            };
            this.dataCache = new DataCache({
                widget : this
            });

            this._attrs = attrFns.fromGrid(this);

            this.columnFilterInstance = new ColumnFilterCtr({
                widget : this,
                options : this.options
            });

            this.view = new GridView(this);
            this.viewManager = new ViewManager(this);

            this.view.init();
            this._setSortConfig(this.options.sortBy);

            this.recordGrouper = new RecordGrouper({
                widget : this
            });
            this.recordGrouper.updateGroupConfig(this.options.groupBy);

            this.dataRollup = new DataRollup({
                widget : this,
                dataCache : this.dataCache
            });

            this.rolledupRowsets = {};

            this.setLoading(that.options.showLoading);
            this.view.setDisabled();
            this.selection = {
                records : [],
                recordIds : [],
                recordIdMap : {}
            };

            this.focus = {
                record : null,
                recordId : null,
                column : null
            };

            this.checkedRecords = [];
            this.checkedRecordIds = [];

            this.clipboard = {
                records : [],
                mode : 'cut'
            };
            this.recordKeys = this.options.recordKeys;

            this.rootNodes = {};
            this.groupingApplied = false;

            this.defaultRowset = _.filter(this.config.rowsets, function (rowset) {
                return rowset.type === 'flex';
            })[0].id;
            if (this.defaultRowset == null) {
                this.defaultRowset = this.config.rowsets[0].id;
            }
            this.undoStack = [];
            this.redoStack = [];

            this.searchMatcher = null;
            this.searchColumns = [];
            this.searchColumnIndices = [];
            this.matchingRecords = {
                recordOrder : []
            };

            this.columnMatcher = new ColumnMatcher(this);

            this.storeRecords = [];
            this.siblingIds = {};
            this.parentIds = {};
            this.selectionAttributes = {};
            this.recentlyRemoved = {};

            UTILS.determineRowsetPercents(this.config.rowsets);

            var triggerDirty = function () {
                /**
                 * Triggered when the grid enters a 'dirty' state (such as after an edit).
                 * This event is captured after certain action takes place on the grid.
                 *
                 * @event pgbu.grid#is-dirty
                 */
                that._trigger('is-dirty');
            };

            this.element.on('grid-cell-updated', triggerDirty);
            this.element.on('grid-rows-added', triggerDirty);
            this.element.on('grid-rows-deleted', triggerDirty);
            this.element.on('grid-rows-pasted', triggerDirty);
            this.view.resize();

            //>>excludeStart("validation", pragmas.production)
            if (widgetNamespace !== 'GBU') {
                window['grid' + this.uuid] = this;
                window.grid = this;
            }
            //>>excludeEnd("validation")

            this.syncedGrid = null;

            this.timeouts = {};
            this.rowSelectedTriggerArgs = {};

            this.debouncedTriggerRowSelected = _.debounce(
                    function () {
                        /**
                         * Triggered when a single row or multiple rows are selected.
                         * This event is captured after certain action takes place on the grid.
                         *
                         * @event pgbu.grid#rows-selected
                         * @param {Object} event
                         * @param {Object} triggerArgs
                         */
                        that._trigger('rows-selected', that.rowSelectedTriggerArgs.event, that.rowSelectedTriggerArgs.triggerArgs);
                    }, that.options.performance.selectDebounceInterval, that.options.performance.selectDebounceOptions);

            this.debouncedCheckActionState = _.debounce(function () {
                that.checkActionState();
            }, that.options.performance.actionStateInterval, that.options.performance.actionStateDebounceOptions);

            this._debouncedFns.push(this.debouncedTriggerRowSelected);
            this._debouncedFns.push(this.debouncedCheckActionState);
            addBulkOperationMethods(this);
        },

        _resetInternalStorage : function () {
            this.rootNodes = {};
            this.fakeChildren = {};
            this.groupingApplied = false;
            this.recordGrouper.resetGroupStorage();
            this.dataCache.resetDataCache();
            this._resetProxyRecord();
            this.undoStack = [];
            this.redoStack = [];

            this.searchMatcher = null;
            this.searchColumns = [];
            this.searchColumnIndices = [];
            this.matchingRecords = {
                recordOrder : []
            };

            this.storeRecords = [];
            this.siblingIds = {};
            this.parentIds = {};
            this.selectionAttributes = {};
            this.recentlyRemoved = {};
            this.checkedRecords = [];
            this.checkedRecordIds = [];
            this.view.clearOriginalSortOrder();
            this.view.resetVirtualRows();
            this.clipboard = {
                records : [],
                mode : 'cut'
            };

            this.rolledupRowsets = {};
        },

        _resetView : function () {
            var that = this;
            var oldEscapeMode = this.view.escapeMode;
            // Lots of view destruction and recreation
            var originalFocusId = this.focus.recordId;
            var originalColumnId = this.focus.columnId;
            this.config.groups = [];
            this.config.rowsets = [];
            this.view.closeEditor(true);
            this.view.closeMenus();
            this.view.$grid.remove();
            var virtualRows = _.clone(this.view.virtualRows);
            this.view.destroy();
            delete this.view;
            this.view = new GridView(this);
            this.view.init();
            // Only update the virtualRows for rowsets that exist still.
            _.forEach(virtualRows, function (rows, key) {
                if (that.view.virtualRows[key] != null) {
                    that.view.virtualRows[key] = rows;
                }
            });
            this.view.setEscapeMode(oldEscapeMode);
            UTILS.determineRowsetPercents(this.config.rowsets);
            this._addSortIcons();
            if (this.isLoaded()) {
                this.viewManager.queue.repopulate = true;
                this.viewManager.args.repopulate = {
                    record : originalFocusId,
                    dataIndex : originalColumnId
                };
            }
            this.viewManager.queue.resize = true;
        },

        _setOptions : function (/* options */) {
            $.Widget.prototype._setOptions.apply(this, arguments);
            this.viewManager.refresh();
        },

        /**
         * Used to update the value for a specific option in the grid.  Please refer to {@link grid#options} for more information on possible options.
         *
         * @method grid#_setOption
         * @private
         * @param {String} key - The index of the option to update.
         * @param {*} value - The value to give the option.
         * @example
         * // base level option update
         * gridWidget._setOption('filldown', true);
         *
         * // nested option update
         * gridWidget._setOption('stackedCells', {enabled: true});
         */
        _setOption : function (key, value) {
            var newOption = value;
            var that = this;
            if (_.isObject(this.options[key]) && !_.isArray(this.options[key])) {
                newOption = {};
                if (key === 'columns') {
                    var optionsToReset = ['selectValues'];
                    _.forEach(value, function (columnOptions, columnId) {
                        for (var i = 0; i < optionsToReset.length; i++) {
                            var optionToReset = optionsToReset[i];
                            if (columnOptions[optionToReset] != null && that.options[key] !== value && that.options[key][columnId] != null) {
                                that.options[key][columnId][optionToReset] = null;
                            }
                        }
                    });
                    var viewConfigColumns = this.getViewConfig().columns;
                    var prune = [];
                    _.forEach(viewConfigColumns, function (column, dataIndex) {
                        if (value[dataIndex] == null && that.options.columns[dataIndex] == null) {
                            prune.push(dataIndex);
                        }
                    });

                    var columnsToApply = _.omit(viewConfigColumns, prune);
                    _.merge(newOption, this.options[key], columnsToApply, value, function (a, b) {
                        if (_.isArray(a)) {
                            return b;
                        }
                    });
                } else {
                    _.merge(newOption, this.options[key], value, function (a, b) {
                        if (_.isArray(a)) {
                            return b;
                        }
                    });
                }
            }
            var oldProperty = this.options[key];

            $.Widget.prototype._setOption.apply(this, [key, newOption]);
            switch (key) {
            case 'layout' :
                this.view.setRowHeight();
                if (value.rowsets == null && value.columnGroups == null) {
                    this.viewManager.queue.resize = true;
                    break;
                }
                if (value.columnGroups != null) {
                    this.viewManager.queue.deleteColumnGroups = true;
                }
                this.viewManager.queue.resetView = true;
                this.viewManager.queue.applyViewConfig = true;
                break;
            case 'contextMenu':
                this.viewManager.queue.resetView = true;
                this.viewManager.queue.applyViewConfig = true;
                break;
            case 'theme':
                this.view.updateGroupBandClasses();
                break;
            case 'iconColumns':
                /* falls through */
            case 'columns':
                this.viewManager.queue.resetView = true;
                break;
            case 'grouping':
                if (this.isLoaded()) {
                    this.view.updateGroupBandClasses();
                    this._setGroupingOptions(oldProperty, newOption);
                }
                break;

            case 'rows':
                this.viewManager.queue.redrawRows = true;
                break;
            case 'sortBy':
                if (this.isLoaded()) {
                    this._updateSortConfig(this.options.sortBy);
                } else {
                    this._setSortConfig(this.options.sortBy);
                }
                break;
            case 'groupBy' :
                this._setGroupConfig(value);
                break;
            case 'showVerticalLines' :
                this.view.applyVerticalLines();
                break;
            case 'timephased' :
                var timephasedChanged = this.options.timephased.enabled !== oldProperty.enabled;
                if (this.options.timephased.enabled || timephasedChanged) {
                    var rangeChanged = value.range != null && (value.range.start != null || value.range.end != null);
                    if (rangeChanged || value.usePeriodLabels !== oldProperty.usePeriodLabels || timephasedChanged) {
                        this.viewManager.queue.resetView = true;
                        this.viewManager.queue.applyViewConfig = true;
                    } else {
                        this.viewManager.queue.setTimescaleView = this.options.timephased.enabled;
                    }
                    if (value.view != null && value.view.start != null) {
                        this.viewManager.queue.setTimescaleView = this.options.timephased.enabled;
                    }
                }
                if (timephasedChanged) {
                    this._syncTimephasedOptions(newOption.enabled, newOption.enabled);
                }
                break;
            case 'disabled' :
                if (oldProperty !== value) {
                    this.view.setDisabled();
                    this.viewManager.queue.redrawRows = true;
                }
                break;
            case 'data' :
                this._resetInternalStorage();
                this.viewManager.queue.repopulate = true;
                break;
            case 'editable':
                if (oldProperty !== value) {
                    this.viewManager.queue.repopulate = true;
                }
                break;
            case 'enableQuickAddRow':
                    /* falls through */
            case 'quickAdd':
                if (oldProperty !== value) {
                    if (key === 'quickAdd') {
                        // Clear selection if quickAdd record is no longer available
                        if (!this.options.quickAdd.enabled && this.focus.recordId === constants.DUMMY_RECORD_ID) {
                            this.focus.recordId = null;
                            this.focus.record = null;
                        }
                    }
                    this.viewManager.queue.resetView = true;
                    this.viewManager.queue.applyViewConfig = true;
                }
                break;
            case 'tree':
                if (oldProperty !== value) {
                    this._resetInternalStorage();
                    this.viewManager.queue.repopulate = true;
                }
                break;
            case 'headerOverlay':
                this.view.updateHeaderOverlay();
                break;
            case 'search': // LUX-506
                if (!_.isUndefined(value.columnFilterMatchAllColumns)) {
                    this._columnBasedFilter({ optionUpdate : true });
                }
                break;

            }
        },

            /**
             * Function to delete the grid from the DOM and remove all listeners.
             *
             * @method grid#destroy
             * @public
             */
        destroy : function () {
            //>>excludeStart("validation", pragmas.production)
            if (widgetNamespace !== 'GBU') {
                if (_.get(window, 'grid.uuid') === this.uuid) {
                    delete window.grid;
                }
                delete window['grid' + this.uuid];
            }
            //>>excludeEnd("validation")

            var $window = $(window);
            $window.off('unload' + this.eventNamespace + '-tear-down');
            $window.off('resize' + this.eventNamespace + '-main');

            // remove any editors and menus
            $window.off('unload' + this.eventNamespace + '-tear-down');
            $window.off('resize' + this.eventNamespace + '-main');
            this.view.destroy();
            _.forEach(this.timeouts, function (timeout) {
                clearTimeout(timeout);
            });
            _.forEach(this._debouncedFns, function (debouncedFn) {
                debouncedFn.cancel();
            });
            this.element.children().each(function () {
                $(this).remove();
            });

            this.element.empty();
            this._tearDownTriggers();
            // Does nice things like unbind all namespaced events on the original element

            if (this.store && this.store.destroy) {
                this.store.destroy();
            }
            delete this.store;
            delete this.view;
            $.Widget.prototype.destroy.call(this);
        },

        _tearDownTriggers : function () {
            this.element.off('grid-cell-updated');
            this.element.off('grid-rows-added');
            this.element.off('grid-rows-deleted');
            this.element.off('grid-rows-pasted');
            $(window).off('resize' + this.eventNamespace + '-main');
            this.element.off(this.widgetEventPrefix);
        },

        //* ***********************************************************
        // METHODS TO CALL FROM STORE
        //* ***********************************************************

        /**
         * Function used to inform the grid that a field has been updated.
         *
         * @method grid#_recordUpdated
         * @param {Object} record - The record that was updated.
         * @param {String} dataIndex - The index of the field that was updated.
         * @private
         * @example
         * // base level index updates
         * gridWidget._recordUpdated(record, 'foo');
         *
         * // nested index updates
         * gridWidget._recordUpdated(record, 'foo.bar);
         * @fires grid#event:cell-updated
         */
        _recordUpdated : function (record, dataIndex) {
            var recordId = this._getRecordId(record);
            var that = this;
            var rowsetId = this._getRecordPlacement(record).rowsetId;

            if (this.groupingApplied && !this.quickAddInProgress) {
                this.recordGrouper.rerollGroupValues({
                    record : record,
                    recordId : recordId,
                    dataIndex : this._getDataIndexToReroll(dataIndex),
                    rowsetId : rowsetId
                });
            } else if (this.options.dataRollup.enabled) {
                this.dataRollup.recordUpdated({
                    recordId : recordId,
                    dataIndex : this._getDataIndexToReroll(dataIndex)
                });
            }

            // If rolluped rowset recalc the rollup
            var rowsetRollupGroup = that.rolledupRowsets[rowsetId];
            if (rowsetRollupGroup != null) {
                var indices = dataIndex != null ? [dataIndex] : null;
                that.recordGrouper.rollupFromLeafs([rowsetRollupGroup], indices, that.recordGrouper.recordGroups[rowsetRollupGroup].rowsetId);
            }
            this.view.updateRow(recordId, rowsetId);
            var triggerArgs = {
                recordId : recordId,
                dataIndex : dataIndex,
                record : record
            };

            /**
             * Triggered when the data of a given cell has been updated.
             * This event is captured after certain action takes place on the grid.
             *
             * @event pgbu.grid#cell-updated
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {String} triggerArgs.recordId - ID of record that was updated
             * @param {String} triggerArgs.dataIndex - Index of field that was updated.
             * @param {Object} triggerArgs.record - Record that was updated.
             */
            this._trigger('cell-updated', null, triggerArgs);
        },

        /**
         * Function used to get the chained fields that need to be rerolled after one field is edited
         *
         * @method grid#_getDataIndexToReroll
         * @param {String} dataIndex - The index of the field that was updated.
         * @private
         */
        _getDataIndexToReroll : function (dataIndex) {
            var indexes = [dataIndex];
            if (this.columnDefinitions[dataIndex] != null && this.columnDefinitions[dataIndex].rollupOnChange != null) {
                indexes = indexes.concat(this.columnDefinitions[dataIndex].rollupOnChange);
            }
            return indexes;
        },

        /**
         * Rolls up a rowset total - put in place for bulk operations to be able to handle this after non-total rollups.
         *
         * @method grid#_rollupRowsetTotal
         * @private
         */
        _rollupRowsetTotal : function (group, dataIndices) {
            this.recordGrouper.rollupFromLeafs([group], dataIndices, this.recordGrouper.recordGroups[group].rowsetId);
        },

        /**
         * Function that runs validation on all fields, on all records, currently in the store.
         *
         * Note: Will trigger {@link grid#repopulate repopulate()}
         * @method grid#validateStore
         * @public
         */
        validateStore : function () {
            this.store.iterateStore(this._validateRecord, this);
            this.repopulate();
        },

        _validateRecord : function (record, hideErrors) {
            var proxy = this._getProxyRecord();
            if (proxy != null && this._getRecordId(proxy) === this._getRecordId(record)) {
                return;
            }
            var columnsWithValidation = _.filter(_.values(this.view.columnDefinitions), function (columnDefinition) {
                return (columnDefinition.validator != null);
            });

            for (var i = 0; i < columnsWithValidation.length; i++) {
                this._validateRecordCell({
                    dataIndex : columnsWithValidation[i].dataIndex,
                    record : record,
                    hidden : hideErrors,
                    append : i === columnsWithValidation.length - 1
                });
            }
        },

        /**
         * Used to validate an individual cell in the grid.
         *
         * @method grid#_validateRecordCell
         * @param {Object} options
         * @param {String} options.dataIndex - The data index of the record to be validated.
         * @param {Object} options.record - The record to be validated.
         * @param {Boolean} options.hidden - Whether any errors found should trigger the change bar to be displayed.
         * @param {Boolean} options.append - Whether any errors found should be appended to the change bar.
         * @private
         */
        _validateRecordCell : function (options) {
            if (options.recordId != null && options.record == null) {
                options.record = this._getRecord(options.recordId);
            }

            var column = this.view.columnDefinitions[options.dataIndex];
            var validator = column.validator;
            var recordValue = this.store.getValueFromRecord(options.record, options.dataIndex);
            var recordId = options.recordId || this._getRecordId(options.record);
            var validationResponse;
            if (validator != null) {
                validationResponse = validator.validate(recordValue);
            }
            if (validator == null || validationResponse.valid || this.options.overrideValidation) {
                if (_.isFunction(this.store.validateRecordValue)) {
                    validationResponse = this.store.validateRecordValue(options.record, options.dataIndex, recordValue, validationResponse);
                }
            }
            var validationMessages = null;
            if (validationResponse != null && !validationResponse.valid) {
                validationMessages = {};
                validationMessages[validationResponse.errorType || 'generic'] = {
                    message : validationResponse.message,
                    hidden : options.hidden
                };
                this._clearInvalid(recordId, options.dataIndex, false); // Skip the trigger as mark invalid is about to reset
                this._markInvalid(recordId, options.dataIndex, validationMessages, options.append);
            } else {
                this._clearInvalid(recordId, options.dataIndex, options.append);
            }

            if (column.unique && (validationResponse == null || validationResponse.valid)) {
                if (_.isFunction(this.store.checkRecordUniqueness)) {
                    this.store.checkRecordUniqueness(options.dataIndex, options.record);
                }
            }
        },

        /**
         * Informs the grid that the corresponding records have been added to the store.
         *
         * @method grid#_recordsAdded
         * @param {Array<Object>} records - The records that were added to the store.
         * @param {Boolean} select - Whether or not to auto-select one of the records after adding to the grid.
         * @private
         * @fires grid#event:rows-added
         */
        _recordsAdded : function (records, select) {
            if (this.bypassRowAdd) {
                return;
            }

            var that = this;
            if (select == null) {
                select = this.options.methodDefaults._recordsAdded.select;
            }
            var rowsetsToRender = [];
            var toAdd = [];
            var rowsetTotals = [];

            toAdd = _.map(records, function (record) {
                return {
                    record : record,
                    recordId : this._getRecordId(record)
                };
            }.bind(this));

            // Group the new record and add new groups to the view
            if (that.groupingApplied) {
                that.recordGrouper.recordsAdded(toAdd);
            } else if (that.options.dataRollup.enabled) {
                that.dataRollup.recordsAdded(toAdd);
            }
            toAdd = [];

            _.forEach(records, function (record) {
                var location = that._getRecordPlacement(record),
                    recordId = that._getRecordId(record);

                if (that.view.containsRecord(recordId, location.rowsetId)) {
                    // BPC will always do this because it invokes _recordsAdded on an event and the add method itself
                    //>>excludeStart("validation", pragmas.production)
                    logger.info('grid._recordsAdded | Cant add row record already in view, skipping:' + recordId);
                    //>>excludeEnd("validation")
                    return;
                }

                var addRecord = true;
                that._validateRecord(record, that.options.hideErrorsUntilEdit);
                toAdd.push({
                    record : record,
                    recordId : recordId
                });

                // If rolluped rowset recalc the rollup
                var rowsetRollupGroup = that.rolledupRowsets[location.rowsetId];
                if (rowsetRollupGroup != null) {
                    rowsetTotals.push(rowsetRollupGroup);
                    var group = that.recordGrouper.recordGroups[rowsetRollupGroup];
                    group.records.push(recordId);
                }

                // If there is a parent (tree or group) make sure its visibe and update the row
                // this will redraw proper group colors and expansion buttons
                if (location.parentId != null) {
                    addRecord = !that.view.ensureExpanded({
                        nodeId : location.parentId,
                        rowsetId : location.rowsetId
                    });
                    that._updateParentRow(location.parentId);
                    addRecord = !that.view.containsRecord(that._getRecordId(record), location.rowsetId);
                }
                // If a parent is expanded above no need to add the record in.
                if (addRecord) {
                    // Need to place the record if you can
                    if (location.siblingId == null && !_.has(that.siblingIds, recordId)) {
                        var focusId = that.focus.recordId;
                        if (focusId === constants.DUMMY_RECORD_ID) {
                            focusId = that.view.previousSibling(focusId);
                        }
                        var recordAboveId = focusId;
                        if (location.parentId != null) {
                            recordAboveId = that.view.lastChild(location.parentId, location.rowsetId, that._getRecordTreeAttributes(record).precedence);
                        }
                        var recordBelowId = recordAboveId != null ? that.view.nextSibling(recordAboveId) : null;
                        that._setSiblingIds(record, recordAboveId, recordBelowId);
                        that._setRecordSequence(record, recordAboveId, recordBelowId);
                    }

                    if (!_.includes(rowsetsToRender, location.rowsetId)) {
                        rowsetsToRender.push(location.rowsetId);
                    }
                    that._addRecordToView(record, {
                        location : that._getRecordPlacement(record),
                        bulk : false,
                        select : select
                    });
                }
            });

            // Nothing to do - likely just duplicate rows that caused _recordsAdded to get invoked due to BPC
            if (!toAdd.length) {
                return;
            }
            _.forEach(rowsetTotals, function (rowsetRollupGroup) {
                that._rollupRowsetTotal(rowsetRollupGroup);
            });

            _.forEach(rowsetsToRender, function (rowsetId) {
                that.view.renderRowset(rowsetId);
            });
            this.view.resize();

            if (select) {
                this.selectCell(this._getRecordId(_.last(records)), null, null, null, null, true);
            }
            var triggerArgs = {
                records : records
            };

            /**
             * Triggered when rows are added to the grid.
             * This event is captured after certain action takes place on the grid.
             *
             * @event pgbu.grid#rows-added
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Array<Object>} triggerArgs.records - The records that were added to the grid.
             */
            this._trigger('rows-added', null, triggerArgs);
        },

        /**
         * Used to check uniqueness of all of the unique attributes, of all of the records, in the store.
         *
         * @method grid#_recheckStoreUniqueness
         * @private
         */
        _recheckStoreUniqueness : function () {
            var columnsWithValidation = _.filter(_.values(this.view.columnDefinitions), function (columnDefinition) {
                return (columnDefinition.validator != null);
            });
            for (var i = 0; i < columnsWithValidation.length; i++) {
                var dataIndex = columnsWithValidation[i].dataIndex;
                if (columnsWithValidation[i].unique && _.isFunction(this.store.checkRecordUniqueness)) {
                    this.store.checkRecordUniqueness(dataIndex);
                }
            }
        },

        /**
         * Used to inform the grid that the corresponding records have been removed from the store.
         *
         * @method grid#_recordsRemoved
         * @param {Array<Object>} records - The records that were removed.
         * @private
         * @fires grid#event:rows-deleted
         * @fires grid#event:action-state-update
         */
        _recordsRemoved : function (records) {
            var that = this;
            var recordIds = [],
                recordId,
                shouldResize = false,
                removedAtLeastOneRecord = false;
            if (that.bypassRowAdd) {
                return;
            }

            var modifiedParents = [];
            var recordIdsRemoved = [];
            var rowsetTotals = [];

            records = _.reject(records, function (item) {
                return item[that.recordKeys.id] === constants.DUMMY_RECORD_ID;
            });
            _.forEach(records, function (record) {
                recordId = that._getRecordId(record);
                if (!that.recentlyRemoved[recordId]) {
                    removedAtLeastOneRecord = true;
                    that.recentlyRemoved[recordId] = true;

                    var location = that._getRecordPlacement(record);
                    var existsInView = that.view.containsRecord(recordId, location.rowsetId);

                    recordIdsRemoved.push(recordId);

                    // If rolluped rowset recalc the rollup
                    var rowsetRollupGroup = that.rolledupRowsets[location.rowsetId];
                    if (rowsetRollupGroup != null) {
                        rowsetTotals.push(rowsetRollupGroup);
                    }

                    that._clearInvalid(recordId, undefined, true);
                    recordIds = [recordId];
                    if (location.parentId != null) {
                        var parent = that._getRecord(location.parentId);
                        if (parent != null && !_.includes(modifiedParents, location.parentId)) {
                            modifiedParents.push(location.parentId);
                        }
                    }
                    if (that.matchingRecords.recordOrder.indexOf(recordId) !== -1) {
                        that.matchingRecords.recordOrder.splice(that.matchingRecords.recordOrder.indexOf(recordId), 1);
                    }
                    if (existsInView) {
                        that._setDeletedRecordSiblings(record);
                        that.view.deleteRecords(recordIds);
                        shouldResize = true;
                    }
                    //>>excludeStart("validation", pragmas.production)
                    else {
                        logger.warn('grid._recordsRemoved | Cant remove row, record not in view:' + that._getRecordId(record));
                    }
                    //>>excludeEnd("validation")
                }
            });
            if (!removedAtLeastOneRecord) {
                return;
            }

            that._clearInvalid(that._getRecordId(_.last(records)));
            if (that.groupingApplied) {
                that.recordGrouper.recordsRemoved(recordIdsRemoved);
            } else if (that.options.dataRollup.enabled) {
                that.dataRollup.recordsRemoved(recordIdsRemoved);
            }
            _.forEach(rowsetTotals, function (rowsetTotal) {
                var group = that.recordGrouper.recordGroups[rowsetTotal];
                group.records = _.difference(group.records, recordIdsRemoved);
                that._rollupRowsetTotal(rowsetTotal);
            });

            if (modifiedParents.length) {
                _.forEach(modifiedParents, function (parentId) {
                    that._updateParentRow(parentId);
                });
            }
            if (shouldResize) {
                that.view.resize();
            }

            recordIds = that._getRecordIds(records);
            that.selection.recordIds = _.difference(that.selection.recordIds, recordIds);
            that.selection.records = that._getRecords(that.selection.recordIds);
            _.forEach(recordIds, function (recordId) {
                delete that.selection.recordIdMap[recordId];
            });
            if (_.includes(recordIds, that.focus.recordId)) {
                that.focus.record = null;
                that.focus.recordId = null;
                that.view.selectionModel.clearFocus();
            }
            that.checkedRecordIds = _.difference(that.checkedRecordIds, recordIds);
            that.checkedRecords = that._getRecords(that.checkedRecordIds);

            /**
             * Returns the allowable action state for the selected records for potential grid actions
             * Example : canDelete, canAdd, etc
             * Triggered when there is a potential change in the state of actions due to user interaction
             * Helpful for updating toolbars (enable and disable actions)
             * @event pgbu.grid#action-state-update
             * @param {Object} [event=null]
             * @param {Function} checkActionsFn - Function to enable/disable actions in the context menu for the given records.
             */
            that._trigger('action-state-update', null, that._checkActions());
            var triggerArgs = {
                records : records
            };

            /**
             * Triggered when rows are deleted from the grid.
             * This event is captured after certain action takes place on the grid.
             *
             * @event pgbu.grid#rows-deleted
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Array<Object>} triggerArgs.records
             */
            that._trigger('rows-deleted', null, triggerArgs);
        },

        /**
         * Used to inform the grid that a record was moved (left, right, up, or down).  This only applies to grids that are hierarchical and/or sequenced.
         *
         * @method grid#_recordMoved
         * @param {Object} record - Record that moved.
         * @param {Object} oldRowset - The rowset the record used to belong to.
         * @private
         * @fires grid#rows-moved
         */
        _recordMoved : function (record, oldRowset) {
            var that = this,
                recordId = that._getRecordId(record),
                location = that._getRecordPlacement(record),
                attributes = that._attrs.all(record);
            if (oldRowset == null) {
                oldRowset = this.defaultRowset;
            }

            if (this.groupingApplied) {
                if (!this.recordGrouper.recordMoved(
                    {
                        record : record,
                        recordId : recordId,
                        rowsetId : location.rowsetId,
                        location : location
                    }
                        )) {
                    return;
                }
            } else if (that.options.dataRollup.enabled) {
                that.dataRollup.recordMoved({
                    record : record,
                    recordId : recordId,
                    rowsetId : location.rowsetId,
                    location : location
                });
            }
            if (this.options.tree && !_.includes(that.recordsBeingMoved, record)) {
                var node = this.view.getNode(recordId, location.rowsetId);
                if (node != null && ((node.parent != null && location.parentId == null) || (node.parent != null && node.parent.id !== location.parentId))) {
                    var handleIt = function () {
                        that._setDeletedRecordSiblings(record);
                        var recordAboveId = that.view.lastChild(location.parentId, location.rowsetId);
                        if (recordAboveId === recordId || recordAboveId == null) {
                            recordAboveId = location.parentId;
                        }
                        that._setSiblingIds(record, recordAboveId, null);
                        location = that._getRecordPlacement(record);
                        that.view.moveRecord(recordId, location, location.rowsetId !== oldRowset ? oldRowset : null, attributes);

                        /**
                         * Triggered whens rows are reordered.
                         * This event is captured after certain action takes place on the grid.
                         *
                         * @event pgbu.grid#rows-moved
                         * @param {Object} [event=null]
                         * @param {Object} triggerArgs
                         * @param {Object} triggerArgs.records
                         * @param {Object} triggerArgs.newLocation
                         * @param {Object} triggerArgs.oldRowset
                         */
                        that._trigger('rows-moved', null, {
                            records : record,
                            newLocation : location,
                            oldRowset : oldRowset
                        });
                    };
                    var expanded = that.view.ensureExpanded({
                        nodeId : location.parentId,
                        rowsetId : location.rowsetId
                    });


                    if (_.isBoolean(expanded)) {
                        handleIt();
                    } else {
                        expanded.done(function () {
                            handleIt();
                        });
                    }
                    return;
                }
            }
            that.view.moveRecord(recordId, location, location.rowsetId !== oldRowset ? oldRowset : null, attributes);
            this.view.updateRowsetScrollbar(location.rowsetId);
            this.view.reselectRecords();
            that._trigger('rows-moved', null, {
                records : record,
                newLocation : location,
                oldRowset : oldRowset
            });
        },

        _rollupTimephasedData : function () {
            this.rerunGroupRollups(_.filter(this.columnDefinitions, function (colDef) {
                return colDef.timephased === true && colDef.visible;
            }));
        },

        rerunGroupRollups : function (dataIndices) {
            this.recordGrouper.forceAllRollups({
                dataIndices : dataIndices
            });
        },

        rollupHierarchyData : function (args) {
            if (args && args.clean) {
                this.dataRollup.buildAndRollupTree();
            } else {
                this.dataRollup.forceAllRollups();
            }
        },

        _getGanttWidget : function () {
            return this.element.parents('.scrolly-gantt').parent().data('pgbu-gantt');
        },

        _childrenLoaded : function (recordId) {
            this.view.childrenLoaded(recordId);
        },

        //* ***********************************************************
        // Methods exposed on the widget
        //* ***********************************************************

        /**
         * Used to get the current view configuration of the grid.
         *
         * @method grid#getViewConfig
         * @returns {grid#viewConfig}
         * @public
         */
        getViewConfig : function () {
            var that = this;
            var viewConfig = {
                sortBy : [],
                groupBy : []
            };
            if (this.config.sortBy && this.config.sortBy.length) {
                _.forEach(this.config.sortBy, function (sort) {
                    viewConfig.sortBy.push(_.pick(sort, 'dataIndex', 'order'));
                });
            }

            if (this.config.groupBy && this.config.groupBy.length) {
                viewConfig.groupingOptions = {
                    showColumnTitle : this.options.grouping.showColumnTitle,
                    enableRollup : this.options.grouping.enableRollup,
                    showEmptyBands : this.options.grouping.showEmptyBands,
                    nameFormat : this.options.grouping.nameFormat
                };
                _.forEach(this.config.groupBy, function (group) {
                    viewConfig.groupBy.push(_.pick(group, 'dataIndex', 'level', 'sortOrder', 'toLevel', 'fakeGroup'));
                });
            }

            viewConfig.theme = {
                style : this.options.theme.style,
                colors : this.options.theme.colors,
                readOnlyStyling : this.options.theme.readOnlyStyling
            };

            viewConfig.columnGroups = [];
            _.forEach(this.config.groups, function (group) {
                if (!group.internalDisplayOnly) {
                    var configGroup = {
                        id : group.id,
                        columns : [],
                        width : group.width
                    };
                    if (that.options.stackedCells.enabled) {
                        configGroup.stackedCells = _.clone(group.stackedCells);
                    }
                    _.forEach(group.columns, function (column) {
                        if (!column.internalDisplayOnly) {
                            configGroup.columns.push(column.dataIndex);
                        }
                    });
                    viewConfig.columnGroups.push(configGroup);
                }
            });

            viewConfig.columns = {};
            _.forEach(this.columnDefinitions, function (columnDefinition) {
                if (!columnDefinition.internalDisplayOnly && !columnDefinition.timephased) {
                    viewConfig.columns[columnDefinition.dataIndex] = _.pick(columnDefinition, function (value, property) {
                        switch (property) {
                        case 'visible':
                        case 'width':
                        case 'percent':
                            if (value != null) {
                                return true;
                            }
                        }
                    });
                }
            });

            if (this.options.timephased.enabled || this.options.timephased.supportsTimephased) {
                viewConfig.timephased = {
                    scale : this.options.timephased.scale,
                    start : this.options.timephased.view.start ? this.options.timephased.view.start.toString() : null,
                    end : this.options.timephased.view.end ? this.options.timephased.view.end.toString() : null,
                    usePeriodLabels : this.options.timephased.usePeriodLabels,
                    bucketWidth : this.options.timephased.bucketWidth,
                    enabled : this.options.timephased.enabled
                };
            }

            if (this.options.advancedFilters.enabled && this.view.advancedFilters) {
                viewConfig.advancedFilters = this.view.advancedFilters.getFilterViewData();
            }

            return viewConfig;
        },

        _renameViewColumns : function (columns) {
            return utils.remapColumns(columns, this.options.renamedViewColumns);
        },

        /**
         * Used to apply a new view configuration to the grid.
         *
         * @method grid#applyViewConfig
         * @param {grid#viewConfig} viewConfig
         * @public
         */
        applyViewConfig : function (viewConfig) {
            var that = this;
            if (this.store == null) {
                return;
            }
            if (viewConfig.columnGroups != null) {
                var rejectColumns = function (column) {
                    return that.options.columns[column] == null;
                };
                _.forEach(viewConfig.columnGroups, function (group) {
                    for (var i = 0; i < that.options.layout.columnGroups.length; i++) {
                        var originalGroup = that.options.layout.columnGroups[i];
                        if (originalGroup.id === group.id) {
                            _.defaults(group, originalGroup);
                            group.columns = _.reject(that._renameViewColumns(group.columns), rejectColumns);
                            that.options.layout.columnGroups[i] = group;
                            if (group.stackedCells != null && group.stackedCells.length) {
                                originalGroup.stackedCells = group.stackedCells;
                            }
                        }
                    }
                });
            }
            if (viewConfig.columns != null) { // safety first kids
                var columns = _.reduce(this._renameViewColumns(viewConfig.columns), function (memo, col, dataIndex) {
                    if (that.options.columns[dataIndex]) {
                        memo[dataIndex] = col;
                    }
                    return memo;
                }, {});
                this._setOption('columns', columns);
            }
            if (viewConfig.sortBy != null) {
                this._setSortConfig(viewConfig.sortBy);
            }

            if (viewConfig.theme != null) {
                if (viewConfig.theme.style != null) {
                    this.options.theme.style = viewConfig.theme.style;
                }
                if (viewConfig.theme.colors != null) {
                    this.options.theme.colors = viewConfig.theme.colors;
                }
                if (viewConfig.theme.readOnlyStyling != null) {
                    this.options.theme.readOnlyStyling = viewConfig.theme.readOnlyStyling;
                }
                this.view.updateGroupBandClasses();
                this.view.updateReadOnlyStyling();
            }

            if (viewConfig.groupBy != null) {
                if (viewConfig.groupBy.length === 0 && this.groupingApplied) {
                    this._setOption('groupBy', []);
                } else {
                    if (viewConfig.groupingOptions != null) {
                        var groupingOptions = _.reduce(viewConfig.groupingOptions, function (options, value, key) {
                            if (value != null) {
                                options[key] = value;
                            }
                            return options;
                        }, {});
                        this._setOption('grouping', groupingOptions);
                    }
                    this._setOption('groupBy', viewConfig.groupBy);
                }
            }

            if (viewConfig.timephased != null) {
                var timephasedOptions = {
                    view : {
                        start : moment(viewConfig.timephased.start)
                    },
                    scale : viewConfig.timephased.scale,
                    bucketWidth : viewConfig.timephased.bucketWidth,
                    usePeriodLabels : viewConfig.timephased.usePeriodLabels,
                    enabled : viewConfig.timephased.enabled != null ? viewConfig.timephased.enabled : true
                };
                if (timephasedOptions.bucketWidth == null) {
                    delete timephasedOptions.bucketWidth;
                }
                this._setOption('timephased', timephasedOptions);
            }
            if (viewConfig.advancedFilters != null && this.options.advancedFilters.enabled) {
                this.config.advancedFilters = viewConfig.advancedFilters;
            }
            this.viewManager.refresh();
        },

        /**
         * Method to generate a new record for the store and row in the grid.
         *
         * @method grid#newRow
         * @param {Object} options
         * @param {Array<Object>} options.records
         * @param {Boolean} options.isChild
         * @param {Integer} options.recordsToAdd
         * @param {Object} options.focus
         * @param {Object} options.addAfter
         * @public
         * @fires grid#before-new-row
         * @fires pbgu.grid#event:performance-debug
         */
        newRow : bulkModificationOperationDecorator(function newRow(options) {
            options = this._prepareOptions(options, 'newRow');

            var triggerArgs = {
                args : options,
                selection : options.records
            };
            var that = this,
                focusId,
                actionToCheck = 'newRow';

            if (options != null && options.isChild) {
                actionToCheck = 'newChildRow';
            }

            /**
             * Triggered before a new row is created.
             * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
             *
             * @event pgbu.grid#before-new-row
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Object} triggerArgs.options
             * @param {Array<Object>} triggerArgs.selection
             */
            if (this._checkAction(actionToCheck, this._preCheckActions(options.records, options.focus), options) &&
                    this._trigger('before-new-row', null, triggerArgs) !== false) {
                //>>excludeStart("perf", pragmas.production)
                var start;
                if (this.options.debug) {
                    start = window.performance.now();
                }
                //>>excludeEnd("perf")

                if (options.recordsToAdd == null) {
                    var selection = this.getSelection();
                    options.recordsToAdd = selection.length || 1;

                    if (_.includes(selection, options.focus) && !options.fromQuickAddRow) {
                        options.records = this.view.orderByViewIndex(selection);
                        if (options.addAfter) {
                            options.records = options.records.reverse();
                        }
                    } else {
                        options.batchAddAtFocus = true;
                    }
                }

                var newRecords = [];

                if (options.focus != null) {
                    focusId = that._getRecordId(options.focus);
                }

                // Defer action of new if needed
                var handleNew = function () {
                    for (var i = 0; i < options.recordsToAdd; i++) {
                        var modifiedOptions = _.clone(options);
                        if (!options.batchAddAtFocus) { // If not adding in a batch
                            modifiedOptions.focus = options.records[i];
                            focusId = that._getRecordId(modifiedOptions.focus);
                            if (focusId === constants.DUMMY_RECORD_ID) {
                                modifiedOptions.addAfter = false;
                            }
                        }
                        if (that.options.data != null) {
                            modifiedOptions.focusType = that._getTypeFromId(focusId);
                            if (modifiedOptions.type == null) {
                                modifiedOptions.type = modifiedOptions.focusType;
                            }
                            modifiedOptions.suffix = that.options.data.types[modifiedOptions.type].idSuffix;
                        }
                        var record;
                        if (options.newRecord) {
                            record = options.newRecord;
                        } else {
                            record = that.store.newRecord.call(that.store, modifiedOptions);
                        }
                        newRecords.push(record);

                        // Apply ParentId

                        if (that.options.data != null && that.options.tree && !that.groupingApplied) {
                            that.bypassRowAdd = true; // bypass adding from store addRecords so that it is in the correct place.
                            if (that.store.addRecordsByType != null) {
                                that.store.addRecordsByType(modifiedOptions.type, [record]);
                            } else {
                                that.store.addRecords([record]); // Add them on
                            }
                            that.bypassRowAdd = false;
                        }
                        that._setNewRecordParent({
                            record : record,
                            focus : modifiedOptions.focus,
                            isChild : options.isChild,
                            childType : modifiedOptions.type
                        });

                        var location = that._getRecordPlacement(record);
                        // Set best guess for recordAbove
                        var recordAboveId = options.addAfter ? focusId : that.view.previousSibling(focusId);
                        // If batch adding at the focus connect the new records together
                        if (i > 0) {
                            recordAboveId = that._getRecordId(newRecords[i - 1]);
                        }

                        // If it belongs to a different parent fix it here.
                        if (location.parentId != null && location.parentId === focusId) {
                            recordAboveId = that.view.lastChild(location.parentId, null, that._attrs.tree(record).precedence);
                        }

                        var recordBelowId = recordAboveId != null ? that.view.nextSibling(recordAboveId) : (options.addAfter || options.isChild ? null : focusId);

                        that._setSiblingIds(record, recordAboveId, recordBelowId);
                        that._setRecordSequence(record, recordAboveId, recordBelowId);
                        if (that.options.data != null && that.options.tree && !that.groupingApplied) {
                            that._recordsAdded([record], !options.skipSelect);
                        } else if (that.groupingApplied || options.fromQuickAddRow) {
                            that.bypassRowAdd = true; // bypass adding from store addRecords so that it is in the correct place.
                            var selectRecords = false; // only select for quickAdd from tab
                            that.store.addRecords([record]); // Add them one at a time for sequencing logic to stay sane.
                            that.quickAddInProgress = true;
                            if (that.groupingApplied && !options.fromQuickAddRow) {
                                that.recordGrouper.setGroupBandValues(record, options.focus);
                            }
                            that.bypassRowAdd = false;
                            that._recordsAdded([record], selectRecords);
                            that.quickAddInProgress = false;
                        } else {
                            that.store.addRecords([record]); // Add them one at a time for sequencing logic to stay sane.
                        }
                    }

                    that._pushUndoAction({
                        records : newRecords,
                        type : 'new'
                    });

                    //>>excludeStart("perf", pragmas.production)
                    if (that.options.debug) {
                        /**
                         * Triggered periodically to supply debug information to the console.
                         * This event is captured after certain action takes place on the grid.
                         * Only fires when the debug flag option is set to true.
                         *
                         * @event pgbu.grid#performance-debug
                         * @param {moment} start - When the performance debug started.
                         * @param {moment} end - When the performance debug should stop.
                         * @param {String} name
                         * @param {String} type='widget'
                         */
                        that._trigger('performance-debug', null, {
                            start : start, end : window.performance.now(), name : 'new-row',
                            type : 'widget'
                        });
                    }
                    //>>excludeEnd("perf")
                    // Use settimeout because the grid has to use timeouts to render when scrollbars are triggering due
                    // to browser problems. Setting a timeout of 1 will put it in the stack and occur after the rerender
                    // of the rowsets is done.
                    clearTimeout(that.timeouts.edit);
                    that.timeouts.edit = setTimeout(function () {
                        var columnToEdit = that.view.firstVisibleColumn;
                        if (options.columnToEdit != null) {
                            columnToEdit = options.columnToEdit;
                        }

                        if (!options.skipSelect) {
                            that.selectRows(that._getRecordIds(newRecords));
                            that.selectCell(that._getRecordId(_.first(newRecords)), columnToEdit, null, true);
                        }

                        if (options.skipEdit) {
                            return;
                        }
                        var editOptions = {
                            recordId : that._getRecordId(_.first(newRecords)),
                            columnId : columnToEdit,
                            currentlyTyping : options.currentlyTyping,
                            hideErrors : options.hideErrors
                        };
                        that.editRow(editOptions);
                    }, 10);
                };
                // Check if we need to expand the current row, that way it will be added correctly.
                var expanded = true;
                if (focusId != null && options.isChild) {
                    expanded = that.view.ensureExpanded({
                        nodeId : focusId,
                        rowsetId : that._getRecordRowset(focusId)
                    });
                }
                // If lazy loaded deferr action until its finished
                if (_.isBoolean(expanded)) {
                    handleNew();
                } else {
                    return expanded.done(function () {
                        handleNew();
                    });
                }
            }
        }),

        /**
         * Method called to create and add, a new child record for the store and row in the grid.
         *
         * @method grid#newChildRow
         * @param {Object} options
         * @param {Object} focus - The record currently focused on in the grid.  Usually the parent record.
         * @param {Boolean} focusInSelection - Whether or not the focused record was selected.
         * @param {Object} records - Child records to add to the grid.
         * @public
         */
        newChildRow : function (options) {
            if (options != null) {
                options.isChild = true;
            } else {
                options = {
                    isChild : true
                };
            }
            return this.newRow(options);
        },

        /**
         * Method used to delete to directly delete cells from the grid.
         * Generally, you should not delete cells directly and should instead use the {@link grid#deleteRows deleteRows} method.
         * @method grid#deleteCells
         * @param {Object} options
         * @param {Array<Object>} options.records - The records whose cells need to be deleted.
         * @param {String} options.dataIndex - The index of the cell that should be deleted.
         * @fires grid#event:before-delete-cells
         */
        deleteCells : function (options) {
            options = this._prepareOptions(options, 'deleteCells');

            var dataIndex = options.dataIndex || this.focus.columnId;
            var triggerArgs = { records : options.records, dataIndex : dataIndex };

            /**
             * @event grid#before-delete-cells
             * @param {Object} [event=null]
             * @param {object} triggerArgs
             * @param {Array<Object>} triggerArgs.records - The records whose cells need to be deleted
             * @param {String} triggerArgs.dataIndex - The index of the cell that should be deleted.
             * @public
             */
            if (this._checkAction('deleteCells', this._preCheckActions(options.records, options.focus), options) &&
                    this._trigger('before-delete-cells', null, triggerArgs) !== false) {
                for (var i = 0; i < options.records.length; i++) {
                    var record = options.records[i];
                    if (this.canEditCell(this._getRecordId(record), dataIndex, this._attrs.nonSelection(record)) === true) {
                        var column = this.columnDefinitions[dataIndex];
                        this._cellUpdated({
                            record : record,
                            value : this._getEmptyValue(column),
                            valid : true,
                            column : column
                        });
                    }
                }
            }
        },

        /**
         * Method used to remove the values from selected cells, and store them in a 'clipboard', so that they can be 'pasted' into other cells.
         *
         * Note: This method can be applied to one or many records.
         * @method grid#cutCells
         * @param {Object} options
         * @param {Array<Object>} options.records - The records whose cells should be 'cut'
         * @param {Object} options.focus - The record that has direct focus in the grid
         * @param {String} options.dataIndex - The index of the cell(s) that should be 'cut'
         * @public
         * @fires grid#event:before-cut-cells
         *
         */
        cutCells : function (options) {
            options = this._prepareOptions(options, 'cutCells');

            var dataIndex = options.dataIndex || this.focus.columnId;
            var triggerArgs = { record : options.records, dataIndex : dataIndex };

            /**
             * @event grid#before-cut-cells
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Array<Object>} triggerArgs.record - The records whose cells need to be cut.
             * @param {String} triggerArgs.dataIndex - The index of the cells that need to be cut from the records.
             */
            if (this._checkAction('cutCells', this._preCheckActions(options.records, options.focus), options) &&
                    this._trigger('before-cut-cells', null, triggerArgs) !== false) {
                if (options.records != null && options.records.length > 0) {
                    var recordIds = [];
                    var values = [];
                    for (var i = 0; i < options.records.length; i++) {
                        var record = options.records[i];
                        recordIds.push(this._getRecordId(record));
                        values.push(this.store.getValueFromRecord(record, dataIndex));
                    }
                    this.clipboard.cellValues = values;
                    this.clipboard.cellIndex = dataIndex;
                    this.clipboard.records = options.records;
                    this.clipboard.mode = 'cut';
                    this.clipboard.scope = 'cell';
                    this.clipboard.hideStyling = false;
                    this.view.applyCutCopyClassesToRecords(recordIds);
                }
            }
        },

        /**
         * Method used to store values from the selected cells, in a 'clipboard', which can then be applied, or 'pasted', into other cells.
         *
         * Note: This method can be applied to one or many records.
         * @method grid#copyCells
         * @param {Object} options
         * @param {Array<Object>} options.records - The records whose values need to be copied.
         * @param {String} options.dataIndex - The index of the column whose vlaue needs to be copied.
         * @param {Object} options.focus - The record that has focus in the grid.
         * @public
         * @fires grid#event:before-copy-cells
         */
        copyCells : function (options) {
            options = this._prepareOptions(options, 'copyCells');

            /*
             * Row selector column will not have selection unless they use the checkbox column
             * A typical user will not know they have to use the checkbox first before copy so default the
             * records to copy to the focused record
             */
            if (options.records.length === 0 && options.focus != null) {
                options.records = [options.focus];
            }

            var dataIndex = options.dataIndex || this.focus.columnId;
            var triggerArgs = { record : options.records, dataIndex : dataIndex };

            /**
             * @event grid#before-copy-cells
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Array<Object>} triggerArgs.record - The records whose values need to be copied.
             * @param {String} triggerArgs.dataIndex - The index of the column whose value needs to be copied.
             */
            if (this._checkAction('copyCells', this._preCheckActions(options.records, options.focus), options) &&
                    this._trigger('before-copy-cells', null, triggerArgs) !== false) {
                // Go through and clone all the values
                if (options.records != null && options.records.length > 0) {
                    var recordIds = [];
                    var values = [];
                    for (var i = 0; i < options.records.length; i++) {
                        var record = options.records[i];
                        recordIds.push(this._getRecordId(record));
                        if (_.isFunction(this.store.cloneValueFromRecord)) {
                            values.push(this.store.cloneValueFromRecord(record, dataIndex));
                        } else {
                            values.push(_.cloneDeep(this.store.getValueFromRecord(record, dataIndex)));
                        }
                    }
                    this.clipboard.cellValues = values;
                    this.clipboard.records = options.records;
                    this.clipboard.cellIndex = dataIndex;
                    this.clipboard.mode = 'copy';
                    this.clipboard.scope = 'cell';
                    this.clipboard.hideStyling = false;

                    this.view.applyCutCopyClassesToRecords(recordIds);
                }
            }
        },

        /**
         * Method used to apply the stored value(s) from the clipboard, to the selected cells in the grid.
         *
         * Note: This method can be applied to one or more records.
         * @method grid#pasteCells
         * @param {Object} options
         * @param {Array<Object>} options.records - The selected records in the grid, that the value(s) will be applied to.
         * @param {Object} options.focus - The record that has focus in the grid.
         * @param {Boolean} options.strictSelection
         * @param {Boolean} options.appendToSelection
         * @public
         * @fires grid#event:before-paste-cells
         */
        pasteCells : bulkModificationOperationDecorator(function pasteCells(options) {
            var that = this;
            options = this._prepareOptions(options, 'pasteCells');

            /*
             * Row selector column will not have selection unless they use the checkbox column
             * A typical user will not know they have to use the checkbox first before paste so default the
             * records to paste to the focused record
             */
            if (options.records.length === 0 && options.focus != null) {
                options.records = [options.focus];
            }

            var dataIndex = this.focus.columnId;
            var record, i, column;
            var triggerArgs = {
                copiedValues : that.clipboard.cellValues,
                mode : that.clipboard.mode,
                focus : that.focus.record,
                dataindex : dataIndex,
                records : options.records,
                args : options
            };

            /**
             * @event grid#before-paste-cells
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Array<Object>} triggerArgs.copiedValues - Values to copy into the selected cells
             * @param {String} triggerArgs.mode - Mode of the clipboard.  Will be 'copy' or 'cut'.
             * @param {Object} triggerArgs.focus - Record that has focus in the grid.
             * @param {String} triggerArgs.dataIndex - Index of the column that will have the values pasted into it.
             * @param {Array<Object>} triggerArgs.records - Records that the value(s) should be applied to.
             * @param {Object} triggerArgs.args - Additional arguments.
             */
            if (this._checkAction('pasteCells', this._preCheckActions(options.records, options.focus), options) &&
                    this._trigger('before-paste-cells', null, triggerArgs) !== false) {
                // Find the proper records to paste into. Use the selection if more than 1 or else use the number
                // equal to the copied or cut values.
                var recordsToPasteInto = options.records;
                if (recordsToPasteInto.length === 1 && that.clipboard.cellValues.length > 1 && !options.strictSelection) {
                    var nextRecordId = that._getRecordId(recordsToPasteInto[0]);
                    for (i = 1; i < that.clipboard.cellValues.length; i++) {
                        nextRecordId = that.view.nextSibling(nextRecordId);
                        recordsToPasteInto.push(that._getRecord(nextRecordId));
                    }
                }
                // For the paste application so it can iterate over the selected values if pasting into a larger area
                var nextIndex = 0;
                var getNextValue = function () {
                    var value = that.clipboard.cellValues[nextIndex];
                    nextIndex++;
                    if (nextIndex >= that.clipboard.cellValues.length) {
                        nextIndex = 0;
                    }
                    return value;
                };
                var undoStackLength = this.undoStack.length;

                // Delete the cut values and clean up the cut values so you can not do it again
                if (this.clipboard.mode === 'cut') {
                    for (i = 0; i < this.clipboard.records.length; i++) {
                        record = this.clipboard.records[i];
                        if (this.canEditCell(this._getRecordId(record), dataIndex, this._attrs.nonSelection(record)) === true) {
                            column = this.columnDefinitions[dataIndex];
                            this._cellUpdated({
                                record : record,
                                value : this._getEmptyValue(column),
                                valid : true,
                                column : column
                            });
                        }
                    }
                }

                // Paste the values by iterating over the values to paste and calling cellupdated
                for (i = 0; i < recordsToPasteInto.length; i++) {
                    record = recordsToPasteInto[i];
                    if (this.canEditCell(this._getRecordId(record), dataIndex, this._attrs.nonSelection(record)) === true) {
                        column = this.columnDefinitions[dataIndex];
                        this._cellUpdated({
                            record : record,
                            value : getNextValue(),
                            valid : true,
                            column : column
                        });
                    }
                }
                var idsToPasteInto = [];
                for (var j = 0; j < recordsToPasteInto.length; j++) {
                    var recordToSelect = recordsToPasteInto[j];
                    idsToPasteInto.push(this._getRecordId(recordToSelect));
                }
                this.selectRows(idsToPasteInto, options.appendToSelection);
                var pasteUndoAction;
                if (this.options.undo) {
                    pasteUndoAction = {
                        type : 'paste',
                        batch : this.undoStack.splice(undoStackLength)
                    };
                }

                // Delete the cut values and clean up the cut values so you can not do it again
                if (this.clipboard.mode === 'cut') {
                    undoStackLength = this.undoStack.length;
                    if (this.options.undo) {
                        var cutUndoAction = {
                            type : 'paste',
                            batch : this.undoStack.splice(undoStackLength)
                        };

                        pasteUndoAction = {
                            type : 'paste',
                            batch : [pasteUndoAction, cutUndoAction]
                        };
                    }

                    this.clipboard.records = [];
                    this.clipboard.cellValues = [];
                }

                this._pushUndoAction(pasteUndoAction);

                this.clipboard.hideStyling = true;
            }

            this.view.clearCutCopyCells();
        }),

        _prepareOptions : function (options, methodName) {
            // Make options if they dont exist;
            if (options == null) {
                options = {};
            }

            // use the defaults from the options
            if (this.options.methodDefaults[methodName] != null) {
                _.defaults(options, this.options.methodDefaults[methodName]);
            }

            // if user has specified records, make the focus one of them.
            if (options.records != null && options.focus == null) {
                options.focus = options.records[0];
            }

            // Set the default records to the current grid selection
            _.defaults(options, { records : this.getSelection() });
            options.records = _.without(options.records, this._getDummyRecord());
            // Set the focus default to the current focused record, will be different for

            _.defaults(options, { focus : this.focus.record });

            return options;
        },

        /**
         * Method called to remove the selected rows from the grid and add them to the deletedCollection of the store.
         *
         * @method grid#deleteRows
         * @param {Object} options
         * @param {Array<Object>} options.records - Records to be deleted.
         * @param {Object} options.focus - Record with focus in the grid.
         * @public
         * @fires grid#event:before-delete-rows
         * @fires grid#event:performance-debug
         */
        deleteRows : bulkModificationOperationDecorator(function deleteRows(options) {
            options = this._prepareOptions(options, 'deleteRows');
            var triggerArgs = {
                args : options,
                records : options.records
            };

            this.closeEditor();

            /**
             * Triggered before rows are deleted.
             * This event is fired before action takes place on the grid.
             * Therefore allowing a grid user to prevent default behavior by returning false.
             *
             * @event pgbu.grid#before-delete-rows
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Object} triggerArgs.args - Additional arguments.
             * @param {Array<Object>} triggerArgs.records - Records to be deleted.
             */
            if (this._checkAction('deleteRows', this._preCheckActions(options.records, options.focus)) &&
                    this._trigger('before-delete-rows', null, triggerArgs) !== false) {
                //>>excludeStart("perf", pragmas.production)
                var start;
                if (this.options.debug) {
                    start = window.performance.now();
                }
                //>>excludeEnd("perf")
                var args = {};
                if (this.options.data != null) {
                    args.types = this.options.data.hierarchy.order;
                }
                this.checkRows(this._getRecordIds(options.records), false);
                this.store.deleteRecords(options.records, args);
                this.clipboard.records = _.reject(this.clipboard.records, function (record) {
                    return _.find(options.records, function (deletedRecord) {
                        return this._getRecordId(deletedRecord) === this._getRecordId(record);
                    }.bind(this)) != null;
                }.bind(this));
                this._pushUndoAction({
                    type : 'delete',
                    records : _.clone(options.records)
                });
                //>>excludeStart("perf", pragmas.production)
                if (this.options.debug) {
                    this._trigger('performance-debug', null, {
                        start : start, end : window.performance.now(), name : 'delete-rows',
                        type : 'widget'
                    });
                }
                //>>excludeEnd("perf")
            }
        }),

        /**
         * Method called to remove the selected rows from the grid and store them in a 'clipboard', so that they can be
         * applied, or 'pasted', some where else in the grid.
         *
         * @method grid#cutRows
         * @param {Object} options
         * @param {Array<Object>} options.records - Records to be 'cut' from the grid.
         * @param {Object} options.focus - Record that has focus in the grid.
         * @public
         * @fires grid#event:before-cut-rows
         * @fires grid#event:action-state-update
         * @fires grid#event:performance-debug
         * @fires pgbu-grid#event:rows-cut
         */
        cutRows : function (options) {
            options = this._prepareOptions(options, 'cutRows');
            var that = this;
            var triggerArgs = {
                records : options.records,
                args : options
            };

            /**
             * Triggered before rows are cut
             * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
             *
             * @event pgbu.grid#before-cut-rows
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Array<Object>} triggerArgs.records - Records to be 'cut' from the grid.
             * @param {Object} triggerArgs.args - Additional arguments.
             */
            if (that._checkAction('cutRows', this._preCheckActions(options.records, options.focus)) &&
                    that._trigger('before-cut-rows', null, triggerArgs) !== false) {
                //>>excludeStart("perf", pragmas.production)
                var start;
                if (that.options.debug) {
                    start = window.performance.now();
                }
                //>>excludeEnd("perf")

                this.clipboard.records = _.clone(options.records);
                delete this.clipboard.childRecords;
                this.clipboard.mode = 'cut';
                this.clipboard.scope = 'row';
                this.clipboard.hideStyling = false;

                var recordIds = [];
                _.forEach(options.records, function (record) {
                    recordIds.push(that._getRecordId(record));
                });

                this.view.applyCutCopyClassesToRecords(recordIds);

                /**
                 * Triggered when rows are cut from the grid.
                 * This event is captured after certain action takes place on the grid.
                 *
                 * @event pgbu.grid#rows-cut
                 * @param {Object} [event=null]
                 * @param {Object} triggerArgs
                 * @param {Array<Object>} triggerArgs.records - Records to be 'cut' from the grid.
                 * @param {Object} triggerArgs.args - Additional arguments.
                 */
                this._trigger('rows-cut', null, triggerArgs);
                var actions = {
                    pasteRows : this._checkAction('pasteRows')
                };

                this._trigger('action-state-update', null, actions);
                //>>excludeStart("perf", pragmas.production)
                if (that.options.debug) {
                    that._trigger('performance-debug', null,
                        {
                            start : start,
                            end : window.performance.now(),
                            name : 'cut-Rows',
                            type : 'widget',
                            info : recordIds
                        });
                }
                //>>excludeEnd("perf")
            }
        },

        /**
         * Method used to store the selected records to the 'clipboard', so that they can be added, or 'pasted', somewhere else in the grid.
         * @method grid#copyRows
         * @param {Object} options
         * @param {Array<Object>} options.records - Records to be copied.
         * @param {Object} options.focus - Record with focus in the grid.
         * @public
         * @fires grid#event:before-copy-rows
         * @fires grid#event:rows-copied
         * @fires grid#event:action-state-update
         * @fires grid#event:performance-debug
         */
        // First clones all the current records, this is so when paste occurs it will paste the old values.
        // They are still cloned again on paste as you can paste multiple times.
        copyRows : function (options) {
            var that = this;
            options = this._prepareOptions(options, 'copyRows');
            var triggerArgs = {
                records : options.records,
                args : options
            };

            /**
             * Triggered before rows are copied.
             * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
             * @event pgbu.grid#before-copy-rows
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Array<Object>} triggerArgs.records - Records to be copied.
             * @param {Object} triggerArgs.args - Additional arguments.
             */
            if (that._checkAction('copyRows', this._preCheckActions(options.records, options.focus)) &&
                    that._trigger('before-copy-rows', null, triggerArgs) !== false) {
                //>>excludeStart("perf", pragmas.production)
                var start;
                if (that.options.debug) {
                    start = window.performance.now();
                }
                //>>excludeEnd("perf")
                var clonedChildRecords;
                var clonedIds = [];
                // Function to help with cloning all the children of a record
                var cloneChildren = function (record, recordId) {
                    var childrenToClone = that._getChildRecords(recordId);
                    if (childrenToClone != null && childrenToClone.length) {
                        clonedChildRecords[recordId] = [];
                        _.forEach(childrenToClone, function (child) {
                            clonedChildRecords[recordId].push(_.cloneDeep(child));
                            var childId = that._getRecordId(child);
                            clonedIds.push(childId);
                            cloneChildren(child, childId);
                        });
                    }
                };
                var clonedRecords = [];

                options.records = that.view.orderByViewIndex(options.records);
                _.forEach(options.records, function (record) {
                    var recordId = that._getRecordId(record);
                    // Make sure you do not copy something that has already been cloned, this it to fix
                    // copy of a parent and its child.
                    if (!_.includes(clonedIds, recordId)) {
                        clonedRecords.push(_.cloneDeep(record));
                        clonedIds.push(recordId);
                        if (options.deepCopy) { // If requested clone the children
                            if (that.options.lazyload) {
                                logger.warn('Deep clone and lazy loading should only be used in conjunction if you are cloning already loaded records. Otherwise results may not be desirable.');
                            }
                            clonedChildRecords = clonedChildRecords == null ? {} : clonedChildRecords;
                            cloneChildren(record, recordId);
                        }
                    }
                });

                that.clipboard.records = clonedRecords;
                that.clipboard.childRecords = clonedChildRecords;
                this.clipboard.copiedRecordMap = {};
                that.clipboard.mode = 'copy';
                that.clipboard.scope = 'row';
                this.clipboard.hideStyling = false;
                var recordIds = [];
                _.forEach(options.records, function (record) {
                    recordIds.push(that._getRecordId(record));
                });

                that.view.applyCutCopyClassesToRecords(recordIds);

                /**
                 * Triggered when rows are copied.
                 * This event is captured after certain action takes place on the grid.
                 *
                 * @event pgbu.grid#rows-copied
                 * @param {Object} [event=null]
                 * @param {Object} triggerArgs
                 * @param {Array<Object>} triggerArgs.records - Records to be copied.
                 * @param {Object} triggerArgs.args - Additional arguments.
                 */
                that._trigger('rows-copied', null, triggerArgs);
                var actions = {
                    pasteRows : this._checkAction('pasteRows')
                };
                that._trigger('action-state-update', null, actions);
                //>>excludeStart("perf", pragmas.production)
                if (that.options.debug) {
                    that._trigger('performance-debug', null,
                        {
                            start : start,
                            end : window.performance.now(),
                            name : 'copy-Rows',
                            type : 'widget',
                            info : recordIds
                        });
                }
                //>>excludeEnd("perf")
            }
        },

        /**
         * Method used to add, or 'paste', records into the grid, that are stored in the 'clipboard'.
         *
         * @method grid#pasteRows
         * @param {Object} options
         * @param {Boolean} options.isChild
         * @param {Object} options.focus - Record with focus in the grid.
         * @param {Array<Object>} options.records - Records to be pasted.
         * @public
         * @fires grid#event:before-paste-rows
         * @fires grid#event:copied-rows-pasted
         * @fires grid#event:rows-pasted
         */
        pasteRows : bulkModificationOperationDecorator(function pasteRows(options) {
            var that = this;
            options = that._prepareOptions(options, 'pasteRows');

            // LUX-1990 keyboard Ctrl-V on column filter input field will cause the grid trigger the pasteRows action, and thus cause the exception.
            // The fix is to ignore the case where there is no row selected and focus when pasteRows is triggered.
            if (options && !options.focus && !options.records.length) {
                return;
            }

            var records = this.clipboard.records;
            var isChild = options != null ? options.isChild : false;
            var initialDepth = that._getNodeLevel(options.focus) + (isChild ? 1 : 0);
            var triggerArgs = {
                records : records,
                mode : that.clipboard.mode,
                focus : options.focus,
                args : options
            };

            /**
             * Triggered before rows are pasted.
             * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
             *
             * @event pgbu.grid#before-paste-rows
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Array<Object>} triggerArgs.records - Records to be pasted.
             * @param {String} triggerArgs.mode - Mode of the clipboard.  Will be 'cut' or 'copy'.
             * @param {Object} triggerArgs.focus - Record with focus in the grid.
             * @param {Object} triggerArgs.args - Additional arguments.
             */
            if (that._checkAction('pasteRows', this._preCheckActions(options.records, options.focus)) &&
                    that._trigger('before-paste-rows', null, triggerArgs) !== false) {
                if (that.clipboard.mode === 'copy') {
                    var copiedChildren = {};
                    var clonedRecords = [];
                    var copiedRecordMap = {};
                    var copiedFromStore = [];
                    // If deep clone of an entire structure handle copying the children
                    if (that.clipboard.childRecords != null) {
                        // Clone the children and set the correct parent ids, iterate over all children
                        var cloneChildren = function (oldParentId, newParentId, parentClone, depth) {
                            var recordChildren = that.clipboard.childRecords[oldParentId];
                            if (recordChildren != null && recordChildren.length) {
                                copiedChildren[newParentId] = [];
                                _.forEach(recordChildren, function (recordChild) {
                                    var childClone = _.first(that.store.copyRecords([recordChild], options, parentClone));
                                    if (that.store.linkCopiedRecords != null) {
                                        that.store.linkCopiedRecords({
                                            dataIndex : that.recordKeys.parentId,
                                            childClone : childClone,
                                            parentClone : parentClone,
                                            parentId : newParentId
                                        });
                                    } else {
                                        that.store.setRecordValue(childClone, that.recordKeys.parentId, newParentId, { parentClone : parentClone });
                                    }
                                    copiedChildren[newParentId].push(childClone);
                                    var cloneId = that._getRecordId(childClone);
                                    var originalId = that._getRecordId(recordChild);

                                    copiedRecordMap[originalId] = cloneId;
                                    copiedRecordMap[cloneId] = originalId;
                                    if (depth < (that.options.treeMaxLevel || 1000)) {
                                        cloneChildren(originalId, cloneId, childClone, depth + 1);
                                    }
                                });
                            }
                        };
                        // Clone all the currently cloned root nodes and all their children.
                        _.forEach(records, function (record) {
                            var recordClone = _.first(that.store.copyRecords([record], options, options.focus));
                            clonedRecords.push(recordClone);
                            var recordId = that._getRecordId(record);
                            var cloneId = that._getRecordId(recordClone);
                            copiedRecordMap[recordId] = cloneId;
                            copiedRecordMap[cloneId] = recordId;
                            if (initialDepth < (that.options.treeMaxLevel || 1000)) {
                                cloneChildren(recordId, cloneId, recordClone, initialDepth + 1);
                            }
                        });
                        records = clonedRecords;
                    } else {
                        // Clone all the currently cloned root nodes and all their children.
                        _.forEach(records, function (record) {
                            copiedFromStore = that.store.copyRecords([record], options, options.focus);
                            var recordClone = _.first(copiedFromStore);
                            clonedRecords = clonedRecords.concat(copiedFromStore);
                            var recordId = that._getRecordId(record);
                            var cloneId = that._getRecordId(recordClone);
                            copiedRecordMap[recordId] = cloneId;
                            copiedRecordMap[cloneId] = recordId;
                        });
                        records = clonedRecords;
                    }
                    that.clipboard.copiedRecordMap = copiedRecordMap;
                } else if (that.clipboard.mode === 'cut' && that.options.data || that.options.tree) {
                    var parentsMap = {};
                    var depth = initialDepth;
                    var cutRecords = [];
                    var maxDepth = false;
                    var exceedsMaxDepth = function (record, currentDepth) {
                        if (maxDepth) {
                            return;
                        }

                        var children = that._getChildRecords(that._getRecordId(record));
                        if (children.length) {
                            _.forEach(children, function (child) {
                                if (!maxDepth) {
                                    exceedsMaxDepth(child, currentDepth + 1);
                                }
                            });
                        } else if (currentDepth > that.options.treeMaxLevel) {
                            maxDepth = true;
                        }
                    };
                    for (var i = 0; i < records.length; i++) {
                        var record = records[i];
                        var key = _.get(record, that.options.recordKeys.id);
                        var parentKey = _.get(that._getRecordPlacement(record), 'parentId');
                        _.set(parentsMap, key, parentKey);
                    }
                    records = _.reject(records, function (record) {
                        var parentKey = _.get(that._getRecordPlacement(record), 'parentId');
                        return _.has(parentsMap, parentKey);
                    });
                    if (that.options.treeMaxLevel != null) {
                        _.forEach(records, function (record) {
                            exceedsMaxDepth(record, depth);
                            if (!maxDepth) {
                                cutRecords.push(record);
                            }
                            maxDepth = false;
                        });
                        records = cutRecords;
                    }
                }
                var lastRecordId;
                _.forEach(records, function (record, length) {
                    if (that.clipboard.mode === 'cut') {
                        that.clipboard.records = [];
                        that._setDeletedRecordSiblings(record);
                    }

                    var oldParentId = that.store.getValueFromRecord(record, that.recordKeys.parentId);

                    var handlePaste = function () {
                        var recordType;
                        var parentIdArgs = {
                            record : record,
                            focus : options.focus,
                            isChild : options.isChild
                        };
                        if (that.options.data != null && that.options.tree && that.clipboard.mode === 'copy') {
                            recordType = that._getTypeFromId(that.clipboard.copiedRecordMap[that._getRecordId(record)]);
                            parentIdArgs.childType = recordType;
                        }

                        var newParentId = that._setNewRecordParent(parentIdArgs);
                        var recordAbove, recordBelow;
                        var location = that._getRecordPlacement(record);
                        // Apply sibling id
                        if (location.parentId != null && location.parentId === that._getRecordId(options.focus)) {
                            recordAbove = that.view.lastChild(location.parentId, null, that._attrs.tree(record).precedence);
                            if (recordAbove === that._getRecordId(record)) {
                                recordAbove = that.view.previousSibling(recordAbove);
                                recordBelow = null;
                            } else {
                                recordBelow = recordAbove != null ? that.view.nextSibling(recordAbove) : null;
                            }
                        } else {
                            recordAbove = lastRecordId || that._getRecordId(options.focus);
                            recordBelow = that.view.nextSibling(recordAbove);

                            if (that._getRecordId(options.focus) === constants.DUMMY_RECORD_ID) {
                                recordBelow = recordAbove;
                                recordAbove = that.view.previousSibling(recordAbove);
                            }

                            if (recordBelow === that._getRecordId(record)) {
                                recordBelow = that.view.nextSibling(that._getRecordId(record));
                            }
                        }

                        lastRecordId = that._getRecordId(record);

                        that._setSiblingIds(record, recordAbove, recordBelow);
                        that._setRecordSequence(record, recordAbove, recordBelow);

                        if (that.clipboard.mode === 'copy') {
                            if (that.groupingApplied) {
                                that.bypassRowAdd = true;
                                that.store.addRecords([record]);
                                that.recordGrouper.setGroupBandValues(record);
                                that.bypassRowAdd = false;
                                that._recordsAdded([record]);
                            } else if (that.options.data != null && that.options.tree && that.store.addRecordsByType != null) {
                                that.store.addRecordsByType(recordType, [record]);
                            } else {
                                that.store.addRecords([record]);
                            }
                            // If you had a deep copy need to paste all the child clones into the store
                            if (copiedChildren[lastRecordId] != null) {
                                var addChildrenToStore = function (record) {
                                    var childrenToAdd = copiedChildren[that._getRecordId(record)];
                                    if (childrenToAdd != null && childrenToAdd.length) {
                                        that.store.addRecords(childrenToAdd);
                                        for (var i = 0; i < childrenToAdd.length; i++) {
                                            addChildrenToStore(childrenToAdd[i]);
                                        }
                                    }
                                };
                                // Do not redraw the rows of the children or focus them.
                                that.bypassRowAdd = true;
                                addChildrenToStore(record);
                                that._updateRecordAndNode(lastRecordId);
                                that.bypassRowAdd = false;
                            }

                            /**
                             * @event grid#copied-rows-pasted
                             * @param {Object} [event=null]
                             * @param {Array<Object>} records - Records that were in the clipboard, or 'copied', and were added, or 'pasted', into the grid.
                             */
                            that._trigger('copied-rows-pasted', null, {
                                copiedRecordMap : that.clipboard.copiedRecordMap,
                                records : [records],
                                childRecords : that.clipboard.childRecords
                            });
                        } else {
                            if (that.groupingApplied) {
                                that.recordGrouper.setGroupBandValues(record);
                            }
                            that.store.recordMoved(record, recordAbove, newParentId, oldParentId);
                        }

                        if (that.clipboard.mode === 'cut') {
                            that.selectCell(that._getRecordId(record));
                        }
                        if (length + 1 === records.length) {
                            that.selectRows(that._getRecordIds(records));
                        }
                    };
                    var expanded = true;
                    if (isChild) {
                        var parent, parentId;
                        if (that.options.tree || that.groupingApplied) {
                            // Apply Parent Id
                            // If this should be a child of the focused record.
                            parent = options.focus;
                            if (that._isGroupRecord(parent)) {
                                while (parent != null && that._isGroupRecord(parent)) {
                                    parent = that.recordGrouper.recordGroups[parent.parentId];
                                }
                                if (parent != null) {
                                    parent = that._getRecord(parent);
                                }
                            }
                            if (parent != null) {
                                parentId = that._getRecordId(parent);
                            }
                            // If this is should be sibling of the focused record, set parentId to the siblings parent
                            expanded = that.view.ensureExpanded({
                                nodeId : parentId,
                                rowsetId : that._getRecordRowset(parentId)
                            });
                        }
                    }
                    that.clipboard.hideStyling = true;
                    if (_.isBoolean(expanded)) {
                        handlePaste();
                    } else {
                        expanded.done(function () {
                            handlePaste();
                        });
                    }
                });
                that.view.clearCutCopyRows();

                if (that.clipboard.mode === 'cut' && options.autoCopy) {
                    that.copyRows({
                        records : records
                    });
                }

                /**
                 * Triggered when rows are pasted.
                 * This event is captured after certain action takes place on the grid.
                 *
                 * @event pgbu.grid#rows-pasted
                 * @param {Object} [event=null]
                 * @param {Object} triggerArgs
                 * @param {Array<Object>} triggerArgs.records - Records to be pasted.
                 * @param {String} triggerArgs.mode - Mode of the clipboard.  Will be 'cut' or 'copy'.
                 * @param {Object} triggerArgs.focus - Record with focus in the grid.
                 * @param {Object} triggerArgs.args - Additional arguments.
                 */
                that._trigger('rows-pasted', null, triggerArgs);
            }
        }),

        /**
         * Method called when moving a record left in the grid, or up a level in a hierarchy.
         *
         * Note: This method only applies to hierarchical (Tree) grids.  Additionally, this method can be used on one or more records.
         * @method grid#moveRowLeft
         * @param {Object} options
         * @param {Array<Object>} options.records - Records to move left.
         * @param {Object} options.focus - Record with focus in the grid.
         * @public
         * @fires grid#event:before-move-row-left
         * @fires grid#event:action-state-update
         */
        moveRowLeft : bulkModificationOperationDecorator(function moveRowLeft(options) {
            var that = this;
            options = this._prepareOptions(options, 'moveRowLeft');
            var triggerArgs = {
                args : options
            };

            /**
             * Triggered before rows are moved left on a tree grid
             * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
             *
             * @event pgbu.grid#before-move-row-left
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Object} triggerArgs.args
             * @param {Array<Object>} triggerArgs.args.records - Records to move left.
             * @param {Object} triggerArgs.args.focus - Record with focus in the grid.
             */
            if (that._checkAction('moveRowLeft', this._preCheckActions(options.records, options.focus)) &&
                    that._trigger('before-move-row-left', null, triggerArgs) !== false) {
                var records = this.view.orderByViewIndex(options.records);
                this.recordsBeingMoved = records;
                var firstRecordId = this._getRecordId(_.first(records));
                var parentId = this.view.parentRecordId(firstRecordId);
                var newParentId = this.view.parentRecordId(parentId);
                var recordBelowId = this.view.nextSibling(parentId);
                var recordAboveId = this.view.getMoveLeftSibling(firstRecordId, parentId);
                for (var i = 0; i < records.length; i++) {
                    var recordToMove = records[i];
                    // Fix old sibling relationships
                    this._setDeletedRecordSiblings(recordToMove);

                    // Setup new sibling relationships
                    this._setSiblingIds(recordToMove, recordAboveId, recordBelowId);
                    this._setRecordSequence(recordToMove, recordAboveId, recordBelowId);
                    // Setup new parent relationship
                    newParentId = this._setNewRecordParent({
                        record : recordToMove,
                        focus : newParentId ? this._getRecord(newParentId) : null,
                        isChild : true
                    });

                    // Tell the store what has been done
                    this.store.recordMoved(recordToMove, recordAboveId, newParentId, parentId);

                    this._updateRecordAndNode(parentId);
                    this._updateRecordAndNode(newParentId);
                    recordAboveId = this._getRecordId(recordToMove);
                }

                this._trigger('action-state-update', null, this._checkActions());
            }
            this.recordsBeingMoved = [];
        }),

        /**
         * Method called when moving a record right in the grid, or down a level in a hierarchy.
         *
         * Note: This method only applies to hierarchical (Tree) grids.  Additionally, this method can be used on one or more records.
         * @method grid#moveRowRight
         * @param {Object} options
         * @param {Array<Object>} options.records - Records to move right.
         * @param {Object} options.focus - Record with focus in the grid.
         * @public
         * @fires grid#event:before-move-row-right
         */
        moveRowRight : bulkModificationOperationDecorator(function moveRowRight(options) {
            var that = this;
            options = this._prepareOptions(options, 'moveRowRight');
            var triggerArgs = {
                args : options
            };

            var records = this.view.orderByViewIndex(options.records);
            this.recordsBeingMoved = records;
            var parentId = this.view.parentRecordId(this._getRecordId(_.first(records)));
            var newParentId = this.view.previousSibling(this._getRecordId(_.first(records)));

            triggerArgs.newParentId = newParentId;

            /**
             * Triggered before rows are moved right on a tree grid.
             * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
             *
             * @event pgbu.grid#before-move-row-right
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Object} triggerArgs.args
             * @param {Array<Object>} triggerArgs.args.records - Records to move right.
             * @param {Object} triggerArgs.args.focus - Record with focus in the grid.
             */
            if (that._checkAction('moveRowRight', this._preCheckActions(options.records, options.focus)) &&
                    that._trigger('before-move-row-right', null, triggerArgs) !== false) {
                var expanded = this.view.ensureExpanded({
                    nodeId : newParentId,
                    rowsetId : this._getRecordRowset(newParentId)
                });
                if (_.isBoolean(expanded)) {
                    this._handleMoveRight(records, newParentId, parentId);
                } else {
                    expanded.done(function () {
                        that._handleMoveRight(records, newParentId, parentId);
                    });
                }
            }
            this.recordsBeingMoved = [];
        }),

        _handleMoveRight : function (records, newParentId, parentId) {
            for (var i = 0; i < records.length; i++) {
                var recordToMove = records[i];

                var recordBelowId = null;// No child below since we add to the bottom;
                var recordAboveId = this.view.lastChild(newParentId, null, this._attrs.tree(recordToMove).precedence);

                if (recordAboveId == null) {
                    recordAboveId = newParentId;
                }

                this._setDeletedRecordSiblings(recordToMove);

                // Setup new sibling relationships
                this._setSiblingIds(recordToMove, recordAboveId, recordBelowId);
                this._setRecordSequence(recordToMove, recordAboveId, recordBelowId);
                // Setup new parent relationship
                newParentId = this._setNewRecordParent({
                    record : recordToMove,
                    focus : newParentId ? this._getRecord(newParentId) : null,
                    isChild : true
                });

                // Tell the store what has been done
                this.store.recordMoved(recordToMove, recordAboveId, newParentId, parentId);

                this._updateRecordAndNode(parentId);
                this._updateRecordAndNode(newParentId, true);
            }
            this._trigger('action-state-update', null, this._checkActions());
        },

        /**
         * Method called when moving a record in a grid.  This only applies to hierarchical (Tree) grids and grids that are sequenced.
         *
         * Note: This method can be called on one or many records.
         * @method grid#moveRowUp
         * @param {Object} options
         * @param {Array<Object>} options.records - Records to move up in the grid.
         * @param {Object} options.focus - Record with focus in the grid.
         * @public
         * @fires grid#event:before-move-row-up
         * @fires grid#event:action-state-update
         */
        moveRowUp : bulkModificationOperationDecorator(function moveRowUp(options) {
            var that = this;
            options = this._prepareOptions(options, 'moveRowUp');
            var triggerArgs = {
                args : options
            };

            /**
             * Triggered before rows are moved up a level.
             * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
             *
             * @event pgbu.grid#before-move-row-up
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Object} triggerArgs.args
             * @param {Array<Object>} triggerArgs.args.records - Records to move up in the grid.
             * @param {Object} triggerArgs.args.focus - Record with focus in the grid.
             */
            if (that._checkAction('moveRowUp', this._preCheckActions(options.records, options.focus)) &&
                    that._trigger('before-move-row-up', null, triggerArgs) !== false) {
                var records = this.view.orderByViewIndex(options.records);
                this.recordsBeingMoved = records;
                var recordBelow = that.view.previousSibling(that._getRecordId(_.first(records))),
                    recordAbove = recordBelow != null ? that.view.previousSibling(recordBelow) : null;
                _.forEach(records, function (record) {
                    var recordId = that._getRecordId(record);
                    // If the record below is nothing, or itself, it is already as high as it can go
                    if (recordBelow != null && recordBelow !== recordId) {
                        that._setDeletedRecordSiblings(record);

                        that._setSiblingIds(record, recordAbove, recordBelow);
                        that._swapSequenceNumbers(record, that._getRecord(recordBelow));
                        that.store.recordMoved(record, recordAbove);
                    } else {
                        recordBelow = that.view.nextSibling(recordId);
                    }
                    // Set the following records to have this record as the record above it, making them contiguous
                    recordAbove = that._getRecordId(record);
                });
                this.recordsBeingMoved = [];
                this._trigger('action-state-update', null, this._checkActions());
            }
        }),

        /**
         * Method called when moving a record down in a grid.  This only applies to hieararchical (Tree) grids and grids that are sequenced.
         *
         * Note: This method can be called on one or many records.
         * @method grid#moveRowDown
         * @param {Object} options
         * @param {Array<Object>} options.records - Records to be moved down in the grid.
         * @param {Object} options.focus - Record with focus in the grid.
         * @public
         * @fires grid#event:before-move-row-down
         * @fires grid#event:action-state-update
         */
        moveRowDown : bulkModificationOperationDecorator(function moveRowDown(options) {
            var that = this;
            options = this._prepareOptions(options, 'moveRowDown');
            var triggerArgs = {
                args : options
            };

            /**
             * Triggered before rows are moved down a level.
             * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
             *
             * @event pgbu.grid#before-move-row-down
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Object} triggerArgs.args
             * @param {Array<Object>} triggerArgs.args.records - Records to be moved down in the grid.
             * @param {Object} triggerArgs.args.focus - Record with focus in the grid.
             */
            if (that._checkAction('moveRowDown', this._preCheckActions(options.records, options.focus)) &&
                    that._trigger('before-move-row-down', null, triggerArgs) !== false) {
                var records = this.view.orderByViewIndex(options.records).reverse();
                this.recordsBeingMoved = records;
                var recordAbove = that.view.nextSibling(that._getRecordId(_.first(records)));
                var recordBelow = recordAbove != null ? that.view.nextSibling(recordAbove) : null;
                _.forEach(records, function (record) {
                    var recordId = that._getRecordId(record);
                    // If the record above is nothing, or itself, it is already as low as it can go
                    if (recordAbove != null && recordAbove !== recordId) {
                        that._setDeletedRecordSiblings(record);
                        that._setSiblingIds(record, recordAbove, recordBelow);
                        that._swapSequenceNumbers(that._getRecord(recordAbove), record);
                        that.store.recordMoved(record, recordAbove);
                    } else {
                        recordAbove = that.view.previousSibling(recordId);
                    }

                    // Set the following records to have this record as the record below it, making them contiguous
                    recordBelow = recordId;
                });
                this.recordsBeingMoved = [];
                this._trigger('action-state-update', null, this._checkActions());
            }
        }),

        /**
         * Method used to expand all rows in the grid.  This method only applies to hierarchical (Tree) grids.
         *
         * Note: You can't use this method with lazy loaded grids.
         * @method grid#expandAllRows
         * @returns {DeferredObject} - Resolved deferred object.
         */
        expandAllRows : attrFns.decorateCache('_attrs', 'gridWidget#expandAllRows', function expandAllRows() {
            var deferred = $.Deferred();
            if (!this.options.lazyload) {
                this.expanding = true;
                this.repopulate();
                this.expanding = false;
                this._checkExpandCollapseState();
            } else {
                deferred.resolve();
            }
            return deferred;
        }),

        /**
         * Used to collapse all rows in the grid.  This method only applies to hierarchical (Tree) grids.
         * @method grid#collapseAllRows
         */
        collapseAllRows : function () {
            this.view.collapseAllNodes();
            this._checkExpandCollapseState();
        },

        /**
         * Used to expand a single row in the grid.  This method only applies to hierarchical (Tree) grids.
         * @method grid#expandRow
         * @param {String} recordId - Id of the record to expand.  Should be the selected record in the grid.
         * @param {Boolean} expandChildren - Whether or not to expand any child nodes of the record.
         * @public
         */
        expandRow : attrFns.decorateCache('_attrs', 'gridWidget#expandRows', function expandRow(recordId, expandChildren) {
            var options = {
                expandChildren : expandChildren
            };
            if (recordId != null) { // Convert the old options
                options.focus = this._getRecord(recordId);
                options.records = [options.focus];
            }
            // Call the newly named method for better naming convention
            return this.expandRows(options);
        }),

        /**
         * Used to expand multiple rows in the grid.  This method only applies to hierarchical (Tree) grids.
         * @method grid#expandRows
         * @param {Object} options
         * @param {Array<Object>} options.records - Records to expand in the grid.
         * @param {Object} options.focus - Record with focus in the grid.
         * @param {Boolean} options.expandChildren - Whether or not to expand any child nodes of the records.
         * @returns {Boolean} Whether or not the rows are now expanded.
         * @public
         */
        expandRows : attrFns.decorateCache('_attrs', 'gridWidget#expandRows', function expandRows(options) {
            options = this._prepareOptions(options, 'expandRows');
            var expanded;
            if (this._checkAction('expandRows', this._preCheckActions(options.records, options.focus), options)) {
                for (var i = 0; i < options.records.length; i++) {
                    var recordId = this._getRecordId(options.records[i]);
                    expanded = this.view.ensureExpanded({
                        nodeId : recordId,
                        rowsetId : this._getRecordRowset(recordId),
                        expandChildren : !this.options.lazyload && options.expandChildren
                    });
                }
                this.selectCell(this._getRecordId(options.focus));
                this._checkExpandCollapseState();
                return expanded;
            }
            return false;
        }),

        /**
         * Used to collapse a single record in the grid.  This method only applies to hierarchical (Tree) grids.
         * @method grid#collapseRow
         * @param {String} recordId - ID of the record to collapse in the grid.
         * @param {Boolean} collapseChildren - Whether or not to collapse child nodes of the record.
         * @public
         */
        collapseRow : function (recordId, collapseChildren) {
            var options = {
                collapseChildren : collapseChildren
            };
            if (recordId != null) { // Convert the old options
                options.focus = this._getRecord(recordId);
                options.records = [options.focus];
            }
            // Call the newly named method for better naming convention
            this.collapseRows(options);
        },

        /**
         * Used to collapse multiple records in the grid.  This method only applies to hierarchical (Tree) grids.
         * @method grid#collapseRows
         * @param {Object} options
         * @param {Array<Object>} options.records - Records to collapse in the grid.
         * @param {Object} options.focus - Record with focus in the grid.
         * @param {Boolean} options.collapseChildren - Whether or not to collapse child nodes of the records.
         * @returns {Boolean} Whether or not the rows are now collapsed.
         * @public
         */
        collapseRows : function (options) {
            options = this._prepareOptions(options, 'collapseRows');

            if (this._checkAction('collapseRows', this._preCheckActions(options.records, options.focus), options)) {
                for (var i = 0; i < options.records.length; i++) {
                    var recordId = this._getRecordId(options.records[i]);
                    this.view.collapseNode(recordId, options.collapseChildren);
                    this.view.updateRow(recordId, this._getRecordRowset(recordId));
                }
                this.resize();
                this.selectCell(this._getRecordId(options.focus));
                this._checkExpandCollapseState();
                return true;
            }
            return false;
        },

        /**
         * Used to check if the expandAll or collapseAll actions should be enabled/disabled.
         * @method grid#_checkExpandCollapseState
         * @private
         * @fires grid#event:action-state-update
         */
        _checkExpandCollapseState : function () {
            this._trigger('action-state-update', null, {
                expandAllRows : this._checkAction('expandAllRows'),
                collapseAllRows : this._checkAction('collapseAllRows')
            });
        },

        /**
         * Determines if the record cell should be editable or not.
         * @method grid#canEditCell
         * @param {String} recordId - ID of the record to edit.
         * @param {String} columnId - ID of the column to edit.
         * @param {grid#defaultRecordAttributes} recordAttributes - attributes on the record.
         * @param {String} subRow - Data Index of the stacked cell.  Only applies to stacked cell grids.
         * @returns {Boolean} Whether or not the cell is editable.
         * @public
         */
        canEditCell : function (recordId, columnId, recordAttributes, subRow, fromChangeEvent) {
            if (recordAttributes == null) {
                recordAttributes = this._attrs.nonSelection(this._getRecord(recordId));
            }
            // If a group row, grid is uneditable, or the cell on the record is readOnly don't open the editor
            var column, masterColumn;
            if (subRow != null) {
                column = this.columnDefinitions[subRow];
                masterColumn = this.columnDefinitions[columnId];
            } else {
                column = this.columnDefinitions[columnId];
            }

            var isGroupRecord = this._isGroupRecordId(recordId);
            var canEdit = true;

            // Group record or grid not editable
            if (isGroupRecord || !this.options.editable) {
                canEdit = false;
            }
            // Record not editable
            else if (recordAttributes.editable != null && !recordAttributes.editable) {
                canEdit = false;
            }
            // Column not editable
            else if ((!column.enableEditor && !fromChangeEvent) &&
                !(recordAttributes.editable && !_.includes(recordAttributes.readOnly, columnId) && column.displayType === 'check')) {
                canEdit = false;
            }
            // Column locked
            else if (column.locked) {
                canEdit = false;
            }
            // Cell restricted from recordAttributes
            else if (_.includes(recordAttributes.readOnly, column.dataIndex)) {
                canEdit = false;
            }
            // Cell locked from recordAttributes
            else if (_.includes(recordAttributes.locked, column.dataIndex)) {
                canEdit = false;
            }

            if (recordAttributes.rollup && column.rollupRole != null) {
                canEdit = false;
            }

            // Timephased checking
            if (masterColumn != null && masterColumn.timephased) {
                // Bucket column is readOnly
                if (!masterColumn.enableEditor) {
                    canEdit = false;
                }
                // Bucket column is locked
                else if (masterColumn.locked) {
                    canEdit = false;
                }
                // Stacked Row is readOnly for this row
                else if (_.includes(recordAttributes.timephased.readOnlyStacked, column.dataIndex)) {
                    canEdit = false;
                }
                // Bucket is readOnly for this row
                else if (_.includes(recordAttributes.timephased.readOnlyBuckets, masterColumn.bucketId)) {
                    canEdit = false;
                }
                // Bucket is read only for this subRow
                else if (recordAttributes.timephased.readOnlyMatrix.bySubRow[column.dataIndex] != null &&
                    _.includes(recordAttributes.timephased.readOnlyMatrix.bySubRow[column.dataIndex], masterColumn.bucketId)) {
                    canEdit = false;
                }
                // SubRow is readOnly for this bucket
                else if (recordAttributes.timephased.readOnlyMatrix.byBucket[masterColumn.bucketId] != null &&
                    _.includes(recordAttributes.timephased.readOnlyMatrix.byBucket[masterColumn.bucketId], column.dataIndex)) {
                    canEdit = false;
                }
                // Stacked Row is locked for this row
                else if (_.includes(recordAttributes.timephased.lockedStacked, column.dataIndex)) {
                    canEdit = false;
                }
                // Bucket is locked for this row
                else if (_.includes(recordAttributes.timephased.lockedBuckets, masterColumn.bucketId)) {
                    canEdit = false;
                }
            } else if (masterColumn != null) {
                // Bucket column is readOnly
                if (!masterColumn.enableEditor) {
                    canEdit = false;
                }
                // Bucket column is locked
                else if (masterColumn.locked) {
                    canEdit = false;
                }
                // Sub Row is readOnly for this row
                else if (_.includes(recordAttributes.stackedCells.readOnlyStacked, column.dataIndex)) {
                    canEdit = false;
                }
                // Column is readOnly for this row
                else if (_.includes(recordAttributes.stackedCells.readOnlyColumns, masterColumn.dataIndex)) {
                    canEdit = false;
                }
                // Column is read only for this subRow
                else if (recordAttributes.stackedCells.readOnlyMatrix.bySubRow[column.dataIndex] != null &&
                    _.includes(recordAttributes.timephased.readOnlyMatrix.bySubRow[column.dataIndex], masterColumn.dataIndex)) {
                    canEdit = false;
                }
                // SubRow is readOnly for this bucket
                else if (recordAttributes.stackedCells.readOnlyMatrix.byColumn[masterColumn.dataIndex] != null &&
                    _.includes(recordAttributes.timephased.readOnlyMatrix.byColumn[masterColumn.dataIndex], column.dataIndex)) {
                    canEdit = false;
                }
                // SubRow is locked for this row
                else if (_.includes(recordAttributes.stackedCells.lockedStacked, column.dataIndex)) {
                    canEdit = false;
                }
                // Column is locked for this row
                else if (_.includes(recordAttributes.stackedCells.lockedColumns, masterColumn.dataIndex)) {
                    canEdit = false;
                }
            }

            if (!canEdit) {
                if (!column.hasDisabledMode || isGroupRecord) {
                    return false;
                } else if (!isGroupRecord && !_.includes(recordAttributes.locked, columnId)) {
                    return 'disabled';
                }
            }

            return true;
        },

        /**
         * Determines if a column for multiple selected records, can be edited.
         * @method grid#canEditCells
         * @param {Array<Object>} records - Records selected in the grid.
         * @param {String} columnId - ID of the column to edit.
         * @returns {boolean} Whether or not the specified column can be edited for the selected records.
         * @public
         */
        canEditCells : function (records, columnId) {
            for (var i = 0, len = records.length; i < len; ++i) {
                var record = records[i];
                var recordId = this._getRecordId(record);
                if (this.canEditCell(recordId, columnId, this._attrs.nonSelection(record)) === true) {
                    return true;
                }
            }
            return false;
        },

        /**
         * Used to edit values of the cells selected in the grid.  This method can be applied to one or many records and/or cells.
         * @method grid#editCells
         * @param {Object} options
         * @param {Array<Object>} options.records - Records selected in the grid.
         * @param {Object} options.focus - Record that has focus in the grid.
         * @param {String} options.dataIndex - Index of the field to edit in the record.
         * @param {String} options.columnId - ID of the of the column to edit in the grid.
         * @param {Boolean} options.hideErrors - Whether or not to hide any discovered errors, caused by editing the cells.
         * @public
         * @fires grid#event:before-edit-cells
         */
        editCells : function (options) {
            options = this._prepareOptions(options, 'editCells');

            var dataIndex = options.dataIndex || this.focus.columnId;
            var triggerArgs = { records : options.records, dataIndex : dataIndex };

            /**
             * @event grid#before-edit-cells
             * @param {Object} triggerArgs
             * @param {Array<Object>} triggerArgs.records - Records to edit.
             * @param {String} triggerArgs.dataIndex - Index of the field to edit in the record(s).
             */
            if (this._checkAction('editCells', this._preCheckActions(options.records, options.focus), options) &&
                    this._trigger('before-edit-cells', null, triggerArgs) !== false) {
                this.editRow({
                    columnId : options.columnId || this.focus.columnId,
                    recordId : this._getRecordId(options.focus),
                    hideErrors : options.hideErrors
                });
            }
        },

        /**
         * Used to edit the value of a specific column, for the record specified.
         * @method grid#editRow
         * @param {Object|String} options
         * @param {String} options.recordId - ID of the record to edit.
         * @param {String} options.columnId - ID of the column to edit.
         * @param {Boolean} options.hideErrors - Whether or not to hide any errors discovered, as a result of editing the row.
         * @param {Object} options.event - The event that triggered the call to editRow.
         * @param {Boolean} options.currentlyTyping - Whether or not the user is typing characters into the grid cell.
         * @param {String} options.subRow - Data index of the field corresponding to the stacked cell that is being edited.  Only applies to stacked cell grids.
         * @fires grid#event:before-quick-add-row
         * @fires grid#event:before-open-editor
         * @public
         */
        editRow : function (options) {
            if (strings.isString(options) || arguments.length > 1) {
                options = {
                    recordId : arguments[0],
                    columnId : arguments[1],
                    event : arguments[2],
                    hideErrors : arguments[3]
                };
            }
            if (this.options.selectTextOnEdit && options.selectText == null) {
                options.selectText = true;
            }

            var record = this._getRecord(options.recordId);
            var recordId = options.recordId;
            if (options.recordId === constants.DUMMY_RECORD_ID) {
                if (this._trigger('before-quick-add-row', null, {
                    columnId : options.columnId,
                    dataIndex : options.columnId
                }) === false) {
                    return;
                }
            }

            // If a group row, grid is uneditable, or the cell on the record is readOnly don't open the editor
            var recordAttributes = this._attrs.all(this._getRecord(options.recordId));
            var disabledMode;
            var canEdit = this.canEditCell(options.recordId, options.columnId, recordAttributes, options.subRow);
            if (canEdit === false) {
                return;
            }

            if (canEdit === 'disabled') {
                disabledMode = true;
            }

            var group = _.find(this.config.groups, function (g) {
                return _.find(g.columns, function (c) {
                    return c.dataIndex === options.columnId;
                }) != null;
            });
            var data;
            if (group.timephased) {
                data = this._getTimephasedValueFromRecord({
                    recordId : options.recordId,
                    record : record,
                    dataIndex : options.subRow,
                    bucketId : this.columnDefinitions[options.columnId].bucketId,
                    granularity : this.columnDefinitions[options.columnId].granularity
                });
            } else if (group.showStackedCells) {
                data = this._getStackedValueFromRecord(record, options.columnId, options.subRow);
            } else if (options.recordId === constants.DUMMY_RECORD_ID) {
                data = this.store.getValueFromRecord(this._getProxyRecord(), options.columnId);
            } else {
                data = this.store.getValueFromRecord(record, options.columnId);
            }

            var invalid = false;
            if (recordAttributes.invalid != null && recordAttributes.invalid[options.columnId] != null && recordAttributes.invalid[options.columnId].invalidFormat) {
                data = recordAttributes.invalid[options.columnId].invalidFormat.invalidData;
                invalid = true;
            }

            /**
             * @event grid#before-open-editor
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {String} triggerArgs.recordId - ID of the record to edit.
             * @param {Object} triggerArgs.record - Record to edit.
             * @param {String} triggerArgs.dataIndex - Index of the field to edit on the record.
             * @param {Object|Number|String} triggerArgs.currentValue - Current value of the field on the record.
             */
            if (this._trigger('before-open-editor', null, {
                recordId : options.recordId,
                record : record,
                dataIndex : options.columnId,
                currentValue : data
            }) === false) {
                return;
            }
            this.view.setEscapeMode(false);
            var editorOptions = {
                group : group,
                data : data,
                recordId : recordId,
                subRow : options.subRow,
                dataIndex : options.columnId,
                record : record,
                event : options.event,
                disabled : disabledMode,
                invalid : invalid,
                hideErrors : options.hideErrors,
                selectText : options.selectText,
                currentlyTyping : options.currentlyTyping,
                recordAttributes : recordAttributes,
                fromQuickAddRow : options.recordId === constants.DUMMY_RECORD_ID
            };
            this.view.openEditor(editorOptions);
        },

        requestBrowserFocus : function (force, event) {
            if (this.options.focusOnSelection || force === true) {
                if (this.options.focusOnSelection) {
                    this.view.setEscapeMode(false);
                }
                    /**
                     * This event is triggered before the grid element is focused.
                     * @event pgbu.grid#before-request-focus
                     * @param {Object} [event=null] The event object will be provided if there is a user action associated with the event.
                     */
                this._trigger('before-request-focus', event);
                this.view.requestFocus();
                    /**
                     * This event is triggered after the grid element is focused.
                     * @event pgbu.grid#after-request-focus
                     * @param {Object} [event=null] The event object will be provided if there is a user action associated with the event.
                     */
                this._trigger('after-request-focus', event);
            }
        },

        editNextCell : function (moveRight) {
            var that = this;
            var direction = moveRight ? 'moveRight' : 'moveLeft';
            var initialCell = this.getCurrentCell();
            var quickAdd = false;
            var cell;

            if (initialCell.recordId === constants.DUMMY_RECORD_ID) {
                this.moveDummyCell(moveRight);
                quickAdd = true;
            } else {
                cell = this.findEditableCell(direction);
            }

            if (quickAdd || !(cell.current.column === cell.previous.column && cell.current.recordId === cell.previous.recordId)) {
                this._editCell({
                    recordId : that.focus.recordId,
                    columnId : that.focus.columnId,
                    subRow : that.focus.subRow
                });
            }
        },

        _editCell : function (args) {
            var that = this;
            setTimeout(function () {
                that.editRow({
                    recordId : args.recordId,
                    columnId : args.columnId,
                    subRow : args.subRow,
                    selectText : true
                });
            }, 50);
        },

        moveDummyCell : function (moveRight) {
            var direction = moveRight ? 'moveRight' : 'moveLeft';
            var initialCell = this.getCurrentCell();
            var cell = this.findEditableCell(direction);
            if (((initialCell.recordId === constants.DUMMY_RECORD_ID) && moveRight) && this.shouldCommitQuickAdd(initialCell, cell)) {
                this._commitNewRecord({});
                this.selectCell(constants.DUMMY_RECORD_ID, this.view.firstVisibleColumn);
            }
        },

        shouldCommitQuickAdd : function (initialCell, cell) {
            return cell.current.recordId !== constants.DUMMY_RECORD_ID ||
                    cell.previous.column === cell.current.column ||
                    cell.previous.column === this.view.lastVisibleColumn ||
                    (initialCell.column === this.view.lastVisibleColumn && cell.current.column !== initialCell.column);
        },

        findEditableCell : function (direction) {
            var previous;
            var current;
            do {
                previous = this.getCurrentCell();
                this.view.selectionModel.cell[direction](true);
                current = this.getCurrentCell();
            } while (!this.canEditCell(current.recordId, current.column) &&
                ((previous.column !== current.column) || (previous.recordId !== current.recordId)));
            return {
                previous : previous,
                current : current
            };
        },

        getCurrentCell : function () {
            return {
                column : this.focus.columnId,
                recordId : this.focus.recordId
            };
        },

        /**
         * Used to select a specific cell in the grid.  This is called whenever the user clicks on a cell in the grid.
         * @method grid#selectCell
         * @param {String} recordId - ID of the record (row) you are selecting.
         * @param {String} columnId - ID of the column you are selecting.
         * @param {String} location - Where you want the selection to render in the rowset.  Possible values are: 'top', 'bottom' or 'middle'.
         * @param {Boolean} appendToSelection
         * @param {String} subRow - Data index of the stacked cell you are selecting.  This only applies to stacked cell grids.
         * @param {Boolean} preserveTyping
         * @fires grid#event:view-sync-scroll-focus-into-view
         * @fires grid#event:action-state-update
         * @fires grid#event:performance-debug
         * @fires grid#event:cell-selection-changed
         * @public
         */
        // location: top bottom middle (where you want it to render in the rowset)
        selectCell : function (recordId, columnId, location, appendToSelection, subRow, preserveTyping, args) {
            var that = this;
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            if (recordId != null) {
                recordId = this.view.fixRecordIdType(recordId);
                var newFocus = this._getRecord(recordId);
                if (newFocus == null) {
                    return; // trying to select incorrect cells should not go forward
                } else {
                    this.focus.record = newFocus;
                    this.focus.recordId = recordId;
                }
            } else if (this.focus.recordId != null) {
                recordId = this.focus.recordId;
            } else {
                if (this.options.selectRowByDefault) {
                    recordId = this.view.findFirstRow();
                        // If in selectionMode and checkOnSelect
                    if (this.options.selectionMode === 'check' && this.options.checkOnSelect && this.options.checkRowByDefault && recordId != null) {
                        this.checkRows([recordId], true);
                    }
                }
                if (recordId != null) {
                    this.focus.record = this._getRecord(recordId);
                    this.focus.recordId = recordId;
                } else {
                    this.focus.record = null;
                    this.focus.recordId = null;
                }
            }
            var record = recordId && this._getRecord(recordId);
            if (record) {
                this._attrs.clear(record);
            }
            var oldColumnFocus = this.focus.columnId;
            var oldSubRow = this.focus.subRow;
            if (columnId == null) {
                if (this.focus.columnId != null && this.columnDefinitions[this.focus.columnId] != null && this.columnDefinitions[this.focus.columnId].visible && this.columnDefinitions[this.focus.columnId].groupId != null) {
                    columnId = this.focus.columnId;
                } else {
                    columnId = this.view.firstVisibleColumn;
                }
            } else if (this.columnDefinitions[columnId] == null || !this.columnDefinitions[columnId].visible || this.columnDefinitions[columnId].groupId == null) {
                columnId = this.view.firstVisibleColumn;
            } else if (this._isGroupRecordId(recordId) && this.columnDefinitions[columnId].hideGroupCell) {
                columnId = this.view.firstVisibleColumn;
            } else if (columnId === constants.EMPTY_COLUMN) {
                columnId = this.view.lastVisibleColumn;
            }

            var columnGroup = null;
            if (columnId != null) {
                columnGroup = _.find(that.options.layout.columnGroups, function (columnGroup) {
                    if (that.columnDefinitions[columnId] != null) {
                        return columnGroup.id === that.columnDefinitions[columnId].groupId;
                    } else {
                        return false;
                    }
                });
            }
            this.focus.subRow = subRow;
            // If stacked cells and one is not given pick the top one.
            if (columnGroup != null && this.focus.subRow == null && columnGroup.stackedCells != null && columnGroup.stackedCells.length) {
                this.focus.subRow = _.first(columnGroup.stackedCells);
            }

            if (!appendToSelection && !preserveTyping) {
                this.view.currentlyTyping = '';
            }
            this.focus.columnId = columnId;

            if (recordId != null) {
                var rowsetId = this._getRecordPlacement(this._getRecord(recordId)).rowsetId;
                var updateScrollbar = this.view.renderFocus(recordId, rowsetId, columnId, location);
                if (this.view.getRecordIndex(recordId, rowsetId) < 0 && (this.options.tree || this.groupingApplied)) {
                    this.view.expandToRecord(recordId, rowsetId);
                }
                this.selectRows([recordId], appendToSelection, null, columnId, args);
                this.view.scrollFocusIntoView();

                /**
                 * @event grid#view-sync-scroll-focus-into-view
                 * @param {Object} [event=null]
                 * @param {Object} triggerArgs
                 * @param {String} triggerArgs.recordId - ID of the record to select.
                 */
                this._trigger('view-sync-scroll-focus-into-view', null, { recordId : recordId });

                if (updateScrollbar) {
                    this.view.updateRowsetScrollbar(rowsetId);
                }
            } else {
                this._clearRowSelection();
                this._trigger('action-state-update', null, this._checkActions());
            }

            //>>excludeStart("perf", pragmas.production)
            if (this.options.debug) {
                this._trigger('performance-debug', null,
                    {
                        start : start,
                        end : window.performance.now(),
                        name : 'select-cell',
                        type : 'widget',
                        info : 'Record:  ' + recordId + ' Col: ' + columnId
                    });
            }
            //>>excludeEnd("perf")

            this.view.focusCellSelection();
            this.view.resetRowHovered();

            if (this.options.enableCellChangeTrigger && (oldColumnFocus !== this.focus.columnId || oldSubRow !== this.focus.subRow)) {
                /**
                 * @event grid#cell-selection-changed
                 * @param {Object} [event=null]
                 * @param {Object} triggerArgs
                 * @param {Object} triggerArgs.record - Record that was selected.
                 * @param {String} triggerArgs.dataIndex - Index of the column that was selected.
                 * @param {String} triggerArgs.subRow - Data index of the field that was selected for the stacked cell.  Only applies to stacked cell grids.
                 */
                this._trigger('cell-selection-changed', null, {
                    record : this.focus.record,
                    dataIndex : this.focus.columnId,
                    subRow : this.focus.subRow
                });
            }
        },

        selectHeaderCell : function (columnId) {
            this._clearRowSelection();
            this.focus.record = null;
            this.focus.recordId = null;
            this.focus.columnId = columnId;
            this.view.selectHeaderCell(columnId);
            this.view.scrollFocusIntoView();
            this.view.focusCellSelection();
        },

        selectAdvancedFilterCell : function (columnId) {
            this._clearRowSelection();
            this.focus.record = null;
            this.focus.columnId = columnId;
            this.view.advancedFilters.focusFilter(columnId);
            this.view.scrollFocusIntoView();
            this.view.focusCellSelection();
        },

        selectColumnFilterCell : function (columnId) {    // LUX-506
            this._clearRowSelection();
            this.focus.record = null;
            this.focus.columnId = columnId;
            this.view.selectColumnFilterCell(columnId);
            this.view.scrollFocusIntoView();
            this.view.focusCellSelection();    // setting activedescendant on scrolly-grid
        },

        /**
         * Used to select one or more rows in the grid.
         * @method grid#selectRows
         * @param {Array<String>} recordIds - IDs of the records you are selecting in the grid.
         * @param {Boolean} append
         * @param {Object} event - Event that results in selectRows being called.
         * @param {String} columnId - ID of the column being selected.
         * @fires grid#event:before-select-rows
         * @fires grid#event:action-state-update
         * @fires grid#event:performance-debug
         * @public
         */
        selectRows : function (recordIds, append, event, columnId, args) {
            var that = this;
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (that.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            var records = this._getRecords(recordIds);
            var triggerArgs = {
                records : records
            };

            if (!args || !args.byPassBeforeTrigger) {
                /**
                 * Triggered before rows are selected.
                 * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
                 *
                 * @event pgbu.grid#before-select-rows
                 * @param {Object} [event=null]
                 * @param {Object} triggerArgs
                 * @param {Array<Object>} triggerArgs.records - Records to be selected.
                 */
                if (that._trigger('before-select-rows', null, triggerArgs) === false) {
                    return;
                }
            }
            this._clearRecordSelectionAttributes();
            if (recordIds != null && recordIds.length > 0) {
                var deselectedRecords = [];
                if (append == null || !append) {
                    if (recordIds.length !== this.selection.recordIds.length || _.difference(recordIds,
                                that.selection.recordIds).length) {
                        deselectedRecords = this._getRecords(this.selection.recordIds);
                        that._clearRowSelection(true);
                    }
                    // if you are not appending selection make sure that the focus is amoung one of the new items. Case is for programtic selection.
                    if (!_.includes(recordIds, that.focus.recordId)) {
                        that.focus.record = records[0];
                        that.focus.recordId = recordIds[0];
                        that.view.selectionModel.updateFocus(that.focus.record);
                    }
                } else if (that.options.disableMultiRowsetSelection && that.selection.recordIds.length) {
                    if (that._getRecordPlacement(records[0]).rowsetId !== that._getRecordPlacement(that._getRecord(that.selection.recordIds[0])).rowsetId) {
                        deselectedRecords = this._getRecords(this.selection.recordIds);
                        that._clearRowSelection(true);
                        // if you are not appending selection make sure that the focus is amoung one of the new items. Case is for programtic selection.
                        if (!_.includes(recordIds, that.focus.recordId)) {
                            that.focus.record = records[0];
                            that.focus.recordId = recordIds[0];
                            that.view.selectionModel.updateFocus(that.focus.record);
                        }
                    }
                }

                var newlySelected = _.difference(recordIds, this.selection.recordIds);

                if (newlySelected.length > 0) {
                    for (var i = 0; i < newlySelected.length; i++) {
                        var recordId = newlySelected[i];
                        that.selection.recordIds.push(recordId);
                        that.selection.records.push(this._getRecord(recordId));
                        that.selection.recordIdMap[recordId] = true;
                    }

                    that.view.reselectRecords();

                    triggerArgs = {
                        deselectedRecords : deselectedRecords,
                        selectedRecords : that.selection.records,
                        selectedRecordIds : that.selection.recordIds,
                        focusedRecord : this.focus.record,
                        focusedRecordId : this.focus.recordId
                    };

                    this.rowSelectedTriggerArgs.event = event;
                    this.rowSelectedTriggerArgs.triggerArgs = triggerArgs;
                    this.debouncedTriggerRowSelected(); // For performance with ajax loads when selects occur

                    this.debouncedCheckActionState();
                } else if (columnId != null) { // select the cell if the row is already selected
                    that.view.reselectRecords();
                }
            } else {
                that._clearRowSelection();
                this.debouncedCheckActionState();
            }

            //>>excludeStart("perf", pragmas.production)
            if (that.options.debug) {
                that._trigger('performance-debug', null,
                    {
                        start : start,
                        end : window.performance.now(),
                        name : 'select-rows',
                        type : 'widget',
                        info : _.slice(recordIds, 100)
                    });
            }
            //>>excludeEnd("perf")
        },

        /**
         * Returns first row of the first (non empty) rowset based on current order of rows.
         * @method grid#findFirstRow
         * @param {Boolean} excludeGroups - When true, returns the first data row, instead of a header row.
         * @returns {String} recordId - Id of the first record.
         * @public
         */
        findFirstRow : function (excludeGroups) {
            if (excludeGroups) {
                return this.view.findFirstNonGroupRow();
            } else {
                return this.view.findFirstRow();
            }
        },

        /**
         * Used to un-select the selected row(s) in the grid.
         * @method grid#deselectRows
         * @param {Array<String>} recordIds - IDs of the records to un-select in the grid.
         * @param {Object} event - Event that caused deselectRows to be called.
         * @fires grid#event:before-deselect-rows
         * @fires grid#event:action-state-update
         * @fires grid#event:performance-debug
         * @public
         */
        deselectRows : function (recordIds, event) {
            var that = this;
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (that.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            var records = that.store.getRecords(recordIds);
            var triggerArgs = {
                records : records
            };

            /**
                 * Triggered before rows are deselected.
                 * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
                 *
                 * @event pgbu.grid#before-deselect-rows
                 * @param {Object} [event=null]
                 * @param {Object} triggerArgs
                 * @param {Array<Object>} triggerArgs.records - Records to be un-selected.
                 */
            if (that._trigger('before-deselect-rows', null, triggerArgs) === false) {
                return;
            }
            if (recordIds != null && recordIds.length > 0) {
                var selected = _.intersection(recordIds, that.selection.recordIds);

                that.selection.recordIds = _.difference(that.selection.recordIds, recordIds);
                that.selection.records = that._getRecords(that.selection.recordIds);
                _.forEach(recordIds, function (recordId) {
                    delete that.selection.recordIdMap[recordId];
                });
                that.view.reselectRecords();
                if (selected.length > 0) {
                    triggerArgs = {
                        deselectedRecords : that._getRecords(selected),
                        selectedRecords : this.selection.records,
                        focusedRecord : this.focus.record,
                        deselectedRecordIds : selected,
                        selectedRecordIds : this.selection.recordIds
                    };
                    this.rowSelectedTriggerArgs.event = event;
                    this.rowSelectedTriggerArgs.triggerArgs = triggerArgs;
                    this.debouncedTriggerRowSelected();
                    that._trigger('action-state-update', event, that._checkActions());
                }
            }

            //>>excludeStart("perf", pragmas.production)
            if (that.options.debug) {
                that._trigger('performance-debug', null,
                    {
                        start : start,
                        end : window.performance.now(),
                        name : 'deselect-rows',
                        type : 'widget',
                        info : recordIds
                    });
            }
            //>>excludeEnd("perf")
        },

        closeMenus : function () {
            this.view.closeMenus();
        },

        /**
         * Used to check (select), one or more rows in the grid, by clicking on the check box in the column header or on each individual row.
         *
         * Note: This only applies to grids that have their selectionMode set to 'check'.
         * @method grid#checkRows
         * @param {Array<String>} recordIds - IDs of the records that are being selected.
         * @param {Boolean} check - When true, the user has checked a checkbox, otherwise false.
         * @param {Boolean} all - When true, the user has checked the checkbox in the column header, and is selecting all of the rows.  Otherwise false.
         * @public
         * @fires grid#event:rows-checked
         * @fires grid#event:action-state-update
         * @example
         * // grid config with seletionMode set to check.
         * // selectionMode should be at the root level of the grid config.
         * var gridConfig = {
         *     selectionMode: 'check'
         * };
         */
        checkRows : function (recordIds, check, all) {
            var that = this;

            recordIds = _.without(recordIds, constants.DUMMY_RECORD_ID);
            if (that.options.selectionMode === 'check') {
                var records = [];
                if (!all || check) {
                    records = this._getRecords(recordIds);
                }
                if (check == null || check) {
                    if (all) {
                        that.checkedRecordIds = recordIds;
                        that.checkedRecords = records;
                        that.view.setHeaderCheckbox('row-selector', true);
                    } else {
                        var newlyChecked = _.difference(recordIds, that.checkedRecordIds);
                        if (newlyChecked.length) {
                            records = this._getRecords(newlyChecked);
                            that.checkedRecordIds = that.checkedRecordIds.concat(newlyChecked);
                            that.checkedRecords = that.checkedRecords.concat(records);
                            if (that.checkedRecords.length === _.flatten(_.pluck(_.values(that.view.virtualRows), 'records')).length) {
                                that.view.setHeaderCheckbox('row-selector', true);
                            } else {
                                that.view.setHeaderCheckbox('row-selector', 'mixed');
                            }
                        }
                    }
                } else {
                    if (all) {
                        that.checkedRecords = [];
                        that.checkedRecordIds = [];
                    } else {
                        that.checkedRecords = _.difference(that.checkedRecords, records);
                        that.checkedRecordIds = _.difference(that.checkedRecordIds, recordIds);
                    }

                    if (that.checkedRecords.length > 0) {
                        that._setHeaderCheckbox('row-selector', 'mixed');
                    } else {
                        that._setHeaderCheckbox('row-selector', false);
                    }
                }
                if (all) {
                    that._forceViewRender();
                } else {
                    that.view.checkRecords(recordIds, check);
                    if (recordIds.length) {
                        that.view.updateRow(recordIds[0], that._getRecordRowset(recordIds[0]));
                    }
                }
                var triggerArgs = {
                    checkedRecords : this.checkedRecords
                };

                /**
                     * Triggered when rows are checked using the row-selector column.
                     * This event is captured after certain action takes place on the grid.
                     *
                     * @event pgbu.grid#rows-checked
                     * @param {Object} [event=null]
                     * @param {Object} triggerArgs
                     * @param {Array<Object>} triggerArgs.checkedRecords - Record(s) that are checked in the grid.
                     */
                that._trigger('rows-checked', null, triggerArgs);
                this._trigger('action-state-update', null, this._checkActions());
            }
        },

        /**
         * Used to get the currently selected records (rows) in the grid.
         * @method grid#getSelection
         * @returns {Object|Array<Object>} Record(s) that are selected/checked in the grid.
         * @public
         */
        getSelection : function () {
            switch (this.options.selectionMode) {
            case 'default' :
                return this._getRecords(this.selection.recordIds);
            case 'check' :
                return this._getRecords(this.checkedRecordIds);
            }

            throw new Error('Selection mode is not set correctly');
        },

        _getSelectionAny : function () {
            var selectedRecords = this._getRecords(this.selection.recordIds);
            switch (this.options.selectionMode) {
            case 'default' :
                return selectedRecords;
            case 'check' :
                var checkedRecords = this._getRecords(this.checkedRecordIds);
                if (checkedRecords.length > 0)
                    return checkedRecords;
                else if (selectedRecords.length > 0)
                    return selectedRecords;
                else
                    return checkedRecords;
            }

            throw new Error('Selection mode is not set correctly');
        },

        getSelectionIds : function () {
            switch (this.options.selectionMode) {
            case 'default' :
                return this.selection.recordIds;
            case 'check' :
                return this.checkedRecordIds;
            }

            throw new Error('Selection mode is not set correctly');
        },

        /**
         * Method used to get the row with that currently has focus in the grid.
         * @method grid#getRowFocus
         * @returns {Object} Record that currently has focus.
         * @public
         */
        getRowFocus : function () {
            return this.focus;
        },

        /**
         * Method to deselect the currently selected row(s) in the grid.
         * @method grid#clearSelection
         * @param {Boolean} skipTrigger - Whether or not to skip any associated triggered events.
         * @public
         * @fires grid#event:rows-checked
         */
        clearSelection : function (skipTrigger) {
            switch (this.options.selectionMode) {
            case 'default' :
                this._clearRowSelection(skipTrigger);
                this.focus.record = null;
                this.focus.recordId = null;
                break;
            case 'check' :
                this.checkedRecords = [];
                this.checkedRecordIds = [];
                this.view.clearChecked();
                if (!skipTrigger) {
                    var triggerArgs = {
                        checkedRecords : this.checkedRecords
                    };
                    this._trigger('rows-checked', null, triggerArgs);
                }
                break;
            }
        },

        /**
         * Used to resize the grid widget.
         * @method grid#resize
         * @public
         */
        resize : function () {
            // update the config
            this.view.resize();
        },

        /**
         * Used to set the store and mediator to be used with the grid.
         *
         * Note: This method will call repopulate().
         * @method grid#setStore
         * @param {Object} store - Store to be used.
         * @param {Object} CustomMediator - Mediator to be used.
         * @param {Object} options - Additional options for the mediator.
         * @returns {Object} The new mediator for the grid.
         * @fires grid#event:store-created
         * @fires grid#event:performance-debug
         * @fires grid#event:action-state-update
         * @public
         */
        setStore : attrFns.decorateCache('_attrs', 'gridWidget#setStore', function setStore(store, CustomMediator, options) {
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")

            if (this.store && _.isFunction(this.store.destroy)) {
                this.store.destroy();
            }
            if (CustomMediator != null) {
                this.store = new CustomMediator(store, this, options);
            } else if (this.options.tree) {
                this.store = new DefaultTreeMediator(store, this, options);
            } else {
                this.store = new DefaultMediator(store, this, options);
            }

            /**
             * @event grid#store-created
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Object} triggerArgs.store - New mediator that was created for the grid.
             */
            this._trigger('store-created', null, { store : this.store });
            this._resetProxyRecord();
            this.recordGrouper.setStore(this.store);
            this.dataRollup.setStore(this.store);
            this._resetInternalStorage();


            if (this.config.groupBy != null && this.config.groupBy.length) {
                this.groupRows(this.config.groupBy, true);
            }

            this.dataRollup.buildAndRollupTree();
            this.repopulate();

            //Do after grouping to keep the repopulate from running and rolling up the tree. Note this may break the sort order slightly, investigation to be continued
            if (this.config.sortBy != null && this.config.sortBy.length) {
                this._updateSortConfig(this.config.sortBy, true);
            }


            this.focus.record = null;
            this.focus.recordId = null;
            this.focus.columnId = null;
            this.store.attachListeners();
            this.selectCell();

            //>>excludeStart("perf", pragmas.production)
            if (this.options.debug) {
                this._trigger('performance-debug', null,
                    {
                        start : start,
                        end : window.performance.now(),
                        name : 'set-store',
                        type : 'widget'
                    });
            }
            //>>excludeEnd("perf")
            this.view.setLoading(false);
            this._trigger('action-state-update', null, this._checkActions());
            return this.store;
        }),

        /**
         * Used to reset the mediator associated with the grid.  This is called when ever the
         * user cancels any changes from the changebar.
         * @method grid#reset
         * @fires grid#event:before-reset
         * @fires grid#event:reset
         * @first grid#event:is-clean
         * @public
         */
        reset : function () {
            /**
                 * Triggered before the grid is reset.
                 * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
                 *
                 * @event pgbu.grid#before-reset
                 */
            if (this._trigger('before-reset') === false) {
                return;
            }
            this.store.reset();
                /**
                 * Triggered when 'reset' is called on the grid.
                 * This event is captured after certain action takes place on the grid.
                 *
                 * @event pgbu.grid#reset
                 */
            this._trigger('reset');
                /**
                 * Triggered when the grid enters a 'clean' state (such as after a commit).
                 * This event is captured after certain action takes place on the grid.
                 *
                 * @event pgbu-grid#is-clean
                 */
            this._trigger('is-clean');
        },

        /**
         * Method to commit all changes to the grid and bring the grid back to a clean state.
         * This is called after a successful page save.
         * @method grid#commit
         * @fires grid#event:before-commit-grid
         * @fires grid#event:commit
         * @public
         */
        commit : function () {
            /**
                 * Triggered before commit is called.
                 * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
                 * @event pgbu.grid#before-commit-grid
                 */
            if (this._trigger('before-commit-grid') === false) {
                return;
            }
            this.store.commit();
                /**
                 * Triggered when the grid is commited.
                 * This event is captured after certain action takes place on the grid.
                 *
                 * @event pgbu.grid#commit
                 */
            this._trigger('commit');
        },

        getRecordOrder : function () {
            var that = this;
            var recordOrder = {};
            _.forEach(this.config.rowsets, function (rowset) {
                if (rowset.id !== 'headers') {
                    recordOrder[rowset.id] = that.view.virtualRows[rowset.id].records;
                }
            });
            return recordOrder;
        },

        /**
         * Method used to sort the grid by column.
         * @method grid#sortRows
         * @param {String} dataIndex - Index of the column that the grid is to be sorted by.
         * @param {Boolean} append
         * @param {String} order
         * @param {Boolean} silent
         * @public
         */
        sortRows : attrFns.decorateCache('_attrs', 'gridWidget#sortRows', function sortRows(dataIndex, append, order, silent) {
            var columnDef = this.view.columnDefinitions[dataIndex];
            var group = _.find(this.config.groups, function (g) {
                return _.find(g.columns, function (c) {
                    return c.dataIndex === dataIndex;
                }) != null;
            });
            if (columnDef.sortable && !group.timephased && !group.showStackedCells) {
                this._updateColumnSort(dataIndex, append, order);
                this._applySortConfig({ surpressTrigger : silent, columnId : dataIndex, userAction : true });
            }
        }),

        _setSortConfig : function (sortConfigs) {
            var configBefore = this.getViewConfig().sortBy;
            this.config.sortBy = [];
            if (sortConfigs != null) {
                for (var i = 0; i < sortConfigs.length; i++) {
                    var sortConfig = sortConfigs[i];
                    this._updateColumnSort(sortConfig.dataIndex, true, sortConfig.order);
                }
            }
            this.viewManager.queue.sortBy = !_.isEqual(this.getViewConfig().sortBy, configBefore);
        },

        _setGroupingOptions : function (oldOptions, newOptions) {
            if (newOptions.enableRollup != null && (oldOptions.enableRollup !== newOptions.enableRollup)) {
                this.viewManager.queue.group = true;
            }
            if (newOptions.showEmptyBands != null && (oldOptions.showEmptyBands !== newOptions.showEmptyBands)) {
                this.viewManager.queue.group = true;
            }
        },

        _setGroupConfig : function (groupConfig) {
            var currentConfig = this.getViewConfig().groupBy;
            this.recordGrouper.updateGroupConfig(groupConfig);
            if (!this.viewManager.queue.group) {
                this.viewManager.queue.group = this.isLoaded() && !_.isEqual(this.getViewConfig().groupBy, currentConfig);
            }
        },

        _updateSortConfig : function (sortConfigs, silent) {
            this._setSortConfig(sortConfigs);
            this._applySortConfig({ surpressTrigger : silent, userAction : false });
        },

        _updateColumnSort : function (dataIndex, append, order) {
            // Update the sort config
            var columnSort = _.first(_.filter(this.config.sortBy, function (sort) {
                return sort.dataIndex === dataIndex;
            }));

            if (append == null || !append) {
                this.config.sortBy = _.intersection(this.config.sortBy, [columnSort]);
            }

            if (columnSort != null) {
                // If order is specified by the user/function use it.
                if (order != null) {
                    if (columnSort.order === order) {
                        this.config.sortBy = _.without(this.config.sortBy, columnSort);
                    } else {
                        columnSort.order = order;
                        if (this.config.sortBy.length === 1) {
                            this.reverseSortOrder = true;
                        }
                    }
                } else {
                    var nextOrderIndex = _.indexOf(this.options.sortClickOrder, columnSort.order) + 1;
                    if (nextOrderIndex === this.options.sortClickOrder.length) {
                        nextOrderIndex = 0;
                    }
                    var nextOrder = this.options.sortClickOrder[nextOrderIndex];
                    if ((columnSort.order === 'asc' && nextOrder === 'dsc') ||
                            (columnSort.order === 'dsc' && nextOrder === 'asc')) {
                        if (this.config.sortBy.length === 1) {
                            this.reverseSortOrder = true;
                        }
                    } else if (nextOrder === 'none') {
                        this.config.sortBy = _.without(this.config.sortBy, columnSort);
                    }
                    columnSort.order = nextOrder;
                }
            } else {
                var column = this.view.columnDefinitions[dataIndex];
                if (column) {
                    var sortConfigOptions;
                    if (column.dataType === 'string') {
                        sortConfigOptions = { numeric : column.numericSearch && this.options.search.numericSearchEnabled };
                    }
                    this.config.sortBy.push(
                        {
                            dataIndex : dataIndex,
                            order : order != null ? order : this.options.sortClickOrder[0],
                            sorter : column.sorter || this.options.sorters[column.dataType] || sorters(column.dataType),
                            getSortValue : column.getSortValue,
                            options : sortConfigOptions
                        });
                }
                //>>excludeStart("validation", pragmas.production)
                else {
                    logger.info('Requested sort on column ' + dataIndex + ', which did not exist; ignoring');
                }
                //>>excludeEnd("validation")
            }
        },

        _applySortConfig : function (options) {
            if (this.store != null && (this.store.sort == null || this.store.sort(this.config.sortBy) === false)) {
                /**
                 * @event grid#before-apply-sort
                 * @param {Object} [event=null]
                 * @param {Object} triggerArgs
                 * @param {Array<Object>} triggerArgs.sortBy
                 * @param {String} triggerArgs.sortBy.dataIndex - Index of the column that the grid should be sorted by.
                 * @param {String} triggerArgs.sortBy.order - Whether the sort should be ascending or descending.  Possible values: 'asc' or 'desc'.
                 */
                if (this._trigger('before-apply-sort', null, { sortBy : this.config.sortBy }) === false) {
                    this._addSortIcons();
                    return;
                }
                var headerToReselect = this.view.selectionModel.focus.columnId;

                if (this.config.sortBy.length) {
                    this.sortingApplied = true;

                    if (this.options.tree || this.groupingApplied) {
                        this.repopulate(this.focus.recordId, options.columnId || this.focus.columnId);
                    } else {
                        this._sortFlatRecords();
                        this._forceViewRender();

                        if (options.columnId != null) {
                            this.selectCell(null, options.columnId);
                        }
                    }
                } else {
                    this.sortingApplied = false;
                    this.repopulate(this.focus.recordId, options.columnId || this.focus.columnId);
                }

                if (this.view.reselectHeaderAfterSort) {
                    var that = this;
                    this.requestBrowserFocus(true);
                    that.selectHeaderCell(headerToReselect);
                    clearTimeout(that.timeouts.selectHeaderCell);
                    that.timeouts.selectHeaderCell = setTimeout(function () { // Ie is terrible and does not work without this timeout.
                        that.selectHeaderCell(headerToReselect);
                    }, 1);
                    this.view.reselectHeaderAfterSort = false;
                }

                this._addSortIcons();

                if (options == null || !options.surpressTrigger) {
                    this._trigger('view-updated', null,
                        {
                            action : 'column-sort',
                            userAction : options.userAction,
                            sort : _.clone(this.config.sortBy),
                            recordOrder : this.getRecordOrder()
                        });
                }
            }
        },

        _addSortIcons : function () {
            this.view.setColumnsSorted();
        },

        /**
         * Used to group the records in the grid, based on a specific field.
         * @method grid#groupRows
         * @param {Array} groupConfig
         * @param {String} groupConfig[i].dataIndex - Index of the field that the records should be grouped by.
         * @param {Function} groupConfig[i].getGroupValue - optional - Function called to get the value for a group record.
         * @param {String} groupConfig[i].title - Title for a group record.
         * @param {Boolean} silent
         * @fires grid#event:view-updated
         * @fires grid#event:performance-debug
         * @public
         */
        groupRows : function (groupConfig, silent) {
            var that = this;
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (that.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            if (this.store != null && this.store.group != null && this.store.group(groupConfig) !== false) {
                return;
            }
            this.view.resetVirtualRows();
            this.recordGrouper.groupRecords(groupConfig, silent);

            if (silent == null || !silent) {
                /**
                     * Triggered when the view is updated
                     * This event is captured after certain action takes place on the grid.
                     *
                     * @event pgbu.grid#view-updated
                     * @param {Object} [event=null]
                     * @param {Object} triggerArgs
                     * @param {String} triggerArgs.action - Action performed that requires the view to be updated.
                     * @param {Array<Object>} triggerArgs.group
                     * @param {String} triggerArgs.group.dataIndex - Index of the field to group the grid by.
                     * @param {Number} triggerArgs.group.level
                     * @param {String} triggerArgs.group.sortOrder - Whether to sort ascending or descending.  Possible values: 'asc' or 'desc'.
                     */
                that._trigger('view-updated', null,
                    {
                        action : 'column-group',
                        userAction : false,
                        group : _.clone(this.config.groupBy)
                    });
            }
            //>>excludeStart("perf", pragmas.production)
            if (that.options.debug) {
                that._trigger('performance-debug', null,
                    {
                        start : start,
                        end : window.performance.now(),
                        name : 'group-rows',
                        type : 'widget',
                        info : groupConfig
                    });
            }
            //>>excludeEnd("perf")
            return;
        },

        getRecordsFromNode : function (args) {
            return this.getRecordsFromGroup(args);
        },

        getRecordsFromGroup : function (args) {
            var children;
            var record = args.record;
            if (args.recordId != null) {
                record = this._getRecord(args.recordId);
            }
            if (this._isGroupRecord(record)) {
                if (args.allChildren) {
                    children = this.recordGrouper.getAllChildRecordsFromGroup(record);
                } else if (args.immediateOrAll) {
                    children = this.recordGrouper.getRecordsFromGroup(record);
                    if (!children.length) {
                        children = this.recordGrouper.getAllChildRecordsFromGroup(record);
                    }
                } else {
                    children = this.recordGrouper.getRecordsFromGroup(record);
                }
            } else {
                children = [];
            }
            return children;
        },

        getHierarchyChildrenFromGroup : function (args) {
            var children = [];
            var group = args.record;
            if (args.recordId != null) {
                group = this._getRecord(args.recordId);
            }
            if (this._isGroupRecord(group)) {
                children = this.recordGrouper.getHierarchyChildrenFromGroup(group);
            }
            return children;
        },

        getLastRecord : function (args) {
            var rowsetId = this.defaultRowset;
            var record;
            if (args && args.rowsetId) {
                rowsetId = args.rowsetId;
            }
            var lastRecordId = _.last(this.view.virtualRows[rowsetId].records);
            if (lastRecordId != null) {
                record = this._getRecord(lastRecordId);
            }
            return record;
        },

        undo : function (change) {
            var fromBatch = change != null;
            if (fromBatch || this._checkAction('undo')) {
                change = change || this.undoStack.pop();
                if (change != null) {
                    var redoAction = _.clone(change);
                    switch (change.type) {
                    case 'edit' :
                        redoAction.data = _.clone(this.store.getValueFromRecord(change.record, change.dataIndex));
                        this._cellUpdated({
                            record : change.record,
                            value : change.data,
                            valid : true,
                            column : this.columnDefinitions[change.dataIndex],
                            skipUndo : true
                        });
                        this.selectCell(this._getRecordId(change.record), change.dataIndex, null, fromBatch);
                        break;
                    case 'delete' :
                        this.store.addRecords(change.records.reverse());
                        this.selectRows(this._getRecordIds(change.records));
                        break;
                    case 'new' :
                        this.store.deleteRecords(change.records);
                        break;
                    case 'paste' :
                        this.clearSelection();
                        var redoStackLength = this.redoStack.length;
                        for (var i = 0; i < change.batch.length; i++) {
                            var edit = change.batch[i];
                            this.undo(edit);
                        }
                        redoAction = {
                            type : 'paste',
                            batch : this.redoStack.splice(redoStackLength).reverse()// reverse to get the proper selection to occur via redo
                        };
                        break;
                    }
                    this.redoStack.push(redoAction);

                    var actions = {
                        undo : this._checkAction('undo'),
                        redo : this._checkAction('redo')
                    };
                    this._trigger('action-state-update', null, actions);
                }
            }
        },

        redo : function (change) {
            var fromBatch = change != null;
            if (fromBatch || this._checkAction('redo')) {
                change = change || this.redoStack.pop();
                if (change != null) {
                    var undoAction = _.clone(change);
                    switch (change.type) {
                    case 'edit' :
                        undoAction.data = _.clone(this.store.getValueFromRecord(change.record, change.dataIndex));
                        this._cellUpdated({
                            record : change.record,
                            value : change.data,
                            valid : true,
                            column : this.columnDefinitions[change.dataIndex],
                            skipUndo : true
                        });
                        this.selectCell(this._getRecordId(change.record), change.dataIndex, null, fromBatch);
                        break;
                    case 'delete' :
                        this.store.deleteRecords(change.records.reverse());
                        break;
                    case 'new' :
                        this.store.addRecords(change.records.reverse());
                        this.selectRows(this._getRecordIds(change.records));
                        break;
                    case 'paste' :
                        this.clearSelection();
                        var undoStackLength = this.undoStack.length;
                        for (var i = 0; i < change.batch.length; i++) {
                            var edit = change.batch[i];
                            this.redo(edit);
                        }
                        undoAction = {
                            type : 'paste',
                            batch : this.undoStack.splice(undoStackLength).reverse() // reverse to get the proper selection to occur via undo
                        };
                        break;
                    }

                    this._pushUndoAction(undoAction, false);

                    var actions = {
                        undo : this._checkAction('undo'),
                        redo : this._checkAction('redo')
                    };
                    this._trigger('action-state-update', null, actions);
                }
            }
        },

        /**
         * Method used to completely redraw the grid.
         *
         * Note: Please use sparingly.  In most cases, you don't need to redraw the entire grid
         * and could achieve your desired result by using other functions like recordUpdated, when updating a value on a record.
         *
         * @method grid#repopulate
         * @param {String} recordId - ID of the record that should be selected after redrawing the grid.
         * @param {String} columnId - ID of the column to be selected, in the selected record, after redrawing the grid.
         * @param {Object} args
         * @fires grid#event:performance-debug
         * @public
         */
        repopulate : attrFns.decorateCache('_attrs', 'gridWidget#repopulate', function repopulate(recordId, columnId, args) {
            if (this._trigger('before-repopulate', null, {
                recordId : recordId,
                columnId : columnId,
                args : args
            }) === false) {
                return;
            }

            var isPaused = this.viewManager.isPaused();
            if (!this.isLoaded() || isPaused) {
                if (isPaused) {
                    this.viewManager.queue.repopulate = true;
                }
                return;
            }

            var that = this;

            //>>excludeStart("perf", pragmas.production)
            var start;
            if (that.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")

            if (that._getProxyRecord() == null) {
                that._resetProxyRecord();
            }
            this.recentlyRemoved = {};
            that.view.closeMenus(); // Close this stuff before in case of crazy
            that.view.closeEditor();

            // detach tables from DOM
            that.view.attachDOMTables(false, '$tbody');

            that.view.clearTables();
            // Clear focus
            that.focus.record = null;
            that.focus.recordId = null;
            that.focus.columnId = null;

            // LUX-1966 we clear all selections regardless of the selectionMode setting.
            that.clearSelection();

            if (!this.options.search.columnFilter) {     // LUX-506
                that.view.selectionModel.clearFocus();
            }

            that.storeRecords = [];
            that.view.clearOriginalSortOrder();

            // add rows to grid
            var defaultOptions = {
                bulk : true
            };

            var mergedOptions = {};
            _.merge(mergedOptions, defaultOptions, args);
            // If search is applied
            if (!that.options.search.showAll && that._hasFilters() && (!this.options.tree || this.groupingApplied)) {
                if (that.groupingApplied) {
                    _.forEach(that.config.rowsets, function (rowset) {
                        var nodes = that.rootNodes['rowset-' + rowset.id] || [];
                        for (var i = 0; i < nodes.length; i++) {
                            if (that.matchingRecords[nodes[i]] != null) {
                                that._addRecordToView(that._getRecord(nodes[i]), mergedOptions);
                            }
                        }
                    });
                } else {
                    _.forEach(that.matchingRecords.recordOrder, function (recordId) {
                        that._addRecordToView(that._getRecord(recordId), mergedOptions);
                    });
                }
            } else {
                // If grouping is applied, look at the root nodes. This may need to change if tree grouping gets more complex
                if (that.groupingApplied) {
                    _.forEach(that.config.rowsets, function addRootsForRowset(rowset) {
                        var nodes = that.rootNodes['rowset-' + rowset.id] || [];
                        for (var i = 0; i < nodes.length; i++) {
                            that._addRecordToView(that._getRecord(nodes[i]), mergedOptions);
                        }
                    });
                } else {
                    if (this.options.tree && this.options.data != null) {
                        that.storeRecords = this._getRecordsByData({ types : this.options.data.rootTypes });
                    } else {
                        that.store.iterateStore(this._getRecordFromStore, this, mergedOptions);
                    }

                    if (!that.options.recordKeys.numericalId && that.storeRecords.length && _.isNumber(that._getRecordId(that.storeRecords[0]))) {
                        that.options.recordKeys.numericalId = true;
                        logger.warn('Numerical Id detected without proper option set, changing recordKeys.numericalId to true. Please update your options.');
                    }
                    // Check to see if the root nodes need to be sorted, also check that the rowset they are going into should be sorted
                    if (this.options.tree) {
                        this.siblingIds = {};
                        this.parentIds = {};
                        this._addRootNodesForTree(that.storeRecords, mergedOptions);
                    } else {
                        var sortedRecords = that._assignSiblingIds(that.storeRecords);
                        for (var i = 0; i < sortedRecords.length; i++) {
                            that._addRecordToView(sortedRecords[i], mergedOptions);
                        }
                    }
                    that.storeRecords = [];
                }
            }

            if (that._hasFilters() && !that.options.search.showAll) {
                if (!that.matchingRecords.recordOrder.length) {
                    that.view.setRowsetMessage(that.options.locale.noResultsFound);
                } else {
                    that.view.setRowsetMessage();
                }
            }

            if (this.options.quickAdd.enabled && !that._hasFilters()) {
                if (that._checkAction('newRow')) {
                    if (this._getProxyRecord() == null) {
                        this._newProxyRecord();
                    }
                    that._addRecordToView(that._getDummyRecord(), mergedOptions);
                }
            }
            that.rollupApplied = false;
            _.forEach(that.config.rowsets, function (rowset) {
                // Rollup values
                if (rowset.rollupRowset != null) {
                    //>>excludeStart("validation", pragmas.production)
                    logger.assert(rowset.id.indexOf('-') === -1, 'if using a total row, its ID must not contain a dash');
                    //>>excludeEnd("validation")

                    that.rollupApplied = true;
                    var id = 'group-' + rowset.id + '-rollup-' + rowset.rollupRowset;
                    that.rolledupRowsets[rowset.rollupRowset] = id;
                    var totalRecords = _.clone(that.view.virtualRows[rowset.rollupRowset].records);
                    var rollupGroup = {
                        id : id,
                        records : totalRecords,
                        orderedRecords : totalRecords,
                        rollupValues : {},
                        timephasedRollupValues : {},
                        stackedRollupValues : {},
                        isGroup : true,
                        level : 0,
                        childGroups : [],
                        hierarchyGroups : [],
                        rootQualifier : id,
                        dataIndex : 'name',
                        formatValue : function () {
                            return rowset.rollupName;
                        },
                        hideColumnTitle : true,
                        rowsetId : rowset.id,
                        isRollupGroup : true
                    };
                    that.rootNodes[id] = [id];
                    that.recordGrouper.recordGroups[id] = rollupGroup;
                    that.recordGrouper.rollupFromLeafs([rollupGroup.id]);
                    that._addRecordToView(that._getRecord(rollupGroup.id), mergedOptions);
                }

                if (!that._hasFilters() && rowset.emptyMessage != null && !that.view.virtualRows[rowset.id].records.length) {
                    that.view.setRowsetMessage(rowset.emptyMessage, rowset.id);
                }
            });

            // size grid
            that.view.resize();

            if (!this.options.tree && !this.groupingApplied) {
                this._sortFlatRecords();
            }
            that._forceViewRender();

            // attach rows to tables
            that.view.attachDOMTables(true, '$tbody', '$thead');

            // Check if it exists in the grid
            if (recordId == null) {
                recordId = this.focus.recordId;
            }

            if (recordId != null) {
                var record = this._getRecord(recordId);
                if (record != null) {
                    var rowsetId = this._getRecordPlacement(record).rowsetId;
                    if (this.options.tree || this.groupingApplied) {
                        if (this.view.getRecordIndex(recordId, rowsetId) < 0) {
                            this.view.expandToRecord(recordId, rowsetId);
                        }
                        if (this.view.getRecordIndex(recordId, rowsetId) < 0) {
                            recordId = null;
                        }
                    } else if (this.view.getRecordIndex(recordId, rowsetId) < 0) {
                        recordId = null;
                    }
                } else {
                    recordId = null;
                }
            }
            that.selectCell(recordId, columnId);
            that.view.updateRecordCount();
            that._trigger('repopulated', null, {
                recordId : recordId,
                columnId : columnId,
                args : args
            });

            //>>excludeStart("perf", pragmas.production)
            if (that.options.debug) {
                that._trigger('performance-debug', null,
                    {
                        start : start,
                        end : window.performance.now(),
                        name : 'repopulate',
                        type : 'widget',
                        info : recordId + '-' + columnId
                    });
            }
            //>>excludeEnd("perf")
        }),

        _getRecordsByData : function (args) {
            var records = [];
            if (this.store.getRecordsByType == null) {
                return records;
            }
            var parentType, parentDefinition, childTypes, parent;
            if (args.parentId != null) {
                parentType = args.parentType || this._getTypeFromId(args.parentId);
                parent = args.parent || this._getRecord(args.parentId);
                parentDefinition = this.options.data.types[parentType];
                childTypes = args.types || _.intersection(parentDefinition.childTypes, this.options.data.hierarchy.order);
            } else {
                childTypes = args.types || _.intersection(this.options.data.rootTypes, this.options.data.hierarchy.order);
            }

            for (var i = 0; i < childTypes.length; i++) {
                var childType = childTypes[i];
                var typeDefinition = this.options.data.types[childType];
                var joinValue, foreignKey, constraints, indexed, childMapping;
                if (parent == null) {
                    joinValue = typeDefinition.root.joinValue;
                    foreignKey = typeDefinition.root.localKey;
                    indexed = typeDefinition.root.indexed;
                } else {
                    childMapping = parentDefinition.mapping[childType];
                    joinValue = this.store.getValueFromRecord(parent, childMapping.localKey);
                    foreignKey = childMapping.foreignKey;
                    constraints = childMapping.constraints;
                    indexed = childMapping.indexed;
                }
                records = records.concat(this.store.getRecordsByType({
                    type : childType,
                    foreignKey : foreignKey,
                    joinValue : joinValue,
                    constraints : constraints,
                    parentType : parentType,
                    indexed : indexed
                }));
            }
            return records;
        },

        /*
         * For a tree the root nodes can be ordered by precedence to support multiple types. In this case
         * sorting and sequencing are done for those of each type and then they are tied together.
         */
        _addRootNodesForTree : function (rootRecords, mergedOptions) {
            var that = this;
            // Do each rowset at a time
            var recordsGroupedByRowset = _.groupBy(rootRecords, function (record) {
                return that._getRecordPlacement(record).rowsetId;
            });

            _.forEach(recordsGroupedByRowset, function addRootTreeNodesLoop(records, rowsetId) {
                var rowsetConfig = _.find(that.config.rowsets, function (rowset) {
                    return rowset.id === rowsetId;
                });

                var record, recordsByPrecedence = {}, i;
                for (i = 0; i < records.length; i++) {
                    record = records[i];
                    var attrs = that._attrs.treeAttrs(record);
                    var sortable = attrs.sortable;
                    var precedence = attrs.precedence;
                    // Create the precedence group if it doesnt exist.
                    if (recordsByPrecedence[precedence] == null) {
                        recordsByPrecedence[precedence] = {
                            precedence : precedence,
                            sortableRecords : [],
                            unsequencedRecords : [],
                            orderedIds : []
                        };
                    }
                    // If the grid is sorted and this record can be sorted add it to the list to sort.
                    if (that.sortingApplied && sortable && rowsetConfig.sortable) {
                        recordsByPrecedence[precedence].sortableRecords.push(record);
                        // If it cant be sorted prepare to order the records by sequence, or order if no sequence key
                    } else {
                        recordsByPrecedence[precedence].unsequencedRecords.push(record);
                    }
                }
                // Build the correct record order, going in order of prcedence 1 is higher than 2 and will show up first
                var properOrderedRecords = [];
                _.forEach(_.sortBy(_.values(recordsByPrecedence), 'precedence'), function (precedenceGroup) {
                    // First add the unsortable records
                    var orderedRecords = that._assignSiblingIds(precedenceGroup.unsequencedRecords);
                    properOrderedRecords = properOrderedRecords.concat(orderedRecords);

                    // Next add any records that could be sorted
                    if (that.sortingApplied) {
                        that._sortRecords(precedenceGroup.sortableRecords, true);
                    }
                    properOrderedRecords = properOrderedRecords.concat(precedenceGroup.sortableRecords);
                });

                that._assignSiblingsLoop(properOrderedRecords);

                // Add after siblingids have been fixed add them to the view. This will chain add children if needed
                for (i = 0; i < properOrderedRecords.length; i++) {
                    if (!that._hasFilters() || that.matchingRecords[that._getRecordId(properOrderedRecords[i])] != null) {
                        that._addRecordToView(properOrderedRecords[i], mergedOptions);
                    }
                }
            });
        },

        /**
         * Method used to close the editor in the currently selected cell.
         *
         * Note: This method calls repopulate().
         * @method grid#closeEditor
         * @param {Boolean} commit - Whether or not to commit the value in the editor.
         * @public
         */
        closeEditor : function (commit) {
            this.view.closeEditor(!commit);
        },

        /* eslint-disable no-useless-escape */
            /**
             * Method used to clear currently applied column filters.
             *
             * @method pgbu.grid#clearColumnFilter
             * @public
             */
        clearColumnFilter : function () {
            this.columnFilterInstance._clearColumnFilter();
        },
            /**
             * Method used to get the currently applied column filters.
             *
             * @method pgbu.grid#getColumnFilter
             * @returns {object} Object specifying current column filters.
             * @public
             */
        getColumnFilter : function () {
            return this.columnFilterInstance._getColumnFilter();
        },

            /**
             * Method used to set new column filters.
             *
             * @method pgbu.grid#setColumnFilter
             * @param {Object} columnFilter - dataIndex keyed object specifying current column filters.
             * @param {Object} columnFilter.dataIndex - dataIndex of the column.
             * @param {Object} columnFilter.term - term to be filtered.
             * @public
             */
        setColumnFilter : function (args) {
            this.columnFilterInstance._setColumnFilter(args);
        },

        applyAdvancedFilters : function (filters) {
            if (this.options.advancedFilters.client) {
                //do client filter
            } else if (_.isFunction(this.store.applyAdvancedFilters)) {
                this.store.applyAdvancedFilters(filters);
            }
        },

        _columnBasedFilter : function (args) {
            /**
             * @event pgbu.grid#before-column-filter
             * @param {Object} [event=null]
             * @param {Object} columnFilter - dataIndex keyed object specifying current column filters.
             * @param {Object} columnFilter.dataIndex - dataIndex of the column.
             * @param {Object} columnFilter.term - term to be filtered.
             */
            if (this._trigger('before-column-filter', null, args) === false) {
                return;
            }
            this.columnFilterInstance._columnBasedFilter(args);
            /**
             * @event pgbu.grid#after-column-filter
             * @param {Object} [event=null]
             * @param {Number} numberOfRows - number of rows which matched the filter criteria. (0 indicates that no rows match the filter criteria, -1 indicates that no filters have been applied.)
             */
            this._trigger('after-column-filter', null, { 'numberOfRows': this.columnFilterInstance._getMatchingCount() });
        },


        /**
         * Method used to filter the grid rows based on the term the user is searching for.
         * @method grid#searchRows
         * @param {String} term - Filter criteria the user has entered.
         * @param {Object} args
         * @param {Boolean} args.matchAllTerms - Whether or not matching records should match the all of the terms entered.
         * @returns {Boolean} - Whether or not the search was performed successfully.  Will return false, if called with no 'term'.
         * @public
         */
        searchRows : function searchRows(term, args) {
            this.searchTerm = term;
            this.searchArgs = args;
            this.searchColumns = _.sortBy(generateSearchableColumns({
                groups : this.options.layout.columnGroups,
                columns : this.columnDefinitions,
                searchHidden : this.options.searchHidden
            }), 'searchPrecedence');
            this.searchColumnIndices = _.pluck(this.searchColumns, 'dataIndex');

            if (args != null && args.matchAllTerms != null) {
                this.options.matchAllTerms = args.matchAllTerms;
            }
            if (term == null) {
                this.clearSearch(FROM_EMPTY_TERM_ARG);
                return false;
            }
            term = term.trim();
            if (!term.length || _.unescape(term).length < this.options.performance.minimumSearchCharacters) {
                this.clearSearch(FROM_EMPTY_TERM_ARG);
                return false;
            }

            this.searchMatcher = searchBuilder(term, { matchOnAny: !this.options.matchAllTerms });
            this._executeFiltering();
        },

        addColumnFilter : function (columnId, value) {
            var column = this.view.columnDefinitions[columnId];
            this.columnMatcher.setFilter(column, value);
            this._executeFiltering();
        },
        removeColumnFilter : function (columnId) {
            this.columnMatcher.removeFilter(columnId);
            if (this._hasFilters()) {
                this._executeFiltering();
            } else {
                this.clearSearch(FORCE_CLEAR_ARG);
            }
        },

        _executeFiltering : function () {
            var that = this;
            this.matchingRecords = { recordOrder : [] };
            if (this.groupingApplied) {
                _.forEach(that.config.rowsets, function (rowset) {
                    if (rowset.rollupRowset != null) {
                        return;
                    }
                    var nodes = that.rootNodes['rowset-' + rowset.id] || [];
                    for (var i = 0; i < nodes.length; i++) {
                        that._matchGroupRecord(that._getRecord(nodes[i]));
                    }
                });
            } else if (this.options.tree && (!this.options.lazyload || this.options.search.forceLazyload)) {
                this.storeRecords = [];
                if (this.options.tree && this.options.data != null) {
                    that.storeRecords = this._getRecordsByData({});
                } else {
                    this.store.iterateStore(this._getRecordFromStore, this);
                }

                _.forEach(this.storeRecords, function (record) {
                    that._matchTreeRecord(record);
                });
            } else {
                this.store.iterateStore(this._matchRecord, this);
            }

            var oldOption = this.options.focusOnSelection;
            this.options.focusOnSelection = false;
            if (this.options.search.showAll) { // TODO applies to filtering, too
                this._forceViewRender();
            } else {
                this.repopulate();
            }
            this.options.focusOnSelection = oldOption;
        },

        _matchTreeRecord : function (record) {
            var recordId = this._getRecordId(record);
            var recordMatches = (!this.options.search.columnFilter) ? this._matchRecord(record) : this.columnFilterInstance._matchRecordColumnBasedFilter(record);
            var childMatches = false;
            var childIds = this._getTreeChildIds(recordId, FOR_SEARCH_ARG);
            for (var i = 0; i < childIds.length; i++) {
                var childRecord = this._getRecord(childIds[i]);
                if (this._matchTreeRecord(childRecord)) {
                    childMatches = true;
                }
            }
            if (childMatches && !recordMatches) {
                this.matchingRecords[recordId] = [];
            }
            return recordMatches || childMatches;
        },

        _matchGroupRecord : function (record) {
            var recordId = this._getRecordId(record);
            var recordMatches = (!this.options.search.columnFilter) ? this._matchRecord(record) : this.columnFilterInstance._matchRecordColumnBasedFilter(record);
            var childMatches = false;
            if (this._isGroupRecord(record)) {
                var childIds = this._getChildIds(this._getRecordId(record), FOR_SEARCH_ARG);
                for (var i = 0; i < childIds.length; i++) {
                    var childRecord = this._getRecord(childIds[i]);
                    if (this._matchGroupRecord(childRecord)) {
                        childMatches = true;
                    }
                }
                if (childMatches && !recordMatches) {
                    this.matchingRecords[recordId] = [];
                }
            }

            return recordMatches || childMatches;
        },

        _hasFilters : function () {
            return Boolean(this.searchMatcher || this.columnMatcher.hasFilters() || this.columnFilterInstance.hasFilters());
        },

        /**
         * Method used to clear the current search criteria, and bring the grid back to an unfiltered state.
         *
         * Note: This method calls repopulate().
         * @method grid#clearSearch
         * @public
         */
        clearSearch : function (args) {
            var cleared = false;
            if (this.searchMatcher) {
                cleared = true;
                this.searchMatcher = null;
                this.searchColumns = [];
                this.searchColumnIndices = [];
            }
            if (this.columnMatcher.hasFilters() && !(args && args.fromEmptyTerm)) {
                cleared = true;
                this.columnMatcher.clear();
            }
            if (cleared || (args && args.force)) {
                this.matchingRecords = {
                    recordOrder : []
                };
                this.view.setRowsetMessage();
                this.repopulate();
            }
        },

        toggleColumns : function (columns, toggle) {
            this.view.toggleColumns(columns, toggle);
            this._addSortIcons();
            if (this.syncedGrid != null) {
                this.syncedGrid.view.toggleColumns(columns, toggle);
            }
        },

        /**
         * Method to determine if the grid has been initialized and has a store attached to it.
         * @method grid#isLoaded
         * @returns {Boolean}
         * @public
         */
        isLoaded : function () {
            return this.store != null;
        },

        /**
         * Method used to display a loading spinner over the grid, to indicate to the user that data is being loaded.
         * @method grid#setLoading
         * @param {Boolean} loading - true to show the loading spinner, false to hide the loading spinner.
         * @public
         */
        setLoading : function (loading) {
            this.view.setLoading(loading);
        },

        /**
         * Method to enable/disable the grid.
         * @method grid#setDisabled
         * @param {Boolean} disabled - true to disable the grid, false to enable the grid.
         * @public
         */
        setDisabled : function (disabled) {
            this.options.disabled = disabled;
            this.view.setDisabled();
        },

        /**
         * Method to display all validation errors for a specific cell.
         * The cell is determined based on the intersection of the recordId and columnId.
         * @method grid#showCellErrors
         * @param {String} recordId - ID of the record to show the errors for.
         * @param {String} columnId - ID of the column to show the errors for.
         * @public
         */
        showCellErrors : function (recordId, columnId) {
            var subRow;
            var realDataIndex = columnId;

            recordId = this.view.fixRecordIdType(recordId);

            if (columnId.indexOf('$$') !== -1) {
                realDataIndex = columnId.substring(0, columnId.indexOf('$$'));
                subRow = columnId.substring(columnId.indexOf('$$') + 2);
            }
            var column = this.columnDefinitions[realDataIndex];
            if (column != null && !column.visible && this.options.addColumnOnShowCellError) {
                var columnGroup = _.find(this.config.groups, function (group) {
                    return group.id === column.groupId;
                });
                if (columnGroup == null) { // use the first flexible group if it is not already in a group
                    columnGroup = _.find(this.config.groups, function (group) {
                        return group.type === 'flex';
                    });
                }
                column.groupId = columnGroup.id;
                var columnIndex = columnGroup.columns.indexOf(column);
                if (columnIndex !== -1) {
                    columnGroup.columns.splice(columnIndex, 1);
                }
                columnGroup.columns.unshift(column);
                this.toggleColumns([realDataIndex], true);
            }
            this.selectCell(recordId, realDataIndex, null, null, subRow);
            this.view.showCellErrors(recordId, columnId);
            this.requestBrowserFocus(true);
        },

        /**
         * Method used to open a print preview in the browser and allow the user to print the currently loaded grid.
         * @method grid#print
         * @param {Boolean} skipPrint
         * @public
         */
        print : function (skipPrint) {
            this.view.openPrintView(skipPrint);
        },

        /**
         * Method to export the data from the grid, into an exportable format.
         * Currently the grid supports: Excel
         * @method grid#exportData
         * @param options
         */
        exportData : function (options) {
            if (this.options.timephased.enabled || this.options.stackedCells.enabled) {
                logger.warn('Exporting not currently supported by stacked or timephased grids.');
                return;
            }
            switch (options.exportTo) {
            case 'csv':
                this.view.exportToCSV();
                break;
            default:
                this.view.exportToExcel();
            }
        },

        /**
         * Method used to get the content of the page, that can be 'printed'.
         *
         * Note: This is not currently supported by stacked cell or timephased grids.
         * @method grid#getPrintContent
         * @returns {String}
         * @public
         */
        getPrintContent : function () {
            return this.view.getPrintContent();
        },

        setRowsetMessage : function (message, rowsetId) {
            this.view.setRowsetMessage(message, rowsetId);
        },

        _syncTimephasedOptions : function (showTimephased, showStackedCells) {
            var disabled = !showTimephased;
            _.forEach(this.options.layout.columnGroups, function (columnGroup) {
                if (columnGroup.showStackedCells || columnGroup.timephased) {
                    columnGroup.disabled = disabled;
                }
                if (showStackedCells && !columnGroup.timephased && columnGroup.showStackedCells) {
                    columnGroup.disabled = false;
                }
            });
            this.options.stackedCells.enabled = showTimephased || showStackedCells;
        },

        showTimephasedColumns : function (showTimephased, showStackedCells) {
            if (showTimephased !== this.options.timephased.enabled || (showStackedCells != null && showStackedCells !== this.options.stackedCells.enabled)) {
                this._syncTimephasedOptions(showTimephased, showStackedCells);
                this.options.layout.rowHeight = 26;
                this.option({ timephased : { enabled : showTimephased } });
            }
        },

        _matchRecord : function (record) {
            var recordId = this._getRecordId(record);
            if (this.matchingRecords[recordId]) {
                return true;
            }

            if (!this.columnMatcher.test(record)) {
                return false;
            }
            if (this.searchMatcher) {
                var cols = this._searchOptionsChain(record);
                if (!this.searchMatcher.test(cols)) {
                    return false;
                }
            }

            // ignore which columns were matched (for searchRows) - matcher#surroundMatches is not too slow
            this.matchingRecords[recordId] = this.searchColumnIndices;
            this.matchingRecords.recordOrder.push(recordId);
            return true;
        },

        _searchOptionsChain : function (record) {
            // allows searchMatcher lazy stuff to come into play, only evaluating cols until it finds a single good one
            /* eslint-disable lodash/unwrap, lodash/no-single-chain */
            var that = this;
            return _(this.searchColumns)
                .map(function (column) {
                    var value = that.store.getValueFromRecord(record, column.dataIndex);
                    if (_.isFunction(column.getSearchValue)) {
                        return column.getSearchValue(record, column.dataIndex, column);
                    } else if (_.isFunction(column.generateSearchFn)) {
                        return genSearchFnFactory(column, value, {
                            dataIndex : column.dataIndex,
                            record : record,
                            column : column,
                            value : value
                        });
                    } else if ((column.displayType === 'text' || column.displayType === 'link' || column.displayType === 'textArea') && column.dataType === 'string') {
                        return value;
                    } else if (column.formattedSearch) {
                        return column.formatter.format(value, column);
                    } else if (column.displayType === 'select' && !_.isFunction(column.selectValues) && _.isObject(column.selectValues)) {
                        var selectValues = column.selectValues;
                        var path;
                        if (value && value.id) {
                            path = value.id;
                        } else {
                            path = value;
                        }
                        return _.get(selectValues, path + '.name', '');
                    } else if (column.displayType === 'picker') {
                        return value && value.name;
                    } else if (column.displayType === 'number') {
                        return value;
                    }
                    return '';
                });
            /* eslint-enable lodash/unwrap lodash/no-single-chain */
        },

        // Misc view interaction methods

        _getRecordId : function (record) {
            if (this._isGroupRecord(record)) {
                return record.id;
            } else {
                return this.store.getValueFromRecord(record, this.recordKeys.id);
            }
        },

        _getRecordIds : function (records) {
            var that = this;
            return _.map(records, function (record) {
                return that._getRecordId(record);
            });
        },

        _getRecordRowset : function (recordId) {
            return this._getRecordPlacement(this._getRecord(recordId)).rowsetId;
        },

        _getNodeLevel : function (record) {
            return this.view.getNodeLevel(this._getRecordId(record), this._getRecordPlacement(record).rowsetId);
        },

        _getTypeFromId : function (recordId) {
            var type;
            var storeType;
            if (recordId == null) {
                type = this.options.data.baseTypes[0];
            } else if (this.store.getTypeFromId != null) {
                type = this.store.getTypeFromId(recordId, this.options.data.hierarchy.order);
            } else if (this.options.data.typesBySuffix != null) {
                storeType = this.options.data.typesBySuffix[recordId.substring(recordId.indexOf(this.options.data.idSeparator))];
                if (storeType != null) {
                    type = storeType;
                }
            }
            if (type == null) {
                type = this.options.data.baseTypes[0];
            }
            return type;
        },

        _getRecord : function (recordId) {
            if (recordId === constants.DUMMY_RECORD_ID) {
                return this._getDummyRecord();
            }
            //>>excludeStart("validation", pragmas.production)
            logger.assert(!_.isObject(recordId), 'recordIds cannot be objects - the method you invoked expected ids');
            //>>excludeEnd("validation")
            if (this._isGroupRecordId(recordId)) {
                return this.recordGrouper.recordGroups[recordId];
            } else if (this.options.data != null && this.store.getRecordByType != null) {
                return this.store.getRecordByType(this._getTypeFromId(recordId), recordId);
            } else if (this.options.rowExpansion.enabled && recordId.indexOf('EXPANDED-') === 0) {
                return this.fakeChildren[recordId];
            } else {
                if (this.options.recordKeys.numericalId && strings.isString(recordId) && digitCheck.test(recordId)) {
                    recordId = parseInt(recordId, 10);
                }
                return this.store.getRecord(recordId);
            }
        },

        _getRecords : function (recordIds) {
            var records = [],
                that = this;
            _.forEach(recordIds, function (recordId) {
                records.push(that._getRecord(recordId));
            });
            return records;
        },

        _getEmptyValue : function (columnDef) {
            var value;
            if (_.has(columnDef, 'emptyValue')) {
                value = _.clone(columnDef.emptyValue);
            } else if (_.has(this.options, 'emptyValue')) {
                value = _.clone(this.options.emptyValue);
            }
            return value;
        },

        _pushUndoAction : function (undoAction, clearRedo) {
            if (this.options.undo) {
                if (clearRedo == null || clearRedo) {
                    this.redoStack = [];
                }
                this.undoStack.push(undoAction);
                var actions = {
                    undo : this._checkAction('undo'),
                    redo : this._checkAction('redo')
                };
                this._trigger('action-state-update', null, actions);
            }
        },

        _getColumnValidator : function (dataIndex) {
            return this.view.columnDefinitions[dataIndex].validator;
        },

        _getTimephasedValueFromRecord : function (args) {
            if (_.isFunction(this.store.getTimephasedValueFromRecord)) {
                return this.store.getTimephasedValueFromRecord(args);
            } else {
                return null;
            }
        },

        _getTimephasedDataFromRecord : function (args) {
            if (_.isFunction(this.store.getTimephasedDataFromRecord)) {
                return this.store.getTimephasedDataFromRecord(args);
            } else {
                return null;
            }
        },

        _getStackedValueFromRecord : function (record, dataIndex, subRow) {
            if (_.isFunction(this.store.getStackedValueFromRecord)) {
                return this.store.getStackedValueFromRecord(record, dataIndex, subRow);
            } else {
                return null;
            }
        },

        _multiCellUpdated : function (recordIds, args) {
            var originalUndoLength = this.undoStack.length;
            /* eslint-disable no-continue */
            for (var i = 0; i < recordIds.length; i++) {
                var record = this._getRecord(recordIds[i]);
                // if editor is open when BPC/store is cleaned out (may not happen in practice), just ignore any possible change
                if (!record) {
                    continue;
                }
                if (this.canEditCell(this._getRecordId(record), args.column.dataIndex, null, args.subRow)) {
                    args.record = record;
                    this._cellUpdated(args);
                }
            }
            /* eslint-enable no-continue */
            if (this.options.undo) {
                var undoAction = {
                    type : 'paste',
                    batch : this.undoStack.splice(originalUndoLength)
                };
                this._pushUndoAction(undoAction);
            }
        },

        // Called from grid view when a change is made
        _cellUpdated : function (args) {
            var record = args.record,
                dataIndex = args.column.dataIndex,
                originalValue = _.clone(this.store.getValueFromRecord(record, dataIndex)),
                newValue = args.value,
                validator = args.column.validator,
                validationResponse = {
                    valid : args.valid,
                    message : args.column.invalidFormatError ? args.column.invalidFormatError : this.options.locale.invalidFormatError
                },
                recordId = this._getRecordId(record);

            if (args.timephased) {
                originalValue = this._getTimephasedValueFromRecord({
                    recordId : recordId,
                    record : record,
                    dataIndex : dataIndex,
                    bucketId : args.timephasedColumn.bucketId,
                    granularity : args.timephasedColumn.granularity
                });
            } else if (args.stackedCell) {
                originalValue = this._getStackedValueFromRecord(record, args.masterColumn, args.subRow);
            }

            if (this._trigger('before-cell-updated', null, {
                record : record,
                dataIndex : dataIndex,
                newValue : args.value,
                originalValue : originalValue,
                validationResponse : validationResponse,
                valid : args.valid

            }) === false) {
                return;
            }

            if (validationResponse.valid) { // Formatter says it is a valid value
                if (validator != null) {
                    validationResponse = validator.validate(newValue);
                }
                if (validationResponse.valid || this.options.overrideValidation) {
                    if (args.timephased && _.isFunction(this.store.validateRecordTimephasedValue)) {
                        validationResponse = this.store.validateRecordTimephasedValue({
                            record : record,
                            recordId : recordId,
                            value : newValue,
                            dataIndex : dataIndex,
                            bucketId : args.timephasedColumn.bucketId,
                            granularity : args.timephasedColumn.granularity,
                            validationResponse : validationResponse
                        });
                    } else if (args.stackedCell && _.isFunction(this.store.validateRecordStackedValue)) {
                        validationResponse = this.store.validateRecordStackedValue({
                            record : record,
                            recordId : recordId,
                            value : newValue,
                            subRow : args.subRow,
                            dataIndex : args.masterColumn,
                            validationResponse : validationResponse
                        });
                    } else if (_.isFunction(this.store.validateRecordValue)) {
                        validationResponse = this.store.validateRecordValue(record, dataIndex, newValue, validationResponse, args.subRow);
                    }
                }

                var validationMessages = null;
                if (validationResponse != null && (!validationResponse.valid || validationResponse.warning)) {
                    validationMessages = {};
                    if (this.options.hideErrorsUntilEdit && args.hideError && (args.value == null || args.value === '')) {
                        validationResponse.hidden = true;
                    }
                    validationMessages[validationResponse.errorType || 'generic'] = _.omit(validationResponse, ['valid', 'errorType']);
                    this._clearInvalid(recordId, args.subRow || dataIndex, false, args.masterColumn); // Skip the trigger as mark invalid is about to reset
                    this._markInvalid(recordId, args.subRow || dataIndex, validationMessages, true, args.masterColumn);
                } else {
                    this._clearInvalid(recordId, args.subRow || dataIndex, true, args.masterColumn);
                }
                var setValue = !_.isEqual(originalValue, newValue);
                if (setValue && originalValue == null && (newValue === '' || newValue == null)) {
                    setValue = false;
                }
                if (setValue || this.options.alwaysResetValue) {
                    if (!args.skipUndo) {
                        this._pushUndoAction(
                            {
                                type : 'edit',
                                dataIndex : dataIndex,
                                data : originalValue,
                                record : record
                            });
                    }
                    if (args.timephased) {
                        this.store.updateRecordTimephasedValue({
                            record : record,
                            recordId : recordId,
                            value : newValue,
                            dataIndex : dataIndex,
                            bucketId : args.timephasedColumn.bucketId,
                            granularity : args.timephasedColumn.granularity,
                            bucketStart : args.timephasedColumn.start,
                            bucketEnd : args.timephasedColumn.end
                        });
                    } else if (args.stackedCell) {
                        this.store.updateRecordStackedValue({
                            record : record,
                            recordId : recordId,
                            value : newValue,
                            subRow : args.subRow,
                            dataIndex : args.masterColumn
                        });
                    } else {
                        this._updateRecord(record, dataIndex, newValue);
                    }
                } else { // Redraw the row even if the values are the same to keep errors from hanging around.
                    this._recordUpdated(record, dataIndex);
                }
                if (args.column.unique && _.isFunction(this.store.checkRecordUniqueness)) {
                    this.store.checkRecordUniqueness(dataIndex, record);
                }
            } else {
                this._clearInvalid(recordId, args.subRow || dataIndex, false, args.masterColumn);// Skip the trigger as markInvalid will trigger it.
                this._markInvalid(recordId, args.subRow || dataIndex,
                    {
                        invalidFormat : {
                            message : validationResponse.message,
                            invalidData : newValue
                        }
                    }, true, args.masterColumn);
                this._recordUpdated(record, dataIndex);
            }
            if (this._selectionHasRecord(recordId)) {
                this.view.reselectRecords(SKIP_FOCUS_ARG);
            }
        },

        _prepareNewRecord : function () {
            return this._getProxyRecord();
        },

        _newRecordCellUpdated : function (args) {
            var record = this._getProxyRecord();
            this.quickAddVisited = true;
            this.quickAddInProgress = true;
            this.store.updateRecord(record, args.dataIndex, args.value);
            this.view.updateRow(constants.DUMMY_RECORD_ID, constants.QUICK_ADD_ROWSET_ID);
            this.quickAddInProgress = false;
        },

        _commitNewRecord : function (options) {
            options = this._prepareOptions(options, 'newRow');
            var focus = this._getQuickAddFocus();
            var record = this._prepareNewRecord();
            var dataIndex = options.column && options.column.dataIndex;
            var recordId = this._getRecordId(record);
            var updateScrollbar;
            var rowsetId = this._getRecordPlacement(record).rowsetId;
            var addResult;
            var afterAdd = function () {
                this._clearRecordSelectionAttributes();
                if (this.quickAddVisited) {
                    this.quickAddVisited = false;
                    this._validateRecord(this._getRecord(recordId), false);
                    this.view.updateRow(recordId, rowsetId);
                }
                if (options.focusAfterAdd) {
                    this.selectCell(recordId, options.columnToEdit || dataIndex || this.view.firstVisibleColumn);
                } else {
                    updateScrollbar = this.view.renderRecord(recordId, rowsetId, 'bottom');
                    /**
                     * @event pgbu.grid#view-sync-scroll-focus-into-view
                     * @param {Object} [event=null]
                     * @param {Object} triggerArgs
                     * @param {String} triggerArgs.recordId - ID of the record to select.
                     */
                    this._trigger('view-sync-scroll-focus-into-view', null, { recordId : recordId });

                    if (updateScrollbar) {
                        this.view.updateRowsetScrollbar(rowsetId);
                    }
                }
            }.bind(this);
            var args = {
                newRecord : record,
                records : [focus],
                focus : focus,
                skipSelect : true,
                skipEdit : true,
                fromQuickAddRow : true,
                quickAdd : {
                    dataIndex : dataIndex,
                    value : options.value,
                    cellUpdatedOptions : options
                }
            };

            this.generateProxyRecord();
            if (this.store.quickAddRecord != null) {
                addResult = this.store.quickAddRecord(args);
            } else {
                addResult = this.newRow(args);
            }
            if (addResult && _.isFunction(addResult.done)) {
                addResult.done(afterAdd);
            } else {
                afterAdd();
            }
        },

        /**
         * Method used to update a field value on a record in the grid.
         * @method grid#_updateRecord
         * @param {Object} record - Record to update.
         * @param {String} dataIndex - Index of the field that is being updated.
         * @param {Object|String|Number} newValue - New value for the field.
         * @private
         * @fires grid#event:before-update-record
         */
        _updateRecord : function (record, dataIndex, newValue) {
            /**
             * @event grid#before-update-record
             * @param {Object} [event=null]
             * @param {Object} triggerArgs
             * @param {Object} triggerArgs.record - Record to be updated.
             * @param {String} triggerArgs.dataIndex - Index of the field to be updated.
             * @param {Object|String|Number} triggerArgs.newValue - New value for the field.
             * @param {Object|String|Number} triggerArgs.oldValue - Old value for the field.
             */
            var triggerResponse = this._trigger('before-update-record', null, {
                record : record,
                dataIndex : dataIndex,
                newValue : newValue,
                oldValue : this.store.getValueFromRecord(record, dataIndex)
            });
            if (triggerResponse !== false) {
                this.store.updateRecord(record, dataIndex, newValue);
            }
        },

        /**
         * Used to set a record as invalid, and to display an indication of being invalid in the UI.
         * @method grid#_markInvalid
         * @param {String} recordId - ID of the record to mark as invalid.
         * @param {String} dataIndex - Field on the record that is invalid.
         * @param {Object} validationMessages
         * @param {Boolean} skipValidation - Whether or not to skip re-validating the record.
         * @param masterColumn
         * @private
         */
        _markInvalid : function (recordId, dataIndex, validationMessages, skipValidation, masterColumn) {
            if (_.isFunction(this.store.markInvalid)) {
                if (masterColumn != null) {
                    dataIndex = masterColumn + '$$' + dataIndex;
                }
                this.store.markInvalid(recordId, dataIndex, validationMessages, false, skipValidation);
            }
            //>>excludeStart("validation", pragmas.production)
            else {
                logger.warn('recordValidtion update failed due to missing method on the mediator markInvalid(recordId, dataIndex, validationMessages)');
            }
            //>>excludeEnd("validation")
        },

        /**
         * Method used to remove any indications that a field is invalid.
         * @method grid#_clearInvalid
         * @param {String} recordId - ID of the record to clear.
         * @param {String} dataIndex -Index of the field to clear validation on.
         * @param {Boolean} skipValidation - Whether or not to skip re-validating the record.
         * @param {String} masterColumn
         * @private
         */
        _clearInvalid : function (recordId, dataIndex, skipValidation, masterColumn) {
            if (_.isFunction(this.store.clearInvalid)) {
                if (masterColumn != null) {
                    dataIndex = masterColumn + '$$' + dataIndex;
                }
                this.store.clearInvalid(recordId, dataIndex, skipValidation);
            }
            //>>excludeStart("validation", pragmas.production)
            else {
                logger.warn('recordValidtion update failed due to missing method on the mediator clearInvalid(recordId, dataIndex)');
            }
            //>>excludeEnd("validation")
        },

        _addRecordToView : function (record, args) {
            var that = this;

            var argsBulk, location, argsSelect;
            if (args == null) {
                argsBulk = true;
                location = this._getRecordPlacement(record);
                argsSelect = false;
            } else {
                argsBulk = args.bulk;
                argsSelect = args.select;
                location = args.location;
                if (location == null) {
                    location = this._getRecordPlacement(record);
                }
            }

            var recordId = this._getRecordId(record);

            if (this.groupingApplied || this.options.tree) {
                var expanded;
                if (this._hasFilters() || this.expanding) {
                    expanded = true;
                }
                this.view.updateNode(recordId, location, this._attrs.treeAttrs(record), expanded);
            }

            if (argsBulk) {
                if (this.view.populateRecords(recordId, location) && !record.isRollupGroup) {
                    var matchingChildren = this._getChildIds(recordId);
                    _.forEach(matchingChildren, function bulkAddToViewLoop(childId) {
                        if (that._hasFilters() && that.matchingRecords[childId] == null) {
                            return;
                        }
                        that._addRecordToView(that._getRecord(childId));
                    });
                }
            } else {
                this.view.addRecord({
                    recordId : recordId,
                    location : location,
                    select : argsSelect
                });
            }
        },

        _getRecordFromStore : function (record) {
            this.storeRecords.push(record);
        },

        _clearRowSelection : function (skipTrigger) {
            var deselectedRecords = this.selection.records;
            var deselectedIds = this.selection.recordIds;
            this.selection.records = [];
            this.selection.recordIds = [];
            this.selection.recordIdMap = {};
            this.view.clearSelection();

            if (!skipTrigger) {
                var triggerArgs = {
                    selectedRecords : this.selection.records,
                    selectedRecordIds : this.selection.recordIds,
                    focusedRecord : null,
                    focusedRecordId : null,
                    deselectedRecords : deselectedRecords,
                    deselectedRecordIds : deselectedIds
                };
                this.rowSelectedTriggerArgs.event = null;
                this.rowSelectedTriggerArgs.triggerArgs = triggerArgs;
                this.debouncedTriggerRowSelected();
            }
        },

        _updateRecordAndNode : function (recordId, expanded) {
            if (recordId != null) {
                var record = this._getRecord(recordId),
                    attributes = this._attrs.treeAttrs(record),
                    location = this._getRecordPlacement(record);
                if (this.view.virtualRows[location.rowsetId].nodes[recordId] != null) {
                    this.view.updateNode(recordId, location, attributes, expanded);
                }
                this.view.updateRow(recordId, location.rowsetId);
            }
        },

        _setHeaderCheckbox : function (dataIndex, checked) {
            this.view.setHeaderCheckbox(dataIndex, checked);
        },

        _updateParentRow : function (parentId) {
            var parent = this._getRecord(parentId);
            var location;

            if (parent != null) {
                location = this._getRecordPlacement(parent);
                this.view.updateNode(parentId, location, this._attrs.treeAttrs(parent));
                this.view.updateRow(parentId, location.rowsetId);
            } else {
                logger.warn('Attempting to update parent row that does not exist.');
            }
        },

        _setSiblingIds : function (newRecord, recordAboveId, recordBelowId) {
            var recordId = this._getRecordId(newRecord);
            this.siblingIds[recordId] = recordAboveId;
            if (recordBelowId != null) {
                this.siblingIds[recordBelowId] = recordId;
            }
        },

        _setRecordSequence : function (record, recordAboveId, recordBelowId) {
            if (this.recordKeys.sequence != null && (!this.groupingApplied || this.options.grouping.enableSequence)) {
                var topSequence = recordAboveId != null ? this.store.getValueFromRecord(this._getRecord(recordAboveId), this.recordKeys.sequence) : null;
                var bottomSequence = recordBelowId != null ? this.store.getValueFromRecord(this._getRecord(recordBelowId), this.recordKeys.sequence) : null;
                var recordSequence = this.store.getValueFromRecord(record, this.recordKeys.sequence);
                var newRecordSequence;

                if (topSequence == null && bottomSequence == null && recordSequence == null) {
                    // Default record sequence to be 100 - will resequence when comparison results in equal sequences
                    newRecordSequence = 100;
                }
                if (topSequence != null && (recordSequence == null || recordSequence < topSequence)) {
                    newRecordSequence = topSequence + 100;
                }
                if (bottomSequence != null && (recordSequence == null || recordSequence > bottomSequence)) {
                    if (bottomSequence - topSequence > 1) {
                        newRecordSequence = topSequence + parseInt((bottomSequence - topSequence) / 2, 10);
                    } else {
                        this._resequenceRecords(record, recordAboveId, recordBelowId, topSequence, bottomSequence);
                        return;
                    }
                }

                if (newRecordSequence != null) {
                    this.store.setRecordValue(record, this.recordKeys.sequence, newRecordSequence);
                }
            }
        },
        // If there is not enough space resequence the ids, it is a greedy approach and searches both directions for
        // freespace then reallocates in one direction once it is found.
        _resequenceRecords : function (record, recordAboveId, recordBelowId, topSequence, bottomSequence) {
            //>>excludeStart("perf", pragmas.compiled)
            window.incrementCallCount('_resequenceRecords');
            //>>excludeEnd("perf")

            var that = this,
                nextRecordAboveId = recordAboveId,
                nextRecordBelowId = recordBelowId,
                spaceAbove = 0,
                spaceBelow = 0,
                nextTopSequence, nextBottomSequence,
                recordsAbove = [],
                recordsBelow = [],
                originalBottom = bottomSequence,
                originalTop = topSequence,
                recordAbove,
                recordBelow, recordsFixed = false;
            if (record != null) {
                recordsAbove = [record];
                recordsBelow = [record];
            }
            // How to fix the records once they have been found. Find the step by dividing up the free space and reset all records passed in.
            var sequenceKey = this.recordKeys.sequence;
            var fixRecords = function (records, startNumber, endNumber) {
                var len = records.length;
                var step = Math.floor((endNumber - (startNumber - 1)) / len);
                for (var i = 0; i < len; ++i) {
                    var record = records[i];
                    that.store.updateRecord(record, sequenceKey, startNumber + (step * i), true);
                }
            };

            // Check while there are still more to look for in one direction.
            var siblingIdArray, siblingIdArrayLength, recordBelowIndex, recordAboveIndex;
            while (!recordsFixed && (nextRecordAboveId != null || nextRecordBelowId != null)) {
                if (nextRecordAboveId != null) {
                    recordAbove = that._getRecord(nextRecordAboveId);
                    recordsAbove.push(recordAbove);
                    nextTopSequence = that.store.getValueFromRecord(recordAbove, that.recordKeys.sequence);
                }
                if (nextRecordBelowId != null) {
                    recordBelow = that._getRecord(nextRecordBelowId);
                    recordsBelow.push(recordBelow);
                    nextBottomSequence = that.store.getValueFromRecord(recordBelow, that.recordKeys.sequence);
                }

                spaceAbove = topSequence - nextTopSequence;
                spaceBelow = nextBottomSequence - bottomSequence;
                // Favor adding space via the records below.
                if (spaceBelow > 1 || spaceAbove > 1) {
                    if (spaceBelow >= spaceAbove) {
                        fixRecords(recordsBelow, originalBottom, nextBottomSequence);
                    } else {
                        fixRecords(recordsAbove.reverse(), nextTopSequence, originalTop);
                    }
                    recordsFixed = true;
                } else {
                    if (siblingIdArray == null) {
                        siblingIdArray = that.view.siblingIdsOfRecordAtLevel(recordBelowId || recordAboveId);
                        siblingIdArrayLength = siblingIdArray.length;
                        recordBelowIndex = recordBelowId != null ? siblingIdArray.indexOf(recordBelowId) : null;
                        recordAboveIndex = recordAboveId != null ? siblingIdArray.indexOf(recordAboveId) : null;
                    }
                    nextRecordAboveId = recordAboveIndex > 0 ? siblingIdArray[--recordAboveIndex] : null;
                    nextRecordBelowId = recordBelowIndex < (siblingIdArrayLength - 1) ? siblingIdArray[++recordBelowIndex] : null;

                    // If there is no bottom fix it that way as you can add more space and it will end up in less nodes being modified
                    if (nextRecordBelowId == null) {
                        fixRecords(recordsBelow, originalBottom, originalBottom + recordsBelow.length * 100);
                        recordsFixed = true;
                    }
                    topSequence = nextTopSequence;
                    bottomSequence = nextBottomSequence;
                }
            }
            // If there is no space redo them all and add lots of extra space!
            if (!recordsFixed) {
                recordsAbove.splice(0, 1);
                var allRecords = recordsAbove.concat(recordsBelow);
                fixRecords(allRecords, 0, allRecords.length * 100);
            }
        },

        _swapSequenceNumbers : function (firstRecord, secondRecord) {
            if (this.recordKeys.sequence != null) {
                var firstRecordSequence = this.store.getValueFromRecord(firstRecord, this.recordKeys.sequence);
                var secondRecordSequence = this.store.getValueFromRecord(secondRecord, this.recordKeys.sequence);
                if (firstRecordSequence === secondRecordSequence) {
                    // Fix the sequences and then perform the swap that was intended to happen
                    this._resequenceRecords(null, this._getRecordId(firstRecord), this._getRecordId(secondRecord), firstRecordSequence, secondRecordSequence);
                    this._swapSequenceNumbers(firstRecord, secondRecord);
                } else {
                    this.store.updateRecord(firstRecord, this.recordKeys.sequence, secondRecordSequence, true);
                    this.store.updateRecord(secondRecord, this.recordKeys.sequence, firstRecordSequence, true);
                }
            }
        },

        _setNewRecordParent : function (args) {
            var parent, sibling, parentId;
            if (this.options.data != null && this.options.tree) {
                parentId = this._setDataTreeValues(args);
            } else if (this.options.tree || this.groupingApplied) {
                // Apply Parent Id
                // If this should be a child of the focused record.
                if (args.isChild || this._isGroupRecord(args.focus)) {
                    parent = args.focus;
                    if (this._isGroupRecord(parent)) {
                        return;
                    }
                    if (parent != null) {
                        parentId = this._getRecordId(parent);
                    }
                // If this is should be sibling of the focused record, set parentId to the siblings parent
                } else {
                    sibling = args.focus;
                    parentId = this.store.getValueFromRecord(sibling, this.recordKeys.parentId);
                }
                this.store.setRecordValue(args.record, this.recordKeys.parentId, parentId);
            }
            return parentId;
        },

        _setDataTreeValues : function (args) {
            if (this.options.data != null && !this.groupingApplied) {
                var parent, parentType, parentId, joinValue, foreignKey;
                var recordId = this._getRecordId(args.record);
                var childType = args.childType || this._getTypeFromId(recordId);
                // Adding root items
                if (args.focus == null) {
                    var typeDefinition = this.options.data.types[childType];
                    this.store.setRecordValue(args.record, typeDefinition.root.localKey, typeDefinition.root.joinValue, childType);
                    this.parentIds[recordId] = parentId;
                    if (typeDefinition.root.constraints != null) {
                        this._applyDataTreeContraints({
                            record : args.record,
                            constraints : typeDefinition.root.constraints
                        });
                    }
                } else {
                    if (args.isChild) {
                        parent = args.focus;
                        parentId = this._getRecordId(parent);
                    } else {
                        parentId = this.parentIds[this._getRecordId(args.focus)];
                        if (parentId != null) {
                            parent = this._getRecord(parentId);
                        }
                    }
                    if (parentId != null) {
                        parentType = args.parentType || this._getTypeFromId(parentId);
                        var parentDefinition = this.options.data.types[parentType];
                        this.parentIds[recordId] = parentId;

                        if (childType == null || parentType == null || _.get(parentDefinition.mapping, childType, null) == null) {
                            return;
                        }
                        joinValue = this.store.getValueFromRecord(parent, parentDefinition.mapping[childType].localKey);
                        foreignKey = parentDefinition.mapping[childType].foreignKey;
                        this.store.setRecordValue(args.record, foreignKey, joinValue, childType);
                        if (parentDefinition.mapping[childType].constraints != null) {
                            this._applyDataTreeContraints({
                                record : args.record,
                                constraints : parentDefinition.mapping[childType].constraints
                            });
                        }
                    }
                }
                this.dataCache.getRecordPlacement(recordId, { clear : true });
                return parentId;
            }
        },

        _applyDataTreeContraints : function (args) {
            var that = this;
            _.forEach(args.constraints, function (constraint) {
                that.store.setRecordValue(args.record, constraint.key, constraint.value);
            });
        },

        _setDeletedRecordSiblings : function (deletedRecord) {
            var deletedRecordId = this._getRecordId(deletedRecord);
            var previousSiblingId = this.view.previousSibling(deletedRecordId);
            var nextSiblingId = this.view.nextSibling(deletedRecordId);
            if (nextSiblingId != null) {
                this.siblingIds[nextSiblingId] = previousSiblingId;
            }

            return previousSiblingId;
        },

        /*
         * Sorts an array of recordIds using the current
         */
        _sortRecords : function (records, isRecord) {
            var that = this;
            var appliedSort = that.config.sortBy;
            var store = that.store;
            timsort.sort(records, function sortCallback(left, right) {
                var leftRecord = isRecord ? left : that._getRecord(left);
                var rightRecord = isRecord ? right : that._getRecord(right);
                for (var i = 0, len = appliedSort.length; i < len; ++i) {
                    var sortConf = appliedSort[i];
                    var dataIndex = sortConf.dataIndex;
                    var a = store.getValueFromRecord(leftRecord, dataIndex);
                    var b = store.getValueFromRecord(rightRecord, dataIndex);
                    if (sortConf.getSortValue) {
                        a = sortConf.getSortValue(a, {
                            value : a,
                            record : leftRecord,
                            dataIndex : dataIndex,
                            column : that.columnDefinitions[dataIndex]
                        });
                        b = sortConf.getSortValue(b, {
                            value : b,
                            record : rightRecord,
                            dataIndex : dataIndex,
                            column : that.columnDefinitions[dataIndex]
                        });
                    }
                    var asc = sortConf.order === 'asc';
                    var match = sortConf.sorter.call(that, a, b, asc, sortConf.options);
                    if (match !== 0) {
                        return match;
                    }
                }
                return 0;
            });
        },

        /*
         * Sort records when the view is flat. Loops through each rowset
         * and applies sorting. Or clears it as necesarry
         */
        _sortFlatRecords : function () {
            var that = this;
            var appliedSort = that.config.sortBy;
            if (appliedSort.length) {
                _.forEach(that.config.rowsets, function (rowset) {
                    if (rowset.sortable) {
                        //>>excludeStart("perf", pragmas.production)
                        var start;
                        if (that.options.debug) {
                            start = window.performance.now();
                        }
                        //>>excludeEnd("perf")
                        var records = that.view.getRecordOrder(rowset.id);
                        var hadDummy = false;
                        if (_.last(records) === constants.DUMMY_RECORD_ID || _.first(records) === constants.DUMMY_RECORD_ID) { // Strip out the dummy row
                            records = _.without(records, constants.DUMMY_RECORD_ID);
                            hadDummy = true;
                        }
                        if (records.length) {
                            if (that.reverseSortOrder) {
                                records.reverse();
                            } else {
                                that._sortRecords(records);
                            }
                        }
                        if (hadDummy) { // Readd the dummy row if needed
                            records.push(constants.DUMMY_RECORD_ID);
                        }

                        //>>excludeStart("perf", pragmas.production)
                        if (that.options.debug) {
                            var info = _.pluck(appliedSort, 'dataIndex');
                            info.splice(0, 0, rowset.id);
                            that._trigger('performance-debug', null,
                                {
                                    start : start,
                                    end : window.performance.now(),
                                    name : 'sort-Records',
                                    type : 'widget',
                                    info : info
                                });
                        }
                        //>>excludeEnd("perf")
                        that.view.applySortOrder(rowset.id, records);
                    }
                });
                that.reverseSortOrder = false;
            } else {
                that.view.clearSort();
            }
            this._trigger('action-state-update', null, this._checkActions());
        },

        /*
         * Called from dragging and dropping rows, location should have the new siblingId
         */
        _moveRows : function (location) {
            var that = this;
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (that.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            var destinationRecord = this._getRecord(location.siblingId);
            var placeBefore = false;
            if (location.siblingId == null) {
                destinationRecord = this._getRecord(this.view.virtualRows[location.rowsetId].records[0]);
                placeBefore = true;
            }
            var triggerValues = { records : this.getSelection(), afterRecord : destinationRecord };

            /**
                 * Triggered before row are moved.
                 * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
                 *
                 * @event pgbu.grid#before-move-rows
                 * @param {Object} [event=null]
                 * @param {Object} triggerArgs
                 * @param {Array<Object>} triggerArgs.records - Records to be moved.
                 * @param {Object} triggerArgs.afterRecord - Record to immediately follow the moved records.
                 */
            if (this._trigger('before-move-rows', null, triggerValues) === false || _.includes(this.getSelection(), destinationRecord)) {
                return;
            }
            var records = this.getSelection();
            _.forEach(records, function (record) {
                that._setDeletedRecordSiblings(record);

                var recordAbove, recordBelow;

                if (placeBefore) {
                    recordBelow = that._getRecordId(destinationRecord);
                } else {
                    recordAbove = that._getRecordId(destinationRecord);
                    recordBelow = that.view.nextSibling(recordAbove);
                }

                that._setSiblingIds(record, recordAbove, recordBelow);
                that._setRecordSequence(record, recordAbove, recordBelow);
                that.store.recordMoved(record, recordAbove);
                destinationRecord = record;
                that.selectCell();
                that.resize();
            });

            this._trigger('rows-moved', null, triggerValues);
            that.selectCell();
            //>>excludeStart("perf", pragmas.production)
            if (that.options.debug) {
                that._trigger('performance-debug', null,
                    {
                        start : start,
                        end : window.performance.now(),
                        name : 'move-Rows',
                        type : 'widget',
                        info : this.getSelection()
                    });
            }
            //>>excludeEnd("perf")
        },
        /*
         * Called from drag and drop code when moving into a new rowset,
         * destination should be the new rowsetId
         */
        _dropRows : function (origin, destination) {
            var that = this;

            //>>excludeStart("perf", pragmas.production)
            var start;
            if (that.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")

            var triggerArgs = {
                records : that.getSelection(),
                origin : origin, dropRowset : destination
            };
            /**
                 * Triggered before a user drops a row after dragging it from one rowset to another.
                 * This event is fired before action takes place on the grid. Therefore allowing a grid user to prevent default behavior by returning false.
                 *
                 * @event pgbu.grid#before-drop-rows
                 * @param {Object} [event=null]
                 * @param {Object} triggerArgs
                 * @param {Array<Object>} triggerArgs.records - Records to be dropped.
                 * @param {Object} triggerArgs.origin - Rowset the records originally belonged to.
                 * @param {Object} triggerARgs.dropRowset - Rowset the records are to be dropped into.
                 */
            if (that._trigger('before-drop-rows', null, triggerArgs) === false || destination == null) {
                return;
            }

            var lastRecordId = _.last(that.view.virtualRows[destination].records);

            var records = this.getSelection();
            _.forEach(records.reverse(), function (record) {   // LUX-266  DND MULTIPLE ROWS TO DIFFERENT ROWSET PUTS ROWS BACKWARDS
                that._setDeletedRecordSiblings(record);
                that._setSiblingIds(record, lastRecordId, null);
                that._setRecordSequence(record, lastRecordId);
                that.store.recordMoved(record, lastRecordId, null, null, destination);
            });

            that.resize();

            /**
                 * Triggered after rows are dragged-and-dropped.
                 * This event is captured after certain action takes place on the grid.
                 *
                 * @event pgbu.grid#rows-dropped
                 * @param {Object} [event=null]
                 * @param {Object} triggerArgs
                 * @param {Array<Object>} triggerArgs.records - Records to be dropped.
                 * @param {Object} triggerArgs.origin - Rowset the records originally belonged to.
                 * @param {Object} triggerARgs.dropRowset - Rowset the records are to be dropped into.
                 */
            that._trigger('rows-dropped', null, triggerArgs);
            //>>excludeStart("perf", pragmas.production)
            if (that.options.debug) {
                that._trigger('performance-debug', null,
                    {
                        start : start,
                        end : window.performance.now(),
                        name : 'drop-Rows',
                        type : 'widget',
                        info : that.getSelection()
                    });
            }
            //>>excludeEnd("perf")
        },
        _getDefaultRecordAttributes : function (recordId /*, record */) {
            var defaultAttributes = defaultRecordAttributes();
            if (this.options.tree && this.options.data) {
                var recordType = this._getTypeFromId(recordId);
                defaultAttributes.precedence = this.options.data.hierarchy.order.indexOf(recordType);
                var typeDefinition = this.options.data.types[recordType];
                if (typeDefinition.childTypes != null && typeDefinition.childTypes.length && _.intersection(this.options.data.hierarchy.order, typeDefinition.childTypes).length) {
                    defaultAttributes.isNode = true;
                    defaultAttributes.hasChildren = true;
                } else {
                    defaultAttributes.isNode = false;
                    defaultAttributes.hasChildren = false;
                }
            }
            return defaultAttributes;
        },

        _getDefaultRecordTreeAttributes : function (recordId /*, record */) {
            var defaultAttributes = defaultRecordTreeAttributes();
            if (this.options.tree && this.options.data) {
                var recordType = this._getTypeFromId(recordId);
                defaultAttributes.precedence = this.options.data.hierarchy.order.indexOf(recordType);
                var typeDefinition = this.options.data.types[recordType];
                if (typeDefinition.childTypes != null && typeDefinition.childTypes.length &&
                        _.intersection(this.options.data.hierarchy.order, typeDefinition.childTypes).length) {
                    defaultAttributes.isNode = true;
                    defaultAttributes.hasChildren = true;
                } else {
                    defaultAttributes.isNode = false;
                    defaultAttributes.hasChildren = false;
                }
            }
            return defaultAttributes;
        },

        /*
         * Get record attributes for the view, this done so we can give group nodes record attributes and inject
         * selection and focus data onto what the store returns.
         */
        _getRecordAttributes : function (record, args) {
            var attributes, that = this, recordAboveId, recordBelowId;
            var recordId = that._getRecordId(record);
            var isProxy = false;
            var proxyRecord;

            if (this._isGroupRecord(record)) {
                attributes = {
                    sortable : true,
                    readOnly : true,
                    isNode : true,
                    hasChildren : record.records.length > 0 || record.childGroups.length > 0 || record.hierarchyGroups.length > 0,
                    modified : false,
                    childrenLoaded : true,
                    expanded : !this.options.grouping.collapseByDefault,
                    disableRowMenu : record.isRollupGroup ? true : false,
                    stackedCells : defaultStackedCellAttributes(),
                    timephased : defaultTimephasedAttributes()
                };
            } else {
                proxyRecord = this._getProxyRecord();
                if (recordId === constants.DUMMY_RECORD_ID || (proxyRecord != null && this._getRecordId(record) === this._getRecordId(proxyRecord))) {
                    attributes = this.store.getRecordAttributes(proxyRecord, this._getDefaultRecordAttributes(this._getRecordId(proxyRecord), proxyRecord));
                    attributes.modified = {};
                    attributes.newRow = false;
                    attributes.isNode = false;
                    attributes.disableRowMenu = true;
                    isProxy = true;
                } else {
                    attributes = this.store.getRecordAttributes(record, this._getDefaultRecordAttributes(recordId, record));
                }

                if (this.options.rowExpansion.enabled) {
                    if (attributes.expandableRow && !_.get(record, 'fakeChild')) {
                        _.assign(attributes, {
                            isNode : true,
                            hasChildren : true,
                            childrenLoaded : true,
                            sortable : true
                        });
                    }
                    if (_.get(record, 'fakeChild')) {
                        attributes.editable = false;
                    }
                }

                if (this.tree && this._hasFilters()) {
                    attributes.expanded = true;
                }
                if (this.options.timephased.enabled) {
                    if (this.store.getRecordTimephasedAttributes != null) {
                        attributes.timephased = this.store.getRecordTimephasedAttributes({
                            record : record,
                            recordId : recordId,
                            defaultArgs : defaultTimephasedAttributes(),
                            granularity : this.view.timephasedColumnGroup.appliedTimescale.options.granularity,
                            buckets : this.view.timephasedColumnGroup.getColumnDefinitions()
                        });
                    } else {
                        attributes.timephased = defaultTimephasedAttributes();
                    }
                }
                if (this.options.stackedCells.enabled) {
                    if (this.store.getRecordStackedCellAttributes != null) {
                        attributes.stackedCells = this.store.getRecordStackedCellAttributes({
                            record : record,
                            recordId : recordId,
                            defaultArgs : defaultStackedCellAttributes()
                        });
                    } else {
                        attributes.stackedCells = defaultStackedCellAttributes();
                    }
                }

                var scope = this.clipboard.scope;
                if (!this.clipboard.hideStyling) {
                    if (_.find(this.clipboard.records, function (clipboardRecord) {
                        return that._getRecordId(clipboardRecord) === recordId;
                    })) {
                        var type = this.clipboard.mode === 'copy' ? 'copied' : 'cut';
                        var classPrefix = scope + '-' + type;
                        var classes = classPrefix + ' ';

                        if (this.clipboard.records.length > 1) {
                            recordAboveId = that.view.recordAbove(recordId);
                            var recordAboveCopied = _.find(this.clipboard.records, function (clipboardRecord) {
                                return that._getRecordId(clipboardRecord) === recordAboveId;
                            });
                            if (recordAboveCopied == null) {
                                classes += classPrefix + '-top ';
                            }

                            recordBelowId = that.view.recordBelow(recordId);
                            var recordBelowCopied = _.find(this.clipboard.records, function (clipboardRecord) {
                                return that._getRecordId(clipboardRecord) === recordBelowId;
                            });

                            if (recordBelowCopied == null) {
                                classes += classPrefix + '-bottom ';
                            }
                        } else {
                            classes += classPrefix + '-top ';
                            classes += classPrefix + '-bottom ';
                        }
                        attributes.cutCopyClasses = classes;
                    }
                }
            }

            if (args == null || args.includeSelection) {
                _.assign(attributes, this._getRecordSelectionAttributes(recordId));
                if (this.options.selectionMode === 'check') {
                    attributes.checked = _.includes(this.checkedRecordIds, recordId);
                }
                if (isProxy) {
                    attributes.selected = this._selectionHasRecord(constants.DUMMY_RECORD_ID);
                }
            }

            if (this._hasFilters()) {
                attributes.found = this.matchingRecords[recordId];
            }

            return attributes;
        },

        _getRecordTreeAttributes : function (record) {
            var attributes;
            var recordId = this._getRecordId(record);

            if (this._isGroupRecord(record)) {
                attributes = {
                    sortable : true,
                    isNode : true,
                    hasChildren : record.records.length > 0 || record.childGroups.length > 0 || record.hierarchyGroups.length > 0,
                    childrenLoaded : true,
                    expanded : !this.options.grouping.collapseByDefault
                };
            } else if (record === this.dummyRecord) {
                attributes = this.dummyRecordAttributes || defaultRecordAttributes();
                attributes.isNode = false;
            } else if (this.store.getRecordTreeAttributes != null) {
                attributes = this.store.getRecordTreeAttributes(record, this._getDefaultRecordTreeAttributes(recordId, record));
            } else {
                attributes = this.store.getRecordAttributes(record, this._getDefaultRecordAttributes(recordId, record));
            }

            return attributes;
        },

        _getRecordSelectionAttributes : function (recordId) {
            if (this.selectionAttributes[recordId] != null) {
                return this.selectionAttributes[recordId];
            }

            var testId = recordId;
            var proxyRecord = this._getProxyRecord();
            if (proxyRecord && recordId === this._getRecordId(proxyRecord)) {
                testId = constants.DUMMY_RECORD_ID;
            }

            if (!this._selectionHasRecord(testId)) {
                return {
                    selected : false,
                    focused : false
                };
            } else {
                var optClasses = this.options.classes;
                var attrs = {
                    selected : true,
                    focused : this.focus.recordId === recordId,
                    cellSelectedClasses : optClasses.cellSelected,
                    rowSelectedClasses : optClasses.rowSelected
                };

                var recordAboveId = this.view.recordAbove(recordId);
                var recordAboveSelected = this._selectionHasRecord(recordAboveId);

                if (!recordAboveSelected) {
                    attrs.cellSelectedClasses += ' ' + optClasses.cellSelectedTop;
                    attrs.rowSelectedClasses += ' ' + optClasses.rowSelectedTop;
                }

                var recordBelowId = this.view.recordBelow(recordId);
                var recordBelowSelected = this._selectionHasRecord(recordBelowId);

                if (!recordBelowSelected) {
                    attrs.cellSelectedClasses += ' ' + optClasses.cellSelectedBottom;
                    attrs.rowSelectedClasses += ' ' + optClasses.rowSelectedBottom;
                }

                this.selectionAttributes[recordId] = attrs;
                return attrs;
            }
        },

        _clearRecordSelectionAttributes : function () {
            this.selectionAttributes = {};
        },

        _selectionHasRecord : function (recordId) {
            return this.selection.recordIdMap[recordId];
        },

        _getRecordPlacement : function (record, skipGrouping) {
            var location;
            var recordId;

            if (this.options.quickAdd.enabled && record != null && record === this.dummyRecord) {
                return {
                    rowsetId : constants.QUICK_ADD_ROWSET_ID,
                    table : '$tbody'
                };
            }

            if (this._isGroupRecord(record)) {
                location = this.recordGrouper.getGroupPlacement(record, skipGrouping);
            } else if (this.groupingApplied && (skipGrouping == null || !skipGrouping)) {
                location = this.recordGrouper.getRecordPlacement(record);
            }
            if (record && record.fakeChild) {
                location = record.location;
            }
            if (record === this.dummyRecord) {
                location = {
                    rowsetId : constants.QUICK_ADD_ROWSET_ID,
                    table : '$tbody'
                };
            } else if (location == null) {
                location = this.store.placeRecord(record);
                recordId = this._getRecordId(record);
                location.siblingId = this.siblingIds[recordId];
                if (this.options.tree && this.options.data != null) {
                    location.parentId = this.parentIds[recordId] || _.get(this.dataCache.recordCache[recordId], 'parentId');
                }
            }
            return location;
        },

        _assignSiblingIds : function (records) {
            this._sortRecordsBySequence(records);
            return this._assignSiblingsLoop(records);
        },
        _sortRecordsBySequence : function (records) {
            if (this.recordKeys.sequence != null) { // todo: filter then sort
                var intSorter = sorters('integer');
                var store = this.store;
                var seqKey = this.recordKeys.sequence;
                timsort.sort(records, function (left, right) {
                    var a = store.getValueFromRecord(left, seqKey),
                        b = store.getValueFromRecord(right, seqKey);
                    return intSorter(a, b, 'asc');
                });
            }
            return records;
        },
        _assignSiblingsLoopIds : function (ids) {
            var previousId = null;
            for (var i = 0, len = ids.length; i < len; ++i) {
                var nextId = ids[i];
                this.siblingIds[nextId] = previousId;
                previousId = nextId;
            }
        },
        _assignSiblingsLoop : function (records) {
            var previousId = null;
            for (var i = 0, len = records.length; i < len; ++i) {
                var recordId = this._getRecordId(records[i]);
                this.siblingIds[recordId] = previousId;
                previousId = recordId;
            }
            return records;
        },

        _getChildRecords : function (recordId) {
            if (this.options.data != null) {
                return this._getRecordsByData({ parentId : recordId });
            } else if (this.options.rowExpansion.enabled) {
                var children = this.fakeChildren[recordId] || [];
                if (!children.length) {
                    var parentPlacement = this._getRecordPlacement(this._getRecord(recordId));
                    for (var i = 0; i < this.options.rowExpansion.heightInRows; i++) {
                        children.push(this._getFakeChild(recordId, i, parentPlacement));
                    }
                    this.fakeChildren[recordId] = children;
                }
                return children;
            } else {
                return this.store.getChildRecords(recordId);
            }
        },

        _getFakeChild : function (recordId, counter, parentPlacement) {
            var fake = {
                parentId : recordId,
                location : {
                    parentId : recordId,
                    rowsetId : parentPlacement.rowsetId,
                    table : parentPlacement.table
                },
                fakeChild : true
            };
            var id = 'EXPANDED-' + recordId + '-' + counter;
            fake[this.options.recordKeys.id] = id;
            this.fakeChildren[id] = fake;
            return fake;
        },

        _orderChildIds : function (args) {
            var that = this;
            // Find the correct rowsetId and config to see if it can be sorted
            var rowsetId = this._getRecordRowset(args.recordId);
            var rowsetConfig = _.find(this.config.rowsets, function (rowset) {
                return rowset.id === rowsetId;
            });
            var recordsByPrecedence = {},
                childId, record, i, len;
            for (i = 0, len = args.childRecords.length; i < len; i++) {
                record = args.childRecords[i];
                childId = this._getRecordId(record);
                if (this.options.data != null && !this.groupingApplied) {
                    this.parentIds[childId] = args.recordId;
                }
                var attrs = that._attrs.treeAttrs(record);
                var precedence = attrs.precedence;
                var sortable = attrs.sortable;
                // Create the precedence group if it doesnt exist.
                if (recordsByPrecedence[precedence] == null) {
                    recordsByPrecedence[precedence] = {
                        precedence : precedence,
                        sortableIds : [],
                        unsequencedRecords : [],
                        orderedIds : []
                    };
                }

                if (this.sortingApplied && sortable && rowsetConfig.sortable) { // placerecord, config.rowsets[location.rowsetId]
                    recordsByPrecedence[precedence].sortableIds.push(childId);
                    // Prepare to order the records by the sibling Id
                } else {
                    recordsByPrecedence[precedence].unsequencedRecords.push(record);
                }
            }

            var properOrderedIds = [];

            _.forEach(_.sortBy(_.values(recordsByPrecedence), 'precedence'), function (precedenceGroup) {
                // Find the correct sibling order
                var orderedRecords = that._assignSiblingIds(precedenceGroup.unsequencedRecords);
                for (var j = 0; j < orderedRecords.length; j++) {
                    precedenceGroup.orderedIds.push(that._getRecordId(orderedRecords[j]));
                }
                properOrderedIds = properOrderedIds.concat(precedenceGroup.orderedIds);
                // Sort any records that can be.
                if (that.sortingApplied) {
                    that._sortRecords(precedenceGroup.sortableIds);
                }
                properOrderedIds = properOrderedIds.concat(precedenceGroup.sortableIds);
            });

            if (!args.forSearch && this._hasFilters()) { // If a search is done make sure it matches
                properOrderedIds = _.reject(properOrderedIds, function (idToTest) {
                    return that.matchingRecords[idToTest] == null;
                });
            }

            this._assignSiblingsLoopIds(properOrderedIds);

            return properOrderedIds;
        },

        _getTreeChildIds : function (recordId, options) {
            var that = this;
            var childRecords = this._getChildRecords(recordId);
            var recordsByPrecedence = {};
            var i;

            var parentRecord = this._getRecord(recordId);
            // Find the correct rowsetId and config to see if it can be sorted
            var rowsetId = this.store.placeRecord(parentRecord).rowsetId;
            var rowsetConfig = _.find(this.config.rowsets, function (rowset) {
                return rowset.id === rowsetId;
            });

            var childId, record;
            for (i = 0; i < childRecords.length; i++) {
                record = childRecords[i];
                childId = this._getRecordId(record);
                if (this.options.data != null) {
                    this.parentIds[childId] = recordId;
                }
                var attrs = that._attrs.treeAttrs(record);
                var precedence = attrs.precedence;
                var sortable = attrs.sortable;
                // Create the precedence group if it doesnt exist.
                if (recordsByPrecedence[precedence] == null) {
                    recordsByPrecedence[precedence] = {
                        precedence : precedence,
                        sortableIds : [],
                        unsequencedRecords : [],
                        orderedIds : []
                    };
                }

                // If grouping is applied, find the group for the record
                if (this.groupingApplied) {
                    this.recordGrouper.groupRecord(record);
                    recordsByPrecedence[precedence].orderedIds.push(childId);
                    // If sorting applied and sortable ignore sibling fixing
                } else if (this.sortingApplied && sortable && rowsetConfig.sortable) { // placerecord, config.rowsets[location.rowsetId]
                    recordsByPrecedence[precedence].sortableIds.push(childId);
                    // Prepare to order the records by the sibling Id
                } else {
                    recordsByPrecedence[precedence].unsequencedRecords.push(record);
                }
            }

            var properOrderedIds = [];
            if (this.groupingApplied) {
                properOrderedIds = this.rootNodes[recordId] || [];
            }
            _.forEach(_.sortBy(_.values(recordsByPrecedence), 'precedence'), function (precedenceGroup) {
                // Find the correct sibling order
                var orderedRecords = that._sortRecordsBySequence(precedenceGroup.unsequencedRecords);
                for (var j = 0; j < orderedRecords.length; j++) {
                    precedenceGroup.orderedIds.push(that._getRecordId(orderedRecords[j]));
                }
                properOrderedIds = properOrderedIds.concat(precedenceGroup.orderedIds);
                // Sort any records that can be.
                if (that.sortingApplied) {
                    that._sortRecords(precedenceGroup.sortableIds);
                }
                properOrderedIds = properOrderedIds.concat(precedenceGroup.sortableIds);
            });

            if (options == null && this._hasFilters()) { //If a search is done make sure it matches
                properOrderedIds = _.reject(properOrderedIds, function (idToTest) {
                    return that.matchingRecords[idToTest] == null;
                });
            }

            this._assignSiblingsLoopIds(properOrderedIds);

            return properOrderedIds;
        },

        /**
         * Method to get all of the ids of the children for a specific record.
         *
         * Note: This method only applies to hierarchical (Tree) grids.
         * @method grid#_getChildIds
         * @param {String} recordId - ID of the parent record.
         * @returns {Array<String>} All of the child record IDs.
         * @private
         */
        _getChildIds : function (recordId, rawArgs) {
            var args = rawArgs != null ? rawArgs : {};
            if (this._isGroupRecordId(recordId)) {
                return this.recordGrouper.getGroupChildIds(recordId, args);
            } else {
                return this._getTreeChildIds(recordId, args);
            }
        },

        /**
         * Used to check if a record is a group record.
         * @method pgbug.grid#_isGroupdRecord
         * @param {Object} record - Record to be checked
         * @returns {Boolean} Whether or not the record is a group record.
         * @private
         */
        _isGroupRecord : function (record) {
            //>>excludeStart("validation", pragmas.production)
            if (strings.isString(record)) {
                throw new Error('_isGroupRecordId should have been invoked, not _isGroupRecord.  File a bug against gridapi.');
            }
            //>>excludeEnd("validation")
            if (!this.groupingApplied && !this.rollupApplied) {
                return false;
            }
            if (record != null && record.isGroup != null) {
                return record.isGroup;
            }
            return false;
        },
        /**
         * Used to check if a recordId matches the signature of group records.
         * @method pgbug.grid#_isGroupdRecordId
         * @param {Object} record - Record to be checked
         * @returns {Boolean} Whether or not the record is a group record.
         * @private
         */
        _isGroupRecordId : function (recordId) {
            //>>excludeStart("validation", pragmas.production)
            if (_.isObject(recordId) || (recordId != null && !strings.isString(recordId) && !_.isNumber(recordId))) {
                throw new Error('_isGroupRecord should have been invoked, not _isGroupRecordId.  File a bug against gridapi.');
            }
            //>>excludeEnd("validation")

            if (!this.groupingApplied && !this.rollupApplied) {
                return false;
            }
            return recordId && _.startsWith(recordId, 'group-');
        },

        _forceViewRender : function () {
            var that = this;
            _.forEach(that.config.rowsets, function (rowset) {
                that.view.renderRowset(rowset.id, true);
            });
        },

        _updateRecordIds : function (recordIdMap) {
            var that = this;
            if (that.options.tree || this.options.data != null) {
                var allRowsetNodes = _.pluck(_.values(that.view.virtualRows), 'nodes');
                _.forEach(recordIdMap, function (newId, oldId) {
                    if (newId !== oldId) {
                        // Update the node data
                        _.forEach(allRowsetNodes, function (rowsetNodes) {
                            var node = rowsetNodes[oldId];
                            if (node != null) {
                                node.id = newId;
                                rowsetNodes[newId] = node;
                                delete rowsetNodes[oldId];
                            }
                        });
                    }
                });
            }

            _.forEach(recordIdMap, function (newId, oldId) {
                if (newId !== oldId) {
                    that.siblingIds[newId] = that.siblingIds[oldId];
                    delete that.siblingIds[oldId];

                    that.parentIds[newId] = that.parentIds[oldId];
                    delete that.parentIds[oldId];
                }
            });
            var updatedIds = _.keys(recordIdMap);
            _.forEach(_.keys(that.siblingIds), function (key) {
                if (_.includes(updatedIds, that.siblingIds[key])) {
                    that.siblingIds[key] = recordIdMap[key];
                }
            });
            _.forEach(_.keys(that.parentIds), function (key) {
                if (_.includes(updatedIds, that.parentIds[key])) {
                    that.parentIds[key] = recordIdMap[key];
                }
            });

            if (that.groupingApplied) {
                this.recordGrouper.updateRecordIds(recordIdMap);
            }

            if (that._hasFilters()) {
                _.forEach(recordIdMap, function (newId, oldId) {
                    if (newId !== oldId) {
                        that.matchingRecords[newId] = that.matchingRecords[oldId];
                        that.matchingRecords.recordOrder[that.matchingRecords.recordOrder.indexOf(oldId)] = newId;
                        delete that.matchingRecords[oldId];
                    }
                });
            }
        },

        _silkFindRecord : function (dataIndex, value) {
            var that = this;
            var found = false;
            var column = this.columnDefinitions[dataIndex];
            var checkValue = function (storeValue, expectedValue) {
                return storeValue === expectedValue || (expectedValue === column.formatter.format(storeValue, column));
            };

            function checkChildren(record) {
                var children = that._getChildRecords(that._getRecordId(record));
                if (children == null) {
                    return false;
                }
                for (var i = 0; i < children.length; i++) {
                    var child = children[i];
                    if (checkValue(that.store.getValueFromRecord(child, dataIndex), value)) {
                        that.selectCell(that._getRecordId(child), dataIndex);
                        return true;
                    }
                    if (checkChildren(child)) {
                        return true;
                    }
                }
            }

            this.store.iterateStore(function (record) {
                if (!found && checkValue(that.store.getValueFromRecord(record, dataIndex), value)) {
                    that.selectCell(that._getRecordId(record), dataIndex);
                    found = true;
                }
                if (that.options.tree && !found) {
                    if (checkChildren(record)) {
                        found = true;
                    }
                }
            }, this);
            return found;
        },

        checkActionState : function () {
            var actionState = this._checkActions();
            this._trigger('action-state-update', null, actionState);
            return actionState;
        },

        _preCheckActions : function (selection, focus) {
            var that = this;
            if (selection == null) {
                selection = this.getSelection();
            }
            if (focus == null) {
                focus = that.focus.record;
            }
            var state = {
                groupSelected : false,
                selectedRecords : selection,
                hasSelection : selection.length > 0,
                focusedRecord : focus,
                sorted : this.config.sortBy.length > 0,
                dummySelected : false
            };

            if (focus != null) {
                state.selectedId = this._getRecordId(focus);
                state.rowsetId = this._getRecordPlacement(focus).rowsetId;
            }

            if (state.hasSelection) {
                state.dummySelected = _.includes(selection, this.dummyRecord);
            }

            for (var i = 0; i < selection.length; i++) {
                if (this._isGroupRecord(selection[i])) {
                    state.groupSelected = true;
                    break;
                }
            }

            state.canEditCells = _.once(function (records, dataIndex) {
                return that.canEditCells(records, dataIndex);
            });

            return state;
        },

        _checkActions : function () {
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            var state = this._preCheckActions();
            var actionsMap = {
                cutRows : this._checkAction('cutRows', state),
                copyRows : this._checkAction('copyRows', state),
                pasteRows : this._checkAction('pasteRows', state),
                newRow : this._checkAction('newRow', state),
                deleteRows : this._checkAction('deleteRows', state),
                moveRowUp : this._checkAction('moveRowUp', state),
                moveRowDown : this._checkAction('moveRowDown', state),
                moveRowLeft : this._checkAction('moveRowLeft', state),
                moveRowRight : this._checkAction('moveRowRight', state),
                newChildRow : this._checkAction('newChildRow', state)
            };

            if (this.options.undo) {
                actionsMap.undo = this._checkAction('undo', state);
                actionsMap.redo = this._checkAction('redo', state);
            }
            //>>excludeStart("perf", pragmas.production)
            if (this.options.debug) {
                this._trigger('performance-debug', null, {
                    start : start,
                    end : window.performance.now(),
                    name : 'checkActions',
                    type : 'widget'
                });
            }
            //>>excludeEnd("perf")
            return actionsMap;
        },

        _getFirstRecordId : function () {
            return this.view.findFirstRow(true);
        },

        _getDummyRecord : function () {
            if (this.dummyRecord == null) {
                this.dummyRecord = {};
                this.dummyRecord[this.options.recordKeys.id] = constants.DUMMY_RECORD_ID;
            }
            return this.dummyRecord;
        },

        _getProxyRecord : function () {
            if (this.proxyRecord != null) {
                return this._getRecord(this._getRecordId(this.proxyRecord));
            } else {
                return null;
            }
        },

        _resetProxyRecord : function () {
            var proxy = this._getProxyRecord();
            if (this.proxyRecord != null && proxy != null) {
                this.store.deleteRecords([proxy], { isTemp : true });
            }
            this.proxyRecord = null;
            this.dummyRecord = null;
            this.dummyModified = false;
        },

        _newProxyRecord : function () {
            if (this.options.quickAdd.enabled) {
                this.proxyRecord = this.store.newRecord({
                    focus : this._getQuickAddFocus(),
                    dummyFocus : true
                });
                this.bypassRowAdd = true;
                this.store.addRecords([this.proxyRecord], { isTemp : true });
                this.bypassRowAdd = false;
            }
            return this.proxyRecord;
        },

        generateProxyRecord : function () {
            this._resetProxyRecord();
            this._newProxyRecord();
            this.view.updateRow(constants.DUMMY_RECORD_ID, constants.QUICK_ADD_ROWSET_ID);
        },

        _getQuickAddFocus : function () {
            var focus = this._getDummyRecord();
            var firstRecordId;
            var lastRecord;

            if (!this.options.quickAdd.addRecordAtTop) {
                lastRecord = this.getLastRecord();
                if (lastRecord != null) {
                    focus = lastRecord;
                }
            }
            if (this.options.tree) {
                firstRecordId = this.view.findFirstRow(true);
                if (firstRecordId != null) {
                    focus = this._getRecord(firstRecordId);
                }
            }
            return focus;
        },

        _checkAction : function (action, state, invocationOptions) {
            //>>excludeStart("perf", pragmas.production)
            var start;
            if (this.options.debug) {
                start = window.performance.now();
            }
            //>>excludeEnd("perf")
            if (state == null) {
                state = this._preCheckActions();
            }
            var that = this;
            var selectedRecords = state.selectedRecords;
            var focusedRecord = state.focusedRecord;

            if (!this.options.editable || this.options.disabled) {
                return false;
            }
            if (state.groupSelected || state.dummySelected) {
                switch (action) {
                case 'cutRows' :
                case 'copyRows' :
                case 'deleteRows' :
                case 'cutCells' :
                case 'moveRowLeft' :
                case 'moveRowRight' :
                case 'moveRowUp' :
                case 'moveRowDown' :
                    return false;
                }
            }

            var hasSelection = state.hasSelection;
            var sorted = state.sorted;
            var canDoAction = false;
            var checkActionArgs = {};
            // Allow store to globally prevent actions
            if (this.store.preventAction != null && this.store.preventAction({
                action : action,
                sorted : state.sorted,
                selection : selectedRecords,
                focusedRecord : focusedRecord,
                invocationOptions : invocationOptions
            })) {
                return false;
            }
            switch (action) {
            case 'exportData' :
                canDoAction = true;
                break;
            case 'cutRows' :
                canDoAction = hasSelection && (this.store.canCutRecords == null ||
                            this.store.canCutRecords(selectedRecords));
                break;
            case 'copyRows' :
                canDoAction = hasSelection && (this.store.canCopyRecords == null ||
                            this.store.canCopyRecords(selectedRecords));
                break;
            case 'pasteRows' :
                if (this.clipboard.scope !== 'row') {
                    return false;
                }
                // If cut make sure you are not pasting an node into one of its children
                if (this.clipboard.mode === 'cut' && this.options.tree && this.clipboard.records.length > 0) {
                    var cutIds = [];
                    _.forEach(this.clipboard.records, function (cutRecord) {
                        cutIds.push(that._getRecordId(cutRecord));
                    });
                    if (focusedRecord === null) {
                        var firstRecordId = this.view.findFirstRow(true);
                        focusedRecord = this._getRecord(firstRecordId);
                    }
                    if (this.view.ancestorsBanned(this._getRecordId(focusedRecord), cutIds)) {
                        return false;
                    }
                }
                if (invocationOptions != null && invocationOptions.isChild && that.options.treeMaxLevel != null) {
                    canDoAction = that._getNodeLevel(focusedRecord) < this.options.treeMaxLevel;
                } else {
                    canDoAction = true;
                }
                canDoAction = canDoAction && this.clipboard.records.length > 0 && (this.store.canPasteRecords == null ||
                            this.store.canPasteRecords(_.clone(this.clipboard), focusedRecord, selectedRecords));
                break;

            case 'copyCells' :
                checkActionArgs.records = selectedRecords;
                checkActionArgs.dataIndex = this.focus.columnId;
                canDoAction = hasSelection && (this.store.canCopyCells == null ||
                            this.store.canCopyCells(checkActionArgs));
                break;

            case 'editCells' :
                checkActionArgs.records = selectedRecords;
                checkActionArgs.dataIndex = this.focus.columnId;
                canDoAction = hasSelection && state.canEditCells(checkActionArgs.records, checkActionArgs.dataIndex) && (this.store.canEditCells == null ||
                            this.store.canEditCells(selectedRecords, this.focus.columnId, checkActionArgs));
                break;

            case 'cutCells' :
                checkActionArgs.records = selectedRecords;
                checkActionArgs.dataIndex = this.focus.columnId;
                canDoAction = hasSelection && state.canEditCells(checkActionArgs.records, checkActionArgs.dataIndex) && (this.store.canCutCells == null ||
                            this.store.canCutCells(checkActionArgs));
                break;

            case 'pasteCells' :
                if (this.clipboard.scope !== 'cell') {
                    return false;
                }
                canDoAction = hasSelection && this.clipboard.cellValues != null && this.clipboard.cellValues.length > 0;
                canDoAction = canDoAction &&
                            this.columnDefinitions[this.focus.columnId].displayType === this.columnDefinitions[this.clipboard.cellIndex].displayType;
                checkActionArgs.clipboard = this.clipboard;
                checkActionArgs.records = selectedRecords;
                checkActionArgs.dataIndex = this.focus.columnId;

                canDoAction = canDoAction && state.canEditCells(checkActionArgs.records, checkActionArgs.dataIndex) &&
                            (this.store.canPasteCells == null || this.store.canPasteCells(checkActionArgs));
                break;
            case 'deleteCells' :
                checkActionArgs.records = selectedRecords;
                checkActionArgs.dataIndex = this.focus.columnId;
                var column = this.columnDefinitions[checkActionArgs.dataIndex];
                var allowClearCells = column ? column.allowClearCells : false;
                canDoAction = allowClearCells && hasSelection && state.canEditCells(checkActionArgs.records, checkActionArgs.dataIndex) &&
                            (this.store.canDeleteCells == null ||
                            this.store.canDeleteCells(selectedRecords, this.focus.columnId, checkActionArgs));
                break;
            case 'newRow' :
                if (invocationOptions != null && invocationOptions.isChild && that.options.treeMaxLevel != null) {
                    var selectedLevel = that._getNodeLevel(focusedRecord);
                    canDoAction = selectedLevel < this.options.treeMaxLevel;
                } else {
                    canDoAction = true;
                }
                canDoAction = canDoAction && (this.store.canAddRecord == null ||
                            this.store.canAddRecord(focusedRecord, selectedRecords, invocationOptions));
                break;
            case 'deleteRows' :
                canDoAction = hasSelection && (this.store.canDeleteRecords == null ||
                            this.store.canDeleteRecords(selectedRecords));
                break;
            case 'moveRowLeft' :
                canDoAction = this.options.tree && !this.groupingApplied && this.view.canMoveRecords(selectedRecords, state.rowsetId, 'left') &&
                            (this.store.canMoveRecordsLeft == null || this.store.canMoveRecordsLeft(selectedRecords));
                break;
            case 'moveRowRight' :
                canDoAction = this.options.tree && !this.groupingApplied && this.view.canMoveRecords(selectedRecords, state.rowsetId, 'right');
                if (canDoAction && this.store.canMoveRecordsRight != null) {
                    var newParentId = this.view.previousSibling(this._getRecordId(_.first(this.view.orderByViewIndex(selectedRecords))));
                    var args = {
                        newParentId : newParentId,
                        records : selectedRecords
                    };
                    canDoAction = this.store.canMoveRecordsRight(selectedRecords, args);
                }
                break;
            case 'moveRowUp' :
                canDoAction = !sorted && !this.groupingApplied && this.view.canMoveRecords(selectedRecords, state.rowsetId, 'up') &&
                            (this.store.canMoveRecordsUp == null || this.store.canMoveRecordsUp(selectedRecords));
                break;
            case 'moveRowDown' :
                canDoAction = !sorted && !this.groupingApplied && this.view.canMoveRecords(selectedRecords, state.rowsetId, 'down') &&
                            (this.store.canMoveRecordsDown == null || this.store.canMoveRecordsDown(selectedRecords));
                break;
            case 'newChildRow' :
                canDoAction = (this.options.tree || this.groupingApplied) && hasSelection;
                if (canDoAction && that.options.treeMaxLevel != null) {
                    canDoAction = that._getNodeLevel(focusedRecord) < this.options.treeMaxLevel;
                }
                if (canDoAction && state.groupSelected && this.options.tree === false) {
                    if (this.store.canAddRecord != null) {
                        canDoAction = this.store.canAddRecord(focusedRecord, selectedRecords, invocationOptions);
                    }
                } else if (canDoAction) {
                    canDoAction = (this.store.canAddChildRecord == null || this.store.canAddChildRecord(focusedRecord, selectedRecords, invocationOptions));
                }
                break;
            case 'undo' :
                canDoAction = this.options.undo && this.undoStack.length > 0;
                break;
            case 'redo' :
                canDoAction = this.options.undo && this.redoStack.length > 0;
                break;
            case 'collapseAllRows':
                canDoAction = this.options.tree || this.groupingApplied;
                canDoAction = canDoAction && _.some(_.map(this.view.virtualRows, function (virtualRowSet) {
                    return virtualRowSet.nodes;
                }), function (rowsetNodes) {
                    return _.some(rowsetNodes, function (node) {
                        return node.isNode && node.expanded;
                    });
                });
                break;
            case 'expandAllRows' :
                canDoAction = this.options.tree || this.groupingApplied;
                canDoAction = canDoAction && _.some(_.map(this.view.virtualRows, function (virtualRowSet) {
                    return virtualRowSet.nodes;
                }), function (rowsetNodes) {
                    return _.some(rowsetNodes, function (node) {
                        return node.isNode && !node.expanded;
                    });
                });
                break;
            case 'collapseRows' :
                    /* falls through */
            case 'expandRows' :
                canDoAction = this.options.tree || this.groupingApplied;
                canDoAction = canDoAction && this.view.checkRecordsHaveChildren(selectedRecords, state.rowsetId);
                break;

            }
            //>>excludeStart("perf", pragmas.production)
            if (this.options.debug) {
                this._trigger('performance-debug', null, {
                    start : start,
                    end : window.performance.now(),
                    name : 'checkAction',
                    type : 'widget',
                    info : action
                });
            }
            //>>excludeEnd("perf")
            return canDoAction;
        },

        /**
         * Check if the user can filldown a cell.
         * @method pgbu.grid#_canFillDown
         * @public
         */
        _canFillDown : function (args) {
            var canFillDown = this.options.filldown && args.columnDefinition.filldown;
            if (canFillDown && this.store.canFillDown != null) {
                var record = this._getRecord(args.recordId);
                return this.store.canFillDown({
                    record : record,
                    dataIndex : args.columnDefinition.dataIndex,
                    value : this.store.getValueFromRecord(record, args.columnDefinition.dataIndex)
                });
            }
            return canFillDown;
        },

        /**
         * Used to unhide all of the errors in the grid.
         * @method grid#unhideAllErrors
         * @public
         */
        unhideAllErrors : function () {
            if (this.store.unhideAllErrors != null) {
                this.store.unhideAllErrors();
            }
        },

        /**
         * Gets the ID of the grid.
         * @method grid#getId
         * @returns {String} The id of the grid.
         * @public
         */
        getId : function () {
            return this.id;
        },

        syncGrid : function (gridToSync) {
            var that = this;
            if (this.syncedGrid == null) {
                this.syncedGrid = gridToSync;
                gridToSync.syncedGrid = this;
                this.element.on('grid-view-updated', function (event, args) {
                    switch (args.action) {
                    case 'column-resize' :
                        var option = {};
                        option[args.column] = { width : args.size };
                        that.syncedGrid._setOption('columns', option);
                        that.syncedGrid.resize();
                        break;
                    case 'group-resized':
                        break;
                    default :
                        break;
                    }
                });
            }
        },

        /**
         * Pauses the view manager from executing any actions
         * @method grid#pause
         * @public
         */
        pause : function () {
            this.viewManager.pause();
        },

        /**
         * Tells the view manager to proceed if instructed to refresh
         * @method grid#resume
         * @public
         */
        resume : function () {
            this.viewManager.resume();
            this.viewManager.refresh();
        }
    };

    $.widget(widgetNamespace + '.grid', grid);
    $(window).trigger('gridapi-loaded');
});
